
/**************************************************/
/*                                                */
/* File: DyCm.c                                   */
/* Copyright (C) 2003, D. Verkindt                */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include <fcntl.h> 
#include <stdarg.h>
#include <dirent.h>
#include <assert.h>
#include <unistd.h>
#include "dataDisplay.h"

#ifdef MEMWATCH
#    include "memwatch.h"
#endif


#ifndef NOCM

/*-------------------------------------------------- UFrHandlerCm ----*/

CmMessageStatus UFrHandlerCm (CmMessage message, char *sender, char *serverName)
{
 DyDebugPrint(__FUNCTION__,0);

 UFrCurFrameH = DyFdGetFrameWithSize(message,-1, &UFrCurFrameHSize);

 if (UFrCurFrameH == NULL)
    {printf("UFrHandlerCm> Could not read the frame received!!\n");
     return(CmMessageOk);
    }

 if (strcmp(UFrCurFrameH->name,"NO FRAME") == 0)
    {FrameFree(UFrCurFrameH);
     UFrCurFrameH = NULL;
    }
 else
    {if (((Dy->debugType)&(1<<DBG_CM)) && Dy->debug>0)
        {printf("UFrHandlerCm> Receive frame %d gps%d (%.2fkB) from Cm server %s \n",
                 UFrCurFrameH->frame, UFrCurFrameH->GTimeS, (double)UFrCurFrameHSize/1024. , sender);
        }
     if (Dy->output_flag == ON && Dy->Order == CONTINUE) DyOutputWrite (UFrCurFrameH);
    }

/*  if (sender) strcpy(Dy->CmServer, sender); */

 return(CmMessageBreak);
}

/*------------------------------------------ DyCmWait ---*/

CmConnectCondition DyCmWait (int timeout, char *servername)
{
 int timeout2, iwait;
 CmConnectCondition condition;
 char dummy[LSTRING_MAX];

 fl_activate_object(mainpanel->stop);

 if (timeout == 0) timeout = 99999999;

 if (timeout < 11) timeout2 = timeout / 2;
 else              timeout2 = 10;

 iwait = 1;
 Dy->Order = WAIT;
 while (Dy->Order != STOP)
    {
     condition = CmMessageWaitWithTimeout(1.);
     if (condition == CmConnectBreakDetection) break;
     if (iwait%timeout2 == 0) printf("DyCmWait> No answer from %s since %ld seconds (timeout=%ds) \n", servername, iwait, timeout);
     iwait++;
     if (iwait > timeout) Dy->Order = STOP;
     if (timeout>5 && iwait == 5)
        {fl_set_oneliner_font(FL_TIMESBOLD_STYLE, FL_LARGE_SIZE);
         sprintf(dummy,"Push \042stop\042 button to stop waiting for an answer from %s", servername);
         fl_show_oneliner(dummy, XREF-mainpanel->mainpanel->w-30,YREF-mainpanel->mainpanel->h+30);
        }
     fl_check_forms();
    }

 fl_hide_oneliner();
 fl_set_oneliner_font(FL_TIMESBOLD_STYLE, FL_NORMAL_SIZE);

 return(condition);
}

#endif


/*------------------------------------------ DyCmConnect ---*/

int DyCmConnect (char *CmName)
{
 char localName[LSTRING_MAX], localName2[LSTRING_MAX];

#ifndef NOCM

 DyDebugPrint(__FUNCTION__,0);

 DyDebugTimeNeeded(__FUNCTION__,"start",0);

 /*--- Check that dataDisplay not already Cm connected ---*/

 if (Dy->CmConnect == ON)
    {printf("DyCmConnect> My current CmName is %s: I first disconnect from %s.\n", CmName, Dy->CmServer);
     DyCmDisconnect(CmName);
    }

 /*--- Change CmDomain if requested ---*/

  DyCmSetCmDomain(Dy->CmDomain);

 /*--- if dynamical connection, use a self-made Cm name, otherwise use CmName ---*/

 if (CmName == NULL  ||  (CmName != NULL  &&  strncmp(CmName,DDISPLAY,strlen(DDISPLAY)) == 0))
    {sprintf(localName,"%s_%s_%s_%s_%d", DDISPLAY, getenv("USER"), getenv("HOST"), getenv("DISPLAY"), getpid()); }
 else
    {strcpy(localName, CmName);}

 /*--- Check that the Cm name built does not contains forbidden characters ---*/

 if (DyCheckString(localName))
    {strcpy(localName2, DyCheckString(localName));
     strcpy(localName,localName2);
    }

 /*--- Wait a couple of seconds to let finish previous dynamical connection ---*/

 if (time(NULL)-Dy->timeDisconnect < 2) sleep(2);


 /*--- Now do the Cm connection ---*/
 /*--- (in case of dynamical connection, the call to FdOutSendAdd is done elsewhere) ---*/

 if (!CmMessageOpenServer(localName))
     {printf("DyCmConnect> Could not declare myself with Cm Name %s \n", localName);
      DyInfoLine("This Cm Name may be already used. Try an other one.");
      return(-1);
     }

 /*--- Keep in Dy->CmName the Cm name used, put flag ON, install frame reception handler ---*/

 strcpy(Dy->CmName, localName);
 printf("DyCmConnect> My Cm Name is %s. Now connected.\n", Dy->CmName);
 Dy->CmConnect = ON;

 CmMessageInstallHandler (UFrHandlerCm, CMTYPE_FDFRAME);
 CmMessageInstallHandler (DyInfoHandler, CMTYPE_DYINFO);

 DyDebugTimeNeeded(__FUNCTION__,"stop",0);

#endif

 return(0);
}


/*------------------------------------------ DyCmDisconnect ---*/

void DyCmDisconnect (char *CmName)
{
 char clientName[LSTRING_MAX];

#ifndef NOCM

 DyDebugPrint(__FUNCTION__,0);

 if (Dy->CmConnect == OFF)  return;

 CmMessageUninstallHandler (CMTYPE_FDFRAME);
 CmMessageUninstallHandler (CMTYPE_DYINFO);

 strcpy(clientName,CmConnectGetName(CmConnectWhoAmI()));
 CmMessageCloseServer(clientName);
 printf("DyCmDisconnect> My Cm Name was %s. Now Disconnected.\n", clientName);

 Dy->timeDisconnect = time(NULL);

 Dy->CmConnect = OFF;

#endif

 return;
}


/*------------------------------------------ DyUpdateCmList ---*/

void DyUpdateCmList()
{
#ifndef NOCM

 CmMessage message;
 CmConnectCondition condition;
 CmConnect server;
 char* destination, clientName[LSTRING_MAX];
 int connected=OFF;

 DyDebugPrint(__FUNCTION__,0);

 DyCmSetCmDomain (Dy->CmDomain);

 if (Dy->CmConnect == OFF)
    {if (!CmMessageOpenMultipleServer("DyCmNamesAsker"))
            {printf("DyUpdateCmList> Cannot declare myself with Cm Name DyCmNamesAsker_1 \n");
             return;
            }
     strcpy(clientName,CmConnectGetName(CmConnectWhoAmI()));
     printf("DyUpdateCmList> Now connected. My temporary Cm Name is %s \n",clientName);
     connected = ON;
    }

 CmMessageInstallHandler(DyNamesHandler,"NSNames");

 server      = CmConnectGetNameServer();
 destination = CmConnectGetName(server);

 message = CmMessageNew();
 CmMessageSetType(message, "NSGetNames");
 CmMessageSend(message, destination);
 CmMessageDelete(message);

 condition = DyCmWait (DY_TIMEOUT2, destination);

 if (condition == CmConnectTimeoutDetection)
    {DyInfoLine("Timeout reached. I could not update the list of Cm servers.");}

 CmMessageUninstallHandler("NSNames");

 if (connected == ON)
    {strcpy(clientName,CmConnectGetName(CmConnectWhoAmI()));
     CmMessageCloseServer(clientName);
     printf("DyUpdateCmList> my temporary Cm Name was %s. Now Disconnected\n",clientName);
    }

#endif

 return;
}


/*----------------------------------- DyCmSetCmDomain ---*/

void DyCmSetCmDomain (char* DyCmDomain)
{
 char cmdomain[LSTRING_MAX], dummy[LSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);

 if (DyCmDomain == NULL) return;
 if (strcmp(Dy->CmDomain, DyCmDomain) == 0) return;

 CmConnectSetDomain(Dy->CmDomain);
 CmNameServerGetInfos();
 
 strcpy(cmdomain,getenv("CMDOMAIN"));
 printf("DyCmSetCmDomain> BEFORE ******* cmdomain=%s  DyCmDomain=%s\n", cmdomain, DyCmDomain);
 if (strcmp(cmdomain, DyCmDomain) != 0) {unsetenv("CMDOMAIN"); setenv("CMDOMAIN", DyCmDomain, 1);}
 strcpy(cmdomain,getenv("CMDOMAIN"));
 printf("DyCmSetCmDomain> AFTER ******* cmdomain=%s  DyCmDomain=%s \n", cmdomain, DyCmDomain);

 strcpy(Dy->CmDomain, DyCmDomain);

 return;
}


#ifndef NOCM

/*----------------------------------- DyNamesHandler ---*/

CmMessageStatus DyNamesHandler (CmMessage message, char* sender, char* serverName)
{
 const char *name;
 int ncm;

 DyDebugPrint(__FUNCTION__,0);

 if (Dy->DdNS == ON)
    {
     fl_clear_menu(getdd->datasender_list);
     while (CmMessageGetItemType (message) == CmMessageItemText)
        {name = CmMessageGetText (message);
         if (strncmp(name,DDSERVER,10) == 0)  fl_addto_menu(getdd->datasender_list, name);
        }
    }

 else
    {
     fl_clear_browser(cmservers->liste);
     while (CmMessageGetItemType (message) == CmMessageItemText)
        {name = CmMessageGetText (message);
         if (strcmp(name,"<unknown>") == 0) continue;
         if (strcmp(name,serverName) == 0)  continue;
         if (fl_get_button(cmservers->fullList) == ON)
            {fl_add_browser_line(cmservers->liste, name);}
         else
            {if (strstr(name, "Dy") != NULL)         fl_add_browser_line(cmservers->liste, name);
             else if (strstr(name, "Users") != NULL) fl_add_browser_line(cmservers->liste, name); 
             else if (strstr(name, "Shift") != NULL) fl_add_browser_line(cmservers->liste, name); 
            }
        }
     ncm = fl_get_browser_maxline(cmservers->liste);
     if (ncm > 0)  DyXformSortBrowser(cmservers->liste);
    }

 return (CmMessageBreak);
}


/*----------------------------------- DyInfoHandler ---*/
/* handler Cm to receive an alarm message sent to the dataDisplay user, or to trigger a check
or to produce a jpg picture of current canvas or to stop the dataDisplay */

CmMessageStatus DyInfoHandler (CmMessage message, char* sender, char* serverName)
{
 char dummy[LSTRING_MAX];
 char *infotype;
 int alarm=0;

 DyDebugPrint(__FUNCTION__,0);

 infotype = CmMessageGetText (message);
 alarm    = CmMessageGetInt(message);

 if (infotype == NULL) return(CmMessageOk);

 printf("DyInfoHandler> Received message from %s : %s \n", sender, infotype);

 if (strstr(infotype,"assword") && Dy->Ddpasswd) {free(Dy->Ddpasswd); Dy->Ddpasswd = NULL;}

 if (strcmp(infotype,"quit") == 0)        DyXformQuit(NULL,10);
 else if (strcmp(infotype,"checks") == 0) DyXformChecks(NULL,0);
 else if (strcmp(infotype,"jpeg") == 0)   {sprintf(dummy, "%s/jpeg/%s.jpg", DYLOG, Dy->curConfig); DySavePlot(dummy);}
 else                                     DyInfoPanel(infotype, alarm);

 return (CmMessageOk);
}

#endif


