
/**************************************************/
/*                                                */
/* File: DyConfig.c                               */
/* Copyright (C) 2003, D. Verkindt                */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include <fcntl.h> 
#include <stdarg.h>
#include <dirent.h>
#include <assert.h>
#include <unistd.h>
#include <limits.h>
#include "dataDisplay.h"
#include "URootAccess.h"

#ifdef MEMWATCH
#    include "memwatch.h"
#endif


/*                        *************                     */
/*                        *  CONFIGS  *                     */
/*                        *************                     */

/*----------------------------------------- DyXformSaveConfig ---*/

void DyXformSaveConfig (FL_OBJECT *obj, long arg)
{
 const char *str;
 char dummy[LSTRING_MAX];
 long nString;

 DyDebugPrint(__FUNCTION__,0);

 str = fl_show_input("Config filename", Dy->savedConfig);
 if (str) strcpy(Dy->savedConfig, (char *)str);

 nString = DyMakeSiestaFile(Dy->savedConfig,0);

 if (nString <= 0)
    {DyInfoLine("There is no configuration to save!");}
 else
    {sprintf(dummy,"Configuration saved in file %s", Dy->savedConfig);
     DyInfoLine (dummy);
    }

 return;
}


/*---------------------------------------- DyXformGetConfigFile ---*/

void DyXformGetConfigFile (FL_OBJECT *obj, long arg)
{
 char dummy[LSTRING_MAX], *buffer, pattern_str[LSTRING_MAX], *result, *str, carfile[LSTRING_MAX];
 int result_length;

 DyDebugPrint(__FUNCTION__,0);

 result = (char *)malloc(LSTRING_MAX);

 sprintf(pattern_str,"*%s", CONFIG_SUFFIX);
 result_length = ClXFSelector("Choose a new config file", Dy->config_dir, pattern_str, "", result, LSTRING_MAX);
 if (result_length == 0 || !result) return;
 strcpy(carfile,strtok(result," "));
 if (is_a_file(carfile) == 0) {printf("DyXformGetConfigFile> %s is not a file \n", carfile); return;}
 Dy->config_dir = DyGetDir(Dy->config_dir, carfile);
 printf("DyXformGetConfigFile> result=%s  carfile=%s  config_dir=%s \n", result, carfile, Dy->config_dir);
 free(result);

/**      
 sprintf(pattern_str,"*%s", CONFIG_SUFFIX);
 while (test)
    {carfile = fl_show_fselector("Choose a new config file",".",pattern_str,"");
     if (!carfile) return;
     if (is_a_file((char *)carfile)) test = 0;
     else printf("DyXformGetConfigFile> I need a config file \n");
    }
 DyXformRaz(NULL,0);
 buffer = DyReadConfigFile ((char *)carfile);
**/

 DyXformRaz(NULL,0);

 buffer = DyReadConfigFile (carfile);

 if (buffer != NULL)
    {
     DyGetConfig(buffer);
     free(buffer);
     sprintf(dummy, "Got a new configuration from file %s. Previous plots deleted.", carfile);
     DyInfoLine (dummy);
    }

 return;
}


/*----------------------------------------------- DyReadConfigFile ---*/

char *DyReadConfigFile (char *config_filename)
{
 FILE *fp;
 char *configbuf;
 char s;
 long n;

 DyDebugPrint(__FUNCTION__,0);

 DyDebugTimeNeeded(__FUNCTION__,"start",1);

 if (is_a_file(config_filename) == 0)
    {printf("\nDyReadConfigFile> %s not found\n", config_filename);
     return(NULL);
    }

 fp = fopen(config_filename,"r");
 if (fp == NULL)
    {printf("\nDyReadConfigFile> Could not open %s \n", config_filename);
     return(NULL);
    }

 Dy->config_dir = DyGetDir(Dy->config_dir,config_filename);

 /*--- Get size of config buffer, then malloc ---*/

 n = 0;
 while(!feof(fp))
    {s = getc(fp); 
     n++;
    }
 fclose(fp);
 configbuf = (char *)calloc(n+1,sizeof(char));
 strcpy(configbuf,"");

 /*--- Fill in the config buffer ---*/

 fp = fopen(config_filename,"r");
 while((s = getc(fp)) != EOF)  sprintf(configbuf,"%s%c",configbuf,s);
 fclose(fp);
 sprintf(configbuf,"%s%c",configbuf,'\0');

 DyDebugTimeNeeded(__FUNCTION__,"stop",1);

 return(configbuf);
}


/*----------------------------------------- DyGetConfig ---*/

void DyGetConfig (char *config)
{
 char name[LSTRING_MAX];
 const char *fileName;
 long nfile, nplot, nclock, nframe;
 char *current, **wordlist;
 long length, commentFlag, nword, i, nameLength;
 char* cmacrostart;
 char* cmacroend;
 char* cmacrocopy;
 char *algostart, *algoend, *algotext;
 char algoname[LSTRING_MAX], algofile[LSTRING_MAX], dummy[LSTRING_MAX];
 FILE *fpalgo;

 DyDebugPrint(__FUNCTION__,0);

 wordlist = (char **)calloc(LBIGSTRING_MAX,sizeof(char *));
 length   = strlen(config);

 if (length <= 0)
    {printf("\nDyGetConfig> Configuration file is empty.\n\n");
     return;
    }

 /*--- Get the list of words of the configuration string ---*/

 commentFlag = 0;
 nword       = 0;
 current     = config;

 while (current-config < length)
    {
     sscanf(current,"%s",name);

     nameLength = strlen(name);

     if ((name[0] == '\042'  ||  name[nameLength-1] == '\042')  &&  nameLength > 1)
        {
         if (name[0] == '\042' &&  name[nameLength-1] == '\042')
            {
             wordlist[nword] = (char *)calloc(2, sizeof(char));
             memcpy(wordlist[nword],"\042",1);
             nword++;
             wordlist[nword] = (char *)calloc(nameLength-1, sizeof(char));
             memcpy(wordlist[nword], &name[1], nameLength-2);
             wordlist[nword][nameLength-2] = '\0';
             nword++;
             wordlist[nword] = (char *)calloc(2, sizeof(char));
             memcpy(wordlist[nword],"\042",1);
             nword++;
            }

         else if (name[0] == '\042')
            {wordlist[nword] = (char *)calloc(2, sizeof(char));
             strncpy(wordlist[nword],"\042",1);
             nword++;
             wordlist[nword] = (char *)calloc(nameLength, sizeof(char));
             memcpy(wordlist[nword], &name[1], nameLength-1);
             wordlist[nword][strlen(name)-1] = '\0';
             nword++;
            }

         else if (name[nameLength-1] == '\042')
            {wordlist[nword] = (char *)calloc(nameLength, sizeof(char));
             memcpy(wordlist[nword], name, nameLength-1);
             nword++;
             wordlist[nword] = (char *)calloc(2, sizeof(char));
             memcpy(wordlist[nword],"\042",1);
             nword++;
            }
         current += nameLength+1;
        }

     else
        {
         wordlist[nword] = (char *)calloc(nameLength+1, sizeof(char));
         memcpy(wordlist[nword],name,nameLength);
         wordlist[nword][nameLength] = '\0';
         if (strstr(wordlist[nword],"/*") != NULL  &&
             strstr(wordlist[nword],"/*>") == NULL && strstr(wordlist[nword],"/**>") == NULL && 
             strstr(wordlist[nword],"/*UFrIFileDd") == NULL) commentFlag = 1;
         while(*current==' ' || *current=='\n')            current++;
         current += nameLength+1;
         while(*current==' ' || *current=='\n')            current++;
         if (commentFlag == 0) nword++;
         else if (strstr(wordlist[nword],"*/") != NULL  && strstr(wordlist[nword],"<*/") == NULL  &&
                  strstr(wordlist[nword],"<**/") == NULL)   commentFlag = 0;
        }
    }


 /*--- scan the list of configuration words ---*/

 nclock   = 0;
 nfile    = 0;
 nframe   = 0;
 nplot    = 1+fl_get_browser_maxline(mainbrowser->plots);

 for (i=0; i<nword; i++)
    {
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>3) printf("DyGetConfig>  wordlist[%ld]=%s \n", i, wordlist[i]);

     if (strcmp(wordlist[i],"UJclock") == 0)
        {nclock  = DyGetClocksFromConfig (wordlist, &i, nword);
         if (nclock == 0)
            {printf("DyGetConfig> This is not a dataDisplay configuration file \n");
             return;
            }
        }
     if (strcmp(wordlist[i],"UFrWave") == 0)
        {Dy->InputType = INWAVE;
         nfile = DyGetFilesFromConfig (wordlist, &i, nword, Dy->InputType);
        }
     if (strcmp(wordlist[i],"UFrIFile") == 0)
        {Dy->InputType = INFILE;
         nfile = DyGetFilesFromConfig (wordlist, &i, nword, Dy->InputType);
        }
     if (strcmp(wordlist[i],"UFrIFileOnline") == 0)
        {Dy->InputType = INONLINE;
         nfile = DyGetFilesFromConfig (wordlist, &i, nword, Dy->InputType);
        }
     if (strcmp(wordlist[i],"UFrIFileCm") == 0)
        {Dy->InputType = INCM;
         nfile = DyGetFilesFromConfig (wordlist, &i, nword, Dy->InputType);
        }
     if (strcmp(wordlist[i],"/*UFrIFileDd") == 0)
        {Dy->InputType = INDD;
         nfile = DyGetFilesFromConfig (wordlist, &i, nword, Dy->InputType);
        }
     if (strcmp(wordlist[i],"UFrUnpack") == 0)
        {nplot += DyGetPlotFromConfig (wordlist, &i, nword, nplot);}

#ifdef USE_VEGA
     if (strcmp(wordlist[i],"UFrChannel") == 0)
        {Dy->InputType = INVEGA;
         nfile = DyGetFilesFromConfig (wordlist, &i, nword, Dy->InputType);
        }
#endif
     if (strcmp(wordlist[i],"UFrShm") == 0)
        {Dy->InputType = INSHM;
         nfile = DyGetFilesFromConfig (wordlist, &i, nword, Dy->InputType);
        }

     if (strncmp(wordlist[i],"/*>",3) == 0)
        {nframe = DyGetStartEndFromConfig(wordlist, &i, nword);}

     if (strncmp(wordlist[i],"/**>",4) == 0)
        {DyGetRowColFromConfig(wordlist, &i, nword);}
    }

 if (nframe == 0)
    {printf("DyGetConfig>  no start and end frame available \n");
     Dy->startFrame =  0;
     Dy->startTime  =  0;
     Dy->duration   = -1;
     fl_set_input (mainpanel->startFrame,"0");
     fl_set_input (mainpanel->startTime,"0");
     fl_set_input (mainpanel->startUTCdate, "0");
    }

 /*--- Get data offline from wave audio file ---*/

 if (Dy->InputType == INWAVE)
    {
     fileName = fl_get_browser_line(inputfile->file_list,nfile);
     DyReadFirstFrame((char *)fileName, Dy->InputType);
    }

 /*--- Get frames offline from files ---*/

 if (Dy->InputType == INFILE)
    {
     fileName = fl_get_browser_line(inputfile->file_list,nfile);
     DyReadFirstFrame((char *)fileName, Dy->InputType);
    }

 /*--- Get frames online from files ---*/

 else if (Dy->InputType == INONLINE)
    {
     DyXformAddDir(NULL, 11);
    }

 /*--- Get frames online from a Cm server ---*/

 else if (Dy->InputType == INCM)
    {
     if (strstr(Dy->CmName,"Shift") != 0)
        {if (strstr(Dy->CmName,"Det")  != 0)  strcpy(Dy->CmServer,"FbmDetShift");
         if (strstr(Dy->CmName,"Susp") != 0)  strcpy(Dy->CmServer,"FbmSuspShift");
         if (strstr(Dy->CmName,"Moni") != 0)  strcpy(Dy->CmServer,"FbmMoniShift");
         if (strstr(Dy->CmName,"Main") != 0)  strcpy(Dy->CmServer,"FbmMainShift");
        }
     if (DyCmConnect(Dy->CmName) == -1) return;
     DyReadFirstFrame(NULL, Dy->InputType);
     DyCmDisconnect(Dy->CmName);
    }

 /*--- Get frames online from a Dd server ---*/

 else if (Dy->InputType == INDD)
    {
     DyCmConnect(Dy->CmName);
     DyReadFirstFrame (NULL, Dy->InputType);
     DyCmDisconnect(Dy->CmName);
    }

#ifdef USE_VEGA
 /*--- Get frames offline from a frame channel ---*/

 if (Dy->InputType == INVEGA)
    {
     DyReadFirstFrame(NULL, Dy->InputType);
    }
#endif

 if (Dy->InputType == INSHM)
    {
     DyReadFirstFrame(NULL, Dy->InputType);
    }

 
 /*--- Build UserFct algoFiles from the information hidden inside the config file ---*/

 algostart = strstr(config,ALGO_BEGIN);
 while (algostart)
    {
     sscanf(algostart,"%s %s %s", dummy, algoname, algofile);
     fpalgo = fopen(algofile, "w");
     if (fpalgo == NULL)
        {sprintf(algofile, "%s/%s/%s.C", DYALGO_DIR, getenv("UNAME"), algoname);
         fpalgo = fopen(algofile, "w");
        }
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1) printf("DyGetConfig> recreating the file %s \n", algofile);

     algoend   = strstr(algostart,ALGO_END);
     algostart = strstr(algostart,"\n")+1;

     if (fpalgo != NULL)
        {
         algotext = (char *)malloc(algoend-algostart);
         strncpy(algotext,algostart,algoend-1-algostart);
         algotext[algoend-1-algostart]='\0';
        fprintf(fpalgo, "%s", algotext);
         free(algotext);
         fclose(fpalgo);
        }
     else
        {printf(" DyGetConfig> %s not recreated\n", algofile);}

     algostart = strstr(algoend,ALGO_BEGIN);
    }

 /*--- Now load the algo and get the opstring from each algo header ---*/

 algostart = strstr(config,ALGO_BEGIN);
 while (algostart)
    {sscanf(algostart,"%s %s %s", dummy, algoname, algofile);
     if (strstr(algofile,".C") != NULL) DyLoadScript (algoname);
     algoend   = strstr(algostart,ALGO_END);
     algostart = strstr(algoend,ALGO_BEGIN);
    }

 /*--- Get the additional channels which are results of signals operations ---*/

 for (i=0; i<nword; i++)
    {
     if (strcmp(wordlist[i],"UFrVectOp") == 0) DyGetVectOpFromConfig (wordlist, &i);
    }

 for (i=0; i<nword; i++) free(wordlist[i]);
 free(wordlist);

 /*--- Build a new canvas from the canvas config macro read in the config file */
 
 cmacrostart = strstr(config,"/*<");
 if (cmacrostart)
    {cmacrostart +=3;
     if (Dy->cmacrocopy) free(Dy->cmacrocopy);
     Dy->cmacrocopy = (char *)calloc(strlen(cmacrostart),sizeof(char));
     cmacroend = strstr(cmacrostart,">*/");
     strncpy(Dy->cmacrocopy,cmacrostart,(int)(cmacroend-cmacrostart));
    }

 return;
}


/*------------------------------------------- DyGetStartEndFromConfig ---*/

int DyGetStartEndFromConfig (char **wordlist, long *i, long nword)
{
 char dummy[LSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);

 (*i)++;
 strcpy(Dy->CmServer, wordlist[*i]);

 (*i)++;
 Dy->firstTime = atol(wordlist[*i]);

 (*i)++;
 Dy->startTime = atol(wordlist[*i]);
 if (Dy->startTimeCmd > 0) {Dy->startTime = Dy->startTimeCmd; Dy->startTimeCmd = 0;} /*--- If startTime provided on command line, take it ---*/
 sprintf(dummy, "%d", Dy->startTime);
 fl_set_input(mainpanel->startTime, dummy);
 fl_set_input(mainpanel->startUTCdate, FrStrUTC(Dy->startTime,0));

 (*i)++;
 Dy->duration = atol(wordlist[*i]);
 if (Dy->durationCmd > 0)  {Dy->duration  = Dy->durationCmd;  Dy->durationCmd  = 0;} /*--- If duration provided on command line, take it ---*/
 sprintf(dummy, "%ld", Dy->duration);
 fl_set_input (mainpanel->duration, dummy);

 (*i)++;
 Dy->startRun = atoi(wordlist[*i]);
 fl_set_input(mainpanel->startRun, wordlist[*i]);
 
 (*i)++;
 Dy->startFrame = atoi(wordlist[*i]);
 fl_set_input(mainpanel->startFrame, wordlist[*i]);

 (*i)++;
 

 return(1);
}


/*------------------------------------------- DyGetRowColFromConfig ---*/

int DyGetRowColFromConfig (char **wordlist, long *i, long nword)
{
 char dummy[LSTRING_MAX], name[LSTRING_MAX], filename[LSTRING_MAX];
 int l,j,jp,color;

 DyDebugPrint(__FUNCTION__,0);

 (*i)++;
 Dy->nCol = atol(wordlist[*i]);

 (*i)++;
 Dy->nRow = atol(wordlist[*i]);

 (*i)++;
 
 Dy->triggerType       = TRIG_CH;
 strcpy(Dy->trigAsciiFileName,"./triglist.txt");
 strcpy(Dy->trigEventName,"EVENT_SNR");
 Dy->triggerUsed       = OFF; fl_set_button(dyoptions->trigused, OFF);
 Dy->triggerPause      = OFF; fl_set_button(dyoptions->trigpause, OFF);
 Dy->plotsNotDisplayed = OFF; fl_set_button(dyoptions->noplot, OFF);
 Dy->squarePlot        = OFF; fl_set_button(dyoptions->squareplot, OFF);
 Dy->AutoDeselectPlot  = OFF; fl_set_button(dyoptions->autodeselectplot, OFF);
 Dy->AutoDeselectCh    = OFF; fl_set_button(dyoptions->autodeselectch, OFF);
 Dy->skipFrame         = OFF; fl_set_button(dyoptions->skipFrame, OFF);
 Dy->colorpalette      = 0;
 Dy->output_flag       = OFF;

 while (wordlist[*i] && strcmp(wordlist[*i], ">**/") != 0 && *i<nword)
    {
     if (strncmp(wordlist[*i], "trigeventname=",14) == 0) {sscanf(wordlist[*i],"trigeventname=%s", Dy->trigEventName);}
     if (strncmp(wordlist[*i], "trigasciifile=",14) == 0) {sscanf(wordlist[*i],"trigasciifile=%s", Dy->trigAsciiFileName);}
     if (strncmp(wordlist[*i], "triggertype=",12) == 0)   {sscanf(wordlist[*i],"triggertype=%d", &(Dy->triggerType));}
     if (strcmp(wordlist[*i], "triggerused") == 0)        {Dy->triggerUsed       = ON; fl_set_button(dyoptions->trigused, ON);}
     if (strcmp(wordlist[*i], "triggerpause") == 0)       {Dy->triggerPause      = ON; fl_set_button(dyoptions->trigpause, ON);}
     if (strcmp(wordlist[*i], "noplot") == 0)             {Dy->plotsNotDisplayed = ON; fl_set_button(dyoptions->noplot, ON);}
     if (strcmp(wordlist[*i], "squareplot") == 0)         {Dy->squarePlot = ON; fl_set_button(dyoptions->squareplot, ON);}
     if (strcmp(wordlist[*i], "autodeselectplot") == 0)   {Dy->AutoDeselectPlot  = ON; fl_set_button(dyoptions->autodeselectplot, ON);}
     if (strcmp(wordlist[*i], "autodeselectch") == 0)     {Dy->AutoDeselectCh    = ON; fl_set_button(dyoptions->autodeselectch, ON);}
     if (strcmp(wordlist[*i], "skipframe") == 0)          {Dy->skipFrame         = ON; fl_set_button(dyoptions->skipFrame, ON);}
     if (strncmp(wordlist[*i], "col2d=",6) == 0)          {sscanf(wordlist[*i],"col2d=%d", &(Dy->colorpalette)); DyColorPalette(NULL,Dy->colorpalette);}
     if (strcmp(wordlist[*i], "output_flag") == 0)        {Dy->output_flag       = ON; fl_set_button(dyoutput->output_flag, ON);}
     if (strncmp(wordlist[*i], "output_dir=",11) == 0)    {sscanf(wordlist[*i],"output_dir=%s", &(Dy->output_dir[0]));}
     if (strncmp(wordlist[*i], "output_nframe=",14) == 0) {sscanf(wordlist[*i],"output_nframe=%d", &(Dy->output_nframe_perfile));}
     if (strncmp(wordlist[*i], "output_tag=",11) == 0)    {sscanf(wordlist[*i],"output_tag=%s", &(Dy->output_tag[0])); DyFilterString(Dy->output_tag,'&');}
     if (strncmp(wordlist[*i], "ddpasswd=",9) == 0)       {Dy->Ddpasswd = (char *)malloc(64); sscanf(wordlist[*i],"ddpasswd=%s", &(Dy->Ddpasswd[0]));}
     if (strncmp(wordlist[*i], "ddref=",6) == 0)
        {DyXformRaz(NULL,6);
         jp = 6;
         for (j=6; j<strlen(wordlist[*i]); j++)
            {if (wordlist[*i][j] == '_' && wordlist[*i][j-1] == '_')
               {strncpy(name, &(wordlist[*i][jp]), j-jp-1);
                name[j-jp-1] = '\0';
                if (strlen(name) > 3)
                   {strcpy(filename,name);}
                else
                   {color = atoi(name);
                    DyXformAddToRefList(filename,color);
                    RA_ReadReferencePlot(filename,color);
                    Dy->refplot_dir = DyGetDir(Dy->refplot_dir, filename);
                   }
                jp = j+1;
               }
            }
        }
     (*i)++;
    }

 if (Dy->AutoConfFlag == ON) {sprintf(dummy,"rm -rf %s/Dy-*.gwf", Dy->output_dir); system(dummy);}

 return(Dy->nCol*Dy->nRow);
}


/*-------------------------------------- DyGetVectOpFromConfig ---*/

void DyGetVectOpFromConfig (char **wordlist, long *i)
{
 char *inputVect, className[LSTRING_MAX], dummy[LSTRING_MAX];
 int id;

 DyDebugPrint(__FUNCTION__,0);

 (*i)++;   /* read the clock number   */
 (*i)++;   /* read the id             */
 id = atoi(wordlist[*i]);

 (*i)++;   /* read className          */
 strcpy(className,wordlist[*i]);

 (*i)++;   /* read input vectors list */
 inputVect = (char *)malloc(strlen(wordlist[*i])+1);
 strcpy(inputVect,wordlist[*i]);

 (*i)++;   /* read input param list   */
/**
 if (strstr(wordlist[*i], "REALUNITS") != NULL) Plot[id]->realUnits == ON; 
**/

 (*i)++;   /* read input.frameH       */

 DyXformAddUserChannel (Dy->Frame, mainbrowser->proc, className, inputVect);
 free(inputVect);

 return;
}


/*-------------------------------------- DyGetClocksFromConfig ---*/

long DyGetClocksFromConfig (char **wordlist, long *i, long nword)
{
 long j;

 DyDebugPrint(__FUNCTION__,0);

 (*i)++;
 if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1) printf("DyGetClocksFromConfig> wordlist[%ld]=%s \n",*i,wordlist[*i]);

 if (strcmp(wordlist[*i],"masterClocks") != 0)
    {printf("DyGetClocksFromConfig> no masterClocks in config file. wordlist[%d]=%s\n", *i, wordlist[*i]);
     return(0);
    }

 (*i)++;
 if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1) printf("DyGetClocksFromConfig> wordlist[%ld]=%s \n",*i,wordlist[*i]);
 (*i)++;
 Dy->nClock = atoi(wordlist[*i]);
 if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1) printf("DyGetClocksFromConfig> wordlist[%ld]=%s \n",*i,wordlist[*i]);

 for (j=0; j < Dy->nClock; j++)
    {(*i)++;
     Dy->freqClock[j] = atof(wordlist[*i]);
    }

 if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1) printf("DyGetClocksFromConfig> wordlist[%ld]=%s \n",*i,wordlist[*i]);

 return(Dy->nClock);
}


/*------------------------------------------ DyGetFilesFromConfig ---*/

long DyGetFilesFromConfig (char **wordlist, long *i, long nword, long arg)
{
 long nfile, frame_clock, frameN0;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Get data offline from wave audio file ---*/

 if (arg == INWAVE)
    {
     nfile = 0;
     (*i)++;
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1) printf("DyGetFilesFromConfig> INWAVE> wordlist[%ld]=%s \n",*i,wordlist[*i]);
     if (strcmp(wordlist[*i],"input") != 0)
        {printf("  DyGetFilesFromConfig> no input.\n");
         return(0);
        }
     (*i)++;
     frame_clock = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1) printf("DyGetFilesFromConfig> INWAVE> wordlist[%ld]=%s \n",*i,wordlist[*i]);

     if (frame_clock != Dy->nClock-1)
        {if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>0) printf("DyGetFilesFromConfig> INWAVE> frame clock is not the last one \n");
         /* return(0); */
        }
     (*i)++;
     if (strcmp(wordlist[*i],"\042") == 0) 
        {
         while (strcmp(wordlist[*i+1],"\042") != 0)
            {
             (*i)++;
             DyXformAddToFileList (wordlist[*i]);
             nfile++;
             if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1) printf("DyGetFilesFromConfig> INWAVE> wordlist[%ld]=%s \n",*i,wordlist[*i]);
            }
         (*i)++;
        }
     else
        {DyXformAddToFileList (wordlist[*i]);
         if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1) printf("DyGetFileFromConfig> INWAVE> wordlist[%ld]=%s \n",*i,wordlist[*i]);
        }
    }

 /*--- Get frame offline from files ---*/

 if (arg == INFILE)
    {
     nfile = 0;
     (*i)++;
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1) printf("DyGetFilesFromConfig> INFILE> wordlist[%ld]=%s \n",*i,wordlist[*i]);
     if (strcmp(wordlist[*i],"input") != 0)
        {printf("  DyGetFilesFromConfig> no input.\n");
         return(0);
        }
     (*i)++;
     frame_clock = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1) printf("DyGetFilesFromConfig> INFILE> wordlist[%ld]=%s \n",*i,wordlist[*i]);

     if (frame_clock != Dy->nClock-1)
        {if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>0) printf("DyGetFilesFromConfig> INFILE> frame clock is not the last one \n");
         /* return(0); */
        }
     (*i)++;
     if (strcmp(wordlist[*i],"\042") == 0) 
        {
         while (strcmp(wordlist[*i+1],"\042") != 0)
            {
             (*i)++;
             DyXformAddToFileList (wordlist[*i]);
             nfile++;
             if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1) printf("DyGetFilesFromConfig> INFILE> wordlist[%ld]=%s \n",*i,wordlist[*i]);
            }
         (*i)++;
        }
     else
        {DyXformAddToFileList (wordlist[*i]);
         if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1) printf("DyGetFileFromConfig> INFILE> wordlist[%ld]=%s \n",*i,wordlist[*i]);
        }
    }

 /*--- Get frames online from files ---*/

 if (arg == INONLINE)
    {
     nfile = 0;
     (*i)++;
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1) printf("DyGetFileFromConfig> INONLINE> wordlist[%ld]=%s \n",*i,wordlist[*i]);
     if (strcmp(wordlist[*i],"input") != 0)
        {printf("  DyGetFilesFromConfig> no input.\n");
         return(0);
        }
     (*i)++;
     frame_clock = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1) printf("DyGetFileFromConfig> INONLINE> wordlist[%ld]=%s \n",*i,wordlist[*i]);

     if (frame_clock != Dy->nClock-1)
        {if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>0) printf("DyGetFilesFromConfig> INONLINE> frame clock is not the last one \n");
         /* return(0); */
        }
     (*i)++;
     Dy->Dir = my_strcpy(Dy->Dir, wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1) printf("DyGetFileFromConfig> INONLINE> wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     (*i)++;
     frameN0 = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1) printf("DyGetFileFromConfig> INONLINE> wordlist[%ld]=%s \n",*i,wordlist[*i]);
     fl_set_input(mainpanel->startTime, wordlist[*i]);
     nfile = frameN0 + 1;
    }

 /*--- Get frames online from a Cm server ---*/

 if (arg == INCM)
    {
     nfile = 0;
     (*i)++;
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1) printf("DyGetFileFromConfig> INCM> wordlist[%ld]=%s \n",*i,wordlist[*i]);
     if (strcmp(wordlist[*i],"input") != 0)
        {printf("  DyGetFilesFromConfig> no input.\n");
         return(0);
        }
     (*i)++;
     frame_clock = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1) printf("DyGetFileFromConfig> INCM> wordlist[%ld]=%s \n",*i,wordlist[*i]);
     if (frame_clock != Dy->nClock-1)
        {printf("  DyGetFilesFromConfig> INCM> frame clock is not the last one \n");
         /* return(0); */
        }
     (*i)++;
     (*i)++;
     strcpy (Dy->CmName, wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1) printf("DyGetFileFromConfig> INCM> wordlist[%ld]=%s \n",*i,wordlist[*i]);
    }

 /*--- Get frames online from a Dd server ---*/

 if (arg == INDD)
    {
     nfile = 0;
     (*i)++;
     strcpy (Dy->DdServer, wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1) printf("DyGetFileFromConfig> INDD> wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     strcpy (Dy->Ddinputfile, wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1) printf("DyGetFileFromConfig> INDD> wordlist[%ld]=%s \n",*i,wordlist[*i]);
    }

 /*--- Get frames from shared memory ---*/

 if (arg == INSHM)
    {
     nfile = 0;
     (*i)++;
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1) printf("DyGetFileFromConfig> INSHM> wordlist[%ld]=%s \n",*i,wordlist[*i]);
     if (strcmp(wordlist[*i],"input") != 0)
        {printf("DyGetFilesFromConfig> no input.\n");
         return(0);
        }
     (*i)++;
     frame_clock = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1) printf("DyGetFileFromConfig> INSHM> wordlist[%ld]=%s \n",*i,wordlist[*i]);

     if (frame_clock != Dy->nClock-1)
        {if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>0) printf("DyGetFilesFromConfig> INSHM> frame clock is not the last one \n");
         /* return(0); */
        }
     (*i)++;
     strcpy (Dy->shmName, wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1) printf("DyGetFileFromConfig> INSHM> wordlist[%ld]=%s \n",*i,wordlist[*i]);
    }

 return(nfile);
}


/*------------------------------------------ DyGetPlotFromConfig ---*/

long DyGetPlotFromConfig (char **wordlist, long *i, long nword, long id)
{
 char title[LSTRING_MAX], *suffix, *str, dummy[LSTRING_MAX];
 long tic0, ticmax, clockN;
 int smsPeriod;

 DyDebugPrint(__FUNCTION__,0);

 DyPlotInit(id,0);

 /*--- First, we look for parameters following keyword UFrUnpack ---*/

 (*i)++;
 if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>2) printf("DyGetPlotFromConfig> plotfirstunpack0: wordlist[%ld]=%s \n",*i,wordlist[*i]);

 (*i)++;
 if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>2) printf("DyGetPlotFromConfig> plotfirstunpack1: wordlist[%ld]=%s \n",*i,wordlist[*i]);
 Plot[id]->clockN    = atoi(wordlist[*i]);
 if (Plot[id]->clockN < 0) Plot[id]->clockN = - Plot[id]->clockN;

 Plot[id]->freqClock = Dy->freqClock[Plot[id]->clockN];

 (*i)++;
 if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("DyGetPlotFromConfig> plotfirstunpack2: wordlist[%ld]=%s \n",*i,wordlist[*i]);


 if (strncmp(wordlist[*i],"sms",3) == 0)
    {
     if (strncmp(wordlist[*i],"sms.",4) == 0)
        {sscanf(wordlist[*i], "sms.%s", Plot[id]->vVarx);
         if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>2) printf("DyGetPlotFromConfig> wordlist[%ld]=%s  vVarx=%s\n",*i,wordlist[*i], Plot[id]->vVarx);
        }
     else
        {sscanf(wordlist[*i], "sms%d.%s", &smsPeriod, Plot[id]->vVarx);
         Plot[id]->freqClock = 1./(double)smsPeriod;
         if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>2) printf("DyGetPlotFromConfig> wordlist[%ld]=%s smsPeriod=%d vVarx=%s\n",*i,wordlist[*i], smsPeriod, Plot[id]->vVarx);
        }
    }
 else if (strncmp(wordlist[*i],"adc.",4) == 0) sscanf(wordlist[*i], "adc.%s", Plot[id]->vVarx);
 else if (strncmp(wordlist[*i],"sim.",4) == 0) sscanf(wordlist[*i], "sim.%s", Plot[id]->vVarx);
 else if (strncmp(wordlist[*i],"rec.",4) == 0) sscanf(wordlist[*i], "rec.%s", Plot[id]->vVarx);
 else
    {if ((str=strstr(Plot[id]->vVarx,".")) != NULL) strcpy(wordlist[*i],&str[1]);
     else                                           sscanf(wordlist[*i], "%s", Plot[id]->vVarx);
    }

 (*i)++;
 if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1) printf("DyGetPlotFromConfig> plotfirstunpack3: wordlist[%ld]=%s \n",*i,wordlist[*i]);

 if (strcmp(wordlist[*i],"input.frameH") != 0)
    {printf("DyGetPlotFromConfig> no input.frameH \n");
     return(0);
    }


 /*--- if we have a second UFrUnpack (3D, 2D, TrFCT, COHE or CORR plot) ---*/

 (*i)++;

 if (strcmp(wordlist[*i],"UFrUnpack") == 0)
    {
     (*i)++;
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>2) printf("DyGetPlotFromConfig> plotsecondunpack0: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     (*i)++;
     Plot[id]->clockN2    = atoi(wordlist[*i]);
     if (Plot[id]->clockN2 < 0) Plot[id]->clockN2 = - Plot[id]->clockN2;

     Plot[id]->freqClock2 = Dy->freqClock[Plot[id]->clockN2];

     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>2) printf("DyGetPlotFromConfig> plotsecondunpack1: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     (*i)++;
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>2) printf("DyGetPlotFromConfig> plotsecondunpack2: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     if (strncmp(wordlist[*i],"sms",3) == 0)
        {if (strncmp(wordlist[*i],"sms.",4) == 0)
            {sscanf(wordlist[*i], "sms.%s", Plot[id]->vVary);
             if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>2) printf("DyGetPlotFromConfig> wordlist[%ld]=%s  vVary=%s\n",*i,wordlist[*i], Plot[id]->vVary);
            }
         else
            {sscanf(wordlist[*i], "sms%d.%s", &smsPeriod, Plot[id]->vVary);
             Plot[id]->freqClock2 = 1./(double)smsPeriod;
             if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>2) printf("DyGetPlotFromConfig> wordlist[%ld]=%s smsPeriod=%d vVary=%s\n",*i,wordlist[*i], smsPeriod, Plot[id]->vVary);
            }
        }
     else if (strncmp(wordlist[*i],"adc.",4) == 0) sscanf(wordlist[*i], "adc.%s", Plot[id]->vVary);
     else if (strncmp(wordlist[*i],"sim.",4) == 0) sscanf(wordlist[*i], "sim.%s", Plot[id]->vVary);
     else if (strncmp(wordlist[*i],"rec.",4) == 0) sscanf(wordlist[*i], "rec.%s", Plot[id]->vVary);
     else
        {if ((str=strstr(Plot[id]->vVary,".")) != NULL) strcpy(wordlist[*i],&str[1]);
         else                                           sscanf(wordlist[*i], "%s", Plot[id]->vVary);
        }

     (*i)++;
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>2) printf("DyGetPlotFromConfig> plotsecondunpack3: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     if (strcmp(wordlist[*i],"input.frameH") != 0)
        {printf("DyGetPlotFromConfig> no input.frameH \n");
         return(0);
        }
     (*i)++;

     if (Plot[id]->freqClock2 > Plot[id]->freqClock)
        {Plot[id]->freqClock12 = Plot[id]->freqClock2;
         if (Plot[id]->type == PLOT2D || Plot[id]->type == PLOT3D) Plot[id]->freqClock12 = Plot[id]->freqClock;
        }
     else
        {Plot[id]->freqClock12 = Plot[id]->freqClock;
         if (Plot[id]->type == PLOT2D || Plot[id]->type == PLOT3D) Plot[id]->freqClock12 = Plot[id]->freqClock2;
        }
    }


 /*--- if we have a third UFrUnpack (3D) ---*/

 if (strcmp(wordlist[*i],"UFrUnpack") == 0)
    {
     (*i)++;
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>2) printf("DyGetPlotFromConfig> plotthirdunpack0: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     (*i)++;
     Plot[id]->clockN3    = atoi(wordlist[*i]);
     if (Plot[id]->clockN3 < 0) Plot[id]->clockN3 = - Plot[id]->clockN3;

     Plot[id]->freqClock3 = Dy->freqClock[Plot[id]->clockN3];

     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>2) printf("DyGetPlotFromConfig> plotthirdunpack1: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     (*i)++;
      if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>2) printf("DyGetPlotFromConfig> plotthirdunpack2: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     if (strncmp(wordlist[*i],"sms",3) == 0)
        {if (strncmp(wordlist[*i],"sms.",4) == 0)
            {sscanf(wordlist[*i], "sms.%s", Plot[id]->vVarz);
            }
         else
            {sscanf(wordlist[*i], "sms%d.%s", &smsPeriod, Plot[id]->vVarz);
             Plot[id]->freqClock3 = 1./(double)smsPeriod;
            }
        }
     else if (strncmp(wordlist[*i],"adc.",4) == 0) sscanf(wordlist[*i], "adc.%s", Plot[id]->vVarz);
     else if (strncmp(wordlist[*i],"sim.",4) == 0) sscanf(wordlist[*i], "sim.%s", Plot[id]->vVarz);
     else if (strncmp(wordlist[*i],"rec.",4) == 0) sscanf(wordlist[*i], "rec.%s", Plot[id]->vVarz);
     else
        {if ((str=strstr(Plot[id]->vVarz,".")) != NULL) strcpy(wordlist[*i],&str[1]);
         else                                           sscanf(wordlist[*i], "%s", Plot[id]->vVarz);
        }

     (*i)++;
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>2) printf("DyGetPlotFromConfig> plotthirdunpack3: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     if (strcmp(wordlist[*i],"input.frameH") != 0)
        {printf("DyGetPlotFromConfig> no input.frameH \n");
         return(0);
        }

     (*i)++;

     if (Plot[id]->freqClock3 > Plot[id]->freqClock) Plot[id]->freqClock12 = Plot[id]->freqClock;
     else                       Plot[id]->freqClock12 = Plot[id]->freqClock3;
    }

 if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>0) printf("DyGetPlotFromConfig> --- Get the plot %s \n",wordlist[*i]);


 /*--- Raw Plot ---*/

 if (strcmp(wordlist[*i],"UHplotRaw") == 0)
    {
     Plot[id]->type = PLOTRAW;
     (*i)++;
     clockN = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotraw1: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     (*i)++;
     strcpy(title,wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotraw2: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     tic0 = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotraw3: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotraw4: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     (*i)++;
     DyGetPlotOptions(id, wordlist, i, nword);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotraw5: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     sprintf(dummy, "%sRAW", SEPTITLE);
     suffix = strstr(title,dummy);
     if (suffix)  strncpy (Plot[id]->name,title,strlen(title)-strlen(dummy));
     else         strcpy  (Plot[id]->name,title);
    }

 /*--- Time-Raw Plot ---*/

 if (strcmp(wordlist[*i],"UHplotRawTime") == 0)
    {
     Plot[id]->type = PLOTRAWTIME;
     (*i)++;
     clockN = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotrawtime1: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     (*i)++;
     strcpy(title,wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotrawtime2: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->xnBin = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotrawtime3: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->xMin = atof(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotrawtime4: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->xMax = atof(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotrawtime5: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->timeSize = atof(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotrawtime6: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->resampFactor = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotrawtime7: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     tic0 = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotrawtime8: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotrawtime9: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     (*i)++;
     DyGetPlotOptions(id, wordlist, i, nword);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotrawtime10: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     sprintf(dummy, "%sRAWTIME", SEPTITLE);
     suffix = strstr(title,dummy);
     if (suffix)  strncpy(Plot[id]->name,title,strlen(title)-strlen(dummy));
     else         strcpy(Plot[id]->name,title);
    }


 /*--- Time Plot ---*/

 if (strcmp(wordlist[*i],"UHplotTime") == 0)
    {
     Plot[id]->type = PLOTTIME;
     (*i)++;
     clockN = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plottime1: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     (*i)++;
     strcpy(title,wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plottime2: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->xnBin = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plottime3: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->resampFactor = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plottime4: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     tic0 = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plottime5: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plottime6: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     (*i)++;
     DyGetPlotOptions(id, wordlist, i, nword);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plottime7: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     sprintf(dummy, "%sTIME", SEPTITLE);
     suffix = strstr(title,dummy);
     if (suffix)  strncpy (Plot[id]->name,title,strlen(title)-strlen(dummy));
     else         strcpy  (Plot[id]->name,title);
    }


 /*--- Spectrum Plot ---*/

 if (strcmp(wordlist[*i],"UHplotFFT") == 0)
    {
     Plot[id]->type = PLOTFFT;
     (*i)++;
     clockN = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotfft1: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     (*i)++;
     strcpy(title,wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotfft2: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->nBinP2 = atoi(wordlist[*i]);
     (*i)++;
     Plot[id]->resampFactor = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotfft3: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     tic0 = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotfft4: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotfft5: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     (*i)++;
     DyGetPlotOptions(id, wordlist, i, nword);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotfft6: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     Plot[id]->xnBin = (1 << Plot[id]->nBinP2);

     sprintf(dummy, "%sFFT", SEPTITLE);
     suffix = strstr(title,dummy);
     if (suffix)  strncpy(Plot[id]->name,title,strlen(title)-strlen(dummy));
     else         strcpy(Plot[id]->name,title);
    }


 /*--- Transfert function plot ---*/

 if (strcmp(wordlist[*i],"UHpTfct") == 0)
    {
     Plot[id]->type = PLOTTR;
     (*i)++;
     clockN = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plottrfct1: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     (*i)++;
     (*i)++;
     strcpy(title,wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plottrfct2: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->nBinP2 = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plottrfct3: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->resampFactor = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plottrfct4: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     tic0 = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plottrfct5: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     (*i)++;
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plottrfct6: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     (*i)++;
     DyGetPlotOptions(id, wordlist, i, nword);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plottrfct7: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     Plot[id]->xnBin = (1 << Plot[id]->nBinP2);

     sprintf(dummy, "%sTRFCT", SEPTITLE);
     suffix = strstr(title,dummy);
     if (suffix)  strncpy (Plot[id]->name,title,strlen(title)-strlen(dummy));
     else         strcpy  (Plot[id]->name,title);
    }


 /*--- Coherence function plot ---*/

 if (strcmp(wordlist[*i],"UHpCohe") == 0)
    {
     Plot[id]->type = PLOTCOHE;
     (*i)++;
     clockN = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotcohe1: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     (*i)++;
     (*i)++;
     strcpy(title,wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotcohe2: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->nBinP2 = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotcohe3: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->resampFactor = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotcohe4: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     tic0 = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotcohe5: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     (*i)++;
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotcohe6: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     (*i)++;
     DyGetPlotOptions(id, wordlist, i, nword);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotcohe7: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     Plot[id]->xnBin = (1 << Plot[id]->nBinP2);

     sprintf(dummy, "%sCOHE", SEPTITLE);
     suffix = strstr(title,dummy);
     if (suffix)  strncpy (Plot[id]->name,title,strlen(title)-strlen(dummy));
     else         strcpy  (Plot[id]->name,title);
    }


 /*--- Correlation function plot ---*/

 if (strcmp(wordlist[*i],"UHpCorr") == 0)
    {
     Plot[id]->type = PLOTCORR;
     (*i)++;
     clockN = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotcorr1: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     (*i)++;
     strcpy(title,wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotcorr2: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->nBinP2 = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotcorr3: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->resampFactor = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotcorr4: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     tic0 = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotcorr5: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     (*i)++;
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotcorr6: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     (*i)++;
     DyGetPlotOptions(id, wordlist, i, nword);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotcorr7: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     Plot[id]->xnBin = (1 << Plot[id]->nBinP2);
     Plot[id]->resampFactor = 1;

     sprintf(dummy, "%sCORR", SEPTITLE);
     suffix = strstr(title,dummy);
     if (suffix)  strncpy (Plot[id]->name,title,strlen(title)-strlen(dummy));
     else         strcpy  (Plot[id]->name,title);
    }


 /*--- 3D plot ---*/

 if (strcmp(wordlist[*i],"UHplot3D") == 0)
    {
     Plot[id]->type = PLOT3D;
     (*i)++;
     clockN = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot3d1: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     (*i)++;
     strcpy(title,wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot3d1: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->xnBin = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot3d1: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->xMin = atof(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot3d1: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->xMax = atof(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot3d1: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->ynBin = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot3d1: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->yMin = atof(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot3d1: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->yMax = atof(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot3d1: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     (*i)++;
     (*i)++;
     (*i)++;
     tic0 = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot3d1: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++; 
     ticmax = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot3d1: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     (*i)++;
     DyGetPlotOptions(id, wordlist, i, nword);

     sprintf(dummy, "%s3D", SEPTITLE);
     suffix = strstr(title,dummy);
     if (suffix)  strncpy (Plot[id]->name,title,strlen(title)-strlen(dummy));
     else         strcpy  (Plot[id]->name,title);
    }


 /*--- 2D plot ---*/

 if (strcmp(wordlist[*i],"UHplot2D") == 0)
    {
     Plot[id]->type = PLOT2D;
     (*i)++;
     clockN = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot2d1: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     (*i)++;
     strcpy(title,wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot2d2: wordlist[%ld]=%s \n",*i,wordlist[*i]);  
     (*i)++;
     Plot[id]->xnBin = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot2d3: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->xMin = atof(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot2d4: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->xMax = atof(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot2d5: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->ynBin = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot2d6: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->yMin = atof(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot2d7: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->yMax = atof(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot2d8: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     (*i)++;
     (*i)++;
     tic0 = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot2d9: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++; 
     ticmax = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot2d10: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     (*i)++;
     DyGetPlotOptions(id, wordlist, i, nword);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot2d11: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     sprintf(dummy, "%s2D", SEPTITLE);
     suffix = strstr(title,dummy);
     if (suffix)  strncpy (Plot[id]->name,title,strlen(title)-strlen(dummy));
     else         strcpy  (Plot[id]->name,title);
    }


 /*--- 1D Plot ---*/

 if (strcmp(wordlist[*i],"UHplot1D") == 0)
    {
     Plot[id]->type = PLOT1D;
     (*i)++;
     clockN = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot1d1: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     (*i)++;
     strcpy(title,wordlist[*i]); 
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot1d2: wordlist[%ld]=%s \n",*i,wordlist[*i]); 
     (*i)++;
     Plot[id]->xnBin = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot1d3: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->xMin = atof(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot1d4: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->xMax = atof(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot1d5: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     (*i)++; 
     tic0 = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot1d6: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++; 
     ticmax = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot1d7: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     Plot[id]->resampFactor = 1.;

     (*i)++;
     DyGetPlotOptions(id, wordlist, i, nword);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot1d8: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     sprintf(dummy, "%s1D", SEPTITLE);
     suffix = strstr(title,dummy);
     if (suffix)  strncpy (Plot[id]->name,title,strlen(title)-strlen(dummy));
     else         strcpy  (Plot[id]->name,title);
    }


 /*--- Audio ---*/

 if (strcmp(wordlist[*i],"UHsound") == 0)
    {
     Plot[id]->type = PLOTAUDIO;
     (*i)++;
     clockN = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotaudio1: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     (*i)++;
     Plot[id]->t0 = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotaudio2: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->duration = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotaudio3: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     (*i)++;
     DyGetPlotOptions(id, wordlist, i, nword);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotaudio4: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     strcpy(Plot[id]->name, Plot[id]->vVarx);
    }

 /*--- Time-Frequency Plot ---*/

 if (strcmp(wordlist[*i],"UHplotFFTTime") == 0)
    {
     Plot[id]->type = PLOTFFTTIME;
     (*i)++;
     clockN = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotffttime1: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     (*i)++;
     strcpy(title,wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotffttime2: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->nBinP2 = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotffttime3: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->timeSize = atof(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotffttime4: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->resampFactor = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotffttime5: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     tic0 = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotffttime6: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotffttime7: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     (*i)++;
     DyGetPlotOptions(id, wordlist, i, nword);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotffttime8: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     Plot[id]->xnBin = (1 << Plot[id]->nBinP2);

     sprintf(dummy, "%sFFTTIME", SEPTITLE);
     suffix = strstr(title,dummy);
     if (suffix)  strncpy(Plot[id]->name,title,strlen(title)-strlen(dummy));
     else         strcpy(Plot[id]->name,title);
    }


 /*--- Time-1D Plot ---*/

 if (strcmp(wordlist[*i],"UHplot1DTime") == 0)
    {
     Plot[id]->type = PLOT1DTIME;
     (*i)++;
     clockN = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot1dtime1: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     (*i)++;
     strcpy(title,wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot1dtime2: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->xnBin = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot1dtime3: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->xMin = atof(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot1dtime4: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->xMax = atof(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot1dtime5: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->timeSize = atof(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot1dtime6: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->resampFactor = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot1dtime7: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     tic0 = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot1dtime8: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot1dtime9: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     (*i)++;
     DyGetPlotOptions(id, wordlist, i, nword);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plot1dtime10: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     sprintf(dummy, "%s1DTIME", SEPTITLE);
     suffix = strstr(title,dummy);
     if (suffix)  strncpy(Plot[id]->name,title,strlen(title)-strlen(dummy));
     else         strcpy(Plot[id]->name,title);
    }


 /*--- TrFct-Time Plot ---*/

 if (strcmp(wordlist[*i],"UHpTfctTime") == 0)
    {
     Plot[id]->type = PLOTTRTIME;
     (*i)++;
     clockN = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plottrfcttime1: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     (*i)++;
     strcpy(title,wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plottrfcttime2: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->nBinP2 = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plottrfcttime3: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->timeSize = atof(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plottrfcttime4: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->resampFactor = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plottrfcttime5: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     tic0 = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plottrfcttime6: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     (*i)++;
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plottrfcttime7: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     (*i)++;
     DyGetPlotOptions(id, wordlist, i, nword);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plottrfcttime8: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     Plot[id]->xnBin = (1 << Plot[id]->nBinP2);

     sprintf(dummy, "%sTRFCTTIME", SEPTITLE);
     suffix = strstr(title,dummy);
     if (suffix)  strncpy(Plot[id]->name,title,strlen(title)-strlen(dummy));
     else         strcpy(Plot[id]->name,title);
    }

 /*--- Coherence-Time Plot ---*/

 if (strcmp(wordlist[*i],"UHpCoheTime") == 0)
    {
     Plot[id]->type = PLOTCOHETIME;
     (*i)++;
     clockN = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotcohetime1: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     (*i)++;
     strcpy(title,wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotcohetime2: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->nBinP2 = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotcohetime3: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->timeSize = atof(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotcohetime4: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     Plot[id]->resampFactor = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotcohetime5: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     tic0 = atoi(wordlist[*i]);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotcohetime6: wordlist[%ld]=%s \n",*i,wordlist[*i]);
     (*i)++;
     (*i)++;
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotcohetime7: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     (*i)++;
     DyGetPlotOptions(id, wordlist, i, nword);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  plotcohetime8: wordlist[%ld]=%s \n",*i,wordlist[*i]);

     Plot[id]->xnBin = (1 << Plot[id]->nBinP2);

     sprintf(dummy, "%sCOHETIME", SEPTITLE);
     suffix = strstr(title,dummy);
     if (suffix)  strncpy(Plot[id]->name,title,strlen(title)-strlen(dummy));
     else         strcpy(Plot[id]->name,title);
    }


 /*--- Compute the parameters really used in the panels ---*/

 Plot[id]->tstep = (double)(Plot[id]->refreshPeriod) / Plot[id]->freqClock;

 if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1)
    {printf("DyGetPlotFromConfig> plot%d=%s  tstep=%g  refreshPeriod=%d  freqClock=%g \n",
             id, Plot[id]->name, Plot[id]->tstep, Plot[id]->refreshPeriod, Plot[id]->freqClock);
    }

 if (Plot[id]->type == PLOTTIME  ||  Plot[id]->type == PLOTCORR)
    {
     Plot[id]->sampFreq = Plot[id]->freqClock / Plot[id]->resampFactor;
     Plot[id]->tsize    = (double)Plot[id]->xnBin / Plot[id]->freqClock;
     Plot[id]->tstep    = (double)(Plot[id]->refreshPeriod * Plot[id]->resampFactor)
                        / Plot[id]->freqClock * (100. / Plot[id]->tsize);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1)
        {printf("DyGetPlotFromConfig> PLOTTIME CASE: plot%d=%s  tstep=%g  tsize=%g  refreshPeriod=%d  resampFactor=%d  xnBin=%d  sampFreq=%g  freqClock=%g \n",
                 id, Plot[id]->name, Plot[id]->tstep, Plot[id]->tsize, Plot[id]->refreshPeriod, Plot[id]->resampFactor, Plot[id]->xnBin, Plot[id]->sampFreq, Plot[id]->freqClock);
        }
    }

 if (Plot[id]->type == PLOTFFT || Plot[id]->type == PLOTFFTTIME ||
     Plot[id]->type == PLOTTR  || Plot[id]->type == PLOTCOHE    || Plot[id]->type == PLOTTRTIME || Plot[id]->type == PLOTCOHETIME)
    {
     Plot[id]->sampFreq = Plot[id]->freqClock / Plot[id]->resampFactor;
     Plot[id]->tsize    = (double)Plot[id]->xnBin / Plot[id]->sampFreq;
     Plot[id]->tstep    = (double)(Plot[id]->refreshPeriod * Plot[id]->resampFactor)
                        / Plot[id]->freqClock * (100. / Plot[id]->tsize);
     Plot[id]->dfreq = 0.5/Plot[id]->tsize;
     Plot[id]->dtime = Plot[id]->tsize * (Plot[id]->tstep/100.) * Plot[id]->average_fft;

     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1)
        {printf("DyGetPlotFromConfig> PLOTFFT CASE: plot%d=%s  tstep=%g  tsize=%g  refreshPeriod=%d  resampFactor=%d  xnBin=%d  sampFreq=%g  freqClock=%g \n",
                 id, Plot[id]->name, Plot[id]->tstep, Plot[id]->tsize, Plot[id]->refreshPeriod, Plot[id]->resampFactor, Plot[id]->xnBin, Plot[id]->sampFreq, Plot[id]->freqClock);
        }
    }

 if (Plot[id]->type == PLOT1DTIME)
    {
     Plot[id]->sampFreq = Plot[id]->freqClock / Plot[id]->resampFactor;
     Plot[id]->tsize    = (double)Plot[id]->nBin / Plot[id]->sampFreq;
     Plot[id]->tstep    = 0.;
    }

 if (Plot[id]->type == PLOT1D)
    {
     Plot[id]->sampFreq = Plot[id]->freqClock / Plot[id]->resampFactor;
    }

 DyXformUpdatePlotBrowser(id, ADD_PLOT);

 return(1);
}


/*------------------------------------------ DyGetPlotOptions ---*/

void DyGetPlotOptions (long id, char **wordlist, long *i, long nword)
{
 long ifirst;
 long autoScale = OFF;

 if (strcmp(wordlist[*i],"\042") != 0)  {(*i)--; return;}

 DyDebugPrint(__FUNCTION__,0);
 
 ifirst  = 1;

 while (strcmp(wordlist[*i+1],"\042") != 0)
     {
      (*i)++;

      if (ifirst == 1)
         {
          Plot[id]->refreshPeriod = atol(wordlist[*i]); 
          if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  options0: wordlist[%ld]=%s \n",*i,wordlist[*i]);
          ifirst = 0;
         }
      if (strcmp(wordlist[*i],"NFFT") == 0)
         {(*i)++;
          Plot[id]->average_fft = atoi(wordlist[*i]);
          if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  options2: wordlist[%ld]=%s \n",*i,wordlist[*i]);
         }
      if (strcmp(wordlist[*i],"NFFT_REFRESH") == 0)
         {(*i)++;
          Plot[id]->refresh_fft = atoi(wordlist[*i]);
          if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  options2: wordlist[%ld]=%s \n",*i,wordlist[*i]);
         }
      if (strcmp(wordlist[*i],"FMIN") == 0)
         {(*i)++;
          Plot[id]->xMin = atof(wordlist[*i]);
          if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  options3: wordlist[%ld]=%s \n",*i,wordlist[*i]);
         }
      if (strcmp(wordlist[*i],"FMAX") == 0)
         {(*i)++;
          Plot[id]->xMax = atof(wordlist[*i]);
          if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  options4: wordlist[%ld]=%s \n",*i,wordlist[*i]);
         }
      if (strcmp(wordlist[*i],"ZMIN") == 0)
         {(*i)++;
          Plot[id]->zMin = atof(wordlist[*i]);
          if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  options5: wordlist[%ld]=%s \n",*i,wordlist[*i]);
         }
      if (strcmp(wordlist[*i],"ZMAX") == 0)
         {(*i)++;
          Plot[id]->zMax = atof(wordlist[*i]);
          if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  options5bis: wordlist[%ld]=%s \n",*i,wordlist[*i]);
         }

      if (strcmp(wordlist[*i],"CROSSCOHE_FMIN") == 0)
         {(*i)++;
          Plot[id]->crossCohe_fmin = atof(wordlist[*i]);
          if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  options5ter: wordlist[%ld]=%s \n",*i,wordlist[*i]);
          Plot[id]->crossCohe = ON;
         }
      if (strcmp(wordlist[*i],"CROSSCOHE_FMAX") == 0)
         {(*i)++;
          Plot[id]->crossCohe_fmax = atof(wordlist[*i]);
          if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  options5quart: wordlist[%ld]=%s \n",*i,wordlist[*i]);
         }

      if (strcmp(wordlist[*i],"YMIN") == 0)
         {(*i)++;
          Plot[id]->yMin = atof(wordlist[*i]);
          if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  options6: wordlist[%ld]=%s \n",*i,wordlist[*i]);
         }
      if (strcmp(wordlist[*i],"YMAX") == 0)
         {(*i)++;
          Plot[id]->yMax = atof(wordlist[*i]);
          if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  options7: wordlist[%ld]=%s \n",*i,wordlist[*i]);
         }
      if (strcmp(wordlist[*i],"NSAMPLING") == 0)
         {(*i)++;
          Plot[id]->nSampling = atol(wordlist[*i]);
          if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  options8: wordlist[%ld]=%s \n",*i,wordlist[*i]);
         }

      if (strcmp(wordlist[*i],"FILTERMIN") == 0)
         {(*i)++;
          Plot[id]->filterMin = atof(wordlist[*i]);
          if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  options9: wordlist[%ld]=%s \n",*i,wordlist[*i]);
         }
      if (strcmp(wordlist[*i],"FILTERMAX") == 0)
         {(*i)++;
          Plot[id]->filterMax = atof(wordlist[*i]);
          if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  options10: wordlist[%ld]=%s \n",*i,wordlist[*i]);
         }

      if (strcmp(wordlist[*i],"REFRESH_PERCENT") == 0)
          {(*i)++;
           Plot[id]->refresh_percent = atof(wordlist[*i]);
           if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  options11: wordlist[%ld]=%s \n",*i,wordlist[*i]);
          }

      if (strcmp(wordlist[*i],"N1D") == 0)
          {(*i)++;
           Plot[id]->nBin = atoi(wordlist[*i]);
           if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  options12: wordlist[%ld]=%s \n",*i,wordlist[*i]);
          }

      if (strcmp(wordlist[*i],"YOFFSET") == 0)
         {(*i)++;
          Plot[id]->yoffset = atof(wordlist[*i]);
          if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  options13: wordlist[%ld]=%s \n",*i,wordlist[*i]);
         }
      if (strcmp(wordlist[*i],"YSCALE") == 0)
         {(*i)++;
          Plot[id]->yscale = atof(wordlist[*i]);
          if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  options14: wordlist[%ld]=%s \n",*i,wordlist[*i]);
         }

      if (strcmp(wordlist[*i],"RESAMPFACTOR") == 0)
         {(*i)++;
          Plot[id]->resampFactor = atof(wordlist[*i]);
          if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>10) printf("  options15: wordlist[%ld]=%s \n",*i,wordlist[*i]);
         }

      Plot[id]->soundonline = ON;
      if (strncmp(wordlist[*i],"SOUND16BITS",11) == 0)     Plot[id]->sound16bits     = ON;
      if (strncmp(wordlist[*i],"SOUNDOFF",8) == 0)         Plot[id]->soundonline     = OFF;
      if (strncmp(wordlist[*i],"MINMAX_RESAMPLE",14) == 0) Plot[id]->minmax_resample = ON;
      if (strncmp(wordlist[*i],"REALUNITS",9) == 0)        Plot[id]->realUnits       = ON;
      if (strncmp(wordlist[*i],"X2AV",4)      == 0)        Plot[id]->x2av            = ON;
      if (strncmp(wordlist[*i],"HERTZ",5)     == 0)        Plot[id]->hertz           = ON;
      if (strncmp(wordlist[*i],"RMSFFT",6)    == 0)        Plot[id]->rmsfft          = ON;
      if (strncmp(wordlist[*i],"DECAYFFT",8)  == 0)        Plot[id]->decayfft        = ON;
      if (strncmp(wordlist[*i],"SAME",4)      == 0)        Plot[id]->superposed      = ON;
      if (strncmp(wordlist[*i],"AUTOSCALE",9) == 0)        autoScale                 = ON;
      if (strncmp(wordlist[*i],"NODC",4)      == 0)        Plot[id]->nodc            = ON;
      if (strncmp(wordlist[*i],"BITS",4)      == 0)        Plot[id]->bits            = ON;
      if (strncmp(wordlist[*i],"MODULESQRT",10) == 0)      Plot[id]->modulesqrt      = ON;

      if (strncmp(wordlist[*i],"GRIDX",5) == 0)            Plot[id]->gridx    = ON;
      if (strncmp(wordlist[*i],"GRIDY",5) == 0)            Plot[id]->gridy    = ON;
      if (strncmp(wordlist[*i],"LOGX",4)  == 0)            Plot[id]->logx     = ON;
      if (strncmp(wordlist[*i],"LOGY",4)  == 0)            Plot[id]->logy     = ON;
      if (strncmp(wordlist[*i],"LOGZ",4)  == 0)            Plot[id]->logz     = ON;
      if (strncmp(wordlist[*i],"STAT",4)  == 0)            Plot[id]->stat     = ON;
      if (strncmp(wordlist[*i],"AUTO",4)  == 0)            Plot[id]->autoEdge = ON;
      if (strncmp(wordlist[*i],"CONT",4)  == 0)            Plot[id]->type2d   = CONT;
      if (strncmp(wordlist[*i],"COL",3)   == 0)            Plot[id]->type2d   = COL;
      if (strncmp(wordlist[*i],"LEGO",4)  == 0)            Plot[id]->type2d   = LEGO;
      if (strncmp(wordlist[*i],"SURF",4)  == 0)            Plot[id]->type2d   = SURF;
      if (strncmp(wordlist[*i],"SCAT",4)  == 0)            Plot[id]->type2d   = SCAT;
      if (strncmp(wordlist[*i],"HIDDEN",6)  == 0)          Plot[id]->hidden   = ON;
     }

 if (autoScale == ON)  Plot[id]->autoScale = ON;


 /*--- The second options string of Tfct or Cohe functions ---*/

 if (strcmp(wordlist[*i+2],"\042") == 0)
    {(*i)++;
     (*i)++;
     while (strcmp(wordlist[*i+1],"\042") != 0)
        {
         (*i)++;
         if (strncmp(wordlist[*i],"GRIDX",5) == 0) Plot[id]->gridx2   = ON;
         if (strncmp(wordlist[*i],"GRIDY",5) == 0) Plot[id]->gridy2   = ON;
         if (strncmp(wordlist[*i],"LOGX",4) == 0)  Plot[id]->logx2    = ON;
         if (strncmp(wordlist[*i],"LOGY",4) == 0)  Plot[id]->logy2    = ON;
        }
    }

 (*i)++;
 return;
}





/*                        *************                     */
/*                        *  STRINGS  *                     */
/*                        *************                     */

/*----------------------------------- DyMakeTriggerString ---*/

void DyMakeTriggerString ()
{
 int i, i0, nvar;
 const char *strline;
 long frame_clock;
 char *str, className[LSTRING_MAX], inputVect[LBIGSTRING_MAX], inputPar[LSTRING_MAX];
 int start;

 DyDebugPrint(__FUNCTION__,0);

 frame_clock = Dy->nClock-1;

 i0 = -1;
 nvar = fl_get_browser_maxline(mainbrowser->proc);
 for (i=0; i<nvar; i++)
    {
     strline = fl_get_browser_line(mainbrowser->proc,i+1);
     sscanf(strline,"%s", strline);

     if (strstr(strline,OPSIGNAL) != NULL && strstr(strline,TRIGGER) != NULL)
        {start = strlen(OPSIGNAL);
         str   = strstr(strline,OPSEP);
         strcpy(inputVect,"NULL");
         if (str)
            {strcpy(inputVect,&str[1]);
             strncpy(className,strline+start,str-strline-start);
            }
         className[str-strline-start] = '\0';
         if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1)
            {printf("DyMakeTriggerString> i=%d  strline=%s  str=%s  inputVect=%s  className=%s \n",
                      i, strline, str, inputVect, className);
            }

         strcpy(inputPar," ");
         strcat(inputPar, "REALUNITS");
         strcat(inputPar,"NULL");
         sprintf(Dy->triggerString, "UFrVectOp %ld 999 %s %s %s input.frameH",
                 frame_clock, className, inputVect, inputPar);
        }
    }

 return;
}


/*----------------------------------- DyMakeClockString ---*/

long DyMakeClockString ()
{
 long i, j, nplot, nclock, flag, nTics;
 double freq, startTime;
 const char *fileName;
 struct FrFile *iFile;
 struct FrameH *frame;
 long offset, nfile;
 int newclock;
 double fastest, frameFreq;

 DyDebugPrint(__FUNCTION__,0);

 nplot = fl_get_browser_maxline(mainbrowser->plots);

 /*--- We get the list of clocks ---*/

 frameFreq = 1. / DyGetFramedt(Dy->Frame);
 nclock = 0;
 fflush(stdout);

 for (i=1; i <= nplot; i++)
    {
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1)
        {printf("DyMakeClockString> (%s) Plot[%d]->freqClock=%g  frameFreq=%g \n", Plot[i]->name, i, Plot[i]->freqClock, frameFreq);}
     if (Plot[i]->freqClock > 0. &&  Plot[i]->freqClock > frameFreq)
        {flag = 0;
         for (j=0; j<nclock; j++) {if (Plot[i]->freqClock == Dy->freqClock[j]) flag = 1;}
         if (flag == 0)
            {Dy->freqClock[nclock] = Plot[i]->freqClock;
             nclock++;
            }
        }
     if (Plot[i]->freqClock2 > 0.  &&  Plot[i]->freqClock2 > frameFreq)
        {flag = 0;
         for (j=0; j<nclock; j++) {if (Plot[i]->freqClock2 == Dy->freqClock[j]) flag = 1;}
         if (flag == 0)
            {Dy->freqClock[nclock] = Plot[i]->freqClock2;
             nclock++;
            }
        }
    }

 /*--- We add the frame's clock and compute the number of tics ---*/
 /* Doing this first means computing PPCM with all clocks and
    have the clocks list in right order (frame's clock may not be the slowest) */

 Dy->freqClock[nclock] = 1. / DyGetFramedt(Dy->Frame);
 nclock++;
 Dy->nClock = nclock;

 /*--- Add a new fastest clock if freqClock are not submultiples ---*/

 newclock = DyPPCM(Dy->freqClock, nclock, &fastest);
 if (newclock > nclock)
    {Dy->freqClock[newclock-1] = fastest;
     nclock++;
     Dy->nClock = nclock;
    }

 /*--- We now sort the clocks (and plots) to have the fastest clock first ---*/

 for (i=0; i < nclock-1; i++)
    {for (j=i+1; j < nclock; j++)
        {if (Dy->freqClock[i] < Dy->freqClock[j])
            {freq             = Dy->freqClock[i];
             Dy->freqClock[i] = Dy->freqClock[j];
             Dy->freqClock[j] = freq;
            }
        }
    }

 /*--- We update the plots clocks ---*/

 frameFreq = 1. / DyGetFramedt(Dy->Frame);
 for (i=1; i <= nplot; i++)
    {for (j=0; j < nclock; j++)
        {if (Plot[i]->freqClock   == Dy->freqClock[j])  Plot[i]->clockN   = j;
         if (Plot[i]->freqClock2  == Dy->freqClock[j])  Plot[i]->clockN2  = j;
         if (Plot[i]->freqClock12 == Dy->freqClock[j])  Plot[i]->clockN12 = j;
         if (Plot[i]->freqClock < frameFreq)   Plot[i]->clockN   = nclock-1;
         if (Plot[i]->freqClock2 < frameFreq)  Plot[i]->clockN2  = nclock-1;
         if (Plot[i]->freqClock12 < frameFreq) Plot[i]->clockN12 = nclock-1;
        }
    }

if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>0)
   {printf("DyMakeClockString> nclock=%d (Dy->nClock=%d) \n", nclock, Dy->nClock);}


/*--- Determine the offset between start time and file start time ---*/

 if (Dy->startTime > 0  && (Dy->InputType == INFILE || Dy->InputType == INONLINE || Dy->InputType == INDIR))
    {
     nfile  = fl_get_browser_maxline(inputfile->file_list);
     offset = 0;
     startTime = (double)(Dy->startTime)+0.01;
     for (i=0; i<nfile; i++)
        {
         fileName = fl_get_browser_line(inputfile->file_list, i+1);
         iFile = FrFileINew((char *)fileName);
         if (iFile == NULL) continue;
         frame = FrameReadTChnl(iFile, startTime, "NONE");

         if (frame != NULL)
            {offset = Dy->startTime - frame->GTimeS;
             printf("DyMakeClockString> offset=frame-fileStart=%d-%d=%d\n", frame->GTimeS, Dy->startTime, offset);
             FrameFree(frame);
             FrFileIEnd(iFile);
             break;
            }

         FrFileIEnd(iFile);
        }
    }

 nTics = (long)((Dy->duration+offset) * Dy->freqClock[0]);
 if (nTics <= 0 || nTics==LONG_MAX) nTics = LONG_MAX - 1;


 /*--- We build the clock string ---*/

 sprintf(Dy->clockString, "\nUJclock masterClocks %ld %ld", nTics, Dy->nClock);
									
 for (i=0; i<Dy->nClock; i++)
    {sprintf(Dy->clockString, "%s %12.8f", Dy->clockString, Dy->freqClock[i]);}

 if (newclock > 0)
    {sprintf(Dy->clockString, "%s \nUHdummy 0", Dy->clockString);}

 return(Dy->nClock);
}


/*------------------------------------ DyPPCM ---*/

int DyPPCM (double *freqClock, int nclock, double *fastest)
{
 double ppcm, pgcd, freqmax, freqmin, scale;
 int newclock, i, noppcm;
 double value1, value2;

 if (nclock < 2) return(0);

 freqmin = 1e+30;
 freqmax = freqClock[0];
 for (i=0; i<nclock; i++)
    {if (freqClock[i] < freqmin && freqClock[i] > 0) freqmin = freqClock[i];
     if (freqClock[i] > freqmax) freqmax = freqClock[i];
    }

 noppcm = 1;
 for (i=0; i<nclock; i++)
     {if (freqmax/freqClock[i] - floor(freqmax/freqClock[i]) > 1.e-2 ) noppcm = 0;
      if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1)
         {printf("DyPPCM> freqmax=%f  freqclock[%d]=%f  freqmax/freqClock[i]=%f  floor(freqmax/freqClock[i])=%f  rest=%f  noppcm=%d \n", 
                  freqmax, i, freqClock[i], freqmax/freqClock[i], floor(freqmax/freqClock[i]), freqmax/freqClock[i] - floor(freqmax/freqClock[i]), noppcm);
         }
     }

 if (noppcm == 1)
    {if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>0) printf("DyPPCM> All frequencies are submultiples of freqmax=%g. No PPCM needed.\n", freqmax);
     return(0);
    }

 if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1)
    {for (i=0; i<nclock; i++)
        {printf("DyPPCM> freqClock[%d]=%g  freqmax=%g  freqmin=%g  freqmax/freq=%.9g (%d)\n",
                 i, freqClock[i], freqmax, freqmin, freqmax / freqClock[i], (int)(freqmax / freqClock[i]));}
    }


 scale = 1.;
 if (freqmin < 1.) scale = pow(10,floor(-log10(freqmin)));
 if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1) printf("DyPPCM1> scale=%f freqmin=%f  floor(-log10(freqmin))=%f \n", scale, freqmin, floor(-log10(freqmin)));
 if (freqmin < 1.) scale = pow(10,floor(6.-log10(freqmax)));
 if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1) printf("DyPPCM2> scale=%f freqmax=%f floor(6.-log10(freqmax))=%f \n", scale, freqmax, floor(6.-log10(freqmax)));

 noppcm = 1;
 for (i=0; i<nclock; i++)
    {value1 = ceil(freqmax*scale);
     value2 = ceil(freqClock[i]*scale);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>3)
        {printf("DyPPCM> freqmax=%f  freqmax*scale=%f  ceil(freqmax*scale)=%f \n", freqmax, freqmax*scale, ceil(freqmax*scale));
         printf("DyPPCM> freqClock[i]=%f  freqClock[i]*scale=%f  ceil(freqClock[i]*scale)=%g\n", freqClock[i], freqClock[i]*scale, ceil(freqClock[i]*scale));
        }
     if (fmod(value1,value2) != 0.) {noppcm = 0; break;}
    }
 if (noppcm == 1)
    {if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>0)
        {printf("DyPPCM> All frequencies are submultiples of freqmax=%g. No PPCM needed.\n", freqmax);}
     return(0);
    }

 /* ppcm is the product of the numbers divided by the product of their pgcd */
 /* or ppcm is the result of 2 by 2 iterative ppcm (like below) */

 ppcm = freqClock[0]*scale;
 for (i=1; i<nclock; i++)
    {
     pgcd = (double)compute_pgcd((int)ppcm,(int)(freqClock[i]*scale));
     ppcm = ppcm/pgcd * freqClock[i]*scale;
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1)
        {printf("DyPPCM> freqClock[%d]*scale=%d (freqClock=%f) : pgcd=%f  ppcm=%f\n", i, (int)(freqClock[i]*scale), freqClock[i], pgcd, ppcm);}
    }

 *fastest = ppcm/scale;

 if (*fastest > freqmax && *fastest < 65536*16*16) newclock = nclock+1;
 else                                           newclock = 0;

 if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1)
    { printf("DyPPCM> fastest=ppcm/scale=%f/%f=%f  newclock=%d \n", ppcm, scale, *fastest, newclock);}
 
 return(newclock);
}


/*------------------------------------ compute_pgcd ---*/
/**
unsigned pgcd(unsigned nbr1, unsigned nbr2) { unsigned lePgcd; for (lePgcd = (nbr1 < nbr2) ? nbr1 : nbr2; lePgcd > 1; lePgcd--) { if ((0 == (nbr1 % lePgcd)) && (0 == (nbr2 % lePgcd))) break; } return lePgcd; }
unsigned pgcd(unsigned nbr1, unsigned nbr2) { for (; nbr1 != nbr2; (nbr1 > nbr2) ? (nbr1 -= nbr2) : (nbr2 -= nbr1)); return nbr1; 
unsigned pgcd(unsigned nbr1, unsigned nbr2) { unsigned reste; for (; reste = nbr1 % nbr2; nbr1 = nbr2, nbr2 = reste); return nbr2; }
**/

int compute_pgcd (int nbr1, int nbr2)
{
 int reste, dummy;
 reste = 2;

 if (nbr2 > nbr1) {dummy=nbr2; nbr2=nbr1; nbr1=dummy;}

 while (reste > 0)
     {
      reste = nbr1 % nbr2;
      if (reste > 0)
         {nbr1 = nbr2;
          nbr2 = reste;
         }
     }

 return nbr2;
}


/*------------------------------------ DyMakeFileString ---*/

long DyMakeFileString (long arg)
{
 long i, nfile, frame_clock, totlen;
 const char *filename;
 char tag[LBIGSTRING_MAX];
 double freqFrame;

 DyDebugPrint(__FUNCTION__,0);

 /*--- get the "frame read" clock ---*/

 frame_clock = Dy->nClock - 1;
 freqFrame = 1. / DyGetFramedt(Dy->Frame);
 for (i=0; i<Dy->nClock; i++)
    {if (freqFrame == Dy->freqClock[i]) {frame_clock = i; break;}}

 /*--- Data offline from wave audio file ---*/

 if (arg == INWAVE)
    {
     nfile = fl_get_browser_maxline(inputfile->file_list);
     if (nfile <= 0) return(0);
     totlen = 3;
     for (i=0; i<nfile; i++)
        {filename = fl_get_browser_line(inputfile->file_list, i+1);
         totlen  += strlen(filename) + 1;
        }
     Dy->fileString = (char *) calloc(totlen+64,sizeof(char));
     sprintf(Dy->fileString, "UFrWave input %ld ", frame_clock);
     strcat(Dy->fileString, "\042");
     for (i=0; i<nfile; i++)
        {filename = fl_get_browser_line(inputfile->file_list, i+1);
         strcat(Dy->fileString, filename);
         if (nfile > 1  &&  i<nfile-1) strcat(Dy->fileString, " ");
        }
     strcat(Dy->fileString, "\042");
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>0) printf("DyMakeFileString> input=wave=%s\n", Dy->fileString);
    }

 /*--- Frames offline from files ---*/

 if (arg == INFILE || arg == INDIR)
    {
     nfile = fl_get_browser_maxline(inputfile->file_list);
     if (nfile <= 0) return(0);
     totlen = 3;
     for (i=0; i<nfile; i++)
        {filename = fl_get_browser_line(inputfile->file_list, i+1);
         totlen  += strlen(filename) + 1;
        }
     Dy->fileString = (char *) calloc(totlen+64,sizeof(char));
     sprintf(Dy->fileString, "UFrIFile input %ld ", frame_clock);
     strcat(Dy->fileString, "\042");
     for (i=0; i<nfile; i++)
        {filename = fl_get_browser_line(inputfile->file_list, i+1);
         strcat(Dy->fileString, filename);
         if (nfile > 1  &&  i<nfile-1) strcat(Dy->fileString, " ");
        }
     strcat(Dy->fileString, "\042");
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>0) printf("DyMakeFileString> input=files=%s\n", Dy->fileString);
    }

 /*--- Frames online from files ---*/

 if (arg == INONLINE)
    {
     nfile = 1;
     Dy->fileString = (char *) calloc(LSTRING_MAX, sizeof(char));
     sprintf(Dy->fileString, "UFrIFileOnline input %ld %s 9999999 %d", frame_clock, Dy->Dir, Dy->startTime);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>0) printf("DyMakeFileString> input=filesOnline=%s  startTime=%d\n", Dy->Dir, Dy->startTime);
    }

 /*--- Frames online from a Cm server ---*/

 if (arg == INCM)
    {
     nfile = 1;
     Dy->fileString = (char *) calloc(LSTRING_MAX+strlen(tag), sizeof(char));
     DyMakeTag(tag);
     sprintf(Dy->fileString,"UFrIFileCm input %ld 9999999 %s \042%s\042", frame_clock, Dy->CmName, tag);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>0) printf("DyMakeFileString> input=CmServer=%s\n", Dy->CmServer);
    }

  /*--- Frames online from a Dd server ---*/

 if (arg == INDD)
    {
     nfile = 1;
     Dy->fileString = (char *)calloc(LSTRING_MAX+strlen(tag)+strlen(Dy->Ddinputfile),sizeof(char));
     DyMakeTag(tag);
     sprintf(Dy->fileString,"UFrIFileCm input %ld 9999999 %s \042%s\042", frame_clock, Dy->CmName, tag);
     sprintf(Dy->fileString,"%s\n/*UFrIFileDd %s %s */", Dy->fileString, Dy->DdServer, Dy->Ddinputfile);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>0) printf("DyMakeFileString> input=DataSender=%s\n", Dy->DdServer);
    }

#ifdef USE_VEGA
 if (arg == INVEGA)
    {
     nfile = 1;
     Dy->fileString = (char *) calloc(LSTRING_MAX+strlen(tag),sizeof(char));
     sprintf(Dy->fileString,"UFrChannel input %ld default_viewer",
             frame_clock);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>0) printf("  input=FrameChannel\n");
    }
#endif

 /*--- Frames from a shared memory ---*/

 if (arg == INSHM)
    {
     nfile = 1;
     Dy->fileString = (char *) calloc(LSTRING_MAX, sizeof(char));
     sprintf(Dy->fileString,"UFrShm input %ld %s", frame_clock, Dy->shmName);
     if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>0) printf("DyMakeFileString> input=SharedMemory=%s\n", Dy->shmName);
    }

 return(nfile);
}


/*------------------------------------------- DyMakeTag ---*/

void DyMakeTag (char *tag)
{
 char *smsName, *varName, *adcName, *procName, *simName;
 char *smsName2, *varName2, *adcName2, *procName2, *simName2;
 char *smsName3, *varName3, *adcName3, *procName3, *simName3;
 long id, nplot;
 char chaine[LBIGSTRING_MAX], *vVar, *str;

 DyDebugPrint(__FUNCTION__,0);

 strcpy(tag," ");

 nplot = fl_get_browser_maxline(mainbrowser->plots);

 for (id=1; id<=nplot; id++)
    {
     if (Plot[id]->hidden == ON) continue;

     smsName = NULL; varName = NULL; adcName = NULL; procName = NULL; simName = NULL;
     
     DyDecodeVar(Plot[id]->vVarx, &smsName, &varName, &adcName, &procName, &simName);
     if (adcName)       DyAddToTag(tag, adcName);
     else if (smsName)  DyAddToTag(tag, smsName);
     else if (simName)  DyAddToTag(tag, simName);
     else if (procName)
        {if (strncmp(procName,OPSIGNAL,strlen(OPSIGNAL)) != 0)
            {DyAddToTag(tag, procName);}
         else
            {
             str = strstr(procName,OPSEP);
             strcpy(chaine,&str[1]);
             vVar = strtok(chaine,OPSEP);
             while (vVar != NULL)
                {
                 DyAddToTag(tag, vVar);
                 vVar = strtok(NULL,OPSEP);
                }
            }
        }
     else
        {DyAddToTag(tag, Plot[id]->vVarx);}

     if (adcName)  free(adcName);
     if (smsName)  free(smsName);
     if (varName)  free(varName);
     if (procName) free(procName);
     if (simName)  free(simName);

     if (Plot[id]->type == PLOTTR     ||  Plot[id]->type == PLOT2D   ||  Plot[id]->type == PLOT3D   ||
         Plot[id]->type == PLOTCORR   ||  Plot[id]->type == PLOTCOHE || 
         Plot[id]->type == PLOTTRTIME ||  Plot[id]->type == PLOTCOHETIME)
        {
         smsName2 = NULL; varName2 = NULL; adcName2 = NULL; procName2 = NULL; simName2 = NULL;

         DyDecodeVar(Plot[id]->vVary, &smsName2, &varName2, &adcName2, &procName2, &simName2);
         if (adcName2)       DyAddToTag(tag, adcName2);
         else if (smsName2)  DyAddToTag(tag, smsName2);
         else if (simName2)  DyAddToTag(tag, simName2);
         else if (procName2)
            {
             if (strncmp(procName2,OPSIGNAL,strlen(OPSIGNAL)) != 0)
                {DyAddToTag(tag, procName2);}
             else
                {
                 str = strstr(procName2,OPSEP);
                 strcpy(chaine,&str[1]);
                 vVar = strtok(chaine,OPSEP);
                 while (vVar != NULL)
                    {
                     DyAddToTag(tag, vVar);
                     vVar = strtok(NULL,OPSEP);
                    }
                }
            }
         else
            {DyAddToTag(tag, Plot[id]->vVary);}

         if (adcName2)  free(adcName2);
         if (smsName2)  free(smsName2);
         if (varName2)  free(varName2);
         if (procName2) free(procName2);
         if (simName2)  free(simName2);
        }

     if (Plot[id]->type == PLOT3D)
        {
         smsName3 = NULL; varName3 = NULL; adcName3 = NULL; procName3 = NULL; simName3 = NULL;

         DyDecodeVar(Plot[id]->vVarz, &smsName3, &varName3, &adcName3, &procName3, &simName3);
         if (adcName3)       DyAddToTag(tag, adcName3);
         else if (smsName3)  DyAddToTag(tag, smsName3);
         else if (simName3)  DyAddToTag(tag, simName3);
         else if (procName3)
            {
             if (strncmp(procName3,OPSIGNAL,strlen(OPSIGNAL)) != 0)
                {DyAddToTag(tag, procName3);}
             else
                {
                 str = strstr(procName3,OPSEP);
                 strcpy(chaine,&str[1]);
                 vVar = strtok(chaine,OPSEP);
                 while (vVar != NULL)
                    {
                     DyAddToTag(tag, vVar);
                     vVar = strtok(NULL,OPSEP);
                    }
                }
            }
         else
            {DyAddToTag(tag, Plot[id]->vVarz);}

         if (adcName3)  free(adcName3);
         if (smsName3)  free(smsName3);
         if (varName3)  free(varName3);
         if (procName3) free(procName3);
         if (simName3)  free(simName3);
        }
    }

 if (Dy->output_flag == ON && Dy->output_tag != NULL) strcat(tag, Dy->output_tag);

 if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1) printf("DyMakeTag> tag=%s \n", tag);

 return;
}


/*------------------------------------------- DyAddToTag ---*/

char *DyAddToTag (char *tag, char *name)
{
 char dummy[LSTRING_MAX];
 char *smsName, *varName, *adcName, *procName, *simName;

 smsName = NULL; varName = NULL; adcName = NULL; procName = NULL; simName = NULL;

 DyDecodeVar(name, &smsName, &varName, &adcName, &procName, &simName);
 if (smsName) strcpy(name, smsName);

/*--- HIHI: Uncomment the code below and simplify the code of DyMakeTag ---*/
/** 
 DyDecodeVar(name, &smsName, &varName, &adcName, &procName, &simName);
 if (procName)
    {if (strncmp(procName,OPSIGNAL,strlen(OPSIGNAL)) != 0)
        {DyAddToTag(tag, procName);}
     else
        {
         str = strstr(procName,OPSEP);
         strcpy(chaine,&str[1]);
         vVar = strtok(chaine,OPSEP);
         while (vVar != NULL)
            {
             DyAddToTag(tag, vVar);
             vVar = strtok(NULL,OPSEP);
            }
        }
    }
 else if (smsName) strcpy(name, smsName);
**/

 sprintf(dummy," %s ",name);
 if (strstr(tag, dummy) == NULL) {strcat(tag, name); strcat(tag," ");}

 if (adcName)  free(adcName);
 if (smsName)  free(smsName);
 if (varName)  free(varName);
 if (procName) free(procName);
 if (simName)  free(simName);

 return(tag);
}


/*------------------------------------------- DyMakePlotString ---*/

void DyMakePlotString (long id)
{
 long plottype, t0, tmax, frame_clock;
 double dt;
 char title[LSTRING_MAX], dummy[LSTRING_MAX];
 char variablex[LSTRING_MAX], variabley[LSTRING_MAX],  variablez[LSTRING_MAX];
 char signal[LSTRING_MAX], signalx[LSTRING_MAX], signaly[LSTRING_MAX], signalz[LSTRING_MAX];
 char signalout[LSTRING_MAX], signalxout[LSTRING_MAX];
 char signalyout[LSTRING_MAX], signalzout[LSTRING_MAX];
 char options[LSTRING_MAX], options2[LSTRING_MAX];
 char *smsName=NULL, *varName=NULL, *adcName=NULL, *procName=NULL, *simName=NULL;
 char *str, className[LSTRING_MAX], inputVect[LBIGSTRING_MAX], inputPar[LSTRING_MAX];
 int start, samefreq, j, j0;
 struct FrProcData *proc;

 DyDebugPrint(__FUNCTION__,0);

 if (id <= 0) return;
 plottype = Plot[id]->type;

 if (Dy->Frame == NULL)
    {printf("DyMakePlotString> frame=NULL. This plotString can not be made.\n");
     return;
    }

 dt = DyGetFramedt(Dy->Frame);

/*hihi*/

 if (dt > 0  &&  Dy->InputType != INONLINE)
    {
     t0   = (long)((Dy->startTime - Dy->firstTime) * Plot[id]->freqClock);
     t0   = 0;
     tmax = (long)((Dy->startTime - Dy->firstTime + Dy->duration) * Plot[id]->freqClock);
     if (tmax < t0) tmax = -1;
    }
 else
    {t0   =  0;
     tmax = -1;
    }


 DyMakePlotOptions(id, options, options2);

 /*--- Get the variable name ---*/

 DyDecodeVar(Plot[id]->vVarx, &smsName, &varName, &adcName, &procName, &simName);
 if (smsName && varName)
    {if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1) 
        {printf("DyMakePlotString> smsName=%s  freqClock=%.16lf  int(1/freqClock+0.5)=%d\n", smsName, Plot[id]->freqClock, (int)(1./Plot[id]->freqClock + 0.5));}
     if (Plot[id]->freqClock < 1./dt) sprintf (variablex, "sms%d.%s%s%s", (int)(1./Plot[id]->freqClock + 0.5), smsName, SEP, varName);
     else                             sprintf (variablex, "sms.%s%s%s", smsName, SEP, varName);
    }
 else if (adcName)       sprintf (variablex, "adc.%s", adcName);
 else if (simName)       sprintf (variablex, "sim.%s", simName);
 else if (procName)      sprintf (variablex, "rec.%s", procName);
 else                    sprintf (variablex, "rec.%s",Plot[id]->vVarx);

 frame_clock = Dy->nClock-1;

 /*--- OPSTRING line (before Unpack)---*/
 /*--- for signals that are result of operation on frames signals ---*/

 strcpy(Dy->plotString[id],"");

 if (strstr(Plot[id]->vVarx,OPSIGNAL) != NULL)
    {
     start = strlen(OPSIGNAL);
     str = strstr(&(Plot[id]->vVarx[start]),OPSEP);
     strcpy(inputVect,"NULL");
     if (str)
        {strcpy(inputVect,&str[1]);
         strncpy(className,&(Plot[id]->vVarx[start]),str-&(Plot[id]->vVarx[start]));
        }
     className[str-&(Plot[id]->vVarx[start])] = '\0';

     strcpy(inputPar," ");
     if (Plot[id]->realUnits == ON) strcat(inputPar, "REALUNITS");

     proc = FrProcDataFind(Dy->Frame,Plot[id]->vVarx);
     if (proc && proc->nAuxParam) strcat(inputPar,proc->auxParamNames[0]);
     else                         strcat(inputPar,"NULL");

     sprintf (Dy->plotString[id], "%sUFrVectOp %ld %ld %s %s %s input.frameH",
              Dy->plotString[id], frame_clock, id, className, inputVect, inputPar);
/**
     DyLoadScript (className);
**/
    }

 if (strstr(Plot[id]->vVary,OPSIGNAL) != NULL  &&
     (plottype==PLOTTR || plottype==PLOTCOHE || plottype==PLOTCORR || plottype==PLOT2D || plottype==PLOT3D ||
      plottype==PLOTTRTIME || plottype==PLOTCOHETIME))
    {
     start = strlen(OPSIGNAL);
     str = strstr(&(Plot[id]->vVary[start]),OPSEP);
     strcpy(inputVect,"NULL");
     if (str)
        {strcpy(inputVect,&str[1]);
         strncpy(className,&(Plot[id]->vVary[start]),str-&(Plot[id]->vVary[start]));
        }
     className[str-&(Plot[id]->vVary[start])] = '\0';

     strcpy(inputPar," ");
     if (Plot[id]->realUnits == ON) strcat(inputPar, "REALUNITS_");

     proc = FrProcDataFind(Dy->Frame,Plot[id]->vVary);
     if (proc && proc->nAuxParam) strcat(inputPar,proc->auxParamNames[0]);
     else                         strcat(inputPar,"NULL");

     sprintf (Dy->plotString[id], "%s \nUFrVectOp %ld %ld %s %s %s input.frameH",
              Dy->plotString[id], frame_clock, id, className, inputVect, inputPar);
/**
     DyLoadScript (className);
**/
    }


 if (strstr(Plot[id]->vVarz,OPSIGNAL) != NULL  &&  plottype==PLOT3D)
    {
     start = strlen(OPSIGNAL);
     str = strstr(&(Plot[id]->vVarz[start]),OPSEP);
     strcpy(inputVect,"NULL");
     if (str)
        {strcpy(inputVect,&str[1]);
         strncpy(className,&(Plot[id]->vVarz[start]),str-&(Plot[id]->vVarz[start]));
        }
     className[str-&(Plot[id]->vVarz[start])] = '\0';

     strcpy(inputPar," ");
     if (Plot[id]->realUnits == ON) strcat(inputPar, "REALUNITS_");

     proc = FrProcDataFind(Dy->Frame,Plot[id]->vVarz);
     if (proc && proc->nAuxParam) strcat(inputPar,proc->auxParamNames[0]);
     else                         strcat(inputPar,"NULL");

     sprintf (Dy->plotString[id], "%s \nUFrVectOp %ld %ld %s %s %s input.frameH",
              Dy->plotString[id], frame_clock, id, className, inputVect, inputPar);
    }

 /*--- Determine if superposed plot has same sampling frequency as the reference one ---*/
/**
 samefreq = 1;
 j0 = 0;
 if (Plot[id]->superposed == ON)
    {for (j=id; j>0; j--)
        {if (Plot[j]->superposed == OFF)
            {if (Plot[j]->freqClock != Plot[id]->freqClock) samefreq = 0;
             j0=j;
             break;
            }
        }
    }
 printf("PLOT=%s  samefreq=%d  Plotref=%d=%s\n", Plot[id]->name, samefreq, j0, Plot[j0]->name);
**/

 samefreq = 0;

 /*--- To put correct dephasing of clock of superposed plots ---*/

 if (Plot[id]->superposed == ON && samefreq == 0)
    {Plot[id]->clockN   = - Plot[id]->clockN;
     Plot[id]->clockN2  = - Plot[id]->clockN2;
     Plot[id]->clockN12 = - Plot[id]->clockN12;
    }

 if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1)
    {printf("DyMakePlotString> plot%d=%s  clockN=%d  options=%s\n", id, Plot[id]->name, Plot[id]->clockN, options);}


 /*--- RAW plot ---*/

 if (plottype == PLOTRAW)
    {
     sprintf (signal, "signal%ld",id);
     sprintf (signalout, "signal%ld.out",id);

     sprintf (Dy->plotString[id], "%s \nUFrUnpack %s %ld %s input.frameH",
              Dy->plotString[id], signal, Plot[id]->clockN, variablex);

     sprintf(dummy, "%sRAW", SEPTITLE);
     sprintf (title, "%s%s", Plot[id]->name, dummy);
     sprintf (Dy->plotString[id],"%s \nUHplotRaw %ld %ld %s %ld %s %s",
              Dy->plotString[id], Plot[id]->clockN, id, title, t0, signalout, options);
    }


 /*--- TIME plot ---*/

 if (plottype == PLOTTIME)
    {
     sprintf (signal, "signal%ld",id);
     sprintf (signalout, "signal%ld.out",id);

     sprintf (Dy->plotString[id], "%s \nUFrUnpack %s %ld %s input.frameH",
              Dy->plotString[id], signal, Plot[id]->clockN, variablex);

     sprintf(dummy, "%sTIME", SEPTITLE);
     sprintf (title, "%s%s", Plot[id]->name, dummy);
     sprintf (Dy->plotString[id],"%s \nUHplotTime %ld %ld %s %ld %ld %ld %s %s",
              Dy->plotString[id], Plot[id]->clockN, id, title, 
              Plot[id]->xnBin, Plot[id]->resampFactor, t0, signalout, options);
    }


 /*--- FFT plot ---*/

 if (plottype == PLOTFFT)
    {
     sprintf (signal, "signal%ld",id);
     sprintf (signalout, "signal%ld.out",id);

     sprintf (Dy->plotString[id], "%s \nUFrUnpack %s %ld %s input.frameH",
              Dy->plotString[id], signal, Plot[id]->clockN, variablex);

     sprintf(dummy, "%sFFT", SEPTITLE);
     sprintf (title, "%s%s", Plot[id]->name, dummy);
     sprintf (Dy->plotString[id], "%s \nUHplotFFT %ld %ld %s %ld %ld %ld %s %s",
              Dy->plotString[id], Plot[id]->clockN, id, title,
              Plot[id]->nBinP2, Plot[id]->resampFactor, t0, signalout, options);
    }


 /*--- to use those variables also for vVary ---*/

 if (adcName)  {free(adcName);  adcName  = NULL;}
 if (smsName)  {free(smsName);  smsName  = NULL;}
 if (varName)  {free(varName);  varName  = NULL;}
 if (procName) {free(procName); procName = NULL;}
 if (simName)  {free(simName);  simName  = NULL;}

 /*--- TRansfert function plot ---*/

 if (plottype == PLOTTR)
    {
     DyDecodeVar(Plot[id]->vVary, &smsName, &varName, &adcName, &procName, &simName);
     if (smsName && varName) 
        {if (Plot[id]->freqClock < 1./dt) sprintf (variabley, "sms%d.%s%s%s", (int)(1./Plot[id]->freqClock2 + 0.5), smsName, SEP, varName);
         else                             sprintf (variabley, "sms.%s%s%s", smsName, SEP, varName);
        }
     else if (adcName)       sprintf (variabley, "adc.%s", adcName);
     else if (simName)       sprintf (variabley, "sim.%s", simName);
     else if (procName)      sprintf (variabley, "rec.%s", procName);
     else                    sprintf (variabley, "adc.%s",Plot[id]->vVary);

     /*--- Build final string ---*/

     sprintf (signalx, "signalx%ld",id);
     sprintf (signalxout, "signalx%ld.out",id);
     sprintf (signaly, "signaly%ld",id);
     sprintf (signalyout, "signaly%ld.out",id);
     sprintf (Dy->plotString[id], "%s \nUFrUnpack %s %ld %s input.frameH"
                                  "\nUFrUnpack %s %ld %s input.frameH",
              Dy->plotString[id],
              signalx, Plot[id]->clockN, variablex,
              signaly, Plot[id]->clockN2, variabley);

     sprintf(dummy, "%sTRFCT", SEPTITLE);
     sprintf (title, "%s%s", Plot[id]->name, dummy);
     sprintf (Dy->plotString[id], "%s \nUHpTfct %ld %ld %ld %s %ld %ld %ld %s %s %s %s",
              Dy->plotString[id], Plot[id]->clockN12, id, id+1000, title, 
              Plot[id]->nBinP2, Plot[id]->resampFactor, t0,
              signalyout, signalxout, options, options2);
    }


 /*--- COHErence function plot ---*/

 if (plottype == PLOTCOHE)
    {
     DyDecodeVar(Plot[id]->vVary, &smsName, &varName, &adcName, &procName, &simName);
     if (smsName && varName) 
        {if (Plot[id]->freqClock < 1./dt) sprintf (variabley, "sms%d.%s%s%s", (int)(1./Plot[id]->freqClock2 + 0.5), smsName, SEP, varName);
         else                             sprintf (variabley, "sms.%s%s%s", smsName, SEP, varName);
        }
     else if (adcName)       sprintf (variabley, "adc.%s", adcName);
     else if (simName)       sprintf (variabley, "sim.%s", simName);
     else if (procName)      sprintf (variabley, "rec.%s", procName);
     else                    sprintf (variabley, "adc.%s",Plot[id]->vVary);

     /*--- Build final string ---*/

     sprintf (signalx, "signalx%ld",id);
     sprintf (signalxout, "signalx%ld.out",id);
     sprintf (signaly, "signaly%ld",id);
     sprintf (signalyout, "signaly%ld.out",id);
     sprintf (Dy->plotString[id], "%s \nUFrUnpack %s %ld %s input.frameH"
                                  "\nUFrUnpack %s %ld %s input.frameH",
              Dy->plotString[id],
              signalx, Plot[id]->clockN, variablex,
              signaly, Plot[id]->clockN2, variabley);

     sprintf(dummy, "%sCOHE", SEPTITLE);
     sprintf (title, "%s%s", Plot[id]->name, dummy);
     sprintf (Dy->plotString[id], "%s \nUHpCohe %ld %ld %ld %s %ld %ld %ld %s %s %s %s",
              Dy->plotString[id], Plot[id]->clockN12, id, id+1000, title,
              Plot[id]->nBinP2, Plot[id]->resampFactor, t0,
              signalxout, signalyout, options, options2);
    }


 /*--- CORRelation function plot ---*/

 if (plottype == PLOTCORR)
    {
     DyDecodeVar(Plot[id]->vVary, &smsName, &varName, &adcName, &procName, &simName);
     if (smsName && varName) 
        {if (Plot[id]->freqClock < 1./dt) sprintf (variabley, "sms%d.%s%s%s", (int)(1./Plot[id]->freqClock2 + 0.5), smsName, SEP, varName);
         else                             sprintf (variabley, "sms.%s%s%s", smsName, SEP, varName);
        }
     else if (adcName)       sprintf (variabley, "adc.%s", adcName);
     else if (simName)       sprintf (variabley, "sim.%s", simName);
     else if (procName)      sprintf (variabley, "rec.%s", procName);
     else                    sprintf (variabley, "adc.%s",Plot[id]->vVary);

     /*--- Build final string ---*/

     sprintf (signalx, "signalx%ld",id);
     sprintf (signalxout, "signalx%ld.out",id);
     sprintf (signaly, "signaly%ld",id);
     sprintf (signalyout, "signaly%ld.out",id);
     sprintf (Dy->plotString[id], "%s \nUFrUnpack %s %ld %s input.frameH"
                                  "\nUFrUnpack %s %ld %s input.frameH",
              Dy->plotString[id],
             signalx, Plot[id]->clockN, variablex,
             signaly, Plot[id]->clockN2, variabley);

     sprintf(dummy, "%sCORR", SEPTITLE);
     sprintf (title, "%s%s", Plot[id]->name, dummy);
     sprintf (Dy->plotString[id], "%s \nUHpCorr %ld %ld %s %ld %ld %ld %s %s %s",
              Dy->plotString[id], Plot[id]->clockN12, id, title,
              Plot[id]->nBinP2,  Plot[id]->resampFactor, t0, signalxout, signalyout, options);
    }



 /*--- 3D plot ---*/

 if (plottype == PLOT3D)
    {
     DyDecodeVar(Plot[id]->vVarz, &smsName, &varName, &adcName, &procName, &simName);
     if (smsName && varName) 
        {if (Plot[id]->freqClock < 1./dt) sprintf (variablez, "sms%d.%s%s%s", (int)(1./Plot[id]->freqClock3 + 0.5), smsName, SEP, varName);
         else                             sprintf (variablez, "sms.%s%s%s", smsName, SEP, varName);
        }
     else if (adcName)       sprintf (variablez, "adc.%s", adcName);
     else if (simName)       sprintf (variablez, "sim.%s", simName);
     else if (procName)      sprintf (variablez, "rec.%s", procName);
     else                    sprintf (variablez, "adc.%s",Plot[id]->vVarz);

     if (adcName)  {free(adcName);  adcName  = NULL;}
     if (smsName)  {free(smsName);  smsName  = NULL;}
     if (varName)  {free(varName);  varName  = NULL;}
     if (procName) {free(procName); procName = NULL;}
     if (simName)  {free(simName);  simName  = NULL;}

     DyDecodeVar(Plot[id]->vVary, &smsName, &varName, &adcName, &procName, &simName);
     if (smsName && varName) 
        {if (Plot[id]->freqClock < 1./dt) sprintf (variabley, "sms%d.%s%s%s", (int)(1./Plot[id]->freqClock2 + 0.5), smsName, SEP, varName);
         else                             sprintf (variabley, "sms.%s%s%s", smsName, SEP, varName);
        }
     else if (adcName)       sprintf (variabley, "adc.%s", adcName);
     else if (simName)       sprintf (variabley, "sim.%s", simName);
     else if (procName)      sprintf (variabley, "rec.%s", procName);
     else                    sprintf (variabley, "adc.%s",Plot[id]->vVary);

     sprintf (signalx, "signalx%ld",id);
     sprintf (signalxout, "signalx%ld.out",id);
     sprintf (signaly, "signaly%ld",id);
     sprintf (signalyout, "signaly%ld.out",id);
     sprintf (signalz, "signalz%ld",id);
     sprintf (signalzout, "signalz%ld.out",id);

     sprintf (Dy->plotString[id], "%s \nUFrUnpack %s %ld %s input.frameH"
                                  "\nUFrUnpack %s %ld %s input.frameH"
                                  "\nUFrUnpack %s %ld %s input.frameH",
              Dy->plotString[id],
              signalx, Plot[id]->clockN, variablex,
              signaly, Plot[id]->clockN2, variabley,
              signalz, Plot[id]->clockN3, variablez);

     sprintf(dummy, "%s3D", SEPTITLE);
     sprintf (title, "%s%s", Plot[id]->name, dummy);
     sprintf (Dy->plotString[id],"%s \nUHplot3D %ld %ld %s %ld %g %g %ld %g %g %s %s %s %ld %ld %s",
              Dy->plotString[id], Plot[id]->clockN12, id, title,
              Plot[id]->xnBin, Plot[id]->xMin, Plot[id]->xMax,
              Plot[id]->ynBin, Plot[id]->yMin, Plot[id]->yMax,
              signalyout, signalxout, signalzout, t0, tmax, options);
    }

 /*--- 2D plot ---*/

 if (plottype == PLOT2D)
    {
     DyDecodeVar(Plot[id]->vVary, &smsName, &varName, &adcName, &procName, &simName);
     if (smsName && varName) 
        {if (Plot[id]->freqClock < 1./dt) sprintf (variabley, "sms%d.%s%s%s", (int)(1./Plot[id]->freqClock2 + 0.5), smsName, SEP, varName);
         else                             sprintf (variabley, "sms.%s%s%s", smsName, SEP, varName);
        }
     else if (adcName)       sprintf (variabley, "adc.%s", adcName);
     else if (simName)       sprintf (variabley, "sim.%s", simName);
     else if (procName)      sprintf (variabley, "rec.%s", procName);
     else                    sprintf (variabley, "adc.%s",Plot[id]->vVary);

     sprintf (signalx, "signalx%ld",id);
     sprintf (signalxout, "signalx%ld.out",id);
     sprintf (signaly, "signaly%ld",id);
     sprintf (signalyout, "signaly%ld.out",id);

     sprintf (Dy->plotString[id], "%s \nUFrUnpack %s %ld %s input.frameH"
                                  "\nUFrUnpack %s %ld %s input.frameH",
              Dy->plotString[id],
              signalx, Plot[id]->clockN, variablex,
              signaly, Plot[id]->clockN2, variabley);

     sprintf(dummy, "%s2D", SEPTITLE);
     sprintf (title, "%s%s", Plot[id]->name, dummy);
     sprintf (Dy->plotString[id],"%s \nUHplot2D %ld %ld %s %ld %g %g %ld %g %g %s %s %ld %ld %s",
              Dy->plotString[id], Plot[id]->clockN12, id, title,
              Plot[id]->xnBin, Plot[id]->xMin, Plot[id]->xMax,
              Plot[id]->ynBin, Plot[id]->yMin, Plot[id]->yMax,
              signalyout, signalxout, t0, tmax, options);
    }


 /*--- 1D plot ---*/

 if (plottype == PLOT1D)
    {
     sprintf (signal, "signal%ld", id);
     sprintf (signalout, "signal%ld.out", id);

     sprintf (Dy->plotString[id], "%s \nUFrUnpack %s %ld %s input.frameH",
              Dy->plotString[id], signal, Plot[id]->clockN, variablex);

     sprintf(dummy, "%s1D", SEPTITLE);
     sprintf (title, "%s%s", Plot[id]->name, dummy);
     sprintf (Dy->plotString[id], "%s \nUHplot1D %ld %ld %s %ld %g %g %s %ld %ld %s",
              Dy->plotString[id], Plot[id]->clockN, id, title,
              Plot[id]->xnBin, Plot[id]->xMin, Plot[id]->xMax, signalout, t0, tmax, options);
    }


 /*--- AUDIO plot ---*/

 if (plottype == PLOTAUDIO)
    {
     /*--- Build final string ---*/
     sprintf (signal, "signal%ld", id);
     sprintf (signalout, "signal%ld.out", id);

     sprintf (Dy->plotString[id], "%s \nUFrUnpack %s %ld %s input.frameH",
              Dy->plotString[id], signal, Plot[id]->clockN, variablex);

     sprintf (Dy->plotString[id], "%s \nUHsound %ld %s %ld %ld %s",
              Dy->plotString[id], Plot[id]->clockN, signalout,
              Plot[id]->t0, Plot[id]->duration, options);
    }

 /*--- FFTTime plot ---*/

 if (plottype == PLOTFFTTIME)
    {
     sprintf (signal, "signal%ld",id);
     sprintf (signalout, "signal%ld.out",id);

     sprintf (Dy->plotString[id], "%s \nUFrUnpack %s %ld %s input.frameH",
              Dy->plotString[id], signal, Plot[id]->clockN, variablex);

     sprintf(dummy, "%sFFTTIME", SEPTITLE);
     sprintf (title, "%s%s", Plot[id]->name, dummy);
     sprintf (Dy->plotString[id], "%s \nUHplotFFTTime %ld %ld %s %ld %g %ld %ld %s %s",
              Dy->plotString[id], Plot[id]->clockN, id, title,
              Plot[id]->nBinP2, Plot[id]->timeSize,
              Plot[id]->resampFactor, t0, signalout, options);
    }

 /*--- TrFctTime plot ---*/

 if (plottype == PLOTTRTIME)
    {
     DyDecodeVar(Plot[id]->vVary, &smsName, &varName, &adcName, &procName, &simName);
     if (smsName && varName) 
        {if (Plot[id]->freqClock < 1./dt) sprintf (variabley, "sms%d.%s%s%s", (int)(1./Plot[id]->freqClock2 + 0.5), smsName, SEP, varName);
         else                             sprintf (variabley, "sms.%s%s%s", smsName, SEP, varName);
        }
     else if (adcName)       sprintf (variabley, "adc.%s", adcName);
     else if (simName)       sprintf (variabley, "sim.%s", simName);
     else if (procName)      sprintf (variabley, "rec.%s", procName);
     else                    sprintf (variabley, "adc.%s",Plot[id]->vVary);

     /*--- Build final string ---*/

     sprintf (signalx, "signalx%ld",id);
     sprintf (signalxout, "signalx%ld.out",id);
     sprintf (signaly, "signaly%ld",id);
     sprintf (signalyout, "signaly%ld.out",id);
     sprintf (Dy->plotString[id], "%s \nUFrUnpack %s %ld %s input.frameH"
                                  "\nUFrUnpack %s %ld %s input.frameH",
              Dy->plotString[id],
              signalx, Plot[id]->clockN, variablex,
              signaly, Plot[id]->clockN2, variabley);

     sprintf(dummy, "%sTRFCTTIME", SEPTITLE);
     sprintf (title, "%s%s", Plot[id]->name, dummy);
     sprintf (Dy->plotString[id], "%s \nUHpTfctTime %ld %ld %s %ld %g %ld %ld %s %s %s",
              Dy->plotString[id], Plot[id]->clockN, id, title,
              Plot[id]->nBinP2, Plot[id]->timeSize,
              Plot[id]->resampFactor, t0, signalyout, signalxout, options);
    }

 /*--- COHETime plot ---*/

 if (plottype == PLOTCOHETIME)
    {
     DyDecodeVar(Plot[id]->vVary, &smsName, &varName, &adcName, &procName, &simName);
     if (smsName && varName) 
        {if (Plot[id]->freqClock < 1./dt) sprintf (variabley, "sms%d.%s%s%s", (int)(1./Plot[id]->freqClock2 + 0.5), smsName, SEP, varName);
         else                             sprintf (variabley, "sms.%s%s%s", smsName, SEP, varName);
        }
     else if (adcName)       sprintf (variabley, "adc.%s", adcName);
     else if (simName)       sprintf (variabley, "sim.%s", simName);
     else if (procName)      sprintf (variabley, "rec.%s", procName);
     else                    sprintf (variabley, "adc.%s",Plot[id]->vVary);

     /*--- Build final string ---*/

     sprintf (signalx, "signalx%ld",id);
     sprintf (signalxout, "signalx%ld.out",id);
     sprintf (signaly, "signaly%ld",id);
     sprintf (signalyout, "signaly%ld.out",id);
     sprintf (Dy->plotString[id], "%s \nUFrUnpack %s %ld %s input.frameH"
                                  "\nUFrUnpack %s %ld %s input.frameH",
              Dy->plotString[id],
              signalx, Plot[id]->clockN, variablex,
              signaly, Plot[id]->clockN2, variabley);

     sprintf(dummy, "%sCOHETIME", SEPTITLE);
     sprintf (title, "%s%s", Plot[id]->name, dummy);
     sprintf (Dy->plotString[id], "%s \nUHpCoheTime %ld %ld %s %ld %g %ld %ld %s %s %s",
              Dy->plotString[id], Plot[id]->clockN, id, title,
              Plot[id]->nBinP2, Plot[id]->timeSize,
              Plot[id]->resampFactor, t0, signalxout, signalyout, options);
    }

 /*--- 1DTime plot ---*/

 if (plottype == PLOT1DTIME)
    {
     sprintf (signal, "signal%ld",id);
     sprintf (signalout, "signal%ld.out",id);

     sprintf (Dy->plotString[id], "%s \nUFrUnpack %s %ld %s input.frameH",
              Dy->plotString[id], signal, Plot[id]->clockN, variablex);

     sprintf(dummy, "%s1DTIME", SEPTITLE);
     sprintf (title, "%s%s", Plot[id]->name, dummy);
     sprintf (Dy->plotString[id], "%s \nUHplot1DTime %ld %ld %s %ld %g %g %g %ld %ld %s %s",
              Dy->plotString[id], Plot[id]->clockN, id, title,
              Plot[id]->xnBin, Plot[id]->xMin, Plot[id]->xMax, Plot[id]->timeSize,
              Plot[id]->resampFactor, t0, signalout, options);
    }

 /*--- RawTime plot ---*/

 if (plottype == PLOTRAWTIME)
    {
     sprintf (signal, "signal%ld",id);
     sprintf (signalout, "signal%ld.out",id);

     sprintf (Dy->plotString[id], "%s \nUFrUnpack %s %ld %s input.frameH",
              Dy->plotString[id], signal, Plot[id]->clockN, variablex);

     sprintf(dummy, "%sRAWTIME", SEPTITLE);
     sprintf (title, "%s%s", Plot[id]->name, dummy);
     sprintf (Dy->plotString[id], "%s \nUHplotRawTime %ld %ld %s %ld %g %g %g %ld %ld %s %s",
              Dy->plotString[id], Plot[id]->clockN, id, title,
              Plot[id]->xnBin, Plot[id]->xMin, Plot[id]->xMax, Plot[id]->timeSize,
              Plot[id]->resampFactor, t0, signalout, options);
    }

 if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1)
    {printf("DyMakePlotString> plot%d=%s  type=%ld=%s  clockN=%ld \n plotString[%ld]=%s \n",
                             id, Plot[id]->name, Plot[id]->type, Dy->typeName[Plot[id]->type], Plot[id]->clockN, id, Dy->plotString[id]);}

 /*--- To put back positive clock for superposed plots ---*/

 if (Plot[id]->superposed == ON && samefreq == 0)
    {Plot[id]->clockN   = - Plot[id]->clockN;
     Plot[id]->clockN2  = - Plot[id]->clockN2;
     Plot[id]->clockN12 = - Plot[id]->clockN12;
    }

 if (adcName)  {free(adcName);  adcName  = NULL;}
 if (smsName)  {free(smsName);  smsName  = NULL;}
 if (varName)  {free(varName);  varName  = NULL;}
 if (procName) {free(procName); procName = NULL;}
 if (simName)  {free(simName);  simName  = NULL;}

 return;
}


/*------------------------------------------- DyMakeStartEndString ---*/

void DyMakeStartEndString ()
{
 DyDebugPrint(__FUNCTION__,0);

 sprintf(Dy->StartEndString, "/*>  %s  %ld  %ld  %ld  %d  %d  <*/",
         Dy->CmServer, Dy->firstTime, Dy->startTime, Dy->duration, Dy->startRun, Dy->startFrame);

 return;
}


/*------------------------------------------- DyMakeRowColString ---*/

void DyMakeRowColString ()
{
 char dummy[LSTRING_MAX];
 const char *str;
 int nfile, i;

 DyDebugPrint(__FUNCTION__,0);

 sprintf(Dy->RowColString, "/**> %ld  %ld ", Dy->nCol, Dy->nRow);
 if (Dy->triggerUsed == ON)          {sprintf(dummy,"triggertype=%d ", Dy->triggerType); strcat(Dy->RowColString, dummy);}
 if (Dy->triggerType == TRIG_ASCII)  {sprintf(dummy,"trigasciifile=%s ", Dy->trigAsciiFileName); strcat(Dy->RowColString, dummy);}
 if (Dy->triggerType == TRIG_EVENTS)  {sprintf(dummy,"trigeventname=%s ", Dy->trigEventName); strcat(Dy->RowColString, dummy);}
 if (Dy->triggerUsed == ON)       strcat(Dy->RowColString, "triggerused ");
 if (Dy->triggerPause == ON)      strcat(Dy->RowColString, "triggerpause ");
 if (Dy->plotsNotDisplayed == ON) strcat(Dy->RowColString, "noplot ");
 if (Dy->squarePlot == ON)        strcat(Dy->RowColString, "squareplot ");
 if (Dy->AutoDeselectPlot == ON)  strcat(Dy->RowColString, "autodeselectplot ");
 if (Dy->AutoDeselectCh == ON)    strcat(Dy->RowColString, "autodeselectch ");
 if (Dy->skipFrame == ON)         strcat(Dy->RowColString, "skipframe ");
 if (Dy->colorpalette > 0)         {sprintf(dummy,"col2d=%d ", Dy->colorpalette); strcat(Dy->RowColString, dummy);}
 if (Dy->output_flag == ON)
    {sprintf(dummy,"output_flag output_dir=%s output_nframe=%d ", Dy->output_dir, Dy->output_nframe_perfile);
     strcat(Dy->RowColString, dummy);
     sprintf(dummy,"output_tag=%s ", Dy->output_tag);
     DyFilterString(dummy,'&');
     strcat(Dy->RowColString, dummy);
    }

 if (Dy->Ddpasswd) {sprintf(dummy,"ddpasswd=%s ", Dy->Ddpasswd); strcat(Dy->RowColString, dummy);}

 nfile = fl_get_browser_maxline(reffile->file_list);
 if (Dy->refplot_indelebile == ON && nfile>0)
    {strcpy(dummy,"ddref=");
     for (i=0; i<nfile; i++)
        {str = fl_get_browser_line(reffile->file_list, i+1);
         strcat(dummy,(char *)str); strcat(dummy,"__");
        }
     strcat(Dy->RowColString, dummy);
    }
 
 strcat(Dy->RowColString," <**/");

 return;
}


/*------------------------------------------- DyMakeSiestaFile ---*/

long DyMakeSiestaFile (char *fileName, int noroot)
{
 int err;
 long i, nplot, nFile, nClock, ndump;
 FILE *fp;
 const char* canvasmacro;
 time_t tt;
 char date_heure[24];

 DyDebugPrint(__FUNCTION__,0);

 DyDebugTimeNeeded(__FUNCTION__,"start",0);

 nClock = DyMakeClockString();

 DyMakeTriggerString();

#ifdef USE_VEGA
 if (Dy->frameChannel) nFile = DyMakeFileString(4);
 else if (Dy->InputType == INFILE  ||  Dy->InputType == INDIR)  nFile = DyMakeFileString(INFILE);
#else
 nFile = DyMakeFileString(Dy->InputType);
#endif
 
 nplot = fl_get_browser_maxline(mainbrowser->plots);
 ndump = fl_get_browser_maxline(dumpdata->datatodump);

 if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>0)
    {printf("DyMakeSiestaFile> nClock=%ld  nFile=%ld  nPlot=%ld nDump=%ld \n", nClock, nFile, nplot, ndump);}

 if (nClock <= 0 || nFile <= 0 || (nplot <= 0 && ndump <= 0 && Dy->output_flag == OFF)) return(-1);

 fp = fopen(fileName,"w");
 if (fp==NULL)
    {printf("DyMakeSiestaFile> Cannot open file %s to save current configuration.\n", fileName);
     return(0);
    }
 tt = time(NULL);
 strftime(date_heure,24,"%d_%b_%Y_%T",localtime(&tt));
 fprintf(fp,"/*** Made on %s by %s using dataDisplay %s on %s (display=%s) ***/\n\n",
            date_heure, getenv("USER"), DYVERSION, getenv("HOST"), getenv("DISPLAY"));

/** You can use also the C functions: cuserid, gethostname, DisplayString ***/

 fprintf(fp,"%s \n\n", Dy->clockString);
 fprintf(fp,"%s \n\n", Dy->fileString);
 if (Dy->triggerUsed == ON) fprintf(fp,"%s \n\n", Dy->triggerString);
 for (i=1; i<=nplot; i++)
     {DyMakePlotString(i);
      fprintf(fp,"%s \n\n", Dy->plotString[i]);
     }

 DyMakeStartEndString();
 fprintf(fp,"%s \n\n", Dy->StartEndString);
 
 DyMakeRowColString();
 fprintf(fp,"%s \n\n", Dy->RowColString);

 /*--- Save the global canvas configuration as a C++ macro --- */
 if (noroot == 0)
    {canvasmacro = RA_CanvasMacroString();
     fprintf(fp,"/*<\n%s\n>*/\n",canvasmacro);
    }

 /*--- Put also in the .car file all the .C and .h files associated to User Fct used ---*/
 DyMakeAlgoString(fp);

 /*--- Close .car file and check that it was really written on disk ---*/
 err = fprintf(fp,"/* This is the End */\n");
 fclose(fp);
 if (err < 0)   /* err=ERNOSPC would mean that there is no free space left on device */
    {printf("DyMakeSiestaFile> Could not write the file %s. Maybe no more space left on device.\n", fileName);
     return(0);
    }

 if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1)
    {printf("\n\n");
     printf("------------------ Parameters for Siesta Cards File -----------------------\n");
     printf("%s \n", Dy->clockString);
     printf("%s \n", Dy->fileString);
     for (i=1;i<=nplot;i++) printf("    %s \n", Dy->plotString[i]);
     printf("%s \n", Dy->StartEndString);
     printf("------------------ ============================== -----------------------\n\n");
    }

 free(Dy->fileString);

 DyDebugTimeNeeded(__FUNCTION__,"stop",0);

 return(1+nFile+nplot);
}


/*------------------------------------------- DyMakeAlgoString ---*/

void DyMakeAlgoString (FILE *fp)
{
 FILE *fpalgo;
 const char *strproc;
 char str[LBIGSTRING_MAX], algoname[LSTRING_MAX],  algofile[LSTRING_MAX];
 int i, nproc, length;

 DyDebugPrint(__FUNCTION__,0);

 nproc = fl_get_browser_maxline(mainbrowser->proc);
 for (i=1; i<=nproc; i++)
    {
     strproc = fl_get_browser_line(mainbrowser->proc,i);

     if (strncmp(strproc,OPSIGNAL,strlen(OPSIGNAL)) == 0)
        {
         length = strstr(strproc,",")-(strproc+strlen(OPSIGNAL));
         strncpy(algoname,strproc+strlen(OPSIGNAL), length);
         algoname[length]='\0';

         sprintf(algofile, "%s.C", algoname);
         fpalgo = fopen(algofile, "r");
         if (fpalgo == NULL)
            {sprintf(algofile, "%s/%s/%s.C", DYALGO_DIR, getenv("UNAME"), algoname);
             fpalgo = fopen(algofile, "r");
            }
         if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1)
            {printf("DyMakeAlgoString> proc%d: found algo %s... searching for %s\n", i, algoname, algofile);}

         if (fpalgo != NULL)
            {
             fprintf(fp, "\n%s %s %s\n", ALGO_BEGIN, algoname, algofile);
             while (fgets(str,2048,fpalgo)) fprintf(fp, "%s", str);
             fprintf(fp, "\n%s\n", ALGO_END);
            }
         else
            {printf("DyMakeAlgoString> %s not found \n", algofile);}

         sprintf(algofile, "%s.h", algoname);
         fpalgo = fopen(algofile, "r");
         if (fpalgo == NULL)
            {sprintf(algofile, "%s/%s/%s.h", DYALGO_DIR, getenv("UNAME"), algoname);
             fpalgo = fopen(algofile, "r");
            }
         if (((Dy->debugType)&(1<<DBG_CONFIG)) && Dy->debug>1)
            {printf("DyMakeAlgoString> proc%d: found algo %s... searching for %s\n", i, algoname, algofile);}

         if (fpalgo != NULL)
            {
             fprintf(fp, "\n%s %s %s\n", ALGO_BEGIN, algoname, algofile);
             while (fgets(str,2048,fpalgo)) fprintf(fp, "%s", str);
             fprintf(fp, "\n%s\n", ALGO_END);
            }
         else
            {printf("DyMakeAlgoString> %s not found \n", algofile);}
        }
    }

 return;
}


/*------------------------------------------- DyMakePlotOptions ---*/

void DyMakePlotOptions (long id, char *options, char *options2)
{
 char refreshPeriod[LSTRING_MAX];
 double frameFreq;

 /*--- Build options string (first part) ---*/

 strcpy(options, "\042");
 sprintf(refreshPeriod, "%ld", Plot[id]->refreshPeriod);
 strcat(options, refreshPeriod);

 if (Plot[id]->hidden == ON) strcat (options, " HIDDEN");

 if (Plot[id]->superposed == ON)  strcat (options, " SAME");

 if (Plot[id]->realUnits)  strcat (options, " REALUNITS");

 frameFreq = 1./DyGetFramedt(Dy->Frame);
 if ((Plot[id]->freqClock>0. && Plot[id]->freqClock < frameFreq) || (Plot[id]->freqClock12>0. && Plot[id]->freqClock12 < frameFreq))
    {strcat (options, " SMSPERIOD");
     sprintf(options, "%s %d", options, (int)(1./Plot[id]->freqClock + 0.5));
    }

 if (Plot[id]->type == PLOTRAW)
    {
     strcat (options, " NFFT");
     sprintf(options, "%s %ld", options, Plot[id]->average_fft);
     if (Plot[id]->autoScale == ON)  strcat (options, " AUTOSCALE");
     if (Plot[id]->x2av == ON)       strcat (options, " X2AV");
     strcat (options, " YMIN");
     sprintf(options, "%s %g", options, Plot[id]->yMin);
     strcat (options, " YMAX");
     sprintf(options, "%s %g", options, Plot[id]->yMax);
     strcat (options, " COL");
    }

 if (Plot[id]->type == PLOTFFT    ||  Plot[id]->type == PLOTFFTTIME  ||
     Plot[id]->type == PLOTTR     ||  Plot[id]->type == PLOTCOHE     ||
     Plot[id]->type == PLOTTRTIME ||  Plot[id]->type == PLOTCOHETIME)
    {
     strcat (options, " NFFT");
     sprintf(options, "%s %ld", options, Plot[id]->average_fft);
     if (Plot[id]->decayfft == ON) strcat (options, " DECAYFFT");
     if (Plot[id]->refresh_fft > 0)
        {strcat (options, " NFFT_REFRESH");
         sprintf(options, "%s %ld", options, Plot[id]->refresh_fft);
        }

     if (Plot[id]->crossCohe == ON)
        {
         strcat (options, " CROSSCOHE_FMIN");
         sprintf(options, "%s %g", options, Plot[id]->crossCohe_fmin);
         strcat (options, " CROSSCOHE_FMAX");
         sprintf(options, "%s %g", options, Plot[id]->crossCohe_fmax);
        }

     strcat (options, " FMIN");
     sprintf(options, "%s %g", options, Plot[id]->xMin);
     strcat (options, " FMAX");
     sprintf(options, "%s %g", options, Plot[id]->xMax);
     if (Plot[id]->autoScale == ON)  strcat (options, " AUTOSCALE");
     if (Plot[id]->modulesqrt == ON) strcat (options, " MODULESQRT");
     strcat (options, " YMIN");
     sprintf(options, "%s %g", options, Plot[id]->yMin);
     strcat (options, " YMAX");
     sprintf(options, "%s %g", options, Plot[id]->yMax);
    }

 if (Plot[id]->type == PLOTTIME  ||  Plot[id]->type == PLOTCORR) 
    {
     if (Plot[id]->autoScale == ON)  strcat (options, " AUTOSCALE");
     strcat (options, " YMIN");
     sprintf(options, "%s %g", options, Plot[id]->yMin);
     strcat (options, " YMAX");
     sprintf(options, "%s %g", options, Plot[id]->yMax);
     if (Plot[id]->filterMin > 0 && Plot[id]->filterMax > 0)
        {strcat (options, " FILTERMIN");
         sprintf(options, "%s %g", options, Plot[id]->filterMin);
         strcat (options, " FILTERMAX");
         sprintf(options, "%s %g", options, Plot[id]->filterMax);
        }
     if (Plot[id]->yoffset != 0)
        {strcat (options, " YOFFSET");
         sprintf(options, "%s %g", options, Plot[id]->yoffset);
        }
     if (Plot[id]->yscale != 0)
        {
         strcat (options, " YSCALE");
         sprintf(options, "%s %g", options, Plot[id]->yscale);
        }
    }

 if (Plot[id]->type == PLOTTIME    ||  Plot[id]->type == PLOTCORR ||
     Plot[id]->type == PLOTFFTTIME ||
     Plot[id]->type == PLOT1DTIME  ||
     Plot[id]->type == PLOTRAWTIME || Plot[id]->type == PLOTTRTIME || Plot[id]->type == PLOTCOHETIME )
    {strcat (options, " TIMX");}

 /*--- Build options string (second part) ---*/

 if (Dy->plotsNotDisplayed == ON)  strcat (options, " NOPLOT");

 if (Plot[id]->minmax_resample)  strcat (options, " MINMAX_RESAMPLE");

 if (Plot[id]->nodc)             strcat (options, " NODC");
 if (Plot[id]->bits)             strcat (options, " BITS");

 if (Plot[id]->gridx)            strcat (options, " GRIDX");
 if (Plot[id]->gridy)            strcat (options, " GRIDY");
 if (Plot[id]->logx)             strcat (options, " LOGX");
 if (Plot[id]->logy)             strcat (options, " LOGY");
 if (Plot[id]->logz)             strcat (options, " LOGZ");

 if (Plot[id]->type == PLOT1D  ||  Plot[id]->type ==  PLOT2D  ||  Plot[id]->type ==  PLOT3D)
    {
     if (Plot[id]->resampFactor > 1)
        {strcat (options, " RESAMPFACTOR");
         sprintf(options, "%s %ld", options, Plot[id]->resampFactor);
        }

     if (Plot[id]->stat)             strcat (options, " STAT");
     if (Plot[id]->autoEdge)         strcat (options, " AUTO");
     if (Plot[id]->type2d == COL)    strcat (options, " COL");
     if (Plot[id]->type2d == CONT)   strcat (options, " CONT");
     if (Plot[id]->type2d == LEGO)   strcat (options, " LEGO");
     if (Plot[id]->type2d == SURF)   strcat (options, " SURF");
     if (Plot[id]->type2d == SCAT)   strcat (options, " SCAT");

     if (Plot[id]->nSampling > 0)
        {strcat (options, " NSAMPLING");
         sprintf(options, "%s %ld", options, Plot[id]->nSampling);
        }
    }

 if (Plot[id]->type == PLOTFFT)
    {if (Plot[id]->hertz == ON)    strcat (options, " HERTZ");
     if (Plot[id]->rmsfft == ON)   strcat (options, " RMSFFT");
    }

 if (Plot[id]->type == PLOTFFTTIME || Plot[id]->type == PLOT1DTIME || Plot[id]->type == PLOTRAWTIME ||
     Plot[id]->type == PLOTTRTIME || Plot[id]->type == PLOTCOHETIME)
    {
     strcat (options, " COL");
     if (Plot[id]->autoScale != ON)
        {strcat (options, " ZMIN");
         sprintf(options, "%s %g", options, Plot[id]->zMin);
         strcat (options, " ZMAX");
         sprintf(options, "%s %g", options, Plot[id]->zMax);
        }
     else strcat (options, " AUTOSCALE");
     if (Plot[id]->refresh_percent > 0)
        {strcat (options, " REFRESH_PERCENT");
         sprintf(options, "%s %g", options, Plot[id]->refresh_percent);
        }
    }

 if (Plot[id]->type == PLOT1DTIME)
    {
     if (Plot[id]->nBin > 0)
        {strcat (options, " N1D");
         sprintf(options, "%s %ld", options, Plot[id]->nBin);
        }
     if (Plot[id]->autoEdge) strcat (options, " AUTO");
    }

 if (Plot[id]->type == PLOTRAWTIME)
    {
     strcat (options, " NFFT");
     sprintf(options, "%s %ld", options, Plot[id]->average_fft);
     if (Plot[id]->x2av == ON) strcat (options, " X2AV");
    }

 if (Plot[id]->type == PLOTAUDIO)
    {
     strcat (options, " FILENAME ");
     sprintf(options, "%s %s__%ld_to_%ld", options,
                     Plot[id]->vVarx, Dy->startTime, Dy->startTime+Plot[id]->duration);
     if (Plot[id]->sound16bits == ON) strcat (options, " SOUND16BITS");
     if (Plot[id]->soundonline == OFF) strcat (options, " SOUNDOFF");
    }


 strcat (options, "\042");


 /*--- Build options2 string ---*/

 strcpy(options2, "\042");
 if (Plot[id]->gridx2)  strcat (options2, " GRIDX");
 if (Plot[id]->gridy2)  strcat (options2, " GRIDY");
 if (Plot[id]->logx2)   strcat (options2, " LOGX");
 if (Plot[id]->logy2)   strcat (options2, " LOGY");
 if (Plot[id]->superposed == ON)  strcat (options2, " SAME");
 strcat (options2, "\042");

 return;
}
