
/**************************************************/
/*                                                */
/* File: DyDd.c                                   */
/* Copyright (C) 2003, D. Verkindt                */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include <fcntl.h> 
#include <stdarg.h>
#include <dirent.h>
#include <assert.h>
#include <unistd.h>
#include "dataDisplay.h"

#ifdef MEMWATCH
#    include "memwatch.h"
#endif


/*------------------------------------------- DyDdPauseRequest ---*/

void DyDdPauseRequest (char *DdServer)
{
#ifndef NOCM

 CmMessage message;
 char DsSender[256];

 DyDebugPrint(__FUNCTION__,0);

 message = CmMessageNew();
 CmMessagePutText (message,"PAUSE");
 CmMessageSetType (message,DATAREQUEST);

 sprintf(DsSender, "%s_%s",DdServer, Dy->CmName);
 CmMessageSend (message,DsSender);
 CmMessageDelete (message);

#endif

 return;
}


/*------------------------------------------- DyDdContinueRequest ---*/

void DyDdContinueRequest (char *DdServer)
{
#ifndef NOCM

 CmMessage message;
 char DsSender[256];

 DyDebugPrint(__FUNCTION__,0);

 message = CmMessageNew();
 CmMessagePutText (message,"CONTINUE");
 CmMessageSetType (message,DATAREQUEST);

 sprintf(DsSender, "%s_%s",DdServer, Dy->CmName);
 CmMessageSend (message,DsSender);
 CmMessageDelete (message);

#endif

 return;
}

/*------------------------------------------- DyDdFFLRequest ---*/

long DyDdFFLRequest (char *DdServer)
{
#ifndef NOCM

 CmMessage message;
 const char *answer;

 DyDebugPrint(__FUNCTION__,0);

 if (Dy->Ddpasswd == NULL)
    {answer = fl_show_input("You need to provide a password: ", "");
     if (answer) {Dy->Ddpasswd = (char *)malloc(64); strcpy(Dy->Ddpasswd,answer);}
     else        return(0);
    }

 if (DyCmConnect(Dy->CmName) == -1) return(-1);

 CmMessageInstallHandler(DyDdHandlerFFL,DDFFLDIR);

 message = CmMessageNew();
 CmMessagePutText (message,DDFFLDIR);
 CmMessagePutText (message,Dy->Ddpasswd);
 CmMessagePutText (message," ");
 CmMessagePutInt  (message,0);
 CmMessagePutInt  (message,0);
 CmMessagePutInt  (message,0);

 CmMessageSetType (message,DATAREQUEST);
 CmMessageSend    (message,DdServer);
 CmMessageDelete  (message);

 DyCmWait(DY_TIMEOUT1, DdServer);

 CmMessageUninstallHandler(DDFFLDIR);
 DyCmDisconnect(Dy->CmName);

 if (Dy->Order == STOP) return(-1);
 Dy->Order = STOP;

#endif

 return(0);
}


/*------------------------------------------- DyDdChlistRequest ---*/

long DyDdChlistRequest (char *DdServer, char *inputfile, int gpsStart)
{
#ifndef NOCM

 CmMessage message;
 const char *answer;
 char dummy[LSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);

 CmMessageInstallHandler(DyDdHandlerChlist,DDCHLIST);
 CmMessageInstallHandler(DyDdHandlerFilelist,DDFILELIST);

 message = CmMessageNew();

 if (((Dy->debugType)&(1<<DBG_CM)) && Dy->debug>1)
    {printf("DdChlistRequest> Sending Channels list request:  requester=%s  name=%s  inputfile=%s  gpsStart=%d \n",
             Dy->CmName, DDCHLIST, inputfile, gpsStart);
    }

 CmMessagePutText (message,DDCHLIST);
 CmMessagePutText (message,inputfile);
 if (Dy->Ddpasswd) CmMessagePutText (message, Dy->Ddpasswd);
 else              CmMessagePutText (message,"nopasswd");
 CmMessagePutInt  (message,gpsStart);
 CmMessagePutInt  (message,0);
 CmMessagePutInt  (message,0);

 CmMessageSetType (message,DATAREQUEST);
 CmMessageSend    (message,DdServer);
 CmMessageDelete  (message);

 sprintf(dummy,"Requested files list or channels list to %s", Dy->DdServer);
 DyInfoLine (dummy);

 Dy->filelist_flag = 0;

 DyCmWait(DY_TIMEOUT1, DdServer);

 CmMessageUninstallHandler(DDCHLIST);
 CmMessageUninstallHandler(DDFILELIST);

 if (Dy->Order == STOP) return(-1);
 Dy->Order = STOP;

 if (Dy->Ddchlist != NULL)
    {
     if (Dy->Frame) FrameFree(Dy->Frame);
     Dy->Frame = DyCharChListToFrame(Dy->Ddchlist);
     free(Dy->Ddchlist); Dy->Ddchlist = NULL;
     return(0);
    }
 else if (Dy->filelist_flag > 0)
    {return(-2);}
 else
    {return(-1);}

#endif

 return(0);
}


/*------------------------------------------- DyDdDataRequest ---*/

long DyDdDataRequest (char *DdServer, char *inputfile, char *tag, int gpsStart, int nSeconds)
{
#ifndef NOCM

 CmMessage message;
 char DsSender[256], *listofgps, *p;
 int itry, i;

 DyDebugPrint(__FUNCTION__,0);

 message = CmMessageNew();

 if (((Dy->debugType)&(1<<DBG_CM)) && Dy->debug>1)
    {printf("DyDdDataRequest> Sending Data request to %s:  name=%s  inputfile=%s  tag=%s  gpsStart=%d  nSeconds=%d\n",
             Dy->DdServer, Dy->CmName, inputfile, tag, gpsStart, nSeconds);
    }

 CmMessagePutText (message,Dy->CmName);
 CmMessagePutText (message,inputfile);
 CmMessagePutText (message,tag);
 CmMessagePutInt  (message,gpsStart);
 CmMessagePutInt  (message,nSeconds);
 CmMessagePutInt  (message,Dy->skipFrame);

 if (Dy->triggerUsed == ON && Dy->triggerType == TRIG_ASCII) 
    {CmMessagePutText (message,TRIGASCII);
     listofgps = (char *)malloc(10*Dy->nevent_ascii+2);
     p = listofgps;
     for (i=0; i<Dy->nevent_ascii; i++) p += sprintf(p," %d", (int)(Dy->AsciiGps[i]));
     CmMessagePutText (message,listofgps);
    }

 if (Dy->triggerUsed == ON && Dy->triggerType == TRIG_EVENTS) 
    {CmMessagePutText (message,TRIGEVENTS);
     CmMessagePutText (message,Dy->conditionName);
     CmMessagePutDouble (message,Dy->conditionTh);
    }

 CmMessageSetType (message,DATAREQUEST);
 CmMessageSend    (message,DdServer);

 sprintf(DsSender, "%s_%s",DdServer, Dy->CmName);
 itry = 0;
 while (!CmMessageSend(message,DsSender) && itry<5)
    {itry++;
     sleep(1);
    }

 CmMessageDelete (message);

#endif

 return(0);
}


/*------------------------------------------- DyDdSendWaitingTime ---*/

long DyDdSendWaitingTime (double waiting_time)
{
#ifndef NOCM

 CmMessage message;
 char DsSender[256], dummy[LSTRING_MAX];

 sprintf(dummy,"DyDdSendWaitingTime> waiting_time=%fs", waiting_time);
 DyDebugPrint(dummy,0);

 sprintf(DsSender, "%s_%s",Dy->DdServer, Dy->CmName);

 message = CmMessageNew();

 CmMessagePutFloat (message,waiting_time);
 CmMessageSetType (message,DDWAIT);
 CmMessageSend (message,DsSender);

 CmMessageDelete (message);

#endif

 return(0);
}



#ifndef NOCM

/*------------------------------------------- DyDdHandlerFFL ---*/

CmMessageStatus DyDdHandlerFFL (CmMessage message, char* sender, char* servername)
{
 char *ffldir;

 ffldir = CmMessageGetText(message);
 if (ffldir) strcpy(Dy->Ddffldir, ffldir);

 if (((Dy->debugType)&(1<<DBG_CM)) && Dy->debug>1) printf("DyDdHandlerFFL> received ffldir=%x\n", ffldir);

 return(CmMessageBreak);
}

#endif


#ifndef NOCM

/*------------------------------------------- DyDdHandlerChlist ---*/

CmMessageStatus DyDdHandlerChlist (CmMessage message, char* sender, char* servername)
{
 char *chlist;

 DyDebugPrint(__FUNCTION__,0);

 chlist = CmMessageGetText(message);
 if (chlist) Dy->Ddchlist = my_strcpy(Dy->Ddchlist, chlist);

 if (((Dy->debugType)&(1<<DBG_CM)) && Dy->debug>2 && chlist)
    {printf("DyDdHandlerChlist> received chlist=%x (%d bytes)\n", chlist, strlen(chlist));}

 return(CmMessageBreak);
}

#endif


#ifndef NOCM

/*------------------------------------------- DyDdHandlerFilelist ---*/

CmMessageStatus DyDdHandlerFilelist (CmMessage message, char* sender, char* servername)
{
 char *filelist;
 char *str;
 int nfile;

 DyDebugPrint(__FUNCTION__,0);

 filelist = CmMessageGetText(message);

 fl_clear_browser(getdd->liste);

 if (filelist) fl_add_browser_line(getdd->liste,"..");

 str = strtok(filelist," ");
 while (str != NULL)
    {fl_add_browser_line(getdd->liste, str);
     str = strtok(NULL," ");
    }

 nfile = fl_get_browser_maxline(getdd->liste);
 if (nfile > 0) DyXformSortBrowser(getdd->liste);
 if (nfile > 0) Dy->filelist_flag = 1;

 return(CmMessageBreak);
}

#endif


