
/**************************************************/
/*                                                */
/* File: DyFd.c                                   */
/* COPY OF SOME FD LIBRARY FUNCTIONS (v4r13)      */
/* Copyright (C) 2003, D. Verkindt                */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dataDisplay.h>

#ifndef NOCM

/*-------------------------------------------------- FdGetData --------*/

int DyFdGetData (CmMessage message, char **name, 
               int *run, int *fnum, 
               int *gtimeS, int *gtimeN, 
               int *trig, int *nBytes, char **buf)
{

 if (message == NULL) return(1);
 
 /*----- decode Cm message -----*/
   
 *name   = CmMessageGetText  (message);
 *run    = CmMessageGetInt   (message);
 *fnum   = CmMessageGetInt   (message);
 *gtimeS = CmMessageGetInt   (message);
 *gtimeN = CmMessageGetInt   (message);
 *trig   = CmMessageGetInt   (message);
 *nBytes = CmMessageGetInt   (message);
 *buf    = CmMessageGetArray (message, CmMessageChar, nBytes);

/***
 fnum2   = CmMessageGetInt   (message);
 if (fnum2 != *fnum) return(2);
***/

 return(0);
}               

/*--------------------------------------------- FdGetFrameWithSize --------*/

struct FrameH *DyFdGetFrameWithSize (CmMessage message, int compress, int *nBytes)
{
 struct FrameH *frame;
 char *buf, *name;
 int run, fnum, gtimeS, gtimeN, trig;

 if (message == NULL) return(NULL);
 
 /*----- decode Cm message -----*/
   
 DyFdGetData (message, &name, &run, &fnum, &gtimeS, &gtimeN, &trig, nBytes, &buf);

 frame = FrameReadFromBuf (buf, *nBytes, compress);

 return(frame);
}


/*---------------------------------------------- FdOutSendAdd -----------*/

int DyFdOutSendAdd (char* dest, char* serverName, char *tagAdc, char *tagSer,
                  int maxTry, int priority)
{
 int ok;
 CmMessage message;

 DyDebugPrint(__FUNCTION__,0);

 message= CmMessageNew();

 CmMessagePutText (message, serverName);
 CmMessagePutText (message, tagAdc);
 CmMessagePutText (message, tagSer);
 CmMessagePutInt (message, maxTry);
 CmMessagePutInt (message, priority);

 CmMessageSetType (message, CMTYPE_FDADDFRAME);
 ok = CmMessageSend (message, dest);

 CmMessageDelete (message);

 return(ok);
}


/*------------------------------------------------- FdOutSendRemove ----------*/

int DyFdOutSendRemove (char* dest, char* serverName)
{
 int ok;
 CmMessage message;

 DyDebugPrint(__FUNCTION__,0);

 message= CmMessageNew();

 CmMessagePutText (message, serverName);

 CmMessageSetType (message, CMTYPE_FDREMOVEFRAME);
 ok = CmMessageSend (message, dest);

 CmMessageDelete (message);

 return(ok);
}


/*-------------------------------------------- FdSendFrameChannelsRequest ---------*/

int DyFdSendFrameChannelsRequest (char *dest)
{
 CmMessage request;
 int ok;

 DyDebugPrint(__FUNCTION__,0);

 request = CmMessageNew();
 CmMessageSetType (request, CMTYPE_FDFRAMECHANNELS);

 ok = CmMessageSend (request,dest);

 CmMessageDelete(request);

 return(ok);
}

#endif

