
/**************************************************/
/*                                                */
/* File: DyFiles.c                                */
/* Copyright (C) 2003, D. Verkindt                */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include <fcntl.h> 
#include <stdarg.h>
#include <dirent.h>
#include <assert.h>
#include <unistd.h>
#include "fselector.h"
#include "dataDisplay.h"

#ifdef MEMWATCH
#    include "memwatch.h"
#endif


/*                        ***********                     */
/*                        *  FILES  *                     */
/*                        ***********                     */

/*--------------------------------------------------- DyWaveToFrame------*/
struct FrameH *DyWaveToFrame (char *wavefileName, double framedt)
{
 unsigned char valuec;
 short value;
 char *buf, dummy[LSTRING_MAX];
 FILE *file;
 int j, i, is, nData, vectnData;
 struct FrAdcData *adc;
 struct FrameH *frame;

 DyDebugPrint(__FUNCTION__,0);

 file = fopen(wavefileName,"r");
 if (file == NULL)
    {sprintf(dummy,"Cannot open wave audio file %s", wavefileName);
     DyInfoLine(dummy); 
     Dy->firstTime = 0;
     return(NULL);
    }

 buf = (char *)(&(wavehdr));
 fread(buf, 1, sizeof(wavehdr), file);
 DySoundReadHeader(buf);

 if (wavehdr.riff[0] != 'R')
    {sprintf(dummy,"%s if not a wav file", wavefileName);
     DyInfoLine(dummy); 
     return(NULL);
    }
 if (wavehdr.bits_per_samp != 8 && wavehdr.bits_per_samp != 16)
    {sprintf(dummy,"%s is not 8 or 16 bits wav file", wavefileName);
     DyInfoLine(dummy); 
     return(NULL);
    }
 if (wavehdr.format_tag != 1 && wavehdr.format_tag != 259)
    {sprintf(dummy,"%s is not PCM format wav file", wavefileName);
     DyInfoLine(dummy); 
     return(NULL);
    }

 vectnData = (int)(wavehdr.srate * framedt);
 nData     = wavehdr.dlength / (wavehdr.bytes_per_samp / wavehdr.num_chans);

 if (((Dy->debugType)&(1<<DBG_FILE)) && Dy->debug>1)
    {
     if (wavehdr.num_chans == 256 * buf[30] + buf[31])
        {printf("DyWaveToFrame> num_chans=%d  buf[30]=%ud  buf[31]=%ud  We are in big endian\n", wavehdr.num_chans, buf[30], buf[31]);}
     else
        {printf("DyWaveToFrame> num_chans=%d  buf[30]=%ud  buf[31]=%ud  We are in little endian\n", wavehdr.num_chans, buf[30], buf[31]);}
     printf("DyWaveToFrame> Wave file contains %ld bytes with %d bytes per sample (%d bits) and %d channels --> nData=%ld\n",
            wavehdr.dlength, wavehdr.bytes_per_samp, wavehdr.bits_per_samp, wavehdr.num_chans, nData);
     printf("DyWaveToFrame> Frame length = %g --> vectnData=%d\n", framedt, vectnData);
    }

 frame         = FrameHNew("frame_from_wave");
 frame->GTimeS = Dy->startTime;
 Dy->firstTime = Dy->startTime;
 frame->dt     = framedt;
 adc = FrAdcDataNew (frame, "fraudio", wavehdr.srate, vectnData, 16);

 i = 0;
 for (is=0; is<vectnData; is++)
    {
     if (wavehdr.bits_per_samp == 16) fread(&value, sizeof(short), 1, file);
     else                             fread(&valuec, sizeof(unsigned char), 1, file);
     if (wavehdr.num_chans == 2 && i%2 == 1) is--;
     if (wavehdr.bits_per_samp == 16) adc->data->dataS[is] = value;
     else                             adc->data->dataS[is] = (short)(valuec*256 - 32768);
     i++;
     if (i > nData) break;
    }

 if (is < vectnData) {for (j=is; j<vectnData; j++) adc->data->dataS[j]=0;}

 fclose(file);

 return(frame);
}


/*------------------------------------------ DyReadFirstFrame ---*/

int DyReadFirstFrame (char *chosen_file, long arg)
{
 int csmN, endTime, oldstart;
 long nfile,result;
 struct FrFile *iFile;
 char dummy[LSTRING_MAX];
#ifndef NOCM
 CmConnectCondition condition;
#endif
 long iwait, noChList, nadc;
 char *chList, *firstRead;
 int nBytes;
 long shm_iread;
 struct ShmInfo *shm;
 struct FrAdcData *adc;

 DyDebugPrint(__FUNCTION__,0);

 DyDebugTimeNeeded(__FUNCTION__,"start",1);

 Dy->InputType = INOFF;

 if (arg == INFILE  ||  arg == INONLINE || arg == INWAVE)
    {
     nfile = fl_get_browser_maxline(inputfile->file_list);
     if (nfile <= 0) {DyInfoLine ("No file in the list"); return(-1);}
    }

 /*--- data from wave audio file ---*/

 if (arg == INWAVE)
    {
     if (((Dy->debugType)&(1<<DBG_FILE)) && Dy->debug>1) printf("DyReadFirstFrame> Input=wave file\n");
     Dy->startTime = time(NULL) - FRGPSOFF + FRGPSDELTA;
     fl_set_input(mainpanel->startUTCdate, FrStrUTC(Dy->startTime,0));
     sprintf(dummy,"%d", Dy->startTime);
     fl_set_input(mainpanel->startTime, dummy);

     /*--- Get firstFrame and show its time stamp ---*/
     firstRead = (char *)fl_get_browser_line(inputfile->file_list, 1);
     sprintf(dummy,"Using one file: %s", firstRead);
     DyCurrentInput (dummy);
     strcpy(Dy->fileName,firstRead);

     /*--- get channels list and rebuild frame from channels list ---*/
     Dy->Frame = DyWaveToFrame(firstRead,1.);
     Dy->InputType = arg;
     DyXformFillMainBrowser(Dy->Frame);   /* Get channel audio */
    }

 /*--- frame from one or several files ---*/

 if (arg == INFILE)
    {
     if (((Dy->debugType)&(1<<DBG_FILE)) && Dy->debug>1) printf("DyReadFirstFrame> Input=frames files\n");

     /*--- Get firstFrame and show its time stamp ---*/

     firstRead = (char *)fl_get_browser_line(inputfile->file_list, 1);
     iFile  = FrFileINew(firstRead);
     if (iFile == NULL)
        {sprintf(dummy,"Could not open first file %s", firstRead);
         DyInfoLine(dummy);
         Dy->firstTime = 0;
         endTime       = 0;
        }
     else
        {Dy->firstTime = (int) FrFileITStart(iFile);
         endTime       = (int) FrFileITEnd(iFile);
         FrFileIEnd(iFile);
        }

    if (Dy->startTime > endTime || Dy->startTime < Dy->firstTime)
        {
         oldstart = Dy->startTime;
         if (Dy->startTime < Dy->firstTime) Dy->startTime = Dy->firstTime;
         if (Dy->startTime > endTime)       Dy->startTime = endTime - 300;
         if (oldstart > 0)
            {printf("DyReadFirstFrame> Start GPS not within %s range [%d - %d].\n", firstRead, Dy->firstTime, endTime);
             printf("DyReadFirstFrame>I update it from %d to %d (%s)\n", oldstart, Dy->startTime, FrStrGTime(Dy->startTime));
            }
         sprintf(dummy,"%ld",Dy->startTime);
         fl_set_input(mainpanel->startTime, dummy);
         if (FRGPSDELTA <= 13 && Dy->startTime>=820108813)
            {fl_set_input(mainpanel->startUTCdate, FrStrUTC(Dy->startTime-1,0));}
         else
            {fl_set_input(mainpanel->startUTCdate, FrStrUTC(Dy->startTime,0));}
        }

     /*--- get the chosen file of the list ---*/
     if (nfile == 1) {sprintf(dummy,"Using one file: %s", chosen_file); DyCurrentInput (dummy);}
     else            {sprintf(dummy,"Using %ld files of %s", nfile, Dy->Dir); DyCurrentInput (dummy);}
     strcpy(Dy->fileName,chosen_file);

     /*--- get channels list and rebuild frame from channels list ---*/
     iFile = FrFileINew(chosen_file);
     if (iFile == NULL) {sprintf(dummy,"Could not open chosen file %s", chosen_file); DyInfoLine(dummy); return(-1);}

     if (Dy->notoc > 0)
        {Dy->Frame = FrameRead(iFile);}
     else
        {
         if (((Dy->debugType)&(1<<DBG_FILE)) && Dy->debug>1) printf("DyReadFirstFrame> Getting ChannelList for startTime=%d\n", Dy->startTime);
         chList = FrFileIGetChannelList (iFile, Dy->startTime);
         if (chList == NULL) {FrFileIEnd(iFile); return(-1);}
         Dy->Frame = DyCharChListToFrame (chList);
         free(chList);
        }
     FrFileIEnd(iFile);

     if (Dy->Frame == NULL) {sprintf(dummy,"No frame in file %s", chosen_file); DyInfoLine(dummy); return(-1);}
     else                   {sprintf(dummy,"Frame read successfully from file %s", chosen_file); DyInfoLine(dummy);}

     Dy->InputType = arg;
     if (mainbrowser->mainbrowser && fl_form_is_visible(mainbrowser->mainbrowser)) DyXformClosePanel(mainbrowser->close,MAINBROWSER);
     DyXformFillMainBrowser(Dy->Frame);   /* Get channels of the frame */
    }


 /*--- frame from files online ---*/

 if (arg == INONLINE)
    {
     if (((Dy->debugType)&(1<<DBG_FILE)) && Dy->debug>1) printf("DyReadFirstFrame> Input=online written files\n");
     Dy->firstTime = 0;

     /*--- get channels list and rebuild frame from channels list ---*/
     iFile     = FrFileINew(chosen_file);
     chList    = FrFileIGetChannelList (iFile, 0);
     Dy->Frame = DyCharChListToFrame (chList);
     free(chList);

     if (Dy->Frame == NULL) {sprintf(dummy,"No frame in file %s", chosen_file); DyInfoLine (dummy); return(-1);}
     else                   {sprintf(dummy,"Frame read successfully from file %s", chosen_file); DyInfoLine(dummy);}
     strcpy(Dy->fileName,chosen_file);

     sprintf(dummy,"Reading the files written online in %s", Dy->Dir);
     DyCurrentInput (dummy);
     DyInfoLine (dummy);

     Dy->InputType = arg;
     if (mainbrowser->mainbrowser && fl_form_is_visible(mainbrowser->mainbrowser)) DyXformClosePanel(mainbrowser->close,MAINBROWSER);
     DyXformFillMainBrowser(Dy->Frame);    /* Get channels of the frame */
     FrFileIEnd(iFile);
    }


  /*--- frame from a Cm server ---*/

#ifndef NOCM

 if (arg == INCM)
    {
     noChList = 0;

     if (strcmp(Dy->CmServer,DYEMPTY) != 0)
        {
         DyFdSendFrameChannelsRequest (Dy->CmServer);
         sprintf(dummy, "Requesting channels list to %s", Dy->CmServer);
         DyInfoLine (dummy);
         Dy->Order = WAIT;
         condition = CmMessageWaitWithTimeout((double)DY_TIMEOUT3);

         if (condition == CmConnectTimeoutDetection)
            {sprintf(dummy,"Could not get channels list from %s. Request now full frame. Please, wait...");
             DyInfoLine(dummy);
             DyFdOutSendAdd (Dy->CmServer, Dy->CmName, "*", "*", 2, 0);
             if (UFrCurFrameH) {FrameFree(UFrCurFrameH); UFrCurFrameH=NULL;}
             DyCmWait(DY_TIMEOUT1, Dy->CmServer);
             noChList = 1;
            }

         else if (condition != CmConnectBreakDetection)
            {iwait = 1;
             Dy->Order = WAIT;
             while (Dy->Order != STOP) 
                {condition = CmMessageWaitWithTimeout(1.);
                 if (condition == CmConnectBreakDetection) break;
                 if (iwait%DY_TIMEOUT2 == 0)
                    {sprintf(dummy,"No channels list received from %s since %ld seconds (timeout=%d). Requesting again...", Dy->CmServer, iwait, DY_TIMEOUT1);
                     DyInfoLine(dummy);
                     DyFdSendFrameChannelsRequest (Dy->CmServer);
                    }
                 iwait++;
                 if (iwait > DY_TIMEOUT1) Dy->Order = STOP;
                 fl_check_forms();
                }
            }
        }

     else
        {Dy->Order = WAIT;
         DyInfoLine ("Waiting for frames from any frames provider...");
         DyCmWait(0, "any frame provider");
        }

     if (Dy->Order == STOP) return(-1);
     Dy->Order = STOP;

     /*--- Take the frame received, copy it into Dy->Frame ---*/

     if (Dy->Frame) FrameFree(Dy->Frame);
     Dy->Frame = FrameCopy(UFrCurFrameH);

     /*--- if frame received contains only channels list and Sms, reconstruct it ---*/

     if (noChList == 0) Dy->Frame = DyChListToFrame (Dy->Frame);

     if (Dy->startTime <= 0  && Dy->Frame)
        {
         Dy->startTime = Dy->Frame->GTimeS;
         sprintf(dummy,"%ld",Dy->startTime);
         fl_set_input(mainpanel->startTime, dummy);
         if (FRGPSDELTA <= 13 && Dy->startTime>=820108813) fl_set_input(mainpanel->startUTCdate, FrStrUTC(Dy->startTime-1,0));
         else                                              fl_set_input(mainpanel->startUTCdate, FrStrUTC(Dy->startTime,0));
        }

     sprintf(dummy,"Connected online to %s", Dy->CmServer);
     DyCurrentInput (dummy);
     DyInfoLine(dummy);

     Dy->InputType = arg;
     if (mainbrowser->mainbrowser && fl_form_is_visible(mainbrowser->mainbrowser)) DyXformClosePanel(mainbrowser->close,MAINBROWSER);
     DyXformFillMainBrowser(Dy->Frame);   /* Get channels of the frame */

     if (((Dy->debugType)&(1<<DBG_CM)) && Dy->debug>0 && Dy->Frame)
        {nadc = 0; if (Dy->Frame->rawData) {for (adc=Dy->Frame->rawData->firstAdc; adc!=NULL; adc=adc->next) nadc++;}
         printf("DyReadFirstFrame> one frame (fr=%d gps=%d nadc=%d) received from %s\n",
                 Dy->Frame->frame, Dy->Frame->GTimeS, nadc, Dy->CmServer);
        }
    }
#endif


  /*--- frame from the Data Distribution server ---*/

#ifndef NOCM

 if (arg == INDD)
    {
     sprintf(dummy, "Requesting channels list to %s...", Dy->DdServer);
     DyInfoLine (dummy);

     result = DyDdChlistRequest (Dy->DdServer, Dy->Ddinputfile, Dy->startTime);

     if (result >= 0)
        {
         if (Dy->startTime > time(NULL) - FRGPSOFF + FRGPSDELTA - 2400)
            {Dy->startTime = Dy->startTime - 2400;
             sprintf(dummy,"%ld",Dy->startTime);
             fl_set_input(mainpanel->startTime, dummy);
             if (FRGPSDELTA <= 13 && Dy->startTime>=820108813) fl_set_input(mainpanel->startUTCdate, FrStrUTC(Dy->startTime-1,0));
             else                                              fl_set_input(mainpanel->startUTCdate, FrStrUTC(Dy->startTime,0));
            }

         if (Dy->startTime <= 0  && Dy->Frame)
            {
             Dy->startTime = Dy->Frame->GTimeS;
             sprintf(dummy,"%ld",Dy->startTime);
             fl_set_input(mainpanel->startTime, dummy);
             if (FRGPSDELTA <= 13 && Dy->startTime>=820108813)
               {fl_set_input(mainpanel->startUTCdate, FrStrUTC(Dy->startTime-1,0));}
             else
               {fl_set_input(mainpanel->startUTCdate, FrStrUTC(Dy->startTime,0));}
            }
         sprintf(dummy, "Connected to %s", Dy->DdServer);
         DyCurrentInput (dummy);
         DyInfoLine (dummy);
         Dy->InputType = arg;
         if (mainbrowser->mainbrowser && fl_form_is_visible(mainbrowser->mainbrowser)) DyXformClosePanel(mainbrowser->close,MAINBROWSER);
         DyXformFillMainBrowser(Dy->Frame);   /* Get channels of the frame */

         if (((Dy->debugType)&(1<<DBG_CM)) && Dy->debug>0 && Dy->Frame)
            {nadc = 0; if (Dy->Frame->rawData) {for (adc=Dy->Frame->rawData->firstAdc; adc!=NULL; adc=adc->next) nadc++;}
             printf("DyReadFirstFrame> one frame (fr=%d gps=%d nadc=%d) received from %s\n",
                     Dy->Frame->frame, Dy->Frame->GTimeS, nadc, Dy->CmServer);
            }
        }

     else
        {
         if (result == -1) sprintf(dummy, "No list received from %s", Dy->DdServer);
         if (result == -2) sprintf(dummy, "Files list received from %s", Dy->DdServer);
         DyInfoLine (dummy);
         return(result);
        }
    }
#endif


#ifdef USE_VEGA

 if (arg == INVEGA)
    {
     DyInfoLine ("Waiting for frame from Frame Channel");
     if (Dy->Frame) FrameFree(Dy->Frame);
     Dy->Frame = DyExternSource->externReadFirstFrame();

     if (Dy->Frame != 0)
        {
         DyInfoLine ("Frame received from Frame Channel");
         DyCurrentInput ("Using the current frame channel");
         Dy->InputType = arg;
        }
     else
        {
         DyInfoLine ("No frame received from frame channel");
         return(-1);
        }

     if (mainbrowser->mainbrowser && fl_form_is_visible(mainbrowser->mainbrowser)) DyXformClosePanel(mainbrowser->close,MAINBROWSER);
     DyXformFillMainBrowser(Dy->Frame);   /* Get channels of the frame */
    }
#endif

 if (arg == INSHM)
    {
     Dy->Order = WAIT;

     iwait = 0;
     while ((shm = FdShmOpen(Dy->shmName, Dy->CmName)) == NULL  &&  Dy->Order != STOP)
        {fl_check_forms();
         if (iwait%DY_TIMEOUT2 == 0) {sprintf(dummy, "Could not open shared memory %s since %d sec", Dy->shmName, iwait); DyInfoLine(dummy);}
         sleep(1);
         iwait++;
        }
     if (Dy->Order == STOP) return(-1);
     else                  {sprintf(dummy, "Opened shared memory %s", Dy->shmName); DyInfoLine (dummy); sleep(1); FdShmCsmIni(shm, &shm_iread);}
     Dy->Order = WAIT;

     while ((Dy->Frame = FdShmRead(shm, &(shm_iread), -1, &nBytes)) == NULL  &&  Dy->Order != STOP)
        {fl_check_forms();
         sprintf(dummy, "Frame=NULL. Reading frame again from shared memory %s (iread=%d iwrite=%d)", Dy->shmName, shm_iread, shm->iwrite);
         DyInfoLine (dummy);
         sleep(1);
        }
     if (Dy->Order == STOP) {FdShmClose(shm, Dy->CmName); return(-1);}
     FdShmClose(shm, Dy->CmName);

     if (((Dy->debugType)&(1<<DBG_CM)) && Dy->debug>0)
        {printf("DyReadFirstFrame> one frame (fr=%d gps=%d) read in shared memory %s\n", Dy->Frame->frame, Dy->Frame->GTimeS, Dy->shmName);}

     sprintf(dummy,"Connected to shared memory %s", Dy->shmName);
     DyCurrentInput (dummy);
     DyInfoLine (dummy);

     Dy->InputType = arg;
     if (mainbrowser->mainbrowser && fl_form_is_visible(mainbrowser->mainbrowser)) DyXformClosePanel(mainbrowser->close,MAINBROWSER);
     DyXformFillMainBrowser(Dy->Frame);   /* Get channels of the frame */
    }

 /*--- Initialize Run and Frame counters ---*/
 if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>0)
        {printf("DyReadFirstFrame> Updating counters: frame=%d  run=%d  GPS=%d  ULeapS=%d \n",
                 Dy->Frame->frame, Dy->Frame->run, Dy->firstTime, Dy->Frame->ULeapS);
        } 
 DyXformUpdateCounters (Dy->Frame->frame, Dy->Frame->run, Dy->firstTime, Dy->Frame->ULeapS);

/*--- Show channels list and plots browser ---*/
 if (mainbrowser->mainbrowser && fl_form_is_visible(mainbrowser->mainbrowser) == 0 && Dy->AutoConfFlag == OFF) DyXformRaisePanel(NULL,MAINBROWSER);

 DyDebugTimeNeeded(__FUNCTION__,"start",1);

 return(0);
}


/*------------------------------------------- DyGetFramedt ---*/

double DyGetFramedt (struct FrameH *frame)
{
 double dt;

 if (Dy->Frame == NULL)
    {printf("DyGetFramedt> frame is NULL. I force dt=1.\n\n");
     return(1.);
    }
  
 if (Dy->Frame->dt <= 0.)
    {
     if (Dy->Frame->rawData == NULL)
        {printf("DyGetFramedt> frame %d (gps=%d dt=%g). I force to dt=1.\n",Dy->Frame->frame, Dy->Frame->GTimeS, Dy->Frame->dt);
         return(1.);
        }
     if (Dy->Frame->rawData->firstAdc == NULL)
        {printf("DyGetFramedt> frame %d (gps=%d dt=%g). I force to dt=1.\n", Dy->Frame->frame, Dy->Frame->GTimeS, Dy->Frame->dt);
         return(1.);
        }

     dt = Dy->Frame->rawData->firstAdc->data->nData * Dy->Frame->rawData->firstAdc->data->dx[0];

     printf("DyGetFramedt> frame %d (gps=%d dt=%g). I force to dt=%g (adc=%s nData=%d dx=%g)\n\n",
             Dy->Frame->frame, Dy->Frame->GTimeS, Dy->Frame->dt, dt, Dy->Frame->rawData->firstAdc->name,
             Dy->Frame->rawData->firstAdc->data->nData, Dy->Frame->rawData->firstAdc->data->dx[0]);
    }
 else
    {dt = Dy->Frame->dt;}
    
 return(dt);
}


/*------------------------------------------- DyAddDir ---*/
/*                                     arg=0  means Add Files of Dir offline
                                       arg=1  means Add Files of Dir online
                                       arg=11 means Add Files of Dir using the config file */
void DyAddDir (char *directory, char *file, long arg)
{
 char **filename, dummy[LSTRING_MAX];
 long i, nfile;
 DIR  *dirp;
 struct dirent *entry;

 DyDebugPrint(__FUNCTION__,0);

 if (arg == 0)
    {
     sprintf(dummy, "Reading first frame in each file of %s, be patient...", directory);
     DyInfoLine (dummy);
     filename = (char **) calloc(NFILE_MAX, sizeof(char *));
     nfile = DySortFiles (directory, filename);
     for (i=0; i<nfile; i++)  DyXformAddToFileList (filename[i]);
     for (i=0; i<nfile; i++)  free(filename[i]);
     free(filename);
     DyReadFirstFrame (file, INFILE);
    }

 else if (arg == 1)
    {DyXformAddToFileList (file);
     DyReadFirstFrame (file, INONLINE);
    }

 else
    {dirp  = opendir(directory);
     entry = readdir(dirp);
     while(strncmp(entry->d_name,".",1) == 0)  entry = readdir(dirp);
     entry = readdir(dirp);
     entry = readdir(dirp);
     entry = readdir(dirp);
     sprintf(dummy,"%s/%s", directory, entry->d_name);
     closedir(dirp);
     DyXformAddToFileList (dummy);
     DyReadFirstFrame (dummy, INONLINE);
    }

 return;
}


/*------------------------------------------- DyXformAddFile ---*/

void DyXformAddFile (FL_OBJECT *obj, long arg)
{
 char *result, *str, firstFile[LSTRING_MAX], pattern_str[LSTRING_MAX];
 int size, result_length, nFile;

 DyDebugPrint(__FUNCTION__,0);

 size = LSTRING_MAX * NFILE_MAX;
 result = (char *)malloc(size);
 while (result == NULL)
    {size = size / 2;
     result = (char *)malloc(size);
    } 

 if (arg == 1)
    {strcpy(pattern_str, "*.ffl");
     if (is_a_dir(FFLDIR)) result_length = ClXFSelector("choose ffl to read", FFLDIR, pattern_str, "", result, size);
     else                  result_length = ClXFSelector("choose ffl to read", Dy->Dir, pattern_str, "", result, size);
    }
 else if (arg == -1)
    {strcpy(pattern_str, "*.wav");
     result_length = ClXFSelector("choose wave file to read", Dy->Dir, pattern_str, "", result, size);
    }
 else
    {strcpy(pattern_str, "*.gwf");
     result_length = ClXFSelector("choose frames file to add", Dy->Dir, pattern_str, "", result, size);
    }

 if (result_length == 0 || !result) return;

 fl_clear_browser(inputfile->file_list); nFile = 0;  /** remove all files previously selected **/

 nFile = 0;
 str = strtok(result," ");
 while (str != NULL)
    {if (is_a_file(str) == 0) {str = strtok(NULL," "); continue;}
     if (strstr(str,".ffl") != NULL) {fl_clear_browser(inputfile->file_list); nFile = 0;} /** remove all previous files if ffl selected **/
     nFile++;
     if (nFile == 1)  {Dy->Dir = DyGetDir(Dy->Dir, str); strcpy(firstFile,str);}
     if (((Dy->debugType)&(1<<DBG_FILE)) && Dy->debug>1) printf("DyXformAddFile> file%d=%s\n", nFile, str);
     DyXformAddToFileList(str);
     str = strtok(NULL," ");
    }
 free(result);

 DyXformSortFileList();
 if (is_a_file(firstFile))
    {if (arg == -1) DyReadFirstFrame (firstFile, INWAVE);
     else           DyReadFirstFrame (firstFile, INFILE);
    }

 return;
}


/*------------------------------------------- DyXformRemoveFile ---*/

void DyXformRemoveFile (FL_OBJECT *obj, long arg)
{
 long i, i0, nfile;
 const char *str;
 struct FrFile *iFile;

 DyDebugPrint(__FUNCTION__,0);

 nfile = fl_get_browser_maxline(inputfile->file_list);
 if (nfile <= 0) return;

 i0 = fl_get_browser(inputfile->file_list);
 i0--;
 if (i0 < 0) return;

 if (nfile > 1)
    {
     for (i=i0; i<nfile-1; i++)
        {str = fl_get_browser_line(inputfile->file_list,i+2);
         fl_replace_browser_line(inputfile->file_list,i+1,str);
        }
    }

 fl_delete_browser_line(inputfile->file_list,nfile);

 if (i0 == 0  &&  nfile > 1)
    {
     str = fl_get_browser_line(inputfile->file_list,1);
     iFile = FrFileINew((char *)str);
     if (iFile != NULL)
        {Dy->firstTime = FrFileITStart(iFile);
         FrFileIEnd(iFile);
        }
    }

 return;
}


/*------------------------------------------- DyBaseName ---*/

char *DyBaseName (char *allname)
{
 long i,i0;
 const char *filename;

 if (!allname) return(NULL);

 i0 = -1;
 for (i=strlen(allname)-1; i>= 0; i--) {if (allname[i] == '/') {i0 = i; break;}}
 filename = &allname[i0+1];

 return((char *)filename);
}


/*------------------------------------------- DyGetDir ---*/

char *DyGetDir (char *dirname, char *allname)
{
 long i,i0;

 DyDebugPrint(__FUNCTION__,0);

 if (!allname) return(NULL);

 if (dirname) free(dirname);
 dirname = (char *)calloc(strlen(allname)+1,sizeof(char));
 
 i0 = 0;
 for (i=0; i<strlen(allname); i++)
     {if (allname[i] == '/') i0 = i;}
 
 if (i0 > 0)
    {dirname = (char *)calloc(i0+1,sizeof(char));
     strncpy(dirname, allname, i0);
     dirname[i0] = '\0';
    }
 else
    {dirname = (char *)calloc(3,sizeof(char));
     strncpy(dirname,"./",2);
     dirname[2] = '\0';
    }
    
 return(dirname);
}


/*----------------------------------------------- DySortFiles ---*/
/*        Put in fileName the frames time sorted list of the files of dirName */

long DySortFiles (char *dirName, char **fileName)
{
 long nfile, i, j;
 struct FrFile *iFile;
 char inputName[LSTRING_MAX],infileName[LSTRING_MAX],dummy[LSTRING_MAX];
 double gtimeSN_frame1[NFILE_MAX], gts, gtimeSN;
 DIR  *dirp;
 struct dirent *entry;

 sprintf(dummy, "Time ordering the files of %s, be patient...", dirName);
 DyInfoLine (dummy);

 /*--- We get the first frame of all the files of the list ---*/

 nfile = 0;
 dirp  = opendir(dirName);

 while ((entry = readdir(dirp)) != NULL)
   {
    if (strncmp(entry->d_name,".",1) == 0) continue;

    strcpy(infileName,entry->d_name);
    if (is_a_dir(dirName))  sprintf(inputName,"%s/%s",dirName,infileName);
    else                    sprintf(inputName,"%s",infileName);

    iFile = FrFileINew (inputName);
    if (iFile == NULL)
       {printf("SetSortedFileList> Error during opening of file %s\n",inputName);
        continue;
       }

    gtimeSN = FrFileITStart(iFile);

    if (gtimeSN <= 0)
       {printf("SetSortedFileList> No GPS Time found in %s. I skip it.\n",inputName);
        continue;
       }

    /*--- Keep the time of the first frame of the file for the sorting (see below) ---*/

    gtimeSN_frame1[nfile] = gtimeSN;

    fileName[nfile] = (char *) calloc(LSTRING_MAX,sizeof(char));
    strcpy(fileName[nfile], inputName);

    printf("SetSortedFileList> Read %s : gtimeS=%f\n",fileName[nfile],gtimeSN_frame1[nfile]);

    FrFileIEnd(iFile);
    nfile++;
   }

 closedir(dirp);


 if (nfile < 1)
    {sprintf(dummy, "No file readable in directory %s.", dirName);
     DyInfoLine(dummy);
     return(0);
    }

 /*--- Now sorting the files by time ---*/

 for (i=0; i<nfile-1; i++)
    {for (j=i+1; j<nfile; j++)
        {if (gtimeSN_frame1[i] > gtimeSN_frame1[j])
            {strcpy(dummy,fileName[i]);
             strcpy(fileName[i],fileName[j]);
             strcpy(fileName[j],dummy);
             gts               = gtimeSN_frame1[i];
             gtimeSN_frame1[i] = gtimeSN_frame1[j];
             gtimeSN_frame1[j] = gts;
            }
        }
    }

 return(nfile);
}


/*------------------------------------------- is_a_file ---*/

long is_a_file (char *str)
{
 DIR *fd;
 FILE *fp;

 fd = opendir(str);
 if (fd)  {closedir(fd); return 0;}

 fp = fopen(str,"r");
 if (fp)  {fclose(fp); return 1;}

 return 0;
}


/*------------------------------------------- is_a_dir ---*/

long is_a_dir (char *str)
{
 DIR *fd;

 fd = opendir(str);
 if (fd)  {closedir(fd); return 1;}

 return 0;
}



/*------------------------------------------- DyAddShm ---*/

void DyAddShm(char *shmname)
{
 DyDebugPrint(__FUNCTION__,0);

 if (!shmname) return;

 DyXformClosePanel(getshm->close,GETSHM);
 strcpy(Dy->shmName,shmname);
 DyReadFirstFrame (NULL, INSHM);

 return;
}


/*----------------------------------------- DyGetShmList ---*/

int DyGetShmList()
{
 int nshm;
 FILE *fp;
 char buff[256], *shmid, *str2;
 struct ShmInfo *shm;

 nshm = 0;

 fp = popen("ipcs -m","r");
 if (fp)
    {fgets(buff,256,fp);
     while (!feof(fp))
       {if (strncmp(buff,"0x",2) == 0)
           {shmid = strstr(buff," ") + 1; str2 = strstr(shmid," ");
            *str2='\0';
            shm = FdShmOpen(shmid,Dy->CmName);
            if (shm) {strcpy(Dy->shmlist[nshm],shm->producerName); nshm++; FdShmClose(shm,Dy->CmName);}
           }
        fgets(buff,256,fp);
       }
     pclose(fp);
    }
 Dy->nshm = nshm;

 return(nshm);
}


/*----------------------------------------- DyUpdateShmList ---*/

void DyUpdateShmList()
{
 int i;

 Dy->nshm = DyGetShmList();

 fl_clear_browser(getshm->liste);
 for (i=0; i<Dy->nshm; i++) fl_add_browser_line(getshm->liste, Dy->shmlist[i]);
 if (Dy->nshm > 1)  DyXformSortBrowser(getshm->liste);

 return;
}


