#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include <sys/time.h>
#include <fcntl.h> 
#include <stdarg.h>
#include <dirent.h>
#include <assert.h>
#include <unistd.h>
#include "dataDisplay.h"

#ifdef MEMWATCH
#    include "memwatch.h"
#endif


/*-------------------------------------------------- DySoundFillHeader ------*/

void DySoundFillHeader(struct soundhdr *hdr, long srate, long nBytes, short bits_per_sample)
{
 struct soundhdr myhdr = {"RIF",88244,"WAV","fmt",16,1,1,44100,88200,2,16,"dat",88200};
 myhdr.riff[3] = 'F';
 myhdr.wave[3] = 'E';
 myhdr.fmt[3] = ' ';
 myhdr.data[3] = 'a';
 myhdr.format_tag = 1;
 myhdr.num_chans  = 1;
 myhdr.bits_per_samp = bits_per_sample;
 myhdr.bytes_per_samp = myhdr.bits_per_samp / 8;
 myhdr.srate = srate;
 myhdr.bytes_per_sec = srate * myhdr.bytes_per_samp;
 myhdr.dlength = nBytes;
 myhdr.flength = myhdr.dlength + sizeof(myhdr);
 *hdr = myhdr;

 return;
}


/*--------------------------------------------- DySoundReadHeader ---*/

void DySoundReadHeader (char *buffer)
{
 char format_tag_name[16];
 struct soundhdr myhdr;

 memcpy((char *)(&myhdr),buffer,sizeof(myhdr));

 strcpy(format_tag_name,"unknown");
 if (myhdr.format_tag == 1) strcpy(format_tag_name,"PCM");
 if (myhdr.format_tag == 257) strcpy(format_tag_name,"Mu-Law");
 if (myhdr.format_tag == 258) strcpy(format_tag_name,"A-Law");
 if (myhdr.format_tag == 259) strcpy(format_tag_name,"ADPCM");

 printf("DySoundReadHeader> Header size: %d bytes \n", sizeof(myhdr));
 printf("DySoundReadHeader> riff: %c  %c  %c  %c \n", myhdr.riff[0], myhdr.riff[1], myhdr.riff[2], myhdr.riff[3]);
 printf("DySoundReadHeader>       %x %x %x %x \n", buffer[0], buffer[1], buffer[2], buffer[3]);
 printf("DySoundReadHeader> File length: %ld bytes \n", myhdr.flength);
 printf("DySoundReadHeader> wave: %c%c%c%c \n", myhdr.wave[0], myhdr.wave[1], myhdr.wave[2], myhdr.wave[3]);
 printf("DySoundReadHeader> fmt: %c%c%c%c \n", myhdr.fmt[0], myhdr.fmt[1], myhdr.fmt[2], myhdr.fmt[3]);
 printf("DySoundReadHeader> Block size: %ld bytes \n", myhdr.block_size);
 printf("DySoundReadHeader> Format: %d (%s)  \n", myhdr.format_tag, format_tag_name);
 printf("DySoundReadHeader> Number of channels: %d  \n", myhdr.num_chans);
 printf("DySoundReadHeader> Sample Rate: %ld  \n", myhdr.srate);
 printf("DySoundReadHeader> Number of bytes per sec: %ld  \n", myhdr.bytes_per_sec);
 printf("DySoundReadHeader> Number of bytes per sample: %d  \n", myhdr.bytes_per_samp);
 printf("DySoundReadHeader> Number of bits per sample: %d  \n", myhdr.bits_per_samp);
 printf("DySoundReadHeader> data: %c%c%c%c \n", myhdr.data[0], myhdr.data[1], myhdr.data[2], myhdr.data[3]);
 printf("DySoundReadHeader> Data length: %ld  \n", myhdr.dlength);

/*
     header attendu (big-endian):
     R I  F F  file_length W A  V E  f m  t space
     5249 4646 2408 0000   5741 5645 666d 7420
     block_size format nch  sampRate    bytes_per_sec
     1000 0000  0100   0200 2256 0000   8858 0100
     byte_per_sample  bits_per_sample d a  t a  data_length
     0400             1000            6461 7461 0008 0000

     header obtenu sur lappc-vi22 avec un canal a 20kHz:
     4952 4646 58b4 0001 4157 4556 6d66 2074
     0010 0000 0001 0001 49d4 0000 93a8 0000
     0002 0010 6164 6174 5888 0001 a5a7 9a9f
*/

 return;
}


/** Not used for now, will be used when SEPSMS will be used **/
/*------------------------------------------------------------ DySerDataGet ---*/
int DySerDataGet(FrameH *frame, char *name, char **sername, char **param, double *result)
{
 int i, len, ok;
 double value;
 char *serParam;

 if (frame == NULL) return(-3);

 ok = 0;
 len = strlen(name);
 for (i=1; i<len; i++)
    {if(name[i] != '_') continue;
     name[i] = '\0';
     serParam = name+i+1;
     if (result) {if (FrSerDataGet(frame, name, serParam, result) == 0) {ok = 1; break;}}
     else        {if (FrSerDataFind(frame, name, NULL) != NULL) {ok = 1; break;}}
     name[i] = '_';
    }
 if (i != len) {*sername = my_strcpy(*sername,name); *param = my_strcpy(*param,serParam); name[i] = '_';}
 if (ok) printf("DySerDataGet> Found %s = %s..%s\n", name, *sername, *param);

 return(ok);
}


/*------------------------------------------------ DyGrafIni ---*/
long DyGrafIni (long reset)
{
 static long plot=0;

 DyDebugPrint(__FUNCTION__,0);
 
 if (reset < 0)    plot = 0;
 if (reset == -10) RA_MainCanvasDelete();
 if (reset == 0)   plot++;
 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0) printf("DyGrafIni> reset=%ld  plotNumber=%d\n", reset, plot);

 return(plot);
}


/*------------------------------------------- DyTimeFormatToGps ---*/

double DyTimeFormatToGps (const char *timeformat)
{
 char dummy[256], *str;
 double gtime;
 double dec;

  strcpy(dummy, timeformat);
  str = strstr(dummy,"s");
  if (!str) str = strstr(dummy,"GMT");

  if (str)
     {
      if (strstr(dummy,"s") != NULL) sscanf(str+1, "%lf", &dec);
      else                           dec = 0.;
      if (strstr(dummy,"s") != NULL) dummy[str-dummy] = '\0';
      else                           dummy[str-dummy-1] = '\0';
      gtime = DyGTimeFromDate (dummy+2);
      gtime = gtime + dec;
     }
 else
    {gtime = 0.;}

 return(gtime);
}


/*------------------------------------------- DyGpsToTimeString ---*/

char *DyGpsToTimeString (double origin, struct FrameH *frame, double deltat, int nAverage, char *title)
{
 char stringtime[40], title_end[64];
 time_t starttimeutc;
 int origin2;

 if (deltat == 0 && nAverage <= 1)
    {strcpy(title_end,DYEMPTY);
     if (strstr(title,"dt:") != NULL) strcpy(title_end,strstr(title,"dt:"));
    }

 if (origin-(int)origin >= 0.9999) origin = (double)((int)(origin+0.1));

 if (frame) starttimeutc = DyConvertToUTC((int)origin, frame->ULeapS);
 else       starttimeutc = DyConvertToUTC((int)origin,0);
 strftime(stringtime,30,"%b %e %Y %H:%M:%S",gmtime(&starttimeutc));

 origin2 = (int)(10000 * (origin -(int)origin));
 if (origin2%10000 == 9999)                   {origin2 = 0; origin += 1.;}
 else if (origin2-1000*(origin2/1000) == 999)  origin2++;
 else if (origin2-100*(origin2/100) == 99)     origin2++;
 else if (origin2-10*(origin2/10) == 9)        origin2++;

/*
 if (origin2 == 0)             sprintf(title, "%9.0f : %s UTC", origin, stringtime);
 else if (origin2%1000 == 0.)  sprintf(title, "%10.1f : %s UTC", origin, stringtime);
 else if (origin2%100 == 0.)   sprintf(title, "%11.2f : %s UTC", origin, stringtime);
 else if (origin2%10 == 0.)    sprintf(title, "%12.3f : %s UTC", origin, stringtime);
 else                          sprintf(title, "%13.4f : %s UTC", origin, stringtime);
*/

 if (deltat) sprintf(title, "%13.2f : %s UTC", origin, stringtime);
 else        sprintf(title, "%13.4f : %s UTC", origin, stringtime);

 if (deltat)       sprintf(title, "%s dt:%.2fs", title, deltat);
 if (nAverage > 1) sprintf(title, "%s nAv:%d", title, nAverage);

 if (deltat == 0 && nAverage <= 1)
    {if (strcmp(title_end,DYEMPTY) != 0) sprintf(title,"%s %s", title, title_end);}

 return(title);
}

/*------------------------------------------- DyConvertToUTC ---*/

time_t DyConvertToUTC (int gps, int uLeapS)
{
// Converts the gps time (with leap seconds) into utc
// input : gTimeS = GPS time in seconds
//         ULeapS = Leap seconds for the time

 time_t utc;

 if(uLeapS == 0) {uLeapS = FRGPSDELTA+FRGPSTAI; if (uLeapS < 34) uLeapS=34;}   /*-- current best guess --*/

 if  (gps > 914803214);                  /* Sun Jan  1 00:00:00 2009 */
 else if (gps > 820108813) uLeapS = 33;  /* Sun Jan  1 00:00:00 2006 */
 else if (gps > 599184012) uLeapS = 32;  /* Fri Jan  1 00:00:00 1999 */
 else if (gps > 551750411) uLeapS = 31;  /* Tue Jul  1 00:00:00 1997 */
 else if (gps > 504489610) uLeapS = 30;  /* Mon Jan  1 00:00:00 1996 */
 else if (gps > 457056009) uLeapS = 29;  /* Fri Jul  1 00:00:00 1994 */
 else if (gps > 425520008) uLeapS = 28;  /* Thu Jul  1 00:00:00 1993 */

 utc = gps + FRGPSTAI - uLeapS + FRGPSOFF;
 return utc;
}


/*------------------------------------------- DyGetStartTime ---*/

double DyGetStartTime()
{
 return(Dy->startTime);
}


/*------------------------------------------- DyQuitAtClose ---*/

int DyQuitAtClose(FL_FORM *flForm, void *data)
{
#ifndef USE_VEGA
 quitHandler(-2);
#endif
 return(FL_OK);
}


/*------------------------------------------- DyAtClose ---*/

int DyAtClose(FL_FORM *flForm, void *data)
{
 if (flForm == mainbrowser->mainbrowser) DyXformClosePanel (mainbrowser->close,MAINBROWSER);
 if (flForm == userfuncbrowser->userfuncbrowser) DyXformClosePanel (userfuncbrowser->close,USERFUNCBROWSER);
 if (flForm == plotfft->plotfft)         DyXformClosePanel (plotfft->close,PLOTFFT);
 if (flForm == plotffttime->plotffttime) DyXformClosePanel (plotffttime->close,PLOTFFTTIME);
 if (flForm == plot1dtime->plot1dtime)   DyXformClosePanel  (plot1dtime->close,PLOT1DTIME);
 if (flForm == plottrfct->plottrfct)     DyXformClosePanel (plottrfct->close,PLOTTR);
 if (flForm == plottime->plottime)       DyXformClosePanel (plottime->close,PLOTTIME);
 if (flForm == plot1d->plot1d)           DyXformClosePanel (plot1d->close,PLOT1D);
 if (flForm == plot2d->plot2d)           DyXformClosePanel (plot2d->close,PLOT2D);
 if (flForm == plotaudio->plotaudio)     DyXformClosePanel (plotaudio->close,PLOTAUDIO);
 if (flForm == plotgenepar->plotgenepar) DyXformClosePanel (plotgenepar->close,GENEPAR);
 if (flForm == inputfile->inputfile)     DyXformClosePanel (inputfile->close,INPUTFILE);
 if (flForm == dumpdata->dumpdata)       DyXformClosePanel (dumpdata->close,DUMPDATA);
 if (flForm == cmservers->cmservers)     DyXformClosePanel (cmservers->close,CMSERVERS);
 if (flForm == getdd->getdd)             DyXformClosePanel (getdd->close,GETDD);
 if (flForm == getshm->getshm)           DyXformClosePanel (getshm->close,GETSHM);
 if (flForm == plotraw->plotraw)         DyXformClosePanel (plotraw->close,PLOTRAW);
 if (flForm == plotrawtime->plotrawtime) DyXformClosePanel (plotrawtime->close,PLOTRAWTIME);
 if (flForm == helppanel)                DyXformClosePanel (NULL,HELPPANEL);
 if (flForm == userprochelp)             DyXformClosePanel (NULL,USERFCTHELP);
 if (flForm == coffeepanel)              DyXformClosePanel (NULL,COFFEEPANEL);
 if (flForm == dydebug->dydebug)         DyXformClosePanel (dydebug->close,DYDEBUG);
 if (flForm == dyoptions->dyoptions)     DyXformClosePanel (dyoptions->close,DYOPTIONS);
 if (flForm == dyoutput->dyoutput)       DyXformClosePanel (dyoutput->close,OUTPUTDATA);
 if (flForm == reffile->reffile)         DyXformClosePanel (reffile->close,REFFILE);

 return(FL_IGNORE);
}





/*********/
/* INIT  */
/*********/

/*------------------------------------------- DyIni ---*/

void DyIni (char **argv, int argc)
{
 long i;
 char dirname[LSTRING_MAX], dummy[LSTRING_MAX], currentsys[LSTRING_MAX], *minus;
 int i0, i1, dbgLvl;
 struct rlimit *rlim;

 DyDebugPrint(__FUNCTION__,1);

/*
 if (is_a_dir(FFLDIR)) Dy->Dir = my_strcpy(Dy->Dir, FFLDIR);
 else                  Dy->Dir = my_strcpy(Dy->Dir, ".");
*/
 Dy->Dir = my_strcpy(Dy->Dir, ".");


 /*--- Init plot parameters ---*/

 dbgLvl = Dy->debug;
 Dy->debug = 0;
 for (i=0; i<NPLOT_MAX; i++)
    {Plot[i] = (struct DyPlot *)calloc(1,sizeof(struct DyPlot));
     DyPlotInit(i,0);
    }
 Dy->debug = dbgLvl;

 Dy->Framedt           = FRTIME_MAX;
 Dy->skipFrame         = OFF;
 Dy->notoc             = OFF;
 Dy->currentTime       =  0;
 Dy->firstTime         =  0;
 Dy->startRun          = -1;
 Dy->startFrame        =  0;
 Dy->nClock            =  0;
 Dy->Frame             = NULL;
 Dy->y2d               = OFF;
 Dy->CmConnect         = OFF;
 Dy->InputType         = INOFF;
 Dy->Clone             = OFF;
 Dy->Order             = STOP;
 Dy->frameChannel      = 0;
 Dy->userMenuItem      = NULL;
 Dy->nCol              = 0;
 Dy->nRow              = 0;
 Dy->cmacrocopy        = NULL;
 Dy->triggerVect       = NULL;
 Dy->triggerUsed       = OFF;
 Dy->AutoDeselectPlot  = ON;
 Dy->AutoDeselectCh    = OFF;
 Dy->DdNS              = OFF;
 Dy->output_nbytes_tot = 0;
 Dy->Ddpasswd          = NULL;
 Dy->refPlots          = NULL;
 Dy->refplot_color     = 0; 
 Dy->refplot_indelebile = OFF; 
 Dy->nicefont          = OFF; 
 DyOutputDefault();

 strcpy(Dy->gLastTitle," ");
 strcpy(Dy->gLastTitleY," ");
 strcpy(Dy->lastSearchName," ");
 strcpy(Dy->previousVectList," ");
 strcpy(Dy->trigAsciiFileName,"triglist.txt");
 strcpy(Dy->trigEventName,"EVENT_SNR");
 strcpy(Dy->lastOpString,"UserOp = Pr_B1_ACp / Pr_B5_DC");
 strcpy(Dy->lastTrigString,"Qc_Moni..ScienceMode = 1");
 strcpy (Dy->savedConfig,"ddconfig"); strcat(Dy->savedConfig, CONFIG_SUFFIX);
 strcpy (Dy->savedPlots,"ddplots"); strcat (Dy->savedPlots, ROOT_SUFFIX);
 Dy->refplot_dir = my_strcpy (Dy->refplot_dir,".");
 
 strcpy(Dy->shmName,"siesta");
 strcpy(Dy->CmServer,DYEMPTY);
 if (getenv("USER") != NULL)
    {sprintf(Dy->CmName,"%s_%s_%d",(char *)getenv("USER"), getenv("HOST"), getpid());}
 else
    {sprintf (Dy->CmName,"%s_%s_%d",DDISPLAY, getenv("HOST"), getpid());}

 strcpy(Dy->CmDomain,CASCINA);
 strcpy(Dy->DdServer, DDSERVER);
 strcpy(Dy->Ddinputfile, DDFILE);
 strcpy(Dy->DdDir, DDDIR);


#ifndef USE_VEGA

/***
core limit
 if (Dy->debug == 0)
    {
     rlim = (struct rlimit *) malloc(sizeof(struct rlimit));
     getrlimit(RLIMIT_CORE,rlim);
     printf("rlim_cur=%d  rlim_max=%d \n", (int)rlim->rlim_cur, (int)rlim->rlim_max);
     rlim->rlim_max = (rlim_t)0;
     setrlimit(RLIMIT_CORE,rlim);
     Plot[999999]->average_fft = 1;
    }
***/

#endif

 /*--- Open main panel of data Display ---*/

 DyXformRaisePanel(NULL,MAINPANEL);

 /*--- initialize help ballons ---*/

 DyHelpMainPanel(NULL,1);     /*--- initialize help ballons for mainpanel panel   ---*/
 DyHelpMainBrowser(NULL,1);   /*--- initialize help ballons for mainbrowser panel ---*/
 DyHelpPlottime(NULL,1);      /*--- initialize help ballons for plottime panel    ---*/
 DyHelpPlotffttime(NULL,1);   /*--- initialize help ballons for plotffttime panel ---*/
 DyHelpPlot1dtime(NULL,1);    /*--- initialize help ballons for plot1dtime panel  ---*/
 DyHelpPlotrawtime(NULL,1);   /*--- initialize help ballons for plotrawtime panel ---*/
 DyHelpPlottrfct(NULL,1);     /*--- initialize help ballons for plottrfct panel   ---*/
 DyHelpPlotfft(NULL,1);       /*--- initialize help ballons for plotfft panel     ---*/
 DyHelpPlot1d(NULL,1);        /*--- initialize help ballons for plot1d panel      ---*/
 DyHelpPlot2d(NULL,1);        /*--- initialize help ballons for plot2d panel      ---*/
 DyHelpAudio(NULL,1);         /*--- initialize help ballons for plotaudio panel   ---*/
 DyHelpPlotGenePar(NULL,1);   /*--- initialize help ballons for plotgenepar panel ---*/
 DyHelpCmConnect(NULL,1);     /*--- initialize help ballons for plotgenepar panel ---*/
 DyHelpUserFct(NULL,1);       /*--- initialize help ballons for plotgenepar panel ---*/
 DyHelpOptions(NULL,1);       /*--- initialize help ballons for options panel     ---*/
 DyHelpOutput(NULL,1);        /*--- initialize help ballons for output panel      ---*/
 DyHelpDebug(NULL,1);         /*--- initialize help ballons for debug panel       ---*/
 DyHelpReffile(NULL,1);       /*--- initialize help ballons for refplot panel     ---*/

 /*--- Initialize the debug level in GUI with the value provided from command line ---*/
 sprintf(dummy,"%d",Dy->debug); fl_set_input(dydebug->debuglvl,dummy);

 /*--- Last message ---*/
 if (Dy->AutoConfFlag == OFF)
    {sprintf(dummy, "dataDisplay %s ready to get data. Use Input Menu", DYVERSION);
     DyInfoLine(dummy);
    }

#ifdef USE_VEGA
 if (!DyExternSource)
    {
     DyInfoLine ("Warning : opening an empty vega viewer session, without frame channel.");
    }
 else
    {
     Dy->frameChannel = DyExternSource->dyFrameChannel;
     DyReadFirstFrame(NULL,INVEGA);
    }
#endif
    
#ifndef USE_VEGA 
 /*-- Needed for what ? ---*/
 i0 = 0;
 i1 = 0;
 for (i=0; i<strlen(argv[0]); i++)
    {if (argv[0][i] == '/') {i1=i0; i0=i;}}
 strncpy(dirname, argv[0], i1);
 dirname[i1] = '\0';
#endif

/**
 RA_InitDefaultScripts();
**/

 DyXformUpdateUserProcBrowser();

 return;
}


/*--------------------------------------------------- DyRaz ---*/

void DyRaz (long arg, long nplot)
{
 long i;

 DyDebugPrint(__FUNCTION__,0);

 /*--- reset plots ---*/

 if (arg == 2) 
    {for (i=0; i<=nplot; i++) DyPlotInit(i,0);}

 /*--- remove paw windows ---*/

 if (arg == 5 || arg == 2 || arg == 0)
    {if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>1) printf("DyRaz> Reset paw windows \n");
     UJSetIni();          /* init reallocation */
     DyVisuClear (-10);   /* desactivate and close the previous windows */
    }

 /*--- reset datatodump ---*/

 if (arg == 3 || arg == 0)
    {if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>1) printf("DyRaz> Reset datatodump browser \n");
     fl_clear_browser(dumpdata->datatodump);
    }

 /*--- reset datadump ---*/
/***
 if (arg == 4) DyXformDumpData(Dy->Frame,0);
***/

 /*--- Initialize options and buttons of the main panel ---*/

 if (arg == 0)
    {
     Dy->notoc      =  0;
     Dy->firstTime  =  0;
     Dy->startFrame =  0;
     Dy->startTime  = -1;
     Dy->duration   = -1;
     Dy->nClock     =  0;
     Dy->Frame      = NULL;
     Dy->y2d        = OFF;
     Dy->CmConnect  = OFF;
     Dy->InputType  = INOFF;
     Dy->Clone      = OFF;
     Dy->Order      = STOP;
    }

 return;
}


/*------------------------------------------- DyMakeTypeNames ---*/

void DyMakeTypeNames ()
{
 int i;

 for (i=0; i<64; i++) strcpy(Dy->typeName[i], "unknown");

 strcpy(Dy->typeName[MAINPANEL],    "mainpanel");
 strcpy(Dy->typeName[HELPPANEL],    "help");
 strcpy(Dy->typeName[MAINBROWSER],  "mainbrowser");
 strcpy(Dy->typeName[INPUTFILE],    "fileslist");
 strcpy(Dy->typeName[CMSERVERS],    "cmservers");
 strcpy(Dy->typeName[GETDD],        "datasender");
 strcpy(Dy->typeName[DUMPDATA],     "dumpdata");

 strcpy(Dy->typeName[PLOTTIME],     "time");
 strcpy(Dy->typeName[PLOTCORR],     "correlation");
 strcpy(Dy->typeName[PLOTFFT],      "fft");
 strcpy(Dy->typeName[PLOTTR],       "trf.fct");
 strcpy(Dy->typeName[PLOTCOHE],     "coherence");
 strcpy(Dy->typeName[PLOT1D],       "1d");
 strcpy(Dy->typeName[PLOT2D],       "2d");
 strcpy(Dy->typeName[PLOTAUDIO],    "audio");
 strcpy(Dy->typeName[PLOTFFTTIME],  "ffttime");

 strcpy(Dy->typeName[GENEPAR],      "geneparameters");
 strcpy(Dy->typeName[USERFUNCBROWSER], "userfct");
 strcpy(Dy->typeName[USERFCTHELP],  "userfcthelp");
 strcpy(Dy->typeName[COFFEEPANEL],  "coffee");
 strcpy(Dy->typeName[PLOTRAW],      "raw");
 strcpy(Dy->typeName[PLOT1DTIME],   "1dtime");
 strcpy(Dy->typeName[PLOTRAWTIME],  "rawtime");
 strcpy(Dy->typeName[PLOTTRTIME],   "trfcttime");
 strcpy(Dy->typeName[PLOTCOHETIME], "cohetime");
 strcpy(Dy->typeName[PLOT3D],       "3d");
 strcpy(Dy->typeName[OUTPUTDATA],   "outputdata");
 strcpy(Dy->typeName[REFFILE],      "referencefiles");
 strcpy(Dy->typeName[DYOPTIONS],    "dyoptions");
 strcpy(Dy->typeName[DYDEBUG],      "dydebug");

 return;
}



/*                        *************                     */
/*                        *  RUNNING  *                     */
/*                        *************                     */

/*------------------------------------------ DyFirstLast ---*/

void DyFirstLast (double framePeriod)
{
 char dummy[32];

 DyDebugPrint(__FUNCTION__,0);

 if (Dy->AutoConfFlag == OFF)  /*--- If running manually, get parameters from graphical interface ---*/
    {DyXformFirstLast ( &Dy->startTime, &Dy->duration, &Dy->startFrame, &Dy->startRun);}

 if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>0)
    {printf("DyFirstLast> start> startTime=%d duration=%d startFrame=%d startRun=%d \n",
             Dy->startTime, Dy->duration, Dy->startFrame, Dy->startRun);
    }

 /*--- Update graphical interface fields ---*/

 if (Dy->startTime < Dy->firstTime) Dy->startTime = Dy->firstTime;
 sprintf(dummy,"%ld",Dy->startTime);
 fl_set_input(mainpanel->startTime,dummy);

 if (FRGPSDELTA <= 13 && Dy->startTime>=820108813) 
    {fl_set_input(mainpanel->startUTCdate, FrStrUTC(Dy->startTime-1,0));}
 else
    {fl_set_input(mainpanel->startUTCdate, FrStrUTC(Dy->startTime,0));}


 if (Dy->duration <= 0)
    {Dy->duration = DURATION_MAX;
     fl_set_input (mainpanel->duration,"-1");
    }

 if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>0)
    {printf("DyFirstLast> end> startTime=%d duration=%d startFrame=%d startRun=%d \n",
             Dy->startTime, Dy->duration, Dy->startFrame, Dy->startRun);
    }

 return;
}


/*--------------------------------------------------- DyStart ---*/

void DyStart (long nPlot)
{
 long i, nString, nAudio, nrms, nSame, nHidden, nnopad, ntrfct;
 int result;
 char dummy[LSTRING_MAX],fileName[LSTRING_MAX],logName[LSTRING_MAX];
 double dt;
 char *blank;
 int color;

 sprintf(dummy,"DyStart> nPlot=%ld", nPlot);
 DyDebugPrint(dummy,1);

 Dy->output_nframe_written == 0;
 Dy->start_latency = 0;
 Dy->nAnaTime      = 0;
 Dy->AnaTime       = 0.;
 Dy->oldTime       = 0.;
 Dy->currentTime   = Dy->startTime;

 Dy->nplotdrawn = 0;
 Dy->nplot  = 0;
 Dy->nready = 0;
 nAudio     = 0;
 nSame      = 0;
 nHidden    = 0;
 nnopad     = 0;
 ntrfct     = 0;
 nrms       = 0;
 for (i=1; i<=nPlot; i++)
    {if (Plot[i]->type == PLOTAUDIO) nAudio++;
     if (Plot[i]->superposed == ON) nSame++;
     if (Plot[i]->rmsfft == ON) nrms++;
     if (Plot[i]->hidden == ON && Plot[i]->type != PLOTAUDIO) nHidden++;
     if (Plot[i]->hidden == ON || Plot[i]->superposed == ON || Plot[i]->type == PLOTAUDIO) nnopad++;
     if (Plot[i]->type == PLOTTR) ntrfct++;
    }
 Dy->nsame = nSame;
 Dy->nplot = nPlot - nAudio - nHidden;

 for (i=0; i<128; i++) Dy->plotReady[i] = 0;

 if (Dy->debug > 0) 
    {printf("DyStart> Starting with %d pads for %ld+%d plots (%d trfct %d rmsfft %d same %d hidden) and %ld audio \n",
         nPlot-nnopad, nPlot-nAudio, ntrfct, ntrfct, nSame, nHidden, nAudio);
    }

 /*--- If some canvas configuration was taken from a config , use it ---*/

 if (Dy->cmacrocopy)
    {
     RA_MacroCreateCanvas(Dy->cmacrocopy);
     free(Dy->cmacrocopy);
     Dy->cmacrocopy = NULL;
    }

 /*--- Set number of Pads needed in Root Canvas ---*/

 DyWindowsIni (nPlot - nnopad);

 /*--- Choose fast font (20) when running and nice but slow font (62) at stop ---*/

 if (Dy->nicefont == ON) RA_SetLabelFont (DY_FONT_HELVETICA, NULL);
 else                    RA_SetLabelFont (DY_FONT_TIMES, NULL);

 /*--- Print the type of connection ---*/

 if      (Dy->InputType == INSHM)    sprintf(dummy,"Connect to a Shared memory (%s)", Dy->shmName);
 else if (Dy->InputType == INCM)     sprintf(dummy,"Connect to a Cm server (%s)", Dy->CmServer);
 else if (Dy->InputType == INDD)     sprintf(dummy,"Connect to a Data sender (%s)", Dy->DdServer);
 else if (Dy->InputType == INFILE)   sprintf(dummy,"Connect to a File (%s)", Dy->fileName);
 else if (Dy->InputType == INONLINE) sprintf(dummy,"Connect to a Directory (firt file = %s)", Dy->fileName);
 else if (Dy->InputType == INWAVE)   sprintf(dummy,"Connect to a Wave audio file (%s)", Dy->fileName);
 else                                sprintf(dummy,"Connect to unknown");
 DyInfoLine(dummy);

 /*--- Set the current running state ---*/

 Dy->Order = CONTINUE;
 
 /*--- Make the config file using default name ---*/

 result = access(".", W_OK);
 if (result != 0)
    {printf("DyStart> Cannot write in current directory. Files %s%s and %s%s will be put in your home directory.\n",
             Dy->curConfig, CONFIG_SUFFIX, Dy->curConfig, ROOT_SUFFIX);
     sprintf(Dy->curConfig,"%s/%s", (char *)getenv("HOME"), CARFILE);
    }
 else
    {sprintf(Dy->curConfig,"%s", CARFILE);}

 /*--- Get first and last frames to read ---*/

 dt = DyGetFramedt(Dy->Frame);
 DyFirstLast (dt);
 if (Dy->InputType != INCM && Dy->InputType != INDD) DyXformSetNoToc (NULL, 0);

 /*---------- Write global title of plots ----------*/
 
 if (Dy->Frame) RA_GlobalTitleDraw(Dy->startTime,Dy->Frame->ULeapS);
 else RA_GlobalTitleDraw(Dy->startTime,0);

 /*---------- CREATE CONFIG FILE and Remove Windows -----------*/

 sprintf(fileName, "%s%s", Dy->curConfig, CONFIG_SUFFIX);
 nString = DyMakeSiestaFile(fileName,0);

 if (Dy->InputType == INCM || Dy->InputType == INDD)
    {if (is_a_dir(DYLOG))
        {sprintf(logName,"%s/%s%s",DYLOG,Dy->CmName, CONFIG_SUFFIX);
         DyMakeSiestaFile(logName,1);
        }
     else if (is_a_dir("/virgo/virgoLog/Dy"))
        {sprintf(logName,"/virgo/virgoLog/Dy/%s%s",Dy->CmName, CONFIG_SUFFIX);
         DyMakeSiestaFile(logName,1);
        }
    }

 if (nString == 0)
    {sprintf(dummy, "Not been able to create config file %s%s", Dy->curConfig, CONFIG_SUFFIX);
     DyInfoLine (dummy);
     return;
    }

 sprintf(dummy, "Created config file %s%s. Start to read data", Dy->curConfig, CONFIG_SUFFIX);
 DyInfoLine (dummy);

 /*------------------ MAIN LOOP -----------------*/
 if (DyCheckMemory()) {fl_trigger_object(mainpanel->stop); return;}
 Dy->timeOrigin = Dy->startTime;
 DySiestaLike(Dy->curConfig);                          /*** DyReadingFrames(); ***/
 /*----------------------------------------------*/

 fl_trigger_object(mainpanel->stop);

 return;
}


/*--------------------------------------------------- DyReadingFrames ---*/
/***
void DyReadingFrames ()
{

 DyDebugPrint(__FUNCTION__,1);

 if (Dy->InputType == INCM)
    {
#ifndef NOCM
     while (Dy->Order != STOP)
       {
        CmMessageWaitWithTimeout((double)DY_TIMEOUT2);
        if (frameIn != NULL)
           {
            nBytes = DyPlotFrame (frameIn, frameInSize);
            FrameFree(frameIn);
            frameIn = NULL;
           }
       }
#endif
    }

 else
    {
     while (Dy->Order != STOP)
       {sleep(1);
        printf("I must read a file./n");
       }
    }
     
 return;
}
***/


/*--------------------------------------------------- DyPlotFrames ---*/

/***
void DyPlotFrames ()
{
 int nBytes;

 if (Dy->debug>0) printf("DyPlotFrames\n");

 return(nBytes);
}
***/


/*--------------------------------------------------- DyPause ---*/

void DyPause ()
{
 Dy->Order = PAUSE;

 if (Dy->InputType == INDD) DyDdPauseRequest (Dy->DdServer);

 DyInfoLine ("Data read paused");

 return;
}


/*--------------------------------------------------- DyContinue ---*/

void DyContinue ()
{
 char dummy[LSTRING_MAX];

 Dy->Order = CONTINUE;

 if (Dy->InputType == INDD) DyDdContinueRequest (Dy->DdServer);

 sprintf(dummy, "Running using config file %s%s", Dy->curConfig, CONFIG_SUFFIX);
 DyInfoLine (dummy);

 return;
}


/*--------------------------------------------------- DyNext ---*/

void DyNext (long arg)
{
 if (arg == 0) Dy->Order = NEXT_UPDATE;
 if (arg == 1) Dy->Order = NEXT_FRAME;
 if (arg == 2) Dy->Order = NEXT_EVENT;

 if (Dy->InputType == INDD) DyDdContinueRequest (Dy->DdServer);

 return;
}


/*---------------------------------------------------  DyStop ---*/

void DyStop()
{
 char dummy[LSTRING_MAX];
 int i, nPlot, ndeleted, ntrfct, nrms;

 Dy->Order        = STOP;
 Dy->AutoConfFlag = OFF;

 if (Dy->BgRun == ON)      DySavePlot(Dy->AutoConf);
 if (Dy->QuitAtStop == ON) DyXformQuit(NULL,10);

 nPlot  = fl_get_browser_maxline(mainbrowser->plots);
 ntrfct = 0;
 for (i=1; i<=nPlot; i++) if (Plot[i]->type == PLOTTR) ntrfct++;
 nrms = 0;
 for (i=1; i<=nPlot; i++) if (Plot[i]->rmsfft == ON) nrms++;
 ndeleted = RA_DeleteUnusedPlot(Dy->nplot+ntrfct+nrms);
 if (Dy->debug > 0) printf("DyStop> Deleted %d unused plots. Remaining %d+%d visible plots\n", ndeleted, Dy->nplot, ntrfct);

 if (Dy->AnaTime > 30.  &&  Dy->AnaTime / Dy->nAnaTime > Dy->Framedt)
    {
     printf("\n");
     printf("DyStop> *** DataDisplay CONFIGURATION is TOO SLOW to run online\n");
     printf("DyStop> *** Your plots may have too many points or are using too small refresh periods\n");
     printf("DyStop> *** First, check length and refresh period of the TIME plots.\n\n");
/**
     fl_show_alert("DataDisplay CONFIGURATION is TOO SLOW to run online",
                   "Your plots may have too many points or are using too small refresh periods.",
                   "First, check length and refresh period of the TIME plots.",1);
**/
     fl_show_messages("--- CONFIGURATION is TOO SLOW to run online ---\n"
                      "Your plots may have too many points\n"
                      "or are using too small refresh periods\n"
                      "First, check length and refresh period of the TIME plots");
    }

 return;
}


/*------------------------------------------- DySiestaLike ---*/

void DySiestaLike (char *carfile)
{
 static long first_time=1;
 char dummy[LSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);

 /*--- reset graphics windows and plots ---*/

 if (!first_time) UJLibIni2(carfile, 1, NULL);      /* re-initialize only first part of siesta libraries */
 if (first_time)UJLibIni2(carfile, 0, NULL);        /* initialize all siesta libraries */
 if (first_time && ((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>0) printf("DySiestaLike> ******** START ********* \n");

 DyDebugTimeNeeded("DyVisuIni","start",0);
 DyVisuIni(NULL);                                  /* Init visualization canvas */
 DyDebugTimeNeeded("DyVisuIni","stop",0);

 DyDebugTimeNeeded("UJCardRead","start",0);
 UJCardRead();                                     /* Read control cards */
 DyDebugTimeNeeded("UJCardRead","stop",0);

 if (first_time) {UserUJstructH = UJDicAddS("User",NULL,DyInteractionSim,NULL,NULL,NULL); first_time = 0;} /* Adding userSim to the user object ---*/

 UJCObAdd(Dy->nClock-1, NULL, UserUJstructH, NULL);  /* Adding user object to clocks list */

 /*=================================  INIT AND RUN MAIN LOOP ======================================*/
 DyDebugTimeNeeded("UJCObIni","start",0);
 UJCObIni();                                                            /* call all "ini" procedures" */	
 DyDebugTimeNeeded("UJCObIni","stop",0);
 UJCObSim();                                                            /* call all "sim" procedures" */ 
 /*==================================================================================================*/

 /*--- call all "end" procedures" ---*/ 

 if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>1) printf("DySiestaLike> Reading loop finished\n");
 UJpauseAtTheEnd = NO;
 DyDebugTimeNeeded("UJCObEnd","start",0);
 UJCObEnd();                     /* end objects (last updates) */
 DyDebugTimeNeeded("UJCObEnd","stop",0);
 DyDebugTimeNeeded("DyVisuEnd","start",0);
 DyVisuEnd();                   /* end visualization set */
 DyDebugTimeNeeded("DyVisuEnd","stop",0);
 UJSetIni();                    /* init reallocation to be able to read cards again */
 UJEnd();                       /* print end job timing */

 if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>0) printf("DySiestaLike> ******** STOP ********** \n");

 if (Dy->plotsNotDisplayed == ON)
    {sprintf(dummy, "Stopped. Plots saved in file %s%s. Config saved in file %s%s", Dy->curConfig, ROOT_SUFFIX, Dy->curConfig, CONFIG_SUFFIX);}
 else
    {sprintf(dummy, "Stopped. Config saved in file %s%s", Dy->curConfig, CONFIG_SUFFIX);}
 DyInfoLine (dummy);

 return;
}


/*------------------------------------------- DyInteractionSim ---*/

void DyInteractionSim (void *iFile, long time)
{
 struct UJclock *sclock;
 int nch, nPlot;
 char dummy[256];

 DyDebugPrint(__FUNCTION__,0);

 /*--- for Next Frame button ---*/

 if (Dy->Order == PAUSE2)
    {if (Dy->InputType == INDD) DyDdPauseRequest (Dy->DdServer);
     while (Dy->Order == PAUSE2) fl_check_forms();
    }

 /*--- Stop if loop is finished ---*/

 sclock = UJCloGet();
 if (sclock == NULL) UJError(3,"DyInteractionSim","no clock struct");
 if (sclock->obj0->time == sclock->totTime) Dy->Order = STOP;

 /*--- Stop and show message if current frame read has not same length as first frame read ---*/

 if (UFrCurFrameH && Dy->Frame && UFrCurFrameH->dt != Dy->Frame->dt)
        {
         sprintf(dummy,"Reading frames of %g s while configuration was for frames of %g s. Please, do \"Update Channels\" and restart", UFrCurFrameH->dt, Dy->Frame->dt);
         sclock = UJCloGet();
         if (sclock == NULL) UJError(3,"DyInteractionSim","no clock struct");
         sclock->obj0->time = sclock->totTime; Dy->Order = STOP;
         return;
        }

 /*--- Update main panel counters and data dump ---*/

 if (UFrCurFrameH != NULL)
    {DyXformUpdateCounters (UFrCurFrameH->frame, UFrCurFrameH->run, UFrCurFrameH->GTimeS,  UFrCurFrameH->ULeapS);
     if (fl_get_browser_maxline(dumpdata->datatodump) > 0)  DyXformDumpData (UFrCurFrameH,1);
    }

 /*--- Check that number of channels is not greater than nPlot ---*/

 if (UFrCurFrameH != NULL && time<10 && strcmp(Dy->CmServer,"...") != 0)
   {nch = DyCountChannels(UFrCurFrameH);
    nPlot = fl_get_browser_maxline(mainbrowser->plots);
    if (nch > 2*nPlot)
       {sprintf(dummy, "DyInteractionSim> CmServer %s sends frames containing nch=%d > 2*nPlot=%d. I stop.", Dy->CmServer, nch, 2*nPlot);
        DyInfoLine(dummy);
        Dy->Order = STOP;
        return;
       }
   }

 /*--- Do Pause if trigger option chosen, if pause_on_trigger chosen and if trigger start detected ---*/

 if (Dy->triggerUsed == ON && Dy->triggerPause == ON && Dy->triggerEvent == ON)
    {fl_trigger_object(mainpanel->pause);}

 /*--- Checked status of running buttons ---*/

 fl_check_forms();
 while (Dy->Order == PAUSE) fl_check_forms();         /*--- button PAUSE has been pushed ---*/
 if (Dy->Order == NEXT_FRAME)  Dy->Order = PAUSE2;    /*--- buttons NEXT_FRAME has been pushed ---*/
 if (Dy->Order == NEXT_EVENT)  Dy->Order = CONTINUE;  /*--- buttons NEXT_EVENT has been pushed ---*/

 return;
}


/*------------------------------------------- DyCheckRunning ---*/

int DyCheckRunning ()
{
 int flag;
 struct UJclock *clock;

 /*--- Do Pause if trigger option chosen, if pause_on_trigger chosen and if trigger start detected ---*/

 if (Dy->triggerUsed == ON && Dy->triggerPause == ON && Dy->triggerEvent == ON)
    {fl_trigger_object(mainpanel->pause);}

 fl_check_forms();

 while (Dy->Order == PAUSE) fl_check_forms();         /*--- button PAUSE has been pushed ---*/
 if (Dy->Order == NEXT_UPDATE) Dy->Order = PAUSE;     /*--- button NEXT_UPDATE has been pushed ---*/
 if (Dy->Order == NEXT_FRAME)  Dy->Order = PAUSE2;    /*--- buttons NEXT_FRAME has been pushed ---*/
 if (Dy->Order == NEXT_EVENT)  Dy->Order = CONTINUE;  /*--- buttons NEXT_EVENT has been pushed ---*/


 /*--- Stop if button STOP has been pushed (or if last Frame has been reached) ---*/

 if (Dy->Order == STOP)
    {
     clock = UJCloGet();
     if (clock == NULL) UJError(3,"UJCObSim","no clock struct");
     clock->obj0->time = clock->totTime;
/***
     if (UFrCurFrameH != NULL) DyXformDumpData(UFrCurFrameH,0);
     DyXformCmConnect (NULL, 0);
     DyXformDumpData (Dy->Frame,0);
     DyXformFillMainBrowser(Dy->Frame);
     DyXformDumpData(UFrCurFrameH,0);
     DyXformFillMainBrowser(UFrCurFrameH);
     if (Dy->Frame) FrameFree(Dy->Frame);
     Dy->Frame = FrameCopy(UFrCurFrameH);
***/
     flag = 0;
    }

 else  flag = 1;

/*
 *  RA_ProcessRootEvents();
 */

 return(flag);
}



/*                        ************                     */
/*                        *  DIVERS  *                     */
/*                        ************                     */


/*----------------------------------------------------- DyXformQuit ---*/

void DyXformQuit (FL_OBJECT *obj, long arg)
{
 quitHandler(arg);
 return;
}


/*----------------------------------------------------- quitHandler ---*/

void quitHandler (int arg)
{
 int answer, i;
 char name[LSTRING_MAX];

 if (arg != 10 && Dy->BgRun == OFF)
    {
     answer = fl_show_question("Do you want to quit ?",1);
     if (!answer) return;
    }

 /*--- if any plot present, call all "end" procedures and save plots and ntuples ---*/ 

 if (Dy->Order != STOP)
    {fl_trigger_object(mainpanel->stop);
     UJpauseAtTheEnd = NO;
     UJCObEnd();
     DyVisuEnd();
     UJSetIni();                /* init reallocation to be able to read cards again */
     UJEnd();                   /* print end job timing */
     if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>1) printf("quitHandler> Reading loop finished\n");
    }

 /*--- Disconnect from Cm Name Server ---*/ 

 if (Dy->InputType == INCM || Dy->InputType == INDD) DyCmDisconnect(Dy->CmName);
 

 /*--- Remove *.C , *.h and *.so files produced by RA_MakeClass (UserFct) ---*/ 

 DyXformRemoveUserProcFiles();


 /*--- Remove *.lis file and print end informations ---*/ 

 sprintf(name,"%s.lis",Dy->curConfig);
 remove(name);
/*
 sprintf(name, "%s%s", Dy->curConfig, CONFIG_SUFFIX);
 DyMakeSiestaFile(name,0);
*/

 if (Dy->QuitAtStop == OFF)
    {
     printf("\n");
/**     printf("quitHandler>  Plots are in %s%s \n",   Dy->curConfig, ROOT_SUFFIX);  **/
     printf("quitHandler>  Configuration is in %s%s \n", Dy->curConfig, CONFIG_SUFFIX);
     printf("quitHandler>  Aurevoir! \n\n");
    }
 else
    {
     printf("\n");
     printf("quitHandler>  Run in batch mode with config %s \n", Dy->AutoConf);
     printf("quitHandler>  Aurevoir! \n\n");
    }

 UJFree();
 DyFree();
#ifndef USE_VEGA
 exit(0);
#endif
}


/*----------------------------------------------------- DyFree ----*/

void DyFree ()
{
 long i;
 
 if (Dy->Frame) FrameFree(Dy->Frame);
 if (Dy->Dir) free(Dy->Dir);
 if (Dy->smsName) free(Dy->smsName);
 if (Dy->smsvarName) free(Dy->smsvarName);
 if (Dy->adcName) free(Dy->adcName);
 if (Dy->procName) free(Dy->procName);
 if (Dy->dumpfp) free(Dy->dumpfp);
 if (Dy->AutoConf) free(Dy->AutoConf);
 if (Dy->Ddchlist) free(Dy->Ddchlist);

 free(Dy);

 for (i=0; i<NPLOT_MAX; i++) {if (Plot[i]) free(Plot[i]);}

 return;
}


/*------------------------------------------- my_strcpy ---*/

char *my_strcpy (char *dummy, char *str)
{
 if (!str) return(NULL);
 if (dummy) free(dummy);
 dummy = (char *) calloc(strlen(str)+1,sizeof(char));
 strcpy(dummy, str);

 return(dummy);
}


/*------------------------------------------- max (not used) ---*/

long max (long a, long b)
{
 if (a>b) return a;
 return b;
}


/*------------------------------------------ return_val (not used) ---*/

double return_val (struct FrAdcData *adc, long i)
{
 double val;

 switch(adc->data->type)
   {case FR_VECT_C:  val = (double)adc->data->data[i]; break;
    case FR_VECT_2S: val = (double)adc->data->dataS[i]; break;
    case FR_VECT_4S: val = (double)adc->data->dataI[i]; break;
    case FR_VECT_8S: val = (double)adc->data->dataL[i]; break;
    case FR_VECT_1U: val = (double)adc->data->dataU[i]; break;
    case FR_VECT_2U: val = (double)adc->data->dataUS[i]; break;
    case FR_VECT_4U: val = (double)adc->data->dataUI[i]; break;
    case FR_VECT_8U: val = (double)adc->data->dataUL[i]; break;
    case FR_VECT_4R: val = (double)adc->data->dataF[i]; break;
    case FR_VECT_8R: val = (double)adc->data->dataD[i]; break;
    default: printf("Vector type of adc %s unknown !",adc->name); exit(0);
   }

 return val;
}


/*------------------------------------------- DyGetMinMax ---*/

void DyGetMinMax (struct FrVect *vect, double *vmin, double *vmax)
{
 long i;
 double value;

 if (FrVectIsValid(vect) != 0)
    {
     if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>0)
        {printf("DyGetMinMax> Found invalid data or zeros in the vector %s.\n",vect->name);}
     *vmin = 0.;
     *vmax = 0.;
     return;
    }

 for (i=0; i<vect->nData; i++)
    {if (vect->next && vect->next->data[i] <= 0) continue;
     value = FrVectGetV(vect,i);
     if (i == 0 || *vmin > value) *vmin = value;
     if (i == 0 || *vmax < value) *vmax = value;
    }

 return;
}


/*------------------------------------------- DyGetMinMaxFromTrend ---*/

void DyGetMinMaxFromTrend (struct FrameH *frame, char *adcName, double *vmin, double *vmax)
{
 struct FrSerData *serData;
 char dummy[LSTRING_MAX], *minStr, *maxStr;

 if (*vmin != 0.  &&  *vmax != 0.) return;
 if (frame == NULL)                return;
 if (frame->rawData == NULL)       return;

 serData = frame->rawData->firstSer;
 while (serData != NULL)
    {
     if (strcmp(serData->name, adcName) == 0)
        {
         maxStr = strstr(serData->data,"_max");
         if (maxStr != NULL) sscanf(maxStr,"%s %lg",dummy, vmax);
         minStr = strstr(serData->data,"_min");
         if (minStr != NULL) sscanf(minStr,"%s %lg",dummy, vmin);
         if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>1)
            {printf("DyGetMinMaxFromTrend> maxStr=%s\n vymax=%g\n minStr=%s\n vymin=%g\n",maxStr, *vmax, minStr, *vmin);}
         break;
        }
     serData = serData->next;
    }

 return;
}


/*------------------------------------------- DyXformPadsConfig ---*/

void DyXformPadsConfig (FL_OBJECT *obj, long arg)
{
 long ntot;
 int nCol, nRow;
 char dummy[LSTRING_MAX];
 const char *answer;

 DyDebugPrint(__FUNCTION__,0);

 sprintf(dummy,"%ldx%ld  (nCol times nRow)",Dy->nCol, Dy->nRow);
 answer = fl_show_input("Pads Configuration", dummy);
 if (!answer) return;

 if (strstr(answer,"x") == NULL)
    {
     sscanf(answer,"%ld", &ntot);
     DySetRowCol(ntot, &nCol, &nRow);
     Dy->nCol = nCol;
     Dy->nRow = nRow;
    }
 else
    {sscanf(answer,"%ldx%ld", &Dy->nCol, &Dy->nRow);}

 return;
}



/*------------------------------------------- DyXformChecks ---*/

void DyXformChecks (FL_OBJECT *obj, long arg)
{
 long i,nplot,nfile,nnfile,nplotmem;
 const char *filename;
 char dummy[LSTRING_MAX];
 const char *answer;

 if (arg == -1)
    {sprintf(dummy,"%ld",Dy->debug);
     answer = fl_show_input("Debug Level", dummy);
     if (!answer) return;
     if (strstr(answer,"x") == NULL)
        {Dy->debug = atoi(answer);}
     else
        {sscanf(answer,"%ldx%ld", &Dy->nCol, &Dy->nRow);}
     return;
    }


 printf("\n  **************** START TIME and OTHERS ************ \n\n");
 printf("    Dy->startTime=%ld  Dy->duration=%ld  Dy->startFrame=%d \n", Dy->startTime, Dy->duration, Dy->startFrame);
 printf("    Dy->Order=%ld  Dy->AutoConf=%s  Dy->AutoConfFlag=%d \n", Dy->Order, Dy->AutoConf, Dy->AutoConfFlag);
 printf("    Dy->notoc=%ld  Dy->skipFrame=%ld  Dy->y2d=%ld Dy->AutoDeselectPlot=%d  Dy->AutoDeselectCh=%d \n",
             Dy->notoc, Dy->skipFrame, Dy->y2d, Dy->AutoDeselectPlot, Dy->AutoDeselectCh);
 printf("    Dy->triggerPause=%d  Dy->triggerUsed=%d  Dy->triggerType=%d  Dy->trigAsciiFileName=\042%s\042  Dy->trigEventName=%s\n",
             Dy->triggerPause, Dy->triggerUsed, Dy->triggerType, Dy->trigAsciiFileName, Dy->trigEventName);
 printf("    Dy->triggerEvent=%d  Dy->lastTrigString=\042%s\042 \n",
             Dy->triggerEvent, Dy->lastTrigString);


 printf("\n  **************** %ld CLOCKS ************ \n\n", Dy->nClock);
 for (i=0; i<Dy->nClock; i++)
    {printf("    [%ld]  Dy->freqClock=%g  \n", i, Dy->freqClock[i]);}


 printf("\n  **************** INPUTS ************ \n\n");
 printf("    Dy->InputType=%ld  Dy->Dir=%s  Dy->CmServer=%s  Dy->shmName=%s  Dy->DdServer=%s\n",
             Dy->InputType, Dy->Dir, Dy->CmServer, Dy->shmName, Dy->DdServer);
 if (Dy->InputType == INCM)
    {printf("    Data are read online from Cm Server %s (my CmName=%s)\n\n", Dy->CmServer, Dy->CmName);}
 else if (Dy->InputType == INDD)
    {printf("    Data are read online from Data Distribution Server %s\n\n",Dy->DdServer);}
 else if (Dy->InputType == INSHM)
    {printf("    Data are read from shared memory %s\n\n",Dy->shmName);}
 else if (Dy->InputType == INFILE)
    {printf("    Files read offline from some files of directory %s\n\n", Dy->Dir);}
 else if (Dy->InputType == INDIR)
    {printf("    Files read offline from all files of directory %s\n\n", Dy->Dir);}
 else if (Dy->InputType == INONLINE)
    {printf("    Files read online from directory %s\n\n", Dy->Dir);}

 nfile = fl_get_browser_maxline(inputfile->file_list);
 if (nfile > 0) printf("     *** %ld FILES ***\n", nfile);
 if (nfile > 5) nnfile = 5; else nnfile = nfile;
 for (i=0; i<nnfile; i++)
    {filename = fl_get_browser_line(inputfile->file_list, i+1);
     printf("    file %ld = %s \n", i, filename);
    }
 if (nfile > 5) printf("    .....\n");


 printf("\n  **************** OUPUTS ************ \n\n");
 printf("    Dy->output_flag=%d  Dy->output_dir=%s  Dy->output_nframe_perfile=%d  Dy->output_nframe_written=%d \n",
             Dy->output_flag,  Dy->output_dir,  Dy->output_nframe_perfile, Dy->output_nframe_written);
 printf("    Dy->output_tag=%s \n", Dy->output_tag);

 nplot = fl_get_browser_maxline(mainbrowser->plots);
 printf("\n  **************** %ld PLOTS ************ \n", nplot);
 printf("\n ------- Intermediate plot used by Edit Clone is: ---------\n");
 DyCheckPlot(0);
 printf("\n ------- The plots displayed are: ---------\n");
 for (i=1; i<=nplot; i++)  DyCheckPlot(i);
 printf("\n");

 nplotmem = RA_CheckAllPlot(0);
 printf("\n  **************** %d PLOTS IN MEMORY ************ \n\n", nplotmem);
 RA_CheckAllPlot(1+Dy->debug);
 printf("\n");

/***
 if (Dy->Frame)
    {if (Dy->Frame->detectProc)
        {if (! Dy->Frame->detectProc->sData)
            {printf("\n  **************** LAST FRAME READ ************ \n\n");
             FrameDump(Dy->Frame, stdout, 2);
            }
        }
    }
***/

 return;
}


/*------------------------------------------- DyCheckPlot ---*/

void DyCheckPlot (long id)
{
 char truetype[10];

 strcpy (truetype, Dy->typeName[Plot[id]->type]);

 printf("\n    Plot %ld: type=%ld (%s)  name=%s  hidden=%d\n",
        id, Plot[id]->type, truetype, Plot[id]->name, Plot[id]->hidden);

 if (Plot[id]->type == PLOTTIME    || Plot[id]->type == PLOTFFT  || Plot[id]->type == PLOT1D      || 
     Plot[id]->type == PLOTFFTTIME || Plot[id]->type == PLOTRAW  || Plot[id]->type == PLOTRAWTIME || 
     Plot[id]->type == PLOT1DTIME  || Plot[id]->type == PLOTAUDIO)
    {
     printf("            clockN=%ld (%6.6f)\n", Plot[id]->clockN, Plot[id]->freqClock);
     printf("            vVarx=%s \n", Plot[id]->vVarx); 
    }

 if (Plot[id]->type == PLOT2D     || Plot[id]->type == PLOTCOHE     || Plot[id]->type == PLOTTR || 
     Plot[id]->type == PLOTTRTIME || Plot[id]->type == PLOTCOHETIME || 
     Plot[id]->type == PLOT3D     || Plot[id]->type == PLOTCORR)
    {
     printf("            clockN=%ld (%6.6f)  clockN2=%ld (%6.6f)  freq12=%6.6f\n",
           Plot[id]->clockN, Plot[id]->freqClock,
           Plot[id]->clockN2, Plot[id]->freqClock2, Plot[id]->freqClock12);
     printf("            vVarx=%s  vVary=%s \n",
           Plot[id]->vVarx,
           Plot[id]->vVary);
    }

 printf("            t0=%ld  duration=%ld \n",
       Plot[id]->t0,
       Plot[id]->duration); 
 printf("            xnBin=%ld  (nBinP2=%ld)  xMin=%g  xMax=%g \n",
       Plot[id]->xnBin,
       Plot[id]->nBinP2,
       Plot[id]->xMin,
       Plot[id]->xMax);
 printf("            ynBin=%ld  yMin=%g  yMax=%g  nSampling=%ld\n",
       Plot[id]->ynBin,
       Plot[id]->yMin,
       Plot[id]->yMax,
       Plot[id]->nSampling);
 printf("            filterMin=%g  filterMax=%g\n",
       Plot[id]->filterMin,
       Plot[id]->filterMax);
 printf("            tstep=%.4f  tsize=%.4f  timeSize=%.4f  sampFreq=%.4f  dfreq=%.4f  dtime=%.4f\n",
       Plot[id]->tstep,
       Plot[id]->tsize,
       Plot[id]->timeSize,
       Plot[id]->sampFreq,
       Plot[id]->dfreq,
       Plot[id]->dtime);
 printf("            refreshPeriod=%ld  resampFactor=%ld  average_fft=%ld  rmsfft=%d  decayfft=%d  refresh_fft=%d\n",
       Plot[id]->refreshPeriod,
       Plot[id]->resampFactor,
       Plot[id]->average_fft,
       Plot[id]->rmsfft,
       Plot[id]->decayfft,
       Plot[id]->refresh_fft);
 printf("            gridx=%d  gridy=%d  logx=%d  logy=%d  logz=%d \n",
       Plot[id]->gridx,
       Plot[id]->gridy,
       Plot[id]->logx,
       Plot[id]->logy,
       Plot[id]->logz);
 printf("            stat=%d  type2d=%d  autoEdge=%d  autoScale=%d  x2av=%d\n",
       Plot[id]->stat,
       Plot[id]->type2d,
       Plot[id]->autoEdge,
       Plot[id]->autoScale,
       Plot[id]->x2av);

 return;
}


/*-------------------------------------- DyGTimeFromDate ---*/

int DyGTimeFromDate (char *inputDate)   /* inputDate in UTC, output is GTimeS */
{
 char chaine[256];
 char words[32][32], *word;
 int nword, i, dum1, dum2, dum3;
 int year, month, day, hour, minute, second, gtime, length;

 if (inputDate == NULL)             return(0);
 if (strstr(inputDate,":") == NULL) return(0);

/**
 if (strstr(inputDate,"-") != NULL)
    {length = strlen(inputDate);
     for (i=0; i<length; i++) {if (inputDate[i] == '-') inputDate[i] = ' ';}
    }
**/

 strcpy(chaine,inputDate);
 word  = strtok(chaine," ");
 nword = 0;
 while (word != NULL)
    {
     strcpy(words[nword],word);
     nword++;
     word = strtok(NULL," ");
    }

 year = 0; month = 0; day = 0; hour = 0; minute = 0; second = 0;

 for (i=0; i<nword; i++)
    {

     if (strstr(words[i],"/") == NULL  &&  strstr(words[i],"-") == NULL  &&  strstr(words[i],":") == NULL)
        {
         if (strstr(words[i],"Jan")  || strstr(words[i],"jan"))  month =  1;
         if (strstr(words[i],"Feb")  || strstr(words[i],"feb"))  month =  2;
         if (strstr(words[i],"Fev")  || strstr(words[i],"fev"))  month =  2;
         if (strstr(words[i],"Mar")  || strstr(words[i],"mar"))  month =  3;
         if (strstr(words[i],"Apr")  || strstr(words[i],"apr"))  month =  4;
         if (strstr(words[i],"Avr")  || strstr(words[i],"avr"))  month =  4;
         if (strstr(words[i],"May")  || strstr(words[i],"may"))  month =  5;
         if (strstr(words[i],"Mai")  || strstr(words[i],"mai"))  month =  5;
         if (strstr(words[i],"Jun")  || strstr(words[i],"jun"))  month =  6;
         if (strstr(words[i],"Juin") || strstr(words[i],"juin")) month =  6;
         if (strstr(words[i],"Jul")  || strstr(words[i],"jul"))  month =  7;
         if (strstr(words[i],"Juil") || strstr(words[i],"juil")) month =  7;
         if (strstr(words[i],"Aug")  || strstr(words[i],"aug"))  month =  8;
         if (strstr(words[i],"Aout") || strstr(words[i],"aout")) month =  8;
         if (strstr(words[i],"Sep")  || strstr(words[i],"sep"))  month =  9;
         if (strstr(words[i],"Oct")  || strstr(words[i],"oct"))  month = 10;
         if (strstr(words[i],"Nov")  || strstr(words[i],"nov"))  month = 11;
         if (strstr(words[i],"Dec")  || strstr(words[i],"dec"))  month = 12;

         if (atoi(words[i]) > 59)   year = atoi(words[i]);
         else if (atoi(words[i]) > 0) day  = atoi(words[i]);
        }

     dum1 = 0; dum2 = 0; dum3 = 0;
     if (sscanf(words[i], "%d-%d-%d", &dum1, &dum2, &dum3) == 3)
        { 
         if (dum3 > 59)  {day = dum1; month = dum2; year = dum3;}
         if (dum1 > 59)  {year = dum1; month = dum2; day = dum3;}
         else            {hour = dum1; minute = dum2; second = dum3;}
        }

     dum1 = 0; dum2 = 0; dum3 = 0;
     if (sscanf(words[i], "%d/%d/%d", &dum1, &dum2, &dum3) == 3)
        { 
         if (dum3 > 59)  {day = dum1; month = dum2; year = dum3;}
         if (dum1 > 59)  {year = dum1; month = dum2; day = dum3;}
         else            {hour = dum1; minute = dum2; second = dum3;}
        }

     dum1 = 0; dum2 = 0; dum3 = 0;
     if (sscanf(words[i], "%d:%d:%d", &dum1, &dum2, &dum3) == 3)
        {
         if (dum3 > 59)  {day = dum1; month = dum2; year = dum3;}
         if (dum1 > 59)  {year = dum1; month = dum2; day = dum3;}
         else            {hour = dum1; minute = dum2; second = dum3;}
        }
    }

 gtime = (DyJulianDay(year,month,day) - DyJulianDay(1980,1,6)) * 86400.
       + hour*3600. + minute*60. + second + FRGPSDELTA;

 if (FRGPSDELTA <= 13 && gtime>=820108813) gtime += 1.;

 if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>1)
    {printf("DyGTimeFromDate> Found date:  %d/%d/%d  %d:%d:%d\n", day, month, year, hour, minute, second);
     printf("DyGTimeFromDate> Found gtime: %d  (%s)\n", gtime, FrStrUTC(gtime,0));
    }
           
 return(gtime);
}


/*----------------------------------------- DyJulianDay ---*/

int DyJulianDay(int y, int m, int d)
{
 int jd;

 jd = ( 1461 * ( y + 4800 + ( m - 14 ) / 12 ) ) / 4 +
               ( 367 * ( m - 2 - 12 * ( ( m - 14 ) / 12 ) ) ) / 12 -
               ( 3 * ( ( y + 4900 + ( m - 14 ) / 12 ) / 100 ) ) / 4 +
               d - 32075;
 return(jd);
}



/*----------------------------------------- DyMissingFrames ---*/

int DyMissingFrames (int previous, struct FrameH *frame)
{
 int dframe, dgtime, current;
 char errorMsg[LSTRING_MAX];

 if (frame == NULL) return(previous);

 DyDebugPrint(__FUNCTION__,0);

 /*--- Using frame GPS time ---*/

 if (frame->dt > 0.)
    {
     current = frame->GTimeS;
     if ((double)(current-previous) > 1.99*frame->dt  &&  previous > 0)
        {dgtime = (int)(((double)current - (double)previous) / frame->dt)-1;
         sprintf(errorMsg,"%d frames missing (between %d and %d)", dgtime, previous, current);
         DyInfoLine(errorMsg);
        }
    }

 /*--- Using frame number ---*/

 else
    {
     current = frame->frame;
     if (current != previous + 1  &&  previous >= 0)
        {dframe = current - previous;
         if (current < previous) dframe += 32767;
         sprintf(errorMsg,"%d frames missing (between fr%d and fr%d)", dframe, previous, current);
         DyInfoLine(errorMsg);
        }
     }

 return(current);
}


/*-------------------------------------- DyGetMin ---*/

double DyGetMin (double x1, double x2)
{
 if (x1 > x2) return(x2);
 return(x1);
}


/*-------------------------------------- DyGetMax ---*/

double DyGetMax (double x1, double x2)
{
 if (x1 > x2) return(x1);
 return(x2);
}


/*-------------------------------------- DyDebugTime ---*/

double DyDebugTime()
{
 double currentTime, dtime;

 gettimeofday(&dytv, NULL);
 currentTime = dytv.tv_sec + ( dytv.tv_usec * 1.0e-6 );
 dtime = currentTime - dyptime;
 dyptime = currentTime;

 return(dtime);
}


/*-------------------------------------- DyDebugTimeStr ---*/

char *DyDebugTimeStr (char *name, int printflag)
{
 double currentTime, dtime;
 char fctname[256];

 if (Dy->debug < 3)                     return(" ");
 if (!((Dy->debugType)&(1<<DBG_SPEED))) return(" ");

 gettimeofday(&dytv, NULL);
 currentTime = dytv.tv_sec + ( dytv.tv_usec * 1.0e-6 );
 dtime = currentTime - dyptime;

 strcpy(fctname," ");
 if (name) sprintf(fctname, "**%s", name);

 if (dtime > 0.5)      sprintf(dytimestr, "\033[31;1m%s - current_time=%.3f  duration=%.3fs \033[0m \0", fctname, currentTime, dtime);
 else if (dtime > 0.1) sprintf(dytimestr, "\033[34;1m%s - current_time=%.3f  duration=%.3fs \033[0m \0", fctname, currentTime, dtime);
 else                  sprintf(dytimestr, "\033[1m%s - current_time=%.3f  duration=%.5fs \033[0m \0", fctname, currentTime, dtime);

 if (printflag > 0) printf("%s\n",dytimestr);

 dyptime = currentTime;

 return(dytimestr);
}


/*-------------------------------------- DyTimeNeeded ---*/

double DyDebugTimeNeeded (const char *fctname, char *flag, int level)
{
 double currentTime, dtime;

 if (!((Dy->debugType)&(1<<DBG_SPEED))) return(0);

 if (level < 0) level = 0;
 if (level > DBGTIME_LEVELMAX) level = DBGTIME_LEVELMAX;

 if (flag == NULL  || strstr(flag,"start") != NULL)
    {gettimeofday(&dytv, NULL);
     ptime[level] = dytv.tv_sec + ( dytv.tv_usec * 1.0e-6 );
     strcpy(fctnamep[level],fctname);
     dtime = 0.;
    }

 if (strstr(flag, "stop") != NULL)
    {
     if (strcmp(fctname,fctnamep[level]) == 0)
        {gettimeofday(&dytv, NULL);
         currentTime = dytv.tv_sec + ( dytv.tv_usec * 1.0e-6 );
         dtime = currentTime - ptime[level];
         if (dtime > 0.5)      printf("\033[31;1m**%s - time needed=%.3fs  (level=%d) \033[0m \n", fctname, dtime, level);
         else if (dtime > 0.1) printf("\033[34;1m**%s - time_needed=%.3fs  (level=%d) \033[0m \n", fctname, dtime, level);
         else                  printf("\033[1m**%s - time_needed=%.5fs  (level=%d) \033[0m \n", fctname, dtime, level);
        }
     else
        {printf("**%s - time needed=[not evaluated]  -- previous_start[%d]=%s\n",fctname, level, fctnamep[level]);
         dtime = 0.;
        }
     ptime[level] = 0.;
     strcpy(fctnamep[level],DYEMPTY);
    }

 return(dtime);
}


/*-------------------------------------- DyDebugPrint ---*/

void DyDebugPrint (const char *msg, int printflag)
{
 double currentTime;

 if (!((Dy->debugType)&(1<<DBG_CALLS))) return;
 if (Dy->debug <= 0)                    return;

 if (printflag > 0)
    {gettimeofday(&dytv, NULL);
     currentTime = dytv.tv_sec + ( dytv.tv_usec * 1.0e-6 );
     printf("\033[34;1m--%s - current_time=%.3f \033[0m \n", msg, currentTime);
    }
 else
    {printf("\033[34;1m--%s \033[0m \n", msg);}

 return;
}


/*-------------------------------------- DyCheckMemory ---*/

int DyCheckMemory()
{
 char dummy[LSTRING_MAX];
 long i, xnBin, nplot, average_fft, resampFactor, answer=1;
 double dymem, memused, tsize, tstep;

 DyDebugPrint(__FUNCTION__,0);

 nplot = fl_get_browser_maxline(mainbrowser->plots);

 dymem = 0.;
 for (i=1; i<=nplot; i++)
    {
     if (Plot[i]->resampFactor > 0) resampFactor = Plot[i]->resampFactor;
     else                           resampFactor = 1;

     if (Plot[i]->tsize > 0) tsize = Plot[i]->tsize;
     else                    tsize = 1.;

     if (Plot[i]->tstep > 0) tstep = Plot[i]->tstep;
     else                    tstep = 1.;

     if (Plot[i]->average_fft > 0) average_fft = Plot[i]->average_fft ;
     else                          average_fft  = 1.;

     if (Plot[i]->type == PLOT2D || Plot[i]->type == PLOT3D)
        {memused = 8. * Plot[i]->xnBin * Plot[i]->ynBin;}

     else if (Plot[i]->type == PLOTFFTTIME)
        {memused = 8. * (Plot[i]->xnBin/2) * Plot[i]->timeSize / (average_fft * tsize * tstep / 100.);}

     else if (Plot[i]->type == PLOT1DTIME)
        {memused = 8. * Plot[i]->xnBin * Plot[i]->timeSize / tsize;}

     else if (Plot[i]->type == PLOTRAWTIME)
        {memused = 8. * Plot[i]->xnBin * Plot[i]->timeSize / tsize;}

     else
        {memused = 8. * (Plot[i]->nBin + Plot[i]->xnBin) / resampFactor;}

     dymem += memused;

     if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>1)
        {printf("DyCheckMemory> plot%d=%s  nBin=%d  xnBin=%d  ynBin=%d  "
                "tsize=%.4f  timeSize=%.4f  tstep=%.4f  average_fft=%d  resampFactor=%d "
                " memused=%.3fMB=8*%d*%.2f dymem=%.3fMB\n",
                 i, Plot[i]->name, Plot[i]->nBin, Plot[i]->xnBin, Plot[i]->ynBin,
                 tsize, Plot[i]->timeSize, tstep, average_fft, resampFactor,
                 memused/1024./1024., Plot[i]->xnBin/2, Plot[i]->timeSize/(average_fft*tsize*tstep/100.), dymem/1024./1024.);
        }
    }

 dymem += 10.*1024.*1024.;   /* on 8 June 2004: TOC size for rawdata=8MB  trenddata=13MB */

 if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>0 || dymem/1024./1024. > 120.)
    {printf("DyCheckMemory> Estimated memory consumption = %.3f MBytes\n", dymem/1024./1024.);}

 if (dymem/1024./1024. > 512. && Dy->BgRun == OFF)
    {sprintf(dummy, "Memory used: %.3fMB may be too high. Do you really want to start?", dymem/1024./1024.);
     answer = fl_show_question (dummy,1);
     return(!answer);
    }

 return(0);
}


/*-------------------------------------------------- DyCheckString ----*/

char *DyCheckString (char* name)
{
 unsigned char c;
 char *localName;

 if (name == NULL)       return(NULL);
 if (strlen (name) == 0) return(NULL);

 localName = name;

 while (*localName != 0)
   {
    c = (unsigned char) *localName;
    if (strchr (". \t~!^*()-+\"'|\\<>?/", c) != NULL) *localName = '_'; 
    localName++;
   }

 return(name);
}


/*--------------------------------------------------- DyFilterString ----*/

char *DyFilterString (char* name, unsigned char separator)
{
 unsigned char c;
 char *localName;
 int length;

 if (name == NULL)       return(NULL);
 if (strlen (name) == 0) return(NULL);

 length = strlen(name);
 localName = name;

 while (*localName != 0)
   {
    c = (unsigned char) *localName;
    if (c == separator) *localName = ' '; 
    if (c == ' ' && localName > name && localName < name+length-1) *localName = separator; 
    localName++;
   }

 return(name);
}

/*--------------------------------------------------- DyCountChannels ---*/

int DyCountChannels (struct FrameH *frame)
{
 int nadc, nser, nproc, nsim, nch;
 struct FrAdcData *adc;
 struct FrSerData *ser;
 struct FrProcData *proc;
 struct FrSimData *sim;

 if (frame == NULL) return(0);

 nadc = 0;
 nser = 0;
 if (frame->rawData)
    {adc = frame->rawData->firstAdc; while (adc != NULL) {nadc++; adc = adc->next;}
     ser = frame->rawData->firstSer; while (ser != NULL) {nser++; ser = ser->next;}
    }

 nproc = 0;
 proc = frame->procData;
 while (proc != NULL) {nproc++; proc = proc->next;}

 nsim = 0;
 sim = frame->simData;
 while (sim != NULL) {nsim++; sim = sim->next;}

 nch = nadc + nser;

 return(nch);
}


