
/**************************************************/
/*                                                */
/* File: DyVar.c                                  */
/* Copyright (C) 2003, D. Verkindt                */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include <fcntl.h> 
#include <stdarg.h>
#include <dirent.h>
#include <assert.h>
#include <unistd.h>
#include "dataDisplay.h"

#ifdef MEMWATCH
#    include "memwatch.h"
#endif

#ifdef NOCM
#define CHANNELS_LIST   "ChannelsList"  
#endif


/*                        **************                     */
/*                        *  VARIABLE  *                     */
/*                        **************                     */


/*-------------------------------------------------------- DyChangeVarName ---*/
void DyChangeVarName (char *result, char *vVar, int datatype, int datatype_new, double freqClock)
{
 int j, length;
 char *str, namex[LSTRING_MAX], dummy[LSTRING_MAX];

 if (datatype == 2 && datatype_new == 0)  /*--- From trend to raw ---*/
    {length = strlen(vVar);
     if (strncmp(vVar+length-5,"_mean",5) == 0)
        {strncpy(namex,vVar, length-5);
         namex[length-5] = '\0';
        }
     else if (strncmp(vVar+length-4,"_min",4) == 0 || strncmp(vVar+length-4,"_max",4) == 0 || strncmp(vVar+length-4,"_rms",4) == 0) 
        {strncpy(namex,vVar, length-4);
         namex[length-4] = '\0';
        }
     else 
        {strcpy(namex,vVar); namex[length] = '\0';
         if (freqClock <= 1.) 
            {for (j=length-1; j>0; j--)
                {if (namex[j] == '_') {strcpy(dummy,namex+j+1); namex[j] = '\0'; strcat(namex,SEP); strcat(namex,dummy); namex[length+1] = '\0'; break;}}
            }            
        }
     if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>1) printf("DyChangeVarName> trend->raw: Changing name from %s to %s\n", vVar, namex);
     strcpy(result, namex);
    }

 if (datatype == 1 && datatype_new == 0)  /*--- From 50Hz to raw ---*/
    {length = strlen(vVar);
     if (strncmp(vVar+length-5,"_50",3) == 0)
        {strncpy(namex,vVar, length-5);
         namex[length-5] = '\0';
        }
     else
        {strcpy(namex,vVar);
         namex[length] = '\0';
         if (freqClock <= 1.)
            {for (j=length; j>0; j--)
                {if (namex[j] == '_') {strcpy(dummy,namex+j+1); namex[j] = '\0'; strcat(namex,SEP); strcat(namex,dummy); namex[length+1] = '\0';}}
            }
        }
     if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>1) printf("DyChangeVarName> 50Hz->raw: Changing name from %s to %s\n", vVar, namex);
     strcpy(result, namex);
    }

 if (datatype == 2 && datatype_new == 1)  /*--- from trend to 50Hz ---*/
    {length = strlen(vVar);
     if (strncmp(vVar+length-5,"_mean",5) == 0)
        {strncpy(namex,vVar, length-5);
         namex[length-5] = '\0';
         strcat(namex,"_50Hz");
         namex[length-5+5] = '\0';
        }
     else if (strncmp(vVar+length-4,"_min",4) == 0 || strncmp(vVar+length-4,"_max",4) == 0 || strncmp(vVar+length-4,"_rms",4) == 0) 
        {strncpy(namex,vVar, length-4);
         namex[length-4] = '\0';
         strcat(namex,"_50Hz");
         namex[length-4+5] = '\0';
        }
     else
        {strcpy(namex,vVar);}
     if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>1) printf("DyChangeVarName> trend->50Hz: Changing name from %s to %s\n", vVar, namex);
     strcpy(result, namex);
    }

 if (datatype == 1 && datatype_new == 2)  /*--- from 50Hz to trend ---*/
    {length = strlen(vVar);
     if (strncmp(vVar+length-5,"_50",3) == 0)
        {strncpy(namex,vVar, length-5);
         namex[length-5] = '\0';
         strcat(namex,"_mean");
         namex[length-5+5] = '\0';
        }
     else
        {strncpy(namex,vVar, length-5);
         namex[length-5] = '\0';
        }
     if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>1) printf("DyChangeVarName> 50Hz->trend: Changing name from %s to %s\n", vVar, namex);
     strcpy(result, namex);
    }

 if (datatype == 0 && datatype_new == 2)  /*--- from raw to trend ---*/
    {length = strlen(vVar);
     strcpy(namex,vVar);
     namex[length] = '\0';
     if (freqClock > 1.)
        {strcat(namex,"_mean");
         namex[length+5] = '\0';
        }
     else
        {str = strstr(namex, SEP);
         if (str) {strcpy(dummy,str+strlen(SEP)); *str = '\0'; strcat(namex,"_"); strcat(namex,dummy); namex[length-1] = '\0';}
        }
     if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>1) printf("DyChangeVarName> raw->trend: Changing name from %s to %s\n", vVar, namex);
     strcpy(result, namex);
    }

 if (datatype == 0 && datatype_new == 1)  /*--- from raw to 50Hz ---*/
    {length = strlen(vVar);
     strcpy(namex,vVar);
     namex[length] = '\0';
     if (freqClock > 50.)
        {strcat(namex,"_50Hz");
         namex[length+5] = '\0';
        }
     else if (freqClock <= 1.)
        {str = strstr(namex, SEP);
         if (str) {strcpy(dummy,str+strlen(SEP)); *str = '\0'; strcat(namex,"_"); strcat(namex,dummy); namex[length-1] = '\0';}
        }
     if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>1) printf("DyChangeVarName> raw->50Hz: Changing name from %s to %s\n", vVar, namex);
     strcpy(result, namex);
    }

return;
}


/*------------------------------------------- DyUpdateDataType ---*/
/*
  Called when: a new Data Input is chosen
  Called why: to update plot parameters if the new Data Input differs from the old one (raw data --> 50 Hz)
  Call what: DyUpdatePlotName , DyPutDefaultInPlot , DyGetDefaultFromPlot
*/

void DyUpdateDataType (struct FrameH *frame)
{
 struct FrVect *vect, *vect2, *vect3;
 struct FrSerData *sms;
 struct FrAdcData *adc;
 char dummy[LSTRING_MAX], *str;
 char   *smsName=NULL, *varName=NULL, *adcName=NULL, *procName=NULL, *simName=NULL;
 long nplot, i, j;
 int length, datatype, datatype_new, ntry;
 static char *datatype_name[] = {"raw", "50Hz", "trend"};

 if (frame == NULL) return;

 sprintf(dummy,"DyUpdateDataType> frame=%d  dt=%f", frame->GTimeS, frame->dt);
 DyDebugPrint(dummy,0);

 /*--- Find the new data type and compare it to the old one ---*/

 /** datatype_new = 0  raw data **/
 /** datatype_new = 1  50Hz data **/
 /** datatype_new = 2  trend data **/
/**
FrameDump(frame, stdout, Dy->debug);
**/

 if (frame->rawData == NULL) datatype_new = 0;
 else
    {if (frame->rawData->firstAdc == NULL) datatype_new = 0;
     else
        {ntry = 0;
         datatype_new = 0;
         for (adc=frame->rawData->firstAdc; adc != NULL && ntry < 10; ntry++)
            {if (strstr(adc->name,"_50") != NULL)       {datatype_new = 1; break;}
             else if (strstr(adc->name,"_min") != NULL) {datatype_new = 2; break;}
            }
         if (frame->dt > 899.)     datatype_new = 2;
/**
         if (frame->dt > 899.)     datatype_new = 2;
         else if (frame->dt > 9.)  datatype_new = 1;
         else                      datatype_new = 0;
**/
        }
    }

 nplot = fl_get_browser_maxline(mainbrowser->plots);
 if (nplot < 1) return;

 datatype = 0;                                                                               /* raw data */
 for (i=1; i<=nplot; i++)
     {length = strlen(Plot[i]->vVarx);
      if (strncmp(Plot[i]->vVarx+length-5,"_50",3) == 0)           {datatype = 1; break;}    /* 50Hz data */
      else if (strncmp(Plot[i]->vVarx+length-5,"_mean",5) == 0 ||
               strncmp(Plot[i]->vVarx+length-4,"_min",4) == 0 || 
               strncmp(Plot[i]->vVarx+length-4,"_max",4) == 0 || 
               strncmp(Plot[i]->vVarx+length-4,"_rms",4) == 0)     {datatype = 2; ; break;}  /* trend data */
      else if (Plot[i]->freqClock <= 1. && Plot[i]->tsize > 1000)  {datatype = 2; ; break;}  /* trend data */
     }

 if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>0)
    {printf("DyUpdateDataType> datatype=%d=%s  datatype_new=%d=%s  vVarx[%d]=%s  framedt=%f \n",
             datatype, datatype_name[datatype], datatype_new, datatype_name[datatype_new], i, Plot[i]->vVarx, frame->dt);
    }

 if (datatype == datatype_new) return;

 /*--- Update the name of the variables ---*/

 for (i=1; i<=nplot; i++)
    {
     if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>0)
        {printf("DyUpdateDataType> %s->%s : Changing names of plot%d=%s:  vVarx=%s  vVary=%s \n",
                datatype_name[datatype], datatype_name[datatype_new], i, Plot[i]->name, Plot[i]->vVarx, Plot[i]->vVary);}

     /*--- Update name of variable X ---*/
     DyChangeVarName(Plot[i]->vVarx, Plot[i]->vVarx, datatype, datatype_new, Plot[i]->freqClock);

     /*--- Update name of variable Y ---*/
     if (Plot[i]->type != PLOT3D && Plot[i]->type != PLOT2D && Plot[i]->type != PLOTTR && Plot[i]->type != PLOTCOHE &&
         Plot[i]->type != PLOTCORR && Plot[i]->type != PLOTTRTIME && Plot[i]->type != PLOTCOHETIME) continue;
     DyChangeVarName(Plot[i]->vVary, Plot[i]->vVary, datatype, datatype_new, Plot[i]->freqClock2);

     /*--- Update name of variable Z ---*/
     if (Plot[i]->type != PLOT3D) continue;
     DyChangeVarName(Plot[i]->vVarz, Plot[i]->vVarz, datatype, datatype_new, Plot[i]->freqClock3);
    }

 /*--- Update plot names in case of datatype change ---*/

 for (i=1; i<=nplot; i++) DyUpdatePlotName(i);

 /*--- Update plot parameters in case of datatype change ---*/

 DyUpdatePlotParam(frame, -1);

 for (i=1; i<=nplot; i++)
    {
     DyPlotInit(i,1);

     if (Plot[i]->type == PLOT3D)
        {vect = FrameFindVect(frame, Plot[i]->vVarx);
         vect2 = FrameFindVect(frame, Plot[i]->vVary);
         vect3 = FrameFindVect(frame, Plot[i]->vVarz);
         if (vect && vect2 && vect3) DyPutDefaultInPlot(i);
         else                        DyGetDefaultFromPlot(i);
        }

     else if (Plot[i]->type == PLOT2D || Plot[i]->type == PLOTTR || Plot[i]->type == PLOTCOHE || Plot[i]->type == PLOTCORR ||
              Plot[i]->type == PLOTTRTIME || Plot[i]->type == PLOTCOHETIME)
        {vect = FrameFindVect(frame, Plot[i]->vVarx);
         vect2 = FrameFindVect(frame, Plot[i]->vVary);
         if (vect && vect->dx[0] != 0.) Plot[i]->freqClock = 1./vect->dx[0];
         if (vect2 && vect2->dx[0] != 0.) Plot[i]->freqClock2 = 1./vect2->dx[0];
         if (Plot[i]->freqClock > Plot[i]->freqClock2) Plot[i]->freqClock12 = Plot[i]->freqClock;
         else                                          Plot[i]->freqClock12 = Plot[i]->freqClock2;
         if (vect && vect2) DyPutDefaultInPlot(i);
         else               DyGetDefaultFromPlot(i);
        }

     else
        {vect = FrameFindVect(frame, Plot[i]->vVarx);
         if (vect && vect->dx[0] != 0.) Plot[i]->freqClock = 1./vect->dx[0];
         if (vect) DyPutDefaultInPlot(i);
         else      DyGetDefaultFromPlot(i);
         if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>0)
            {printf("DyUpdateDataType> plot%d=%s : vVarx=%s  vect=%x  freqClock=%g  xnBin=%d  tstep=%g  tsize=%g  sampFreq=%g\n",
                    i, Plot[i]->name, Plot[i]->vVarx, vect, Plot[i]->freqClock, Plot[i]->xnBin, Plot[i]->tstep, Plot[i]->tsize, Plot[i]->sampFreq);
            }
        }
    }

 /*--- Remove plots that have no more meaning in case of datatype change ---*/

 for (i=nplot; i>0; i--)
    {if (Plot[i]->xnBin == 0)
        {fl_select_browser_line(mainbrowser->plots,i);
         DyXformRemovePlot(NULL,0);
        }
    }

 return;
}


/*------------------------------------------ DyXformUpdateChList ---*/

void DyXformUpdateChList (FL_OBJECT *fl, long arg)
{
 long nfile, i, fflag;
 int previousTime, firstTime;
 struct FrFileH *fileH;
 struct FrFile *iFile, *iFile2;
 char dummy[LSTRING_MAX];
 char *chList;
 char *firstRead;
 int result, iwait, nBytes, noChList;
 long shm_iread;
 struct ShmInfo *shm;
#ifndef NOCM
 CmConnectCondition condition;
#endif

 DyDebugPrint(__FUNCTION__,0);

 if (Dy->InputType == INSHM)
    {
     Dy->Order = WAIT;
     iwait = 0;
     while ((shm = FdShmOpen(Dy->shmName, Dy->CmName)) == NULL  &&  Dy->Order != STOP)
        {fl_check_forms();
         if (iwait%DY_TIMEOUT2 == 0) {sprintf(dummy, "Could not open shared memory %s since %d sec", Dy->shmName, iwait); DyInfoLine(dummy);}
         sleep(1);
         iwait++;
        }
     if (Dy->Order == STOP) return;
     else                  {sprintf(dummy, "Opened shared memory %s", Dy->shmName); DyInfoLine (dummy); sleep(1); FdShmCsmIni(shm, &shm_iread);}
     Dy->Order = WAIT;
     while ((Dy->Frame = FdShmRead(shm, &(shm_iread), -1, &nBytes)) == NULL  &&  Dy->Order != STOP)
        {fl_check_forms();
         sprintf(dummy, "Frame=NULL. Reading frame again from shared memory %s (iread=%d iwrite=%d)", Dy->shmName, shm_iread, shm->iwrite);
         DyInfoLine (dummy);
         sleep(1);
        }
     if (Dy->Order == STOP) {FdShmClose(shm, Dy->CmName); return;}
     FdShmClose(shm, Dy->CmName);

     sprintf(dummy,"Channels list updated from shared memory %s", Dy->shmName);
     DyInfoLine(dummy);
     DyXformClosePanel(mainbrowser->close,MAINBROWSER);
     DyXformFillMainBrowser(Dy->Frame);
     if (Dy->AutoConfFlag == OFF) DyXformRaisePanel(NULL,MAINBROWSER);
    }


 if (Dy->InputType == INCM)
    {
     sprintf(dummy,"Requested an updated channels list to %s", Dy->CmServer);
     DyInfoLine (dummy);
     sleep(2);
     if (DyCmConnect(Dy->CmName) == -1) return;
     DyFdSendFrameChannelsRequest (Dy->CmServer);
      sprintf(dummy, "Requesting channels list to %s", Dy->CmServer);
      DyInfoLine (dummy);
      Dy->Order = WAIT;
      condition = CmMessageWaitWithTimeout((double)DY_TIMEOUT3);
      noChList = 0;
      if (condition == CmConnectTimeoutDetection)
         {sprintf(dummy,"Could not get channels list from %s. Request now full frame. Please, wait...");
          DyInfoLine(dummy);
          DyFdOutSendAdd (Dy->CmServer, Dy->CmName, "*", "*", 2, 0);
          if (UFrCurFrameH) {FrameFree(UFrCurFrameH); UFrCurFrameH=NULL;}
          DyCmWait(DY_TIMEOUT1, Dy->CmServer);
          noChList = 1;
         }
      else if (condition != CmConnectBreakDetection)
         {iwait = 1;
          Dy->Order = WAIT;
          while (Dy->Order != STOP)
             {condition = CmMessageWaitWithTimeout(1.);
              if (condition == CmConnectBreakDetection) break;
              if (iwait%DY_TIMEOUT2 == 0)
                 {sprintf(dummy,"No channels list received from %s since %ld seconds (timeout=%d). Requesting again...", Dy->CmServer, iwait, DY_TIMEOUT1);
                  DyInfoLine(dummy);
                  DyFdSendFrameChannelsRequest (Dy->CmServer);
                 }
              iwait++;
              if (iwait > DY_TIMEOUT1) Dy->Order = STOP;
              fl_check_forms();
             }
         }
     if (Dy->Order == STOP) return;
     Dy->Order = STOP;

     /*--- Take the frame received, copy it into Dy->Frame ---*/
     if (Dy->Frame) FrameFree(Dy->Frame);
     Dy->Frame = FrameCopy(UFrCurFrameH);

     /*--- if frame received contains only channels list and Sms, reconstruct it ---*/
     if (noChList == 0) Dy->Frame = DyChListToFrame (Dy->Frame);
     if (Dy->startTime <= 0  && Dy->Frame)
        {Dy->startTime = Dy->Frame->GTimeS;
         sprintf(dummy,"%ld",Dy->startTime);
         fl_set_input(mainpanel->startTime, dummy);
         if (FRGPSDELTA <= 13 && Dy->startTime>=820108813)      fl_set_input(mainpanel->startUTCdate, FrStrUTC(Dy->startTime-1,0));
         else if (FRGPSDELTA <= 13 && Dy->startTime>=914803213) fl_set_input(mainpanel->startUTCdate, FrStrUTC(Dy->startTime-2,0));
         else if (FRGPSDELTA == 14 && Dy->startTime>=914803213) fl_set_input(mainpanel->startUTCdate, FrStrUTC(Dy->startTime-1,0));
         else                                                   fl_set_input(mainpanel->startUTCdate, FrStrUTC(Dy->startTime,0));
        }

     DyCmDisconnect(Dy->CmName);
     sprintf(dummy,"Channels list updated from %s", Dy->CmServer);
     DyInfoLine(dummy);
     DyXformClosePanel(mainbrowser->close,MAINBROWSER);
     DyXformFillMainBrowser(Dy->Frame);
     if (Dy->AutoConfFlag == OFF) DyXformRaisePanel(NULL,MAINBROWSER);
     return;
    }

 if (Dy->InputType == INDD)
    {
     sprintf(dummy,"Requested an updated channels list to %s", Dy->DdServer);
     DyInfoLine (dummy);

     sleep(2);
     if (DyCmConnect(Dy->CmName) == -1) return;
     result = DyDdChlistRequest (Dy->DdServer, Dy->Ddinputfile, Dy->startTime);
     DyCmDisconnect(Dy->CmName);

     if (result >= 0)
        {sprintf(dummy,"Channels list received from %s", Dy->DdServer);
         DyInfoLine(dummy);
         DyXformClosePanel(mainbrowser->close,MAINBROWSER);
         DyXformFillMainBrowser(Dy->Frame);
         if (Dy->AutoConfFlag == OFF) DyXformRaisePanel(NULL,MAINBROWSER);
        }
     else
        {if (result == -1) sprintf(dummy,"No list received from %s", Dy->DdServer);
         if (result == -2) sprintf(dummy,"File list received from %s", Dy->DdServer);
         DyInfoLine (dummy);
        }
     return;
    }

 if (Dy->InputType == INFILE)
    {
     /*--- Get file corresponding to startTime ---*/
     nfile = fl_get_browser_maxline(inputfile->file_list);
     previousTime = 0;
     for (i=1; i<=nfile; i++)
        {firstRead = (char *)fl_get_browser_line(inputfile->file_list, i);
         iFile  = FrFileINew(firstRead);
         if (iFile == NULL) {printf("DyXformUpdateChList> No file %s \n",firstRead); return;}
         fflag = 0;
         if (strstr(firstRead,".ffl") != NULL)  /* see FrFileBreakName code in FrameLib */
            {for (fileH = iFile->fileH; fileH != NULL; fileH = fileH->next)
                {if (Dy->startTime >= fileH->tStart  &&  Dy->startTime <= fileH->tStart+fileH->length)
                    {fflag = 1;
                     iFile2 = FrFileINew(fileH->fileName);
                     FrFileIEnd(iFile);
                     iFile = iFile2;
                     break;
                    }
                }
            }
         firstTime = (int) FrFileITStart(iFile);

         if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>1)
            {printf("DyXformUpdateChList> fflag=%d  previousTime=%d  Dy->startTime=%d  firstTime=%d \n", fflag, previousTime, Dy->startTime, firstTime);}

         if ((Dy->startTime > previousTime  &&  Dy->startTime <= firstTime) || fflag != 0) break;
         previousTime = firstTime;
         if (nfile > 1) {FrFileIEnd(iFile); iFile = NULL;}
        }

     if (iFile == NULL) {printf("DyXformUpdateChList> No file for startTime %d\n",Dy->startTime); return;}

     /*--- get channels list and rebuild frame from channels list ---*/

     if (Dy->notoc > 0)
        {Dy->Frame = FrameRead(iFile);}
     else
        {
         /* HIHI to modify */
         chList = FrFileIGetChannelList (iFile, Dy->startTime);
         if (chList == NULL) {FrFileIEnd(iFile); return;}
         Dy->Frame = DyCharChListToFrame (chList);
         free(chList);
        }

     FrFileIEnd(iFile);

     DyXformClosePanel(mainbrowser->close,MAINBROWSER);
     DyXformFillMainBrowser(Dy->Frame);   /* Get channels of the frame */
     if (Dy->AutoConfFlag == OFF) DyXformRaisePanel(NULL,MAINBROWSER);
    }

 return;
}


/*-------------------------------------------------- DyChListToFrame ---*/

struct FrameH *DyChListToFrame (struct FrameH *frame)
{
#ifndef NOCM

 struct FrSerData *smsChList;
 struct FrAdcData *adc;
 struct FrProcData *proc;
 struct FrSimData *sim;
 double chFreq, chMin, chMax;
 int i, nData, nlist;
 char chName[LSTRING_MAX], word0[LSTRING_MAX], type[LSTRING_MAX], chlistName[LSTRING_MAX];
 char *chaine, *sentence;

 if (frame == NULL) return(NULL);

 DyDebugPrint(__FUNCTION__,0);

 nlist = 0;
 while (1)
   {
    sprintf(chlistName,"%s_%d",CHANNELS_LIST, nlist);

    if (nlist == 0) smsChList = FrSerDataFind (frame, CHANNELS_LIST, NULL);
    else            smsChList = FrSerDataFind (frame, chlistName, NULL);

   if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>0) printf("DyChListToFrame> reading %s (smsChList=%x)\n", chlistName, smsChList);

    if (smsChList == NULL) break;

    chaine = (char *)malloc(strlen(smsChList->data)+1);
    strcpy(chaine, smsChList->data);
    sentence = strtok(chaine,"\n");

    while (sentence != NULL)
       {
        sscanf(sentence,"%s",type);

        if (strcmp(type,"ADC") == 0)
           {
            sscanf(sentence,"%s %s %lf %lf %lf", word0, chName, &chFreq, &chMin, &chMax);
            if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>3) printf("DyChListToFrame> %s %s %g %g %g\n", word0, chName, chFreq, chMin, chMax);
            nData = (int)(chFreq*frame->dt);
            if (nData <= 0) nData = 2;
            adc = FrAdcDataNew (frame, chName, chFreq, nData, -32);
            adc->data->dataF[0] = chMin;
            adc->data->dataF[1] = chMax;
            for (i=1; i<nData; i++)  adc->data->dataF[i] = chMax;
           }
        if (strcmp(type,"PROC") == 0)
           {
            sscanf(sentence,"%s %s %lf %lf %lf", word0, chName, &chFreq, &chMin, &chMax);
            if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>3) printf("DyChListToFrame> %s %s %g %g %g\n", word0, chName, chFreq, chMin, chMax);
            nData = (int)(chFreq*frame->dt);
            if (nData <= 0) nData = 2;
            proc = FrProcDataNew (frame, chName, chFreq, nData, -32);
            proc->data->dataF[0] = chMin;
            proc->data->dataF[1] = chMax;
            for (i=1; i<nData; i++)  proc->data->dataF[i] = chMax;
           }
        if (strcmp(type,"SIM") == 0)
           {
            sscanf(sentence,"%s %s %lf %lf %lf", word0, chName, &chFreq, &chMin, &chMax);
            if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>3) printf("DyChListToFrame> %s %s %g %g %g\n", word0, chName, chFreq, chMin, chMax);
            nData = (int)(chFreq*frame->dt);
            if (nData <= 0) nData = 2;
            sim = FrSimDataNew (frame, chName, chFreq, nData, -32);
            sim->data->dataF[0] = chMin;
            sim->data->dataF[1] = chMax;
            for (i=1; i<nData; i++)  sim->data->dataF[i] = chMax;
           }
        sentence = strtok(NULL,"\n");
       }

    free(chaine);
    nlist++;
   }

#endif

 return(frame);
}


/*-------------------------------------------------- DyCharChListToFrame ---*/

struct FrameH *DyCharChListToFrame (char *chaine)
{
 struct FrSerData *sms;
 struct FrAdcData *adc;
 struct FrProcData *proc;
 struct FrSimData *sim;
 double chFreq, chMin, chMax;
 int nData;
 long length;
 char chName[LSTRING_MAX], word0[LSTRING_MAX], type[LSTRING_MAX], dummy[LBIGSTRING_MAX];
 char *sentence, *chData, *str, *str2;
 struct FrameH *frame;

 if (chaine == NULL) return(NULL);

 DyDebugPrint(__FUNCTION__,0);

 frame = FrameHNew("channelsList");

 if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>90) printf("DyCharChListToFrame> channelsList=%s\n", chaine);

 sentence = strtok(chaine,"\n");

 /*** If file contained only serData, the string chaine (chList) returned by
      FrFileIChannelList and passed here contains no SER (to be investigated) ***/

 while (sentence != NULL)
    {
     sscanf(sentence,"%s",type);

     if (strcmp(type,"FIRSTFRAME") == 0)
        {
         sscanf(sentence,"%s %d %d %lf %d %d", word0,
                          &(frame->GTimeS), &(frame->ULeapS), &(frame->dt),
                          &(frame->frame), &(frame->run));
         if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>3)
            {printf("DyCharChListToFrame> FIRSTFRAME: %d %d %g %d %d\n", frame->GTimeS, frame->ULeapS, frame->dt, frame->frame, frame->run);}
        }

/***
     if (strcmp(type,"LASTFRAME") == 0)
        {
         sscanf(sentence,"%s %d %d %lf %d %d", word0,
                          &(frame->GTimeS), &(frame->ULeapS), &(frame->dt),
                          &(frame->frame), &(frame->run));
         if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>3)
            {printf("DyCharChListToFrame> LASTFRAME: %d %d %g %d %d\n", frame->GTimeS, frame->ULeapS, frame->dt, frame->frame, frame->run);}
        }
***/

     if (strcmp(type,"SMS") == 0  ||  strcmp(type,"SER") == 0)
        {
         sscanf(sentence,"%s %s %lf %s", word0, chName, &chFreq, dummy);
         str  = strstr(sentence,dummy);
         if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>4)
            {printf(" DyCharChListToFrame> sentence=%s  \n dummy=%s \n str=%s \n", sentence, dummy, str);}

         if (str)
            {str2 = strstr(str,"\n");
             if (str2 == NULL) length = strlen(sentence)-((long)str-(long)sentence);
             else              length = (long)str2 - (long)str;
             chData = (char *)calloc(1, length+1);
             strncpy(chData,str,length-1);
             chData[length]='\0';
             sms = FrSerDataNew (frame, chName, frame->GTimeS, chData, chFreq);
             if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>5) 
                {printf("  DyCharChListToFrame> word0=%s  chName=%s  chFreq=%.16lf  smsFreq=%.16f  ", word0, chName, chFreq, sms->sampleRate, chData);
                 if (Dy->debug>6) printf("chData=%s \n", chData);
                 else             printf("\n");
                }
             free(chData);
            }
        }

     if (strcmp(type,"ADC") == 0)
        {
         sscanf(sentence,"%s %s %lf %lf %lf", word0, chName, &chFreq, &chMin, &chMax);
         if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>5)
            {printf("  DyCharChListToFrame> word0=%s  chName=%s  chFreq=%.16lf  chMin=%g  chMax=%g\n", word0, chName, chFreq, chMin, chMax);}
         nData = (int)(chFreq*frame->dt);
         if (nData <= 0) nData = 2;
         adc = FrAdcDataNew (frame, chName, chFreq, nData, -32);
         adc->data->nData    = nData;
         adc->data->dataF[0] = chMin;
         adc->data->dataF[1] = chMax;
        }

     if (strcmp(type,"PROC") == 0)
        {
         sscanf(sentence,"%s %s %lf %lf %lf", word0, chName, &chFreq, &chMin, &chMax);
         if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>5)
            {printf("  DyCharChListToFrame> word0=%s  chName=%s  chFreq=%.16lf  chMin=%g  chMax=%g\n", word0, chName, chFreq, chMin, chMax);}
         nData = (int)(chFreq*frame->dt);
         if (nData <= 0) nData = 2;
         proc = FrProcDataNew (frame, chName, chFreq, nData, -32);
         proc->data->nData    = nData;
         proc->data->dataF[0] = chMin;
         proc->data->dataF[1] = chMax;
        }

     if (strcmp(type,"SIM") == 0)
        {
         sscanf(sentence,"%s %s %lf %lf %lf", word0, chName, &chFreq, &chMin, &chMax);
         if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>5) 
            {printf("  DyCharChListToFrame> word0=%s  chName=%s  chFreq=%.16lf  chMin=%g  chMax=%g\n", word0, chName, chFreq, chMin, chMax);}
         nData = (int)(chFreq*frame->dt);
         if (nData <= 0) nData = 2;
         sim = FrSimDataNew (frame, chName, chFreq, nData, -32);
         sim->data->nData    = nData;
         sim->data->dataF[0] = chMin;
         sim->data->dataF[1] = chMax;
        }

     sentence = strtok(NULL,"\n");
    }

 return(frame);
}


/*------------------------------------------- DyDecodePlotName ---*/

void DyDecodePlotName (char *inputString, char **varx, char **vary)
{
 long length;
 char *temp, *temp2;
 char dummy[LSTRING_MAX];

 sprintf(dummy,"DyDecodePlotName> name=%s", inputString);
 DyDebugPrint(dummy,0); 

 temp = strstr(inputString,SEP2VAR);
 temp2 = strstr(inputString,SEP2VARTR);

 if (temp != NULL)
    {*varx = my_strcpy(*varx, &temp[2]);
     length   = strlen(inputString)-strlen(temp);
     *vary = (char *) calloc(length+1,sizeof(char));
     strncpy(*vary,inputString,length);
    }
 else if (temp2 != NULL)
    {*varx = my_strcpy(*varx, &temp2[2]);
     length   = strlen(inputString)-strlen(temp2);
     *vary = (char *) calloc(length+1,sizeof(char));
     strncpy(*vary,inputString,length);
    }
 else
    {*varx  = my_strcpy(*varx, inputString);
     *vary = NULL;
    }

 return;
}


/*------------------------------------------- DyDecodeVar ---*/

void DyDecodeVar (char *inputString, char **smsName, char **varName, char **adcName, char **procName, char **simName)
{
 long length;
 char *temp;
 int ok;
 
 temp = strstr(inputString,SEP);

 if (temp != NULL && strncmp(inputString,OPSIGNAL,strlen(OPSIGNAL)) != 0)
    {*varName = my_strcpy(*varName, &temp[2]);
     length   = strlen(inputString)-strlen(temp);
     *smsName = (char *) calloc(length+1,sizeof(char));
     strncpy(*smsName,inputString,length);
    }

 else
    {
/**
     ok = DySerDataGet(Dy->Frame, inputString, smsName, varName, NULL);
     printf("DyDecodeVar> frame=%d  inputstring=%s  smsName=%s  varName=%s  ok=%d\n", Dy->Frame->GTimeS, inputString, *smsName, *varName, ok);
**/
     ok = 0;
     if (ok != 1)
        {if (FrAdcDataFind(Dy->Frame, inputString)  != NULL)  *adcName  = my_strcpy(*adcName, inputString);
         if (FrProcDataFind(Dy->Frame, inputString) != NULL)  *procName = my_strcpy(*procName, inputString);
         if (FrSimDataFind(Dy->Frame, inputString)  != NULL)  *simName  = my_strcpy(*simName, inputString);
        }
    }

 if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>1)
    {printf("DyDecodeVar>  inputString=%s  smsName=%s  varName=%s  adcName=%s  procName=%s  simName=%s\n",
             inputString, *smsName, *varName, *adcName, *procName, *simName);
    }

 return;
}


/*------------------------------------------ DyXformSetVar ---*/

void DyXformSetVar (FL_OBJECT *obj, long arg)
{
 const char *str=NULL;
 char dummy[LSTRING_MAX];
 long length;
 char *temp;
 int ok;

 DyDebugPrint(__FUNCTION__,0);

 if (Dy->smsName)    {free(Dy->smsName);    Dy->smsName    = NULL;}
 if (Dy->smsvarName) {free(Dy->smsvarName); Dy->smsvarName = NULL;}
 if (Dy->adcName)    {free(Dy->adcName);    Dy->adcName    = NULL;}
 if (Dy->procName)   {free(Dy->procName);   Dy->procName   = NULL;}
 if (Dy->simName)    {free(Dy->simName);    Dy->simName    = NULL;}

 str = fl_get_browser_line(obj,arg);
 sscanf(str,"%s",dummy);

 /*--- Extract one sms channel ---*/

 if (obj == mainbrowser->sms)
    {
/**
     ok = DySerDataGet(Dy->Frame, dummy, &(Dy->smsName), &(Dy->smsvarName), NULL);
     printf("DyXformSetVar> frame=%d  browserline=%s  smsName=%s  varName=%s  ok=%d\n", Dy->Frame->GTimeS, dummy, Dy->smsName, Dy->smsvarName, ok);
**/
     temp = strstr(dummy,SEP);
     if (temp == NULL) {printf("DyXformSetVar> %s has not separator %s\n", dummy, SEP); return;}
     Dy->smsvarName = my_strcpy(Dy->smsvarName, &temp[2]);
     length   = strlen(dummy)-strlen(temp);
     Dy->smsName = (char *) calloc(length+1,sizeof(char));
     strncpy(Dy->smsName,dummy,length);
    }

 /*--- Extract one adc, one proc or one sim ---*/

 if (obj == mainbrowser->adc)  Dy->adcName = my_strcpy(Dy->adcName, dummy);
 if (obj == mainbrowser->proc) Dy->procName = my_strcpy(Dy->procName, dummy);
 if (obj == mainbrowser->sim)  Dy->simName = my_strcpy(Dy->simName, dummy);

 if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>1)
    {printf("DyXformSetVar> sername=%s  varname=%s  adcname=%s  procname=%s  simname=%s\n",
             Dy->smsName, Dy->smsvarName, Dy->adcName, Dy->procName, Dy->simName);
    }

 return;
}


/*------------------------------------------ DyGetSmsVarList ---*/

int DyGetSmsVarList (int *ifirst, char **wordlist, struct FrSerData *smsStation)
{
 char *current, name[LSTRING_MAX], *start;
 long length, nword, i;

 if (smsStation == NULL) return(0);

 /*--- Get the list of words of the string ---*/

 start    = smsStation->data;
 length   = strlen(start);
 current  = start;
 nword    = 0;
 
 while (current-start < length)
    {sscanf(current,"%s",name);
     wordlist[nword] = (char *)calloc(strlen(name)+1, sizeof(char));
     memcpy(wordlist[nword],name,strlen(name));
     while(*current==' ' || *current=='\n') current++;
     current += strlen(name)+1;
     if (current>=start+length) break;
     while(*current==' ' || *current=='\n') current++;
     nword++;
    }

 if (nword < 4)
    {*ifirst = 0;}
 else
    {
     if (strcmp(wordlist[3],"ALL") == 0  ||  strcmp(wordlist[3],"UPD") == 0) *ifirst = 6;
     else                                                                    *ifirst = 0;
    }

 return(nword);
}


/*------------------------------------------- DyXformFillMainBrowser ---*/

void DyXformFillMainBrowser (struct FrameH *frame)
{
 struct FrSerData *sms;
 struct FrAdcData *adc;
 struct FrSimData *sim;
 struct FrProcData *proc;
 char dummy[LSTRING_MAX];
 char procName[LSTRING_MAX], smsName[LSTRING_MAX], adcName[LSTRING_MAX], simName[LSTRING_MAX], *temp;
 const char *smsline, *adcline, *procline, *simline;
 long nadc, nsms, nproc, nsim, i, i0;
 char  **wordlist;
 int nword, ifirst;

 if (frame == NULL) return;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Update the plots already present with the samplingRates of the updated list of channels ---*/

 if (Dy->AutoConfFlag == OFF) DyUpdateDataType(frame);
 DyUpdatePlotParam(frame, -1);

 /*--- Fill PROC browser ---*/

 if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>0) printf("DyXformFillMainBrowser> Filling procData channels browser \n");
 nproc = fl_get_browser_maxline(mainbrowser->proc);
 proc = frame->procData;
/* if (proc && nproc) fl_clear_browser(mainbrowser->proc); */
 while(proc)
    {
     i0 = -1;
     for (i=0; i<nproc; i++)
        {procline = fl_get_browser_line(mainbrowser->proc,i+1);
         sscanf(procline,"%s",procName);
         if (strcmp(procName,proc->name) == 0) {i0=i; break;}
        }
     if (i0 == -1) 
        {sprintf(dummy,"%s : %d samplings/frame at %.2f Hz", proc->name, proc->data->nData, 1./proc->data->dx[0]);
         fl_add_browser_line(mainbrowser->proc,dummy);
        }
     proc = proc->next;
    }
 DyXformSortBrowser(mainbrowser->proc);


 /*--- Fill SIM browser ---*/

 if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>0) printf("DyXformFillMainBrowser> Filling simData channels browser \n");
 nsim = fl_get_browser_maxline(mainbrowser->sim);
 sim = frame->simData;
/* if (sim && nsim) fl_clear_browser(mainbrowser->sim); */
 while(sim)
    {
     i0 = -1;
     for (i=0; i<nsim; i++)
        {simline = fl_get_browser_line(mainbrowser->sim,i+1);
         sscanf(simline,"%s",simName);
         if (strcmp(simName,sim->name) == 0) {i0=i; break;}
        }
     if (i0 == -1) 
        {sprintf(dummy,"%s : %d samplings/frame at %.2f Hz", sim->name, sim->data->nData, 1./sim->data->dx[0]);
         fl_add_browser_line(mainbrowser->sim,dummy);
        }
     sim = sim->next;
    }
 DyXformSortBrowser(mainbrowser->sim);


 /*--- Check if rawdata present ---*/

 if (frame->rawData == NULL)
    {printf("DyFillMainBrowser> no raw data in this frame \n"); return;}


 /*--- Fill SMS browser ---*/

 if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>0) printf("DyXformFillMainBrowser> Filling serData channels browser \n");
 nsms = fl_get_browser_maxline(mainbrowser->sms);
 sms  = frame->rawData->firstSer;
 if (sms && nsms) fl_clear_browser(mainbrowser->sms);
 while (sms != NULL)
    {
     if (strcmp(sms->name,CHANNELS_LIST) != 0)
        {wordlist = (char **)calloc(65536,sizeof(char *));
         nword = DyGetSmsVarList (&ifirst, wordlist, sms);
         for (i=ifirst; i<nword; i=i+2)
            {
/**
sprintf(dummy,"%s%s%s : at %.5f Hz", sms->name, SEPSMS, wordlist[i], sms->sampleRate);
**/
             sprintf(dummy,"%s%s%s : at %.5f Hz", sms->name, SEP, wordlist[i], sms->sampleRate);
             if (strcmp(wordlist[i],"units") == 0) {free(wordlist[i]); continue;}
             if (strcmp(wordlist[i],"slope") == 0) {free(wordlist[i]); continue;}
             if (strcmp(wordlist[i],"bias") == 0)  {free(wordlist[i]); continue;}
             fl_add_browser_line(mainbrowser->sms,dummy);
             free(wordlist[i]);
            }
         free(wordlist);
        }
     sms = sms->next;
    }
 DyXformSortBrowser(mainbrowser->sms);


 /*--- Fill ADC browser ---*/

 if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>0) printf("DyXformFillMainBrowser> Filling adcData channels browser \n");
 nadc = fl_get_browser_maxline(mainbrowser->adc);
 adc  = frame->rawData->firstAdc;
 if (adc && nadc) fl_clear_browser(mainbrowser->adc);
 while(adc)
    {
     sprintf(dummy,"%s : %d samplings/frame at %.2f Hz", adc->name, adc->data->nData, adc->sampleRate);
     fl_add_browser_line(mainbrowser->adc,dummy);
     adc = adc->next;
    }
 DyXformSortBrowser(mainbrowser->adc);

 return;
}



/*-------------------------------------------- DyXformProcRemove ---*/

void DyXformProcRemove (FL_OBJECT *obj, long arg)
{
 const char *str;
 char strnew[LSTRING_MAX], dummy[LBIGSTRING_MAX];
 long nline, i, i0, lastSelected=0;
 struct FrProcData *proc;

 DyDebugPrint(__FUNCTION__,0);

 nline = fl_get_browser_maxline(mainbrowser->proc);
 if (nline <= 0) return;

 i0 = 1;
 while (i0 > 0)
   {
    nline = fl_get_browser_maxline(mainbrowser->proc);
    if (nline <= 0) break;

    i0 = -1;
    for (i=1; i<=nline; i++)
       {if (fl_isselected_browser_line(mainbrowser->proc,i))
           {str = fl_get_browser_line(mainbrowser->proc,i);
            if (strncmp(str,OPSIGNAL,strlen(OPSIGNAL)) == 0)
               {i0 = i;
/*** Removed this code because FrProcDataFree frees all the procData chain after proc ******/
/**
                sscanf(str,"%s",dummy);
                proc = FrProcDataFind(Dy->Frame,dummy);
                if (proc) FrProcDataFree(proc);
**/
                break;
               }
           }
       }

    if (i0 <= 0) break;
    else         lastSelected = i0;

    for (i=i0; i<=nline-1; i++)
      {
       strcpy(strnew, fl_get_browser_line(mainbrowser->proc,i+1));
       fl_replace_browser_line(mainbrowser->proc,i, strnew);
       if (fl_isselected_browser_line(mainbrowser->proc,i+1))
          {fl_select_browser_line(mainbrowser->proc,i);}
       else
          {fl_deselect_browser_line(mainbrowser->proc,i);}
      }

   fl_delete_browser_line(mainbrowser->proc,nline);
  }

 nline = fl_get_browser_maxline(mainbrowser->proc);
 if (nline > 0  &&  lastSelected > 0) fl_select_browser_line(mainbrowser->proc,lastSelected);

 return;
}


