
/**************************************************/
/*                                                */
/* File: DyVisu.c                                 */
/* Copyright (C) 2003, D. Verkindt, D. Buskulic   */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/

#include <stdio.h>
#include <unistd.h>
#include "dataDisplay.h"
#include "URootAccess.h"
#ifdef MEMWATCH
#    include "memwatch.h"
#endif

/*-------------------------------------------- DySavePlot ---*/

int DySavePlot (char *savedPlots)
{
 char dummy[LSTRING_MAX];
 int ok;

 DyDebugPrint(__FUNCTION__,0);

 ok = RA_SavePlots(savedPlots);
// RA_FileOpenCreate(savedPlots);
// RA_FileWrite(savedPlots);
// RA_FileClose(savedPlots);

 sprintf(dummy,"Plots saved in file %s", savedPlots);
 if (ok) DyInfoLine (dummy);

 return(ok);
}


/*------------------------------------------- DyWindowsIni ---*/

void DyWindowsIni (long nwindows)
{
/*
 * Opens a Root canvas and divides it so that there is a suitable number of pads
 */
 int nCol, nRow;
 int padwidth, padheight;
 int winwidth, winheight;
 int i;
 static int nColOld=0, nRowOld=0;
 char canvasTitle[LSTRING_MAX], dummy[LSTRING_MAX];

 sprintf(dummy, "DyWindowsIni> nwindows=%d\n", nwindows);
 DyDebugPrint(dummy,0);

 if (nwindows <=0) return;
 
 /* Calculation of the window size: organization of pad and pad size */

 DySetRowCol(nwindows, &nCol, &nRow);

 if (Dy->nCol > 0 && Dy->nRow > 0 && Dy->nCol*Dy->nRow >= nwindows)
    {
     printf("DyWindowsIni> Canvas format forced to %d columns x %d raws\n", Dy->nCol, Dy->nRow);
     nCol = Dy->nCol; nRow = Dy->nRow;
    }

 if (Dy->squarePlot == ON)
    {
     if (nCol == 1)  padwidth = 700 *0.907;
     if (nCol == 2)  padwidth = 380 *0.907;
     if (nCol == 3)  padwidth = 260 *0.907;
     if (nCol == 4)  padwidth = 200 *0.907;
     if (nCol >= 5)  padwidth = 800/nCol *0.907;

     if (nRow == 1)  padheight = 700;
     if (nRow == 2)  padheight = 380;
     if (nRow == 3)  padheight = 260;
     if (nRow == 4)  padheight = 200;
     if (nRow >= 5)  padheight = 800/nRow;

     if (nRow < nCol) padheight = padwidth;

     /* Set the window size */

     winwidth  = nCol*(padwidth+5)+5;
     winheight = nRow*(padheight+5)+32;
     if (winwidth > 800 * 0.907)  winwidth  = 800 *0.907;
     if (winheight > 800) winheight = 800;
    }

 else
    {
     if (nCol == 1)  padwidth = 900;
     if (nCol == 2)  padwidth = 450;
     if (nCol == 3)  padwidth = 330;
     if (nCol == 4)  padwidth = 250;
     if (nCol >= 5)  padwidth = 1000/nCol;

     if (nRow == 1)  padheight = 700;
     if (nRow == 2)  padheight = 380;
     if (nRow == 3)  padheight = 260;
     if (nRow == 4)  padheight = 200;
     if (nRow >= 5)  padheight = 800/nRow;

     if (nRow < nCol) padheight = padwidth;

     /* Set the window size */

     winwidth  = nCol*(padwidth+5)+5;
     winheight = nRow*(padheight+5)+32;
     if (winwidth > 1000) winwidth  = 1000;
     if (winheight > 800) winheight = 800;
    }

 /* Init the window */
 /* If canvas not divided as before, delete it and reconstruct */

 if (Dy->AutoConf) sprintf(canvasTitle,"Data_Display_%s:_%s", DYVERSION, Dy->AutoConf);
 else              sprintf(canvasTitle,"Data_Display_%s", DYVERSION);
 if (Dy->titleCmd != NULL)
    {if (strncmp(Dy->titleCmd,"::",2) == 0) sprintf(canvasTitle,"Data_Display_%s:_%s", DYVERSION, Dy->titleCmd+2);
     else                                   sprintf(canvasTitle,"Data_Display_%s:_%s", DYVERSION, Dy->titleCmd);
    }

 RA_InitDDWindow(canvasTitle,10,10,winwidth,winheight);

if (Dy->AutoConfFlag == OFF || (Dy->AutoConfFlag == ON && !RA_PadIsDivided(nCol*nRow)))
  {     
   if ((!RA_PadIsDivided(nCol*nRow) || nColOld != nCol || nRowOld != nRow) && RA_PadIsDivided(-1))
      {
       RA_MainCanvasDelete();
       RA_InitDDWindow(canvasTitle,10,10,winwidth,winheight);
       RA_PadDivideWithTitle(nCol,nRow);
      }
   else if (!RA_PadIsDivided(-1))
      {RA_PadDivideWithTitle(nCol,nRow);}
  }

 nColOld = nCol;
 nRowOld = nRow;

 /* Clear the supplementary pads */

 if (nwindows < nCol*nRow)
    {
     for (i=nwindows+1;i<=nCol*nRow;i++) RA_PadCompleteClear(i);
    }
 RA_SetTestTrigPad(-1,0,0);
 
 return;
}


/*-------------------------------- DySetRowCol ---*/
/* author: E. Tournefier */

void DySetRowCol (int ntot, int *nCol, int *nRow)
{
 int n1,n2,n3,n4,a,b;
 double x,y;

 if (ntot <= 0) {*nRow = 1; *nCol = 1; return;}
 if (ntot == 1) {*nRow = 1; *nCol = 1; return;}
 if (ntot == 2) {*nRow = 1; *nCol = 2; return;}
 if (ntot == 5) {*nRow = 2; *nCol = 3; return;}

 y  = ntot;
 x  = sqrt(y);
 n1 = x;
 y  = x-n1;
 if (y > 0.5) n1++;

 n2 = ntot/n1;
 if (n1*n2 < ntot) n2++;

 n3 = n1-1;
 n4 = ntot/n3;
 if (n4*n3 < ntot) n4++;

 a = n1*n2 - ntot;
 b = n3*n4 - ntot;

 if (a <= b)
    {*nRow = n1;
     *nCol = n2;
    }
 else
    {*nRow = n3;
     *nCol = n4;
    }

 if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>2)
    {printf("DySetRowCol> n1=%d n2=%d n3=%d n4=%d a=%d b=%d\n", n1, n2, n3, n4, a, b);
     printf("DySetRowCol> nRow=%d  nCol=%d \n", *nRow, *nCol);
    }

/***
 if (ntot == 1)  {*nCol=1; *nRaw=1;}
 if (ntot == 2)  {*nCol=2; *nRaw=1;}
 if (ntot == 3)  {*nCol=3; *nRaw=1;}
 if (ntot == 4)  {*nCol=2; *nRaw=2;}
 if (ntot == 5)  {*nCol=3; *nRaw=2;}
 if (ntot == 6)  {*nCol=3; *nRaw=2;}
 if (ntot == 7)  {*nCol=4; *nRaw=2;}
 if (ntot == 8)  {*nCol=4; *nRaw=2;}
 if (ntot == 9)  {*nCol=3; *nRaw=3;}
 if (ntot > 9)   {*nCol=4; *nRaw=(ntot-1)/4+1;}
 if (ntot > 12)  {*nCol=5; *nRaw=(ntot-1)/5+1;}
***/

 return;
}


/*-------------------------------------------- DyVisuClear ---*/

void DyVisuClear (long arg)
{
 DyDebugPrint(__FUNCTION__,0);

 DyGrafIni(arg);
 Dy->nRow = 0;
 Dy->nCol = 0;
 return;
}


/*-------------------------------------------- DyVisuIni ---*/

void DyVisuIni (char *fileName)
{
 static long first_time=1;

 DyDebugPrint(__FUNCTION__,0);

 if (!first_time)
    {
     DyGrafIni(-1);
    }
 if (first_time)
    {
     first_time=0;
    }
 return;
}


/*-------------------------------------------- DyVisuEnd ---*/

void DyVisuEnd ()
{
 int result;
 char ddrootFile[LSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);

 RA_DrawReferencePlot();
 RA_MainCanvasUpdate();

 /*--- open file for plots and ntuples and save them ---*/

 result = access(".", W_OK);
 if (result != 0) sprintf(ddrootFile,"%s/%s%s", (char *)getenv("HOME"), CARFILE, ROOT_SUFFIX);
 else             sprintf(ddrootFile,"%s%s", CARFILE, ROOT_SUFFIX);

if (Dy->plotsNotDisplayed == ON)
   {if (result != 0) DyInfoLine ("Could not write in local directory. Files will be written in your home directory.");
    RA_FileOpenCreate(ddrootFile);
    RA_FileWrite(ddrootFile);
    RA_FileClose(ddrootFile);
   }

 return;
}


