/**************************************************/
/*                                                */
/* File: DyXformUser.c                            */
/* Copyright (C) 2003, D. Buskulic, D. Verkindt   */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include <fcntl.h> 
#include <stdarg.h>
#include <dirent.h>
#include <assert.h>
#include <unistd.h>
#include "dataDisplay.h"
#include "URootProcess.h"

#ifdef MEMWATCH
#include "memwatch.h"
#endif


/*                 ******************************                 */
/*                 * User processing and macros *                 */
/*                 ******************************                 */


/*------------------------------------------- DyLoadScript ---*/

void DyLoadScript (char *className)
{
 char dummy[LSTRING_MAX], fileName[LSTRING_MAX];

 sprintf(dummy,"DyLoadScript> className=%s", className);
 DyDebugPrint(dummy,0);

 sprintf(fileName,"%s.C", className);

 if (is_a_file(fileName) == 0)
    {
     if (is_a_dir(DYALGO_DIR))
        {
         sprintf(dummy,"%s/%s", DYALGO_DIR, getenv("UNAME"));
         if (is_a_dir(dummy)) sprintf(fileName, "%s/%s.C", dummy, className);
         else                 sprintf(fileName,"%s/%s.C", DYALGO_DIR, className);
        }
     else
        {printf("DyLoadScript> Algorithm's file not found\n");
         return;
        }
    }

 RA_LoadScript(fileName,1);
 DyXformUpdateUserProcBrowser();

 return;
}


/*------------------------------------------ DyXformMakeNewMacro ---*/

void DyXformMakeNewMacro (FL_OBJECT *obj, long arg)
{
/* Writes a sample macro in a new file. This is just an example macro */
/* that needs to be adapted to a users particular needs               */
/* so it should be edited later, before loading                       */
   
 const char *str;
 long test=1;

 if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>0) printf("DyXformLoadInterpretMacro \n");

 str = fl_show_input("Name of the new processing object",USEROP);
 if (!str) return;
 if (strlen((char *)str) ==0) {
    printf("DyXformMakeNewMacro> need a name \n");
 }

 RA_MakeClass((char*)str, NULL, NULL);

 return;
}

/*------------------------------------------ DyXformLoadInterpretMacro ---*/

void DyXformLoadInterpretMacro (FL_OBJECT *obj, long arg)
{
/* Load a macro without compiling it, i.e. in interpreted mode */

 const char *str;
 long test=1;

 while(test)
    {str = fl_show_fselector("chose a macro file to load",".","*.[c|C|cc]","");
     if (!str) return;
     if (is_a_file((char *)str)) test=0;
     else printf("DyXformLoadInterpretMacro> I need a file \n");
    }

 RA_LoadScript((char*)str, 0);
 DyXformUpdateUserProcBrowser();

/*
 *  DyXformSortBrowser(userfuncbrowser->userFuncLoaded);
 */
 return;
}

/*------------------------------------------ DyXformLoadCompileMacro ---*/

void DyXformLoadCompileMacro (FL_OBJECT *obj, long arg)
{
/* Load a macro and, if it has changed since the last loading, compiles it */
/* prior to loading. That means, creates a new shared library out of       */
/* code contained inside the macro, loads this shared library and update   */
/* the list of already loaded user functions                               */
   
 const char *str;
 long test=1;

 while(test)
    {str = fl_show_fselector("chose a macro file to load",".","*.[c|C|cc]","");
     if (!str) return;
     if (is_a_file((char *)str)) test=0;
     else printf("DyXformLoadCompileMacro> I need a file \n");
    }

 RA_LoadScript((char*)str, 1);
 DyXformUpdateUserProcBrowser();

/*
 *  DyXformSortBrowser(userfuncbrowser->userFuncLoaded);
 */
 return;
}

/*------------------------------------------ DyXformUpdateUserProcBrowser ---*/

void DyXformUpdateUserProcBrowser()
{
/* Updates the user function browser */

 int i, j, j0, nvar;
 char userline[LSTRING_MAX], dummy[LSTRING_MAX], *str3;
 const char *str;

 DyDebugPrint(__FUNCTION__,0);

 RA_GetListOfProc();

 fl_clear_browser(userfuncbrowser->userFuncLoaded);
 fl_clear_menu (mainbrowser->userFct);
 fl_addto_menu (mainbrowser->userFct, SET_PARAMETERS);
 fl_addto_menu (mainbrowser->userFct, OPERATION_STRING);

 fl_addto_menu (mainbrowser->userFct, ADDVECT);
 fl_addto_menu (mainbrowser->userFct, SUBVECT);
 fl_addto_menu (mainbrowser->userFct, MULTVECT);
 fl_addto_menu (mainbrowser->userFct, DIVVECT);

 if (gProcList)
    {
     for (i=0;i<gProcList->numProcess;i++)
        {
         strcpy(dummy,gProcList->name[i]);
         str3 = dummy+strlen(dummy)-1;
         while (atoi(str3) > 0 || *str3 == '0') str3--;
         *(str3+1) = '\0';
         strcpy(userline,dummy);

         if (!gProcList->procCompiled[i]) strcat(userline," (interpreted)");
         nvar = fl_get_browser_maxline(userfuncbrowser->userFuncLoaded);
         j0 = -1;
         for (j=0; j<nvar; j++)
           {str = fl_get_browser_line(userfuncbrowser->userFuncLoaded,j+1);
            if (strcmp(dummy, str) == 0) {j0 = j; break;}
           }
         if (j0 == -1)
            {fl_add_browser_line (userfuncbrowser->userFuncLoaded,userline);
             if (strcmp(userline,ADDVECT) != 0 && strcmp(userline,SUBVECT) != 0 && 
                 strcmp(userline,MULTVECT) != 0 && strcmp(userline,DIVVECT) != 0 && strcmp(userline,TRIGGER) != 0)
                 fl_addto_menu (mainbrowser->userFct, userline);
            }
        }
    }
}


/*------------------------------------------- DyXformSetOp ---*/
                                            /*  called from Main Browser  */
void DyXformSetOp (FL_OBJECT *obj, long arg)
{
 long i, nvar;
 const char *menuItem, *str;
 char dummy[LSTRING_MAX], inputVect[LBIGSTRING_MAX];
 FL_OBJECT *browser;

 DyDebugPrint(__FUNCTION__,0);

 if (fl_get_menu(obj) > 0)
    {menuItem = fl_get_menu_text(obj);
     if (menuItem) Dy->userMenuItem = my_strcpy(Dy->userMenuItem,(char *)menuItem);
    }

 if (Dy->userMenuItem == NULL)
    {DyInfoLine("Choose first a user function to apply to those signals");
     return;
    }

 /*--- If item chosen in the User Fct menu is Set_Parameters or any Operation_String ---*/

 if (strcmp(Dy->userMenuItem,SET_PARAMETERS) == 0) DySetOpPar();
 else                                              DySetOpString(Dy->userMenuItem);

 return;
}

 
/*------------------------------------------- DySetOpString ---*/

void DySetOpString (char *userFct)
{
 long i, ii, i0, length, nvar, number, maxnumber;
 const char *opstring, *str;
 char dummy[LSTRING_MAX], inputVect[LBIGSTRING_MAX], opstring1[LBIGSTRING_MAX], opstring2[LBIGSTRING_MAX];
 char *p, *p1, className[LSTRING_MAX], classNamei[LSTRING_MAX];
 char defaultVectList[LBIGSTRING_MAX], dummy2[LBIGSTRING_MAX];
 char *str1, *str2, *str3, *vectlist2;
 FL_OBJECT *browser;

 if (userFct == NULL) return;

 sprintf(dummy,"DySetOpString> userFct=%s", userFct);
 DyDebugPrint(dummy,0);

 if (Dy->Frame == NULL)
    {printf("DySetOpString> userFct=%s  no frame in memory \n", userFct); return;}

 strcpy(defaultVectList,"");
 strcpy(inputVect,"");
 strcpy(opstring2,"");

 /*--- build the vect list from selected items in the channels browsers ---*/

 if (fl_get_browser(mainbrowser->adc) != 0)
    {browser = mainbrowser->adc;
     nvar    = fl_get_browser_maxline(browser);
     for (i=0; i<nvar; i++)
        {if (fl_isselected_browser_line(browser,i+1))
            {
             str = fl_get_browser_line(browser,i+1);
             sscanf(str,"%s",dummy);
             strcat(defaultVectList," ");
             strcat(defaultVectList,dummy);
            }
        }
     }
 if (fl_get_browser(mainbrowser->proc) != 0)
    {browser = mainbrowser->proc;
     nvar    = fl_get_browser_maxline(browser);
     for (i=0; i<nvar; i++)
        {if (fl_isselected_browser_line(browser,i+1))
            {
             str = fl_get_browser_line(browser,i+1);
             sscanf(str,"%s",dummy);
             if (strncmp(dummy,OPSIGNAL,strlen(OPSIGNAL)) != 0)
                {strcat(defaultVectList," ");
                 strcat(defaultVectList,dummy);
                }
            }
        }
     }
 if (fl_get_browser(mainbrowser->sim) != 0)
    {browser = mainbrowser->sim;
     nvar    = fl_get_browser_maxline(browser);
     for (i=0; i<nvar; i++)
        {if (fl_isselected_browser_line(browser,i+1))
            {
             str = fl_get_browser_line(browser,i+1);
             sscanf(str,"%s",dummy);
             strcat(defaultVectList," ");
             strcat(defaultVectList,dummy);
            }
        }
     }

 strcat(defaultVectList," \0");

 if (defaultVectList  &&  strlen(defaultVectList) > 1)
    {
     if (strcmp(userFct,OPERATION_STRING) == 0) sprintf(dummy2, "%s_%s = %s", getenv("USER"), USEROP, defaultVectList);
     else                                       sprintf(dummy2, "%s = %s", userFct, defaultVectList); 
     strcpy(defaultVectList, dummy2);
    }

 if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>1)
    {printf("defaultVectList=%s  previousVectList=%s  Dy->lastOpString=%s\n",
             defaultVectList, Dy->previousVectList, Dy->lastOpString);
    }


 /*--- If userFct is OPERATION STRING, put vect list in an input field presented to the user ---*/

 if (strcmp(userFct,OPERATION_STRING) == 0)
    {
     if (defaultVectList  &&  strlen(defaultVectList) > 1)
        {    
         if (strcmp(defaultVectList, Dy->previousVectList) != 0)
            {opstring = fl_show_input("Write an operation with correct signals names. Example:", defaultVectList);}
         else
            {opstring = fl_show_input("Write an operation with correct signals names. Example:", Dy->lastOpString);}
        }
     else
        {opstring = fl_show_input("Write an operation with correct signals names. Example:", Dy->lastOpString);}

     if (opstring == NULL) return;
     else                  strcpy(opstring1, opstring);

     if (defaultVectList  &&  strlen(defaultVectList) > 1) strcpy(Dy->previousVectList, defaultVectList);

      strcpy(Dy->lastOpString,opstring1);
    }


 /*--- else build the predefined user function operation string ---*/

 else if (strcmp(userFct,ADDVECT) == 0)
    {if (strstr(defaultVectList,"=") != NULL) {vectlist2 = strstr(defaultVectList,"=")+3; strncpy(opstring1,defaultVectList,vectlist2-defaultVectList);}
     else                                      vectlist2 = defaultVectList;
     length = strlen(vectlist2);
     ii = vectlist2 - defaultVectList;
     for (i=0; i<length-1; i++)
        {if (vectlist2[i] == ' ') {opstring1[ii] = '+'; ii++;}
         else                     {opstring1[ii] = vectlist2[i]; ii++;}
        }
     opstring1[ii] = '\0';
    }

 else if (strcmp(userFct,SUBVECT) == 0)
    {if (strstr(defaultVectList,"=") != NULL) {vectlist2 = strstr(defaultVectList,"=")+3; strncpy(opstring1,defaultVectList,vectlist2-defaultVectList);}
     else                                      vectlist2 = defaultVectList;
     length = strlen(vectlist2);
     ii = vectlist2 - defaultVectList;
     for (i=0; i<length-1; i++)
        {if (vectlist2[i] == ' ') {opstring1[ii] = '-'; ii++;}
         else                     {opstring1[ii] = vectlist2[i]; ii++;}
        }
     opstring1[ii] = '\0';
    }

 else if (strcmp(userFct,MULTVECT) == 0)
    {if (strstr(defaultVectList,"=") != NULL) {vectlist2 = strstr(defaultVectList,"=")+3; strncpy(opstring1,defaultVectList,vectlist2-defaultVectList);}
     else                                      vectlist2 = defaultVectList;
     length = strlen(vectlist2);
     ii = vectlist2 - defaultVectList;
     for (i=0; i<length-1; i++)
        {if (vectlist2[i] == ' ') {opstring1[ii] = '*'; ii++;}
         else                     {opstring1[ii] = vectlist2[i]; ii++;}
        }
     opstring1[ii] = '\0';
    }

 else if (strcmp(userFct,DIVVECT) == 0)
    {if (strstr(defaultVectList,"=") != NULL) {vectlist2 = strstr(defaultVectList,"=")+3; strncpy(opstring1,defaultVectList,vectlist2-defaultVectList);}
     else                                      vectlist2 = defaultVectList;
     length = strlen(vectlist2);
     ii = vectlist2 - defaultVectList;
     for (i=0; i<length-1; i++)
        {if (vectlist2[i] == ' ') {opstring1[ii] = '/'; ii++;}
         else                     {opstring1[ii] = vectlist2[i]; ii++;}
        }
     opstring1[ii] = '\0';
    }

 /*--- else build the user function operation string already used and added in the list ---*/

 else
    {RA_GetListOfProc();
     if (gProcList)
        {maxnumber = -1;
         for (i=0;i<gProcList->numProcess;i++)
            {
             strcpy(dummy,gProcList->name[i]);
             str3 = dummy+strlen(dummy)-1;
             while (atoi(str3) > 0 || *str3 == '0') str3--;
             number = atoi(str3+1);
             *(str3+1) = '\0';
             if (strcmp(dummy, userFct) == 0) {i0 = i; maxnumber = number;}
            }
         if (maxnumber >= 0) strcpy(opstring1,gProcList->opstring[i0]);
         DyOpStringReplaceChannels(opstring1, defaultVectList);
         opstring = fl_show_input("Modify this operation if needed:", opstring1);
         if (opstring == NULL) return;
         else                  strcpy(opstring1, opstring);
        }
    }

 /*--- Build the className ---*/

 if (strstr(opstring1,"=") == NULL)
    {strcpy(className,USEROP);}
 else
    {p  = strstr(opstring1,"="); p--;
     while (*p == ' ' && p >= opstring1) p--;
     p1 = opstring1;
     while (*p1 == ' ' && p1 < opstring1+strlen(opstring1)) p1++;
     strncpy(className,p1,p-p1+1);
     className[p-p1+1] = '\0';
     if (DyCheckString(className)) strcpy(className, DyCheckString(className));
    }

 /*--- To prevent problem coming from one instance of VViewerSelector class per user function --*/
 /*--- we add a numbering to className if it already exists in user functions list ---*/

 RA_GetListOfProc();

 if (gProcList)
    {
     maxnumber = -1;
     for (i=0;i<gProcList->numProcess;i++)
        {
         strcpy(dummy,gProcList->name[i]);
         str3 = dummy+strlen(dummy)-1;
         while (atoi(str3) > 0 || *str3 == '0') str3--;
         number = atoi(str3+1);
         *(str3+1) = '\0';
         if (strcmp(dummy, className) == 0) maxnumber = number;
        }
     if (maxnumber >= 0)
        {if (maxnumber == 0) strcat(className, "2");
         else  sprintf(className, "%s%d", className, maxnumber+1);
        }
    }

 /*--- Create the Operation String (opstring2 is the C code line corresponding to opstring) ---*/

 DyMakeOpCString(opstring1, opstring2, inputVect);

 if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>1)
    {printf("DySetOpString> className=%s \n",className);
     printf("DySetOpString> opstring1=%s \n",opstring1);
     printf("DySetOpString> opstring2=%s \n",opstring2);
     printf("DySetOpString> inputVect=%s \n",inputVect);
    }

 RA_MakeClass(className, opstring2, opstring1);
 DyLoadScript(className);

 /*--- Add the operation string result as a new proc data channel ---*/

 if (inputVect && strlen(inputVect)>1)
    {DyXformAddUserChannel (Dy->Frame, mainbrowser->proc, className, &inputVect[1]);}

 return;
}


/*------------------------------------------- DyMakeOpCString ---*/

void DyMakeOpCString ( char *opstring, char *opstring2, char *inputVect)
{
 int nvect;
 unsigned char c;
 char *p, *p2, *pstart, signalName[LSTRING_MAX];
 char dummy[LSTRING_MAX];

 sprintf(dummy,"DyMakeOpCString> opstring=%s", opstring);
 DyDebugPrint(dummy,0);

 p = opstring;
 if (strstr(opstring,"=") != NULL) {p = strstr(opstring,"="); p++;}
 p2     = opstring2; *p2='('; p2++; *p2='\0';
 pstart = NULL;
 nvect  = 0;

 if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>0) printf("DyMakeOpCString> step0> p=%s  opstring2=%s\n",p,opstring2);

 while (*p != 0)
   {
    if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>2) printf("DyMakeOpCString> step1> p=%s  opstring2=%s\n",p,opstring2);

    c = (unsigned char) *p;

    if (strchr (" \t~!\"'|\\<>=?^", c) != NULL)
       {
        if (pstart != NULL)
           {
            strncpy(signalName,pstart, p-pstart);
            signalName[p-pstart] = '\0';
            if (atof(signalName) == 0.  &&  c != '(' && strncmp(signalName,OPPAR,strlen(OPPAR)) != 0)  /* if this is really a signal name, not a constant or a parameter */
               {
                if (strcmp(signalName,"AND") == 0 || strcmp(signalName,"and") == 0)
                   {strcpy(p2,") & ("); p2 += 5;}
                else if (strcmp(signalName,"OR") == 0 || strcmp(signalName,"or") == 0)
                   {strcpy(p2,") | ("); p2 += 5;}
                else
                   {sprintf(dummy,"%s[%d]", VALUEIN, nvect);
                    strcpy(p2, dummy);
                    strcat(inputVect, OPSEP);
                    strcat(inputVect, signalName);
                    nvect++;
                    p2 += strlen(dummy);
                   }
               }
            else
               {strcpy(p2, signalName);
                p2+=strlen(signalName);
               }
            pstart = NULL;
           }
        else if (c != ' ')
           {*p2 = c; p2++;}
        if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>2) printf("DyMakeOpCString> step2> p=%s  opstring2=%s  inputVect=%s\n",p,opstring2,inputVect);

        p++; continue;
       }
 
    if (strchr ("+-*/(),", c) != NULL)
       {
        if (pstart != NULL)
           {strncpy(signalName,pstart, p-pstart);
            signalName[p-pstart] = '\0';
            /* if this is really a signal name, not a constant or a parameter */
            if (atof(signalName) == 0.  &&  c != '(' && strncmp(signalName,OPPAR,strlen(OPPAR)) != 0)
               {
                if (strcmp(signalName,"AND") == 0 || strcmp(signalName,"and") == 0)
                   {strcpy(p2,") & ("); p2 += 5;}
                else if (strcmp(signalName,"OR") == 0 || strcmp(signalName,"or") == 0)
                   {strcpy(p2,") | ("); p2 += 5;}
                else
                   {sprintf(dummy,"%s[%d]", VALUEIN, nvect);
                    strcpy(p2, dummy);
                    strcat(inputVect, OPSEP);
                    strcat(inputVect, signalName);
                    nvect++;
                    p2+=strlen(dummy);
                   }
               }
            else
               {strcpy(p2, signalName);
                p2+=strlen(signalName);
               }
            pstart = NULL;
           *p2 = c; p2++;
           } 
        else if (c != ' ')
           {*p2 = c; p2++;}
        p++;
        if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>2) printf("DyMakeOpCString> step3> p=%s  opstring2=%s  inputVect=%s\n",p,opstring2,inputVect);
        continue;
       }

    if (pstart == NULL) pstart = p;
    p++;
   }

 if (pstart != NULL)
    {strncpy(signalName,pstart, p-pstart);
     signalName[p-pstart] = '\0';
     if (atof(signalName) == 0. && strncmp(signalName,OPPAR,strlen(OPPAR)) != 0)  /* if this is really a signal name, not a constant or a parameter */
        {
         if (strcmp(signalName,"AND") == 0 || strcmp(signalName,"and") == 0)
            {strcpy(p2,") & ("); p2 += 5;}
         else if (strcmp(signalName,"OR") == 0 || strcmp(signalName,"or") == 0)
            {strcpy(p2,") | ("); p2 += 5;}
         else
            {sprintf(dummy,"%s[%d]", VALUEIN, nvect);
             strcpy(p2, dummy);
             strcat(inputVect, OPSEP);
             strcat(inputVect, signalName);
             nvect++;
             p2+=strlen(dummy);
            }
        }
     else
        {strcpy(p2, signalName);
         p2+=strlen(signalName);
        }
     pstart = NULL;
     if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>2) printf("DyMakeOpCString> step4> p=%s  opstring2=%s  inputVect=%s\n",p,opstring2,inputVect);
    }

 *p2=')'; p2++;
 *p2 = '\0';

 if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>0)
    {printf("DyMakeOpCString> last> opstring=%s  opstring2=%s  inputVect=%s\n",opstring ,opstring2, inputVect);}

 return;
}



/*------------------------------------------- DyOpStringReplaceChannels ---*/

char *DyOpStringReplaceChannels (char *opstring, char *vectlist)
{
 int i, nvect, nch;
 unsigned char c;
 char *p, *p2, *pstart, signalName[LSTRING_MAX];
 char dummy[LSTRING_MAX];
 char *chaine, *name, *vectlist2;
 char *channel[256];
 char opstring2[LBIGSTRING_MAX];

 if (vectlist == NULL) return(NULL);

 /*--- List the channels and put them into an array ---*/

 if (strstr(vectlist,"=") != NULL) vectlist2 = strstr(vectlist,"=")+1;
 else                              vectlist2 = vectlist;

 chaine = (char *)malloc(strlen(vectlist2)+1);
 strcpy(chaine,vectlist2);
 name = strtok(chaine," ");
 nch = 0;

 while (name != NULL)
    {channel[nch] = (char *)malloc(strlen(name)+1);
     strcpy(channel[nch],name);
     nch++;
     name = strtok(NULL,OPSEP);
    }
 free(chaine);
 
 /*--- Scan the opstring and replaced each channel found by channel of the array ---*/

 p = opstring;
 if (strstr(opstring,"=") != NULL) {p = strstr(opstring,"="); p++;}

 strncpy(opstring2,vectlist,p-opstring+1);
 p2 = opstring2; p2 += p-opstring+1;
 pstart = NULL;
 nvect  = 0;

 while (*p != 0)
   {
    c = (unsigned char) *p;

    if (strchr (" \t~!\"'|\\<>=?^", c) != NULL)
       {
        if (pstart != NULL)
           {
            strncpy(signalName,pstart, p-pstart);
            signalName[p-pstart] = '\0';
            if (atof(signalName) == 0.  &&  c != '(' && strncmp(signalName,OPPAR,strlen(OPPAR)) != 0)  /* if this is really a signal name, not a constant or a parameter */
               {
                if (strcmp(signalName,"AND") == 0 || strcmp(signalName,"and") == 0)
                   {strcpy(p2,"AND"); p2 += 3;}
                else if (strcmp(signalName,"OR") == 0 || strcmp(signalName,"or") == 0)
                   {strcpy(p2,"OR"); p2 += 2;}
                else
                   {strcpy(p2, channel[nvect]);
                    if (nvect < nch) p2+=strlen(channel[nvect]);
                    else             p2+=strlen(channel[nch-1]);
                    nvect++;
                   }
               }
            else
               {strcpy(p2, signalName);
                p2+=strlen(signalName);
               }
            pstart = NULL;
           }
        else if (c != ' ')
           {*p2 = c; p2++;}

        p++; continue;
       }
 
    if (strchr ("+-*/(),", c) != NULL)
       {
        if (pstart != NULL)
           {strncpy(signalName,pstart, p-pstart);
            signalName[p-pstart] = '\0';
            if (atof(signalName) == 0.  &&  c != '(' && strncmp(signalName,OPPAR,strlen(OPPAR)) != 0)  /* if this is really a signal name, not a constant or a parameter */
               {
                if (strcmp(signalName,"AND") == 0 || strcmp(signalName,"and") == 0)
                   {strcpy(p2,"AND"); p2 += 3;}
                else if (strcmp(signalName,"OR") == 0 || strcmp(signalName,"or") == 0)
                   {strcpy(p2,"OR"); p2 += 2;}
                else
                   {strcpy(p2, channel[nvect]);
                    if (nvect < nch) p2+=strlen(channel[nvect]);
                    else             p2+=strlen(channel[nch-1]);
                    nvect++;
                   }
               }
            else
               {strcpy(p2, signalName);
                p2+=strlen(signalName);
               }
            pstart = NULL;
           *p2 = c; p2++;
           } 
        else if (c != ' ')
           {*p2 = c; p2++;}
        p++;
        continue;
       }

    if (pstart == NULL) pstart = p;
    p++;
   }

 if (pstart != NULL)
    {strncpy(signalName,pstart, p-pstart);
     signalName[p-pstart] = '\0';
     if (atof(signalName) == 0. && strncmp(signalName,OPPAR,strlen(OPPAR)) != 0)
        {
         if (strcmp(signalName,"AND") == 0 || strcmp(signalName,"and") == 0)
            {strcpy(p2,"AND"); p2 += 3;}
         else if (strcmp(signalName,"OR") == 0 || strcmp(signalName,"or") == 0)
            {strcpy(p2,"OR"); p2 += 2;}
         else
            {strcpy(p2, channel[nvect]);
             if (nvect < nch) p2+=strlen(channel[nvect]);
             else             p2+=strlen(channel[nch-1]);
             nvect++;
            }
        }
     else
        {strcpy(p2, signalName);
         p2+=strlen(signalName);
        }
     pstart = NULL;
    }

 *p2 = '\0';

 strcpy(opstring, opstring2);

 for (i=0; i<nch; i++) free(channel[i]);

 return(opstring);
}


/*------------------------------------------- DySetOpPar ---*/

void DySetOpPar ()
{
 long i, i0, nvar;
 struct FrProcData *proc=NULL;
 const char *str, *paramString=NULL;
 char dummy[LSTRING_MAX], strnew[LBIGSTRING_MAX];
 FL_OBJECT *browser;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Loop over proc browser and add param to the selected UserOp variables ---*/

 browser = mainbrowser->proc;
 nvar = fl_get_browser_maxline(browser);

 for (i=0; i<nvar; i++)
    {if (fl_isselected_browser_line(browser,i+1))
        {str = fl_get_browser_line(browser,i+1);
         sscanf(str,"%s",dummy);
         if (strncmp(dummy,OPSIGNAL,strlen(OPSIGNAL)) == 0)
            {proc = FrProcDataFind(Dy->Frame,dummy); break;}
        }
     }

 if (proc != NULL)
    {if (proc->nAuxParam > 0)
        {paramString = fl_show_input("Write a list of parameters separated by underscores",
                                     (const char *)proc->auxParamNames[0]);
        }
     else
        {paramString = fl_show_input("Write a list of parameters separated by underscores",
                                     "2.17_0.014_7.23");
        }
    }

 if (paramString == NULL) return;

 for (i=0; i<nvar; i++)
    {
     if (fl_isselected_browser_line(browser,i+1))
        {
         str = fl_get_browser_line(browser,i+1);
         sscanf(str,"%s",dummy);
         if (strncmp(dummy,OPSIGNAL,strlen(OPSIGNAL)) != 0) continue;

         proc = FrProcDataFind(Dy->Frame,dummy);
         if (proc)
            {if (proc->auxParamNames == NULL) proc->auxParamNames = (char **)malloc(sizeof(char *));
             FrStrCpy(&(proc->auxParamNames[0]),(char *)paramString);
             proc->nAuxParam = 1;
            }
/*
         strcat(strnew,OPSEP);
         strcat(strnew,OPPAR);
         strcat(strnew,paramString);
         strcat(strnew,strstr(str," "));
         fl_replace_browser_line(browser,i+1,strnew);
*/
         }
     }

 return;
}


/*------------------------------------------ DyXformRemoveUserProcFiles ---*/

void DyXformRemoveUserProcFiles ()
{
/* Remove *.C , *.h and *.so files produced by RA_MakeClass (UserFct) */

 int i;
 char name[LSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);

 RA_GetListOfProc();

 if (gProcList)
    {
     for (i=0;i<gProcList->numProcess;i++)
        {
         if (strcmp(gProcList->name[i], ADDVECT) == 0)    continue;
         if (strcmp(gProcList->name[i], SUBVECT) == 0)    continue;
         if (strcmp(gProcList->name[i], MULTVECT) == 0)   continue;
         if (strcmp(gProcList->name[i], DIVVECT) == 0)    continue;
         if (strcmp(gProcList->name[i], INTEGRATOR) == 0) continue;
         if (strcmp(gProcList->name[i], LOWPASS) == 0)    continue;
         if (strcmp(gProcList->name[i], HIGHPASS) == 0)   continue;

         sprintf(name,"%s.C",gProcList->name[i]);
         remove(name);
         if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>1) printf("DyXformRemoveUserProcFiles> file %s removed \n", name);

         sprintf(name,"%s.h",gProcList->name[i]);
         remove(name);
         if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>1) printf("DyXformRemoveUserProcFiles> file %s removed \n", name);

         sprintf(name,"%s_C.so",gProcList->name[i]);
         remove(name);
         if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>1) printf("DyXformRemoveUserProcFiles> file %s removed \n", name);

         sprintf(name,"%s_C.d",gProcList->name[i]);
         remove(name);
         if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>1) printf("DyXformRemoveUserProcFiles> file %s removed \n", name);
        }
    }
 return;
}


/*------------------------------------------- DyXformSetTrigger ---*/
                                            /*  called from Main Browser  */
void DyXformSetTrigger (FL_OBJECT *obj, long arg)
{
 long i, nvar;
 const char *menuItem, *str, *opstring;
 char dummy[LSTRING_MAX], inputVect[LBIGSTRING_MAX];
 FL_OBJECT *browser;

 DyDebugPrint(__FUNCTION__,0);

 if (fl_get_menu(obj) > 0)
    {menuItem = fl_get_menu_text(obj);
     if (menuItem) Dy->trigMenuItem = my_strcpy(Dy->trigMenuItem,(char *)menuItem);
    }

 if (Dy->trigMenuItem == NULL)
    {DyInfoLine("Choose first a trigger source");
     return;
    }

 if (strcmp(Dy->trigMenuItem,TRIGCH) == 0)
    {
     DySetTriggerString();
     Dy->triggerType = TRIG_CH;
     Dy->triggerUsed = ON;
     fl_set_button(dyoptions->trigused,ON);
    }

 if (strcmp(Dy->trigMenuItem,TRIGASCII) == 0)
    {
     opstring = fl_show_input("Write here the name of the Ascii file that contains triggers GPS times", Dy->trigAsciiFileName);
     if (opstring == NULL) return;
     else                  strcpy(Dy->trigAsciiFileName, opstring);
     DySetTriggerList();
     Dy->triggerType = TRIG_ASCII;
     Dy->triggerUsed = ON;
     fl_set_button(dyoptions->trigused,ON);
     Dy->triggerPause == ON;
     fl_set_button(dyoptions->trigpause,ON);
    }

 if (strcmp(Dy->trigMenuItem,TRIGEVENTS) == 0)
    {
     opstring = fl_show_input("Write here the name of the Ascii file that contains triggers GPS times", Dy->trigEventName);
     if (opstring == NULL) return;
     else                  strcpy(Dy->trigEventName, opstring);
     DyMakeTrigEventList (Dy->trigEventName);
     Dy->triggerType = TRIG_EVENTS;
     Dy->triggerUsed = ON;
     fl_set_button(dyoptions->trigused,ON);
     Dy->triggerPause == ON;
     fl_set_button(dyoptions->trigpause,ON);
    }
 return;
}

/*------------------------------------------- DySetTriggerList ---*/

void DySetTriggerList()
{
 char *trigline, *word, *str;
 int i, triggps, nowgps;
 FILE *trigAsciiFile;

 DyDebugPrint(__FUNCTION__,0);

 trigAsciiFile = fopen(Dy->trigAsciiFileName,"r");
 if (trigAsciiFile == NULL)
    {printf("DySetTriggerList> trigger file %s does not exist.\n", Dy->trigAsciiFileName);
     Dy->triggerUsed == OFF;
     return;
    }

 trigline = (char *)malloc(LSTRING_MAX);

 Dy->nevent_ascii = 0;
 while (trigline && Dy->nevent_ascii < NEVENT_MAX-1)
    {
     trigline = fgets(trigline, LSTRING_MAX, trigAsciiFile);
     if (trigline)
        {word  = strtok(trigline," ");
         while (word != NULL)
            {triggps = atof(word);
             if (triggps == 0)
                {str = strstr(word,"=");
                 if (str) triggps = atof(str+1);
                }
             nowgps  = time(NULL) - FRGPSOFF + FRGPSDELTA + 3600;
             if (triggps > 70000000 && triggps < nowgps) word = NULL;
             else                                        word = strtok(NULL," ");
            }
         free(trigline);
         if (triggps > 70000000 && triggps < nowgps)
            {Dy->AsciiGps[Dy->nevent_ascii] = triggps; Dy->nevent_ascii++;}
        }
    }

 if (Dy->nevent_ascii == 0) Dy->triggerUsed == OFF;
 fclose(trigAsciiFile);

 if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>0)
    {for (i=0; i<100; i++) printf("DySetTriggerList> Event %d/%d : GPS=%f \n", i, Dy->nevent_ascii, Dy->AsciiGps[i]);}

 return;
}


/*------------------------------------------- DySetTriggerString ---*/

void DySetTriggerString()
{
 long i, ii, i0, length, nvar, number, maxnumber;
 const char *opstring, *str;
 char dummy[LSTRING_MAX], inputVect[LBIGSTRING_MAX], opstring1[LBIGSTRING_MAX], opstring2[LBIGSTRING_MAX];
 char *p, *p1, className[LSTRING_MAX], classNamei[LSTRING_MAX];
 char defaultVectList[LBIGSTRING_MAX], dummy2[LBIGSTRING_MAX];
 char *str1, *str2, *str3, *vectlist2;
 FL_OBJECT *browser;
  
 DyDebugPrint(__FUNCTION__,0);

 if (Dy->Frame == NULL) {printf("DySetTrigString> no frame in memory \n"); return;}

 strcpy(defaultVectList,"");
 strcpy(inputVect,"");
 strcpy(opstring2,"");

 /*--- build the vect list from selected items in the channels browsers ---*/

 if (fl_get_browser(mainbrowser->sms) != 0)
    {browser = mainbrowser->sms;
     nvar    = fl_get_browser_maxline(browser);
     for (i=0; i<nvar; i++)
        {if (fl_isselected_browser_line(browser,i+1))
            {str = fl_get_browser_line(browser,i+1);
             sscanf(str,"%s",dummy);
             strcat(defaultVectList," ");
             strcat(defaultVectList,dummy);
            }
        }
     }
 if (fl_get_browser(mainbrowser->adc) != 0)
    {browser = mainbrowser->adc;
     nvar    = fl_get_browser_maxline(browser);
     for (i=0; i<nvar; i++)
        {if (fl_isselected_browser_line(browser,i+1))
            {str = fl_get_browser_line(browser,i+1);
             sscanf(str,"%s",dummy);
             strcat(defaultVectList," ");
             strcat(defaultVectList,dummy);
            }
        }
     }
 if (fl_get_browser(mainbrowser->proc) != 0)
    {browser = mainbrowser->proc;
     nvar    = fl_get_browser_maxline(browser);
     for (i=0; i<nvar; i++)
        {if (fl_isselected_browser_line(browser,i+1))
            {str = fl_get_browser_line(browser,i+1);
             sscanf(str,"%s",dummy);
             if (strncmp(dummy,OPSIGNAL,strlen(OPSIGNAL)) != 0 && strncmp(dummy,TRIGGER,strlen(TRIGGER)) != 0)
                {strcat(defaultVectList," ");
                 strcat(defaultVectList,dummy);
                }
            }
        }
     }
 if (fl_get_browser(mainbrowser->sim) != 0)
    {browser = mainbrowser->sim;
     nvar    = fl_get_browser_maxline(browser);
     for (i=0; i<nvar; i++)
        {if (fl_isselected_browser_line(browser,i+1))
            {str = fl_get_browser_line(browser,i+1);
             sscanf(str,"%s",dummy);
             strcat(defaultVectList," ");
             strcat(defaultVectList,dummy);
            }
        }
     }

 strcat(defaultVectList," \0");

/**
 if (defaultVectList  &&  strlen(defaultVectList) > 1)
    {
     sprintf(dummy2, "%s_%s = %s", getenv("USER"), TRIGGER, defaultVectList);
     strcpy(defaultVectList, dummy2);
    }
**/

 if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>1)
    {printf("DySetTriggerString> defaultVectList=%s  previousVectList=%s  Dy->lastTrigString=%s\n",
             defaultVectList, Dy->previoustrigVectList, Dy->lastTrigString);
    }


 /*--- If Trigger is from Channels, put vect list in an input field presented to the user ---*/

 if (defaultVectList  &&  strlen(defaultVectList) > 1)
    {    
     if (strcmp(defaultVectList, Dy->previoustrigVectList) != 0)
        {opstring = fl_show_input("Write here the trigger condition. Example:", defaultVectList);}
     else
        {opstring = fl_show_input("Write here the trigger condition. Example:", Dy->lastTrigString);}
    }
 else
    {opstring = fl_show_input("Write here the trigger condition. Example:", Dy->lastTrigString);}

 if (opstring == NULL) return;
 else                  strcpy(opstring1, opstring);

 if (defaultVectList  &&  strlen(defaultVectList) > 1) strcpy(Dy->previoustrigVectList, defaultVectList);

 strcpy(Dy->lastTrigString,opstring1);

 /*--- Build the className ---*/

/*
 sprintf(className,"%s_%s", getenv("USER"), TRIGGER);
*/
 strcpy(className, TRIGGER);

 /*--- Create the Trigger C String (opstring2 is the set of C code lines corresponding to opstring1) ---*/

 DyMakeTrigEventList(opstring1);
 DyMakeTrigCString(opstring1, opstring2, inputVect);

 if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>1)
    {printf("SetTriggerString> className=%s \n",className);
     printf("SetTriggerString> opstring1=%s \n",opstring1);
     printf("SetTriggerString> opstring2=%s \n",opstring2);
     printf("SetTriggerString> inputVect=%s \n",inputVect);
    }

 RA_MakeClass(className, opstring2, opstring1);
 DyLoadScript(className);

 /*--- Add the operation string result as a new proc data channel ---*/

 if (inputVect && strlen(inputVect)>1)
    {DyXformAddUserChannel (Dy->Frame, mainbrowser->proc, className, &inputVect[1]);}

 Dy->triggerType = TRIG_CH;
 fl_set_button(dyoptions->trigused,ON);
 Dy->triggerUsed = ON;

 return;
}


/*------------------------------------------- DyMakeTrigCString ---*/

void DyMakeTrigCString ( char *opstring, char *opstring2, char *inputVect)
{
 int nvect;
 unsigned char c;
 char *p, *p2, *pstart, signalName[LSTRING_MAX];
 char dummy[LSTRING_MAX];

 sprintf(dummy,"DyMakeTrigCString> opstring=%s", opstring);
 DyDebugPrint(dummy,0);

 p      = opstring;
 p2     = opstring2; *p2='('; p2++; *p2='\0';
 pstart = NULL;
 nvect  = 0;

 while (*p != 0)
   {
    if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>2) printf("DyMakeTrigCString> step1> p=%s  opstring2=%s\n",p,opstring2);

    c = (unsigned char) *p;

    if (strchr (" \t~!\"'|\\<>=?^", c) != NULL)
       {
        if (pstart != NULL)
           {
            strncpy(signalName,pstart, p-pstart);
            signalName[p-pstart] = '\0';
            if (atof(signalName) == 0.  &&  strncmp(signalName,"0",1) != 0  &&  c != '(' ) /* if this is really a signal name, not a constant */
               {
                if (strcmp(signalName,"AND") == 0 || strcmp(signalName,"and") == 0)
                   {strcpy(p2,") & ("); p2 += 5;}
                else if (strcmp(signalName,"OR") == 0 || strcmp(signalName,"or") == 0)
                   {strcpy(p2,") | ("); p2 += 5;}
                else
                   {sprintf(dummy,"%s[%d]", VALUEIN, nvect);
                    strcpy(p2, dummy);
                    strcat(inputVect, OPSEP);
                    strcat(inputVect, signalName);
                    nvect++;
                    p2 += strlen(dummy);
                   }
               }
            else
               {strcpy(p2, signalName);
                p2+=strlen(signalName);
               }
            pstart = NULL;
           }
        else if (c != ' ')
           {*p2 = c; p2++;}
        if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>2) printf("DyMakeTrigCString> step2> p=%s  opstring2=%s  inputVect=%s\n",p,opstring2,inputVect);

        p++; continue;
       }
 
    if (strchr ("+-*/(),", c) != NULL)
       {
        if (pstart != NULL)
           {strncpy(signalName,pstart, p-pstart);
            signalName[p-pstart] = '\0';
            if (atof(signalName) == 0.  &&  strncmp(signalName,"0",1) != 0  &&  c != '(' ) /* if this is really a signal name, not a constant */
               {
                if (strcmp(signalName,"AND") == 0 || strcmp(signalName,"and") == 0)
                   {strcpy(p2,") & ("); p2 += 5;}
                else if (strcmp(signalName,"OR") == 0 || strcmp(signalName,"or") == 0)
                   {strcpy(p2,") | ("); p2 += 5;}
                else
                   {sprintf(dummy,"%s[%d]", VALUEIN, nvect);
                    strcpy(p2, dummy);
                    strcat(inputVect, OPSEP);
                    strcat(inputVect, signalName);
                    nvect++;
                    p2+=strlen(dummy);
                   }
               }
            else
               {strcpy(p2, signalName);
                p2+=strlen(signalName);
               }
            pstart = NULL;
           *p2 = c; p2++;
           } 
        else if (c != ' ')
           {*p2 = c; p2++;}
        p++;
        if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>2) printf("DyMakeTrigCString> step3> p=%s  opstring2=%s  inputVect=%s\n",p,opstring2,inputVect);
        continue;
       }

    if (pstart == NULL) pstart = p;
    p++;
   }

 if (pstart != NULL)
    {strncpy(signalName,pstart, p-pstart);
     signalName[p-pstart] = '\0';
     if (atof(signalName) == 0.  &&  strncmp(signalName,"0",1) != 0) /* if this is really a signal name, not a constant */
        {
         if (strcmp(signalName,"AND") == 0 || strcmp(signalName,"and") == 0)
            {strcpy(p2,") & ("); p2 += 5;}
         else if (strcmp(signalName,"OR") == 0 || strcmp(signalName,"or") == 0)
            {strcpy(p2,") | ("); p2 += 5;}
         else
            {sprintf(dummy,"%s[%d]", VALUEIN, nvect);
             strcpy(p2, dummy);
             strcat(inputVect, OPSEP);
             strcat(inputVect, signalName);
             nvect++;
             p2+=strlen(dummy);
            }
        }
     else
        {strcpy(p2, signalName);
         p2+=strlen(signalName);
        }
     pstart = NULL;
     if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>2) printf("DyMakeTrigCString> step4> p=%s  opstring2=%s  inputVect=%s\n",p,opstring2,inputVect);
    }

 *p2=')'; p2++; 
 *p2 = '\0';

 if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>0)
    {printf("DyMakeTrigCString> opstring=%s  opstring2=%s\n", opstring, opstring2);}

 return;
}


/*------------------------------------------- DyMakeTrigEventList ---*/

void DyMakeTrigEventList (char *opstring)
{
 char *chaine, *name, *name2, *name3, dummy[LSTRING_MAX];

 if (opstring == NULL) return;

 sprintf(dummy,"DyMakeTrigEventList> opstring=%s", opstring);
 DyDebugPrint(dummy,0);

 chaine = (char *)malloc(strlen(opstring)+1);
 strcpy(chaine, opstring);
 name = strtok(chaine," ");

 while (name != NULL)
    {if (strncmp(name,"EVENT",5) == 0)
        {name2 = strtok(NULL," ");
         if (strcmp(name2,">") == 0 || strcmp(name2,">=") == 0 || strcmp(name2,"=>") == 0)
            {name3 = strtok(NULL," ");
             if (name3)
                {strcpy(Dy->conditionName,name);
                 Dy->conditionTh = atof(name3);
                 if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>2)
                    {printf("DyMakeTrigEventList> opstring=%s chaine=%s  name=%s name2=%s name3=%s conditionName=%s conditionTh=%g \n",
                             opstring, chaine, name, name2, name3,  Dy->conditionName, Dy->conditionTh);
                    }
                 break;
                }
            }
        }
     name  = strtok(NULL," ");
    }
 free(chaine);

 return;
}

