/* ********************************************************************
********** URPlot.h ******************************** URPlot.h *********
********** URPlot.h ******************************** URPlot.h *********
******************************************************************** */
/* by X.Grave, F.Marion and B.Mours LAPP(Annecy) April 07, 1998 */
/* Modified on Nov 26 1997 by D.Castellazzi and D.Verkindt with option sound */
/* Modified on Jan 07 1998 by D.Verkindt to give argument reset to UHplotGrafIni */
/* Modified on April 06 1999 by D.Verkindt to update UHplotTime and UHplotFFT structures */

#if !defined(UHISTOLIB_DEFINED)
#define UHISTOLIB_DEFINED DONE

/*
#include "UJobL.h"
#include "USignalL.h"
*/
#include "UFrameL.h"
#include "dataDisplay.h"

#define NPLOTMAX 64
#define UHMAXDUMP 10
#define SOUND_CONVERT 1
#define SOUND_STOP 2
#define SOUND_WAIT 3
#define AUDIO_BUF_SIZE 1024


/*-------------------------------------- UHdummy -----------------------*/
struct UHdummy{ 
  long   iClock;                /* clock number                           */
  };


/*-------------------------------------- UHplotFFT -----------------------*/
struct UHplotFFT{ 
  long   iClock;                /* clock number                           */
  long   id;                    /*  histo id                              */
  char   *name;                 /* name used in ROOT                      */
  char   *title;                /* histo title                            */
  long   nBinP2;                /* histogram bin number (power of 2)      */
  long   resampFactor;          /* sampling freq / resampling freq        */
  long   t0;                    /* starting time (in clock count)         */
  struct USignal *input;        /* hold input parameters                  */
  char   *options;              /* plot options
                                   (must begin with intervalle value)     */
                                /* ------ end_of_SIO parameters ----------*/
  long   hertz;                 /* flag: >0 if spectrum in 1/Hz           */
  long   nodc;                  /* flag: >0 if spectrum computed without DC */
  long   nBin;                  /* histogram bin number                   */
  double dnu;                   /* frequency bin width                    */
  double dt;                    /* sampling period                        */
  double fmin;                  /* min frequency bin of the spectrum plot */
  double fmax;                  /* max frequency bin of the spectrum plot */
  long nAverage;                /* number of FFT performed                */
  long maxAverage;              /* number of FFT performed before reset   */
  long nRefresh;                /* number of FFT performed                */
  long maxRefresh;              /* number of FFT performed before refresh */
  double norm;                  /* normalization factor                   */
  struct UArrayCD  *vect;       /* vector                                 */
  struct UArrayD   *fft;        /* fft results                            */
  struct UArrayD   *window;     /* haning window                          */
  struct UFrUnpack *FrUnpack;   /* used if no input signal                */
  long   numPicture;            /* number to place the picture            */
  char plotOpt[30];             /* plot options                  */
  long   intervalle;            /* intervalle = number of clock tick 
                                   between two hplot call (if =< 0 the
                                   histo are just saved at the end        */
  int previousGPS;              /* GPS time of previous frame read        */
  long   autoScale;             /* flag for Y axis auto scaling           */
  double yMin;                  /* lower y edge of the plot               */
  double yMax;                  /* upper y edge of the plot               */
  double tmin;                  /* current starting time of time Window   */
  long realUnits;               /* flag to say that we plot in real Units */
  double slope;                 /* slope between units and ADC#           */
  double bias;                  /* bias  between units and ADC#           */

  double localMin;
  double localMax;
  double lastGoodData;

  double smsPeriod;
  double sumvalue;              /* accumulating input values              */
  double corrupt;               /* flag to indicate if last data used are corrupted */
  long   iFirst;                /* flag for first UHplotFTSim pass        */
  long   i;                     /* vector index                           */
  long   j;                     /* vector index                           */
  long   dNbin;                 /* nbr of bins to shift if refresh plot   */
  float  *buff;                 /* buffer for PACK/UNPACK of plot values  */
  int    nsame;                 /* number of plots superposed to this one */
  int    color;                 /* color of the plot                      */
  double decay;                 /* if >0, average is computed with a decay */
  char   *name_rms;             /* name of plot containing integrated rms */
  char   *title_rms;            /* title of plot containing integrated rms */
  long   rmsflag;               /* flag: >0 if rms superposed to spectrum */
  struct UHplotFFT *refplot;    /* pointer to the plot on which is superposed... */
  struct UHplotFFT *next;
  int hidden;
};


/*-------------------------------------- UHplotTime ----------------------*/
struct UHplotTime{ 
  long   iClock;                /* clock number                           */
  long   id;                    /* plot histo id                          */
  char   *name;                 /* name used in ROOT                      */
  char   *title;                /* histo title                            */
  long   nBin;                  /* histogram bin number                   */
  long   resampFactor;          /* sampling freq / resampling freq        */
  long   t0;                    /* starting time (in clock count)         */
  struct USignal *input;        /* hold input parameters                  */
  char   *options;              /* plot options
                                   (must begin with intervalle value)     */
                                /* ------ end_of_SIO parameters ----------*/
  double dt;                    /* sampling period                        */
  struct UFrUnpack *FrUnpack;   /* used if no input signal                */
  long numPicture;	        /* number to place the picture            */
  char plotOpt[30];             /* plot options                           */
  long   intervalle;            /* intervalle = number of clock tick 
                                   between two hplot call (if =< 0 the
                                   histo are just save at the end         */

  double smsPeriod;
  long   minmax_resample;
  double valueMin;              /* used if minmax_resample used           */
  double valueMax;              /* used if minmax_resample used           */
  long   nodc;                  /* flag>0 if time plot done without DC    */
  long realUnits;               /* flag to say that we plot in real Units */
  double slope;                 /* slope between units and ADC#           */
  double bias;                  /* bias  between units and ADC#           */
  double yoffset;               /* user's offset on Y axis                */
  double yscale;                /* user's scaling factor on Y axis        */

  double localMin;
  double localMax;

  double timeOffset;            /* timeOffset between startTime and frame beginning  */
  double tmin;                  /* current starting time of time Window   */
  int previousGPS;              /* GPS time of previous frame read        */
  long   autoScale;             /* flag for Y axis auto scaling           */
  double yMin;                  /* lower y edge of the plot               */
  double yMax;                  /* upper y edge of the plot               */

  double y[3];                  /* pass band filter reg. (y[0] = filtered current value) */
  double filterMin;             /* pass band filter fmin (at -3dB)        */
  double filterMax;             /* pass band filter fmax (at -3dB)        */
  double a1;                    /* pass band filter parameter             */
  double a2;                    /* pass band filter parameter             */
  double G;                     /* pass band filter parameter             */

  double dc;                    /* moving average for DC removal          */
  double sumvalue;              /* accumulating input values              */
  double corrupt;               /* flag to indicate if last data used are corrupted */
  long   iFirst;                /* flag for first UHplotTSim pass         */
  long   i;                     /* current bin number                     */
  long   j;                     /* vector index                           */
  long   dNbin;                 /* nbr of bins to shift if refresh plot   */
  long   buffEndFilled;         /* flag>0 if buffEnd was filled           */
  float *buffEnd;               /* buffer for PACK/UNPACK of last values  */
  float *buff;                  /* buffer for PACK/UNPACK of plot values  */
  float *buffWithDC;            /* buff before DC removal                 */
  int    nsame;                 /* number of plots superposed to this one */
  int    color;                 /* color of the plot                      */
  int    bits;                  /* >0 if bits of values are visualized    */
  struct UHplotTime *refplot;   /* pointer to the plot on which is superposed... */
  struct UHplotTime *next;
  int hidden;
};


/*------------------------------------- UHplot1D --------------------------*/
struct UHplot1D{
  long   iClock;                /* clock number                            */
  long   id;                    /* histo id                                */
  char   *name;                 /* name used in ROOT                       */
  char   *title;                /* histo title                             */
  long   nBin;                  /* histogram bin number                    */
  double xMin;                  /* minimum value of input                  */
  double xMax;                  /* maximum value of input                  */
  struct USignal *input;        /* hold input parameters                   */
  long   t0;                    /* starting time (in clock coun t)         */
  long   tMax;                  /* ending time                             */
  char   *options;              /* plot options
                                   (must begin with intervalle value)      */
                                /* The AUTO option provide an automatic 
                                   definition of the histogram limits. In
                                   this case nBin should be an even number */
                                /* ------- end_of_SIO parameters ----------*/
  double dt;                    /* sampling period                         */
  struct UFrUnpack *FrUnpack;   /* used if no input signal                 */
  long numPicture;              /* number to place the picture             */
  char plotOpt[30];             /* for plot (plot options)                 */
  long   intervalle;            /* intervalle = number of clock tick 
                                   between two hplot call (if =< 0 the
                                   histo are just save at the end          */
  long   resampFactor;          /* sampling freq / resampling freq         */

  double smsPeriod;
  char units[256];              /* copy of the FrAdcData units             */
  long realUnits;               /* flag to say that we plot in real Units  */
  long   iFirst;                /* flag for first UHplot1DSim pass         */
  double slope;                 /* slope between units and ADC#            */
  double bias;                  /* bias  between units and ADC#            */
  double maxofMax;              /* max of maxima on Y axis                 */
  double maxofxMax;             /* max of maxima on X axis                 */
  double minofxMin;             /* min of minima on X axis                 */
  double sumvalue;              /* accumulating input values               */
  double corrupt;               /* flag to indicate if last data used are corrupted */
  long   j;                     /* vector index                            */
  BOOL   adjust;                /* tell if the AUTO option is used         */
  double firstValue;            /* used by the auto option                 */
  double nFirstValue;           /* used by the auto option                 */
  double nData;                 /* used by the auto option                 */ 
  double *data;                 /* histogram buffer in double precision    */
  float  *buff;                 /* histogram buffer (float)                */
  int    nsame;                 /* number of plots superposed to this one  */
  int    color;                 /* color of the plot                       */
  struct UHplot1D *refplot;     /* pointer to the plot on which is superposed... */
  struct UHplot1D *next;
  int hidden;
};


/*------------------------------------- UHplot2D --------------------------*/
struct UHplot2D{
  long   iClock;                /* clock number                            */
  long   id;                    /*  histo id                               */
  char   *title;                /* histo title                             */
  char   *name;                 /* name used in ROOT                       */
  long   nxBin;                 /* histogram bin number for x              */
  double xMin;                  /* lower edge of first x channel           */
  double xMax;                  /* upper edge of first x channel           */
  long   nyBin;                 /* histogram bin number for y              */
  double yMin;                  /* lower edge of first x channel           */
  double yMax;                  /* upper edge of first x channel           */
  struct USignal *xInput;       /* hold x input data                       */
  struct USignal *yInput;       /* hold y input data                       */
  long   t0;                    /* starting time (in clock count)          */
  long   tMax;    	        /* ending time                             */
  char   *options;              /* plot options
                                   (must begin with intervalle value)      */
                                /* ------- end_of_SIO parameters ----------*/
  double dt;                    /* sampling period                         */
  struct UFrUnpack *FrUnpackx;  /* used if no input signal                 */
  struct UFrUnpack *FrUnpacky;  /* used if no input signal                 */
  long numPicture; 	        /* number to place the picture             */
  char plotOpt[30];             /* HPLOPT, plot options                    */
  long   intervalle;            /* intervalle = number of clock tick 
                                   between two hplot call (if =< 0 the
                                   histo are just saved at the end         */
  double previousValue;         /* value nSampling before                  */
  long nSampling;               /* nSampling shift between x and y channels */
  long iSampling;               /* counter to wait nSampling               */
  long   iFirst;                /* flag for first UHplot2DSim pass         */
  long realUnits;               /* flag to say that we plot in real Units  */
  double xslope;                /* x slope between units and ADC#          */
  double xbias;                 /* x bias  between units and ADC#          */
  double yslope;                /* y slope between units and ADC#          */
  double ybias;                 /* y bias  between units and ADC#          */
  double corrupt;               /* flag to indicate if last data used are corrupted */
  BOOL   adjust;                /* tell if the AUTO option is used         */
  struct UHplot2D *next;
  int hidden;
  };


/*------------------------------------- UHplot3D --------------------------*/
struct UHplot3D{
  long   iClock;                /* clock number                            */
  long   id;                    /*  histo id                               */
  char   *title;                /* histo title                             */
  char   *name;                 /* name used in ROOT                       */
  long   nxBin;                 /* histogram bin number for x              */
  double xMin;                  /* lower edge of first x channel           */
  double xMax;                  /* upper edge of first x channel           */
  long   nyBin;                 /* histogram bin number for y              */
  double yMin;                  /* lower edge of first x channel           */
  double yMax;                  /* upper edge of first x channel           */
  struct USignal *xInput;       /* hold x input data                       */
  struct USignal *yInput;       /* hold y input data                       */
  struct USignal *zInput;       /* hold z input data                       */
  long   t0;                    /* starting time (in clock count)          */
  long   tMax;    	        /* ending time                             */
  char   *options;              /* plot options
                                   (must begin with intervalle value)      */
                                /* ------- end_of_SIO parameters ----------*/
  double dt;                    /* sampling period                         */
  struct UFrUnpack *FrUnpackx;  /* used if no input signal                 */
  struct UFrUnpack *FrUnpacky;  /* used if no input signal                 */
  struct UFrUnpack *FrUnpackz;  /* used if no input signal                 */
  long numPicture; 	        /* number to place the picture             */
  char plotOpt[30];             /* HPLOPT, plot options                    */
  long   intervalle;            /* intervalle = number of clock tick 
                                   between two hplot call (if =< 0 the
                                   histo are just saved at the end         */
  double previousValue;         /* value nSampling before                  */
  long nSampling;               /* nSampling shift between x and y channels */
  long iSampling;               /* counter to wait nSampling               */
  long   iFirst;                /* flag for first UHplot3DSim pass         */
  long realUnits;               /* flag to say that we plot in real Units  */
  double xslope;                /* x slope between units and ADC#          */
  double xbias;                 /* x bias  between units and ADC#          */
  double yslope;                /* y slope between units and ADC#          */
  double ybias;                 /* y bias  between units and ADC#          */
  double zslope;                /* z slope between units and ADC#          */
  double zbias;                 /* z bias  between units and ADC#          */
  double corrupt;               /* flag to indicate if last data used are corrupted */
  BOOL   adjust;                /* tell if the AUTO option is used         */
  struct UHplot3D *next;
  int hidden;
  };


/*-------------------------------------- UHpTfct --------------------------*/
struct UHpTfct{ 
  long   iClock;                /* clock number                            */
  long   idModule;              /*  histo id                               */
  long   idPhase;               /*  histo id                               */
  char   *nameModule;           /* name used in ROOT                       */
  char   *namePhase;            /* name used in ROOT                       */
  char   *title;                /* histo title                             */
  long   nBinP2;                /* histogram bin number (power of 2)       */
  long   resampFactor;          /* sampling freq / resampling freq         */
  long   t0;                    /* starting time (in clock count)          */
  struct USignal *input1;       /* hold input parameters                   */
  struct USignal *input2;       /* hold input parameters                   */
  char   *option1;              /* plot options
                                  (must begin with intervalle value)       */
  char   *option2;              /* plot options
                                  (must begin with intervalle value)       */
                                /* ------- end_of_SIO parameters ----------*/
  double tmin;                  /* current starting time of time Window    */
  int previousGPS;              /* GPS time of previous frame read         */
  long   autoScale;             /* flag for Y axis auto scaling            */
  double yMin;                  /* lower y edge of the plot               */
  double yMax;                  /* upper y edge of the plot               */
  long   nBin;                  /* histogram bin number                    */
  double dnu;                   /* frequency bin width                     */
  double dt;                    /* sampling period                         */
  double fmin;                  /* min frequency bin of the plots          */
  double fmax;                  /* max frequency bin of the plots          */
  long   iFirst;                /* flag for first UHpTfctSim pass          */
  long   i;                     /* vector index                            */
  long   j;                     /* vector index                            */
  long nAverage;                /* number of FFT performed                 */
  long maxAverage;              /* number of FFT performed before reset    */
  long nRefresh;                /* number of FFT performed                 */
  long maxRefresh;              /* number of FFT performed before refresh  */
  struct UArrayCD *vect1;       /* vector                                  */
  struct UArrayCD *vect2;       /* vector                                  */
  struct UArrayD *module;       /* fft results                             */
  struct UArrayD *phase;        /* fft results                             */
  struct UArrayD *rer;          /* fft results                             */
  struct UArrayD *imr;          /* fft results                             */
  struct UArrayD *mod1sq;       /* fft results                             */
  struct UArrayD *window;       /* haning window                           */
  struct UFrUnpack *FrUnpack1;  /* used if no input signal                 */
  struct UFrUnpack *FrUnpack2;  /* used if no input signal                 */
  long   numPicture;            /* number to place the picture             */
  char plotOpt1[30];            /* for HPLOPT (plot options)               */
  char plotOpt2[30];            /* for HPLOPT (plot options)               */
  long   intervalle;            /* intervalle = number of clock tick 
                                   between two hplot call (if =< 0 the
                                   histo are just save at the end          */
  long realUnits;               /* flag to say that we plot in real Units  */
  double slope1;                /* channel 1: slope between units and ADC# */
  double bias1;                 /* channel 1: bias  between units and ADC# */
  double slope2;                /* channel 2: slope between units and ADC# */
  double bias2;                 /* channel 2: bias  between units and ADC# */
  double sumvalue1;             /* accumulating input values1              */
  double sumvalue2;             /* accumulating input values2              */
  double corrupt;               /* flag to indicate if last data used are corrupted */
  long   dNbin;                 /* nbr of bins to shift if refresh plot    */
  float  *buff1;                /* buffer for PACK/UNPACK of plot values1  */
  float  *buff2;                /* buffer for PACK/UNPACK of plot values2  */
  int    nsame;                 /* number of plots superposed to this one  */
  int    color;                 /* color of the plot                      */
  double decay;                 /* if >0, average is computed with a decay */
  struct UHpTfct *refplot;      /* pointer to the plot on which is superposed... */
  struct UHpTfct *next;
  int hidden;
};


/*-------------------------------------- UHpCohe --------------------------*/
struct UHpCohe{ 
  long   iClock;                /* clock number                            */
  long   idModule;              /*  histo id                               */
  long   idPhase;               /*  histo id                               */
  char   *nameModule;           /* name used in ROOT                       */
  char   *namePhase;            /* name used in ROOT                       */
  char   *title;                /* histo title                             */
  long   nBinP2;                /* histogram bin number (power of 2)       */
  long   resampFactor;          /* sampling freq / resampling freq         */
  long   t0;                    /* starting time (in clock count)          */
  struct USignal *input1;       /* hold input parameters                   */
  struct USignal *input2;       /* hold input parameters                   */
  char   *option1;              /* plot options
                                  (must begin with intervalle value)       */
  char   *option2;              /* plot options
                                  (must begin with intervalle value)       */
                                /* ------- end_of_SIO parameters ----------*/
  double tmin;                  /* current starting time of time Window    */
  int previousGPS;              /* GPS time of previous frame read         */
  long   autoScale;             /* flag for Y axis auto scaling            */
  int    modulesqrt;            /* flag to use sqrt in computing module    */
  double yMin;                  /* lower y edge of the plot               */
  double yMax;                  /* upper y edge of the plot               */
  long   nBin;                  /* histogram bin number                    */
  double dnu;                   /* frequency bin width                     */
  double dt;                    /* sampling period                         */
  double fmin;                  /* min frequency bin of the plot          */
  double fmax;                  /* max frequency bin of the plot          */
  double crossCohe_fmin;        /* min frequency bin of the 2D crossCoherence plot */
  double crossCohe_fmax;        /* max frequency bin of the 2D crossCoherence plot */
  long   iFirst;                /* flag for first UHpCoheSim pass          */
  long   i;                     /* vector index                            */
  long   j;                     /* vector index                            */
  long maxAverage;              /* number of FFT performed before reset    */
  long nAverage;                /* number of FFT performed                 */
  long nRefresh;                /* number of FFT performed                 */
  long maxRefresh;              /* number of FFT performed before refresh  */
  struct UArrayCD *vect1;       /* vector                                  */
  struct UArrayCD *vect2;       /* vector                                  */
  struct UArrayD *module;       /* fft results                             */
  struct UArrayD *phase;        /* fft results                             */
  struct UArrayD *rer;          /* fft results                             */
  struct UArrayD *imr;          /* fft results                             */
  struct UArrayD *mod1sq;       /* fft results                             */
  struct UArrayD *mod2sq;       /* fft results                             */
  struct UArrayD *window;       /* haning window                           */

  struct UArrayD *module2D;     /* fft results for crosscoherence          */
  struct UArrayD *phase2D;      /* fft results for crosscoherence          */
  struct UArrayD *rer2D;        /* fft results for crosscoherence          */
  struct UArrayD *imr2D;        /* fft results for crosscoherence          */
  struct UArrayD *mod1sq2D;     /* fft results for crosscoherence          */
  struct UArrayD *mod2sq2D;     /* fft results for crosscoherence          */

  struct UFrUnpack *FrUnpack1;  /* used if no input signal                 */
  struct UFrUnpack *FrUnpack2;  /* used if no input signal                 */
  long   numPicture;            /* number to place the picture             */
  char plotOpt1[30];            /* for HPLOPT (plot options)               */
  char plotOpt2[30];            /* for HPLOPT (plot options)               */
  long   intervalle;            /* intervalle = number of clock tick 
                                   between two hplot call (if =< 0 the
                                   histo are just save at the end          */
  long realUnits;               /* flag to say that we plot in real Units  */
  double slope1;                /* channel 1: slope between units and ADC# */
  double bias1;                 /* channel 1: bias  between units and ADC# */
  double slope2;                /* channel 2: slope between units and ADC# */
  double bias2;                 /* channel 2: bias  between units and ADC# */
  double sumvalue1;             /* accumulating input values1              */
  double sumvalue2;             /* accumulating input values2              */
  double corrupt;               /* flag to indicate if last data used are corrupted */
  long   dNbin;                 /* nbr of bins to shift if refresh plot    */
  float  *buff1;                /* buffer for PACK/UNPACK of plot values1  */
  float  *buff2;                /* buffer for PACK/UNPACK of plot values2  */
  int    nsame;                 /* number of plots superposed to this one  */
  int    color;                 /* color of the plot                      */
  double decay;                 /* if >0, average is computed with a decay */
  struct UHpCohe *refplot;      /* pointer to the plot on which is superposed... */
  struct UHpCohe *next;
  int hidden;
};


/*-------------------------------------- UHpCorr --------------------------*/
struct UHpCorr{ 
  long   iClock;                 /* clock number                           */
  long   id;                     /*  histo id                         */
  char   *name;                  /* name used in ROOT                      */
  char   *title;                 /* histo title                            */
  long   nBinP2;                 /* histogram bin number (power of 2)      */
  long   resampFactor;           /* sampling freq / resampling freq        */
  long   t0;                     /* starting time (in clock count)         */
  struct USignal *input1;        /* hold input parameters                  */
  struct USignal *input2;        /* hold input parameters                  */
  char   *options;               /* this contains the plot options. 
                                    Must start with the intervalle value   */
                                 /* ------ end_of_SIO parameters ----------*/
  double timeOffset;             /* Offset startTime - frame start         */
  long   nBin;                   /* time window of the plot (bin number)   */
  long   dNbin;                  /* number of bins on which ffts are done  */
  long   dNbinP2;                /* number of bins (power of 2)            */
  double dnu;                    /* frequency bin width                    */
  double dt;                     /* sampling period                        */
  double tmin;                   /* starting time                          */
  int previousGPS;               /* GPS time of previous frame read        */
  long   autoScale;              /* flag for Y axis auto scaling           */
  double yMin;                   /* lower edge on corr. output value       */
  double yMax;                   /* upper edge on corr. output value       */
  long   iFirst;                /* flag for first UHpCorrSim pass          */
  long   i;                      /* current bin number                     */
  long   i2;                     /* vector index                           */
  long   j;                      /* vector index                           */
  struct UArrayCD *vect1;        /* vector to store first input signal     */
  struct UArrayCD *vect2;        /* vector to store second input signal    */
  struct UArrayD   *window;      /* hanning window                         */
  struct UFrUnpack *FrUnpack1;   /* used if no input signal                */
  struct UFrUnpack *FrUnpack2;   /* used if no input signal                */
  long   numPicture;             /* hplot picture number                   */
  char plotOpt[30];              /* for plot (plot options)                */
  long   intervalle;             /* intervalle = number of clock tick 
                                    between two hplot call (if =< 0 the
                                    histo are just saved at the end)       */
  long realUnits;                /* flag to say that we plot in real Units */
  double slope;                  /* slope between units and ADC#           */
  double bias;                   /* bias  between units and ADC#           */
  int    zoomXminbin;            /* if zoom in x interactively, min bin    */
  int    zoomXmaxbin;            /* if zoom in x interactively, max bin    */
  double sumvalue1;              /* accumulating input values1             */
  double sumvalue2;              /* accumulating input values2             */
  double corrupt;               /* flag to indicate if last data used are corrupted */
  float  *buff1;                 /* buffer for PACK/UNPACK of plot values1 */
  float  *buff2;                 /* buffer for PACK/UNPACK of plot values2 */
  float  *buff;                  /* buffer for PACK/UNPACK of output values */
  int    nsame;                  /* number of plots superposed to this one */
  int    color;                  /* color of the plot                      */
  struct UHpCorr *refplot;       /* pointer to the plot on which is superposed... */
  struct UHpCorr *next;
  int hidden;
};


/*-------------------------------------- UHsound --------------------------*/
struct UHsound {
  char   *name;                 /* name for each instance                 */
  long   iClock;                /* clock number                           */
  struct USignal *in_signal;    /* signal to hear                         */
  long   beginTime;             /* beginning time of sounding (sec.)      */
  long   totTime;               /* total duration time of sounding (sec.) */
  char   *options;              /* this contains the plot options. 
                                   Must start with the intervalle value   */
                                /* ------ end_of_SIO parameters ----------*/
  long   iFirst;                /* flag for first UHsoundSim pass         */
  long   i;                     /* counter of sample written in file      */
  short  previousShort;         /* outshort computed at previous time step*/
  double max_value;             /* used to normalize to 1                 */
  double min_value;             /* used to set to 0                       */
  double Fsamp;                 /* real ADC sampling freq.                */
  int resampFactor;             /* interpolation factor when Fsamp<5512   */
  int encoding;                 /* signal linear encoding 8 or 16 bits    */
  int soundonline;              /* flag > 0 if online audio stream active */
  char outfileName[1024];       /* Audio output file name                 */
  long data_nBytes;             /* Audio output file data length          */
  long  begin_nb;               /* Audio output file begin sampling nbr   */
  long  end_nb;                 /* Audio output file end sampling nbr     */
  unsigned char bufstream[AUDIO_BUF_SIZE];   /* buffer for audio output stream 8 bits  */
  unsigned short bufstreamS[AUDIO_BUF_SIZE]; /* buffer for audio output stream 16 bits */
  int nsamp;                    /* counter to fill the audio device buffer*/
  int fd_audio;                 /* file descriptor for output audio device*/
  FILE  *fout;                  /* output file                            */
  int vecttype;                  /* type of data read (short, int, float...) */
  int hidden;
};


/*------------------------------------ UHplotFFTTime ---------------------*/
struct UHplotFFTTime{ 
  long   iClock;                /* clock number                           */
  long   id;                    /*  histo id                              */
  char   *name;                 /* name used in ROOT                      */
  char   *title;                /* histo title                            */
  long   nBinP2;                /* Y bin number (power of 2)              */
  double timeSize;              /* X time axis size (sec)                 */
  long   resampFactor;          /* sampling freq / resampling freq        */
  long   t0;                    /* starting time (in clock count)         */
  struct USignal *input;        /* hold input parameters                  */
  char   *options;              /* plot options
                                   (must begin with intervalle value)     */
                                /* ------ end_of_SIO parameters ----------*/
  long   nodc;                  /* flag>0 if spectrum computed without DC */
  long   nBin;                  /* histogram bin number                   */
  double dnu;                   /* frequency bin width                    */
  double dt;                    /* sampling period                        */
  double fmin;                  /* min frequency bin of the spectrum plot */
  double fmax;                  /* max frequency bin of the spectrum plot */
  double zmin;                  /* min value on z axis (for colors scale) */
  double zmax;                  /* max value on z axis (for colors scale) */
  long autoScale;               /* flag>0 if z axis is auto-scaled        */
  long nAverage;                /* number of FFT performed                */
  long maxAverage;              /* number of FFT performed before reset   */
  double norm;                  /* normalization factor                   */
  struct UArrayCD  *vect;       /* vector                                 */
  struct UArrayD   *fft;        /* fft results                            */
  struct UArrayD   *window;     /* hanning window                         */
  struct UFrUnpack *FrUnpack;   /* used if no input signal                */
  long   numPicture;            /* number to place the picture            */
  char plotOpt[30];             /* plot options                           */
  long   intervalle;            /* intervalle = number of clock tick 
                                   between two hplot call (if =< 0 the
                                   histo are just saved at the end        */
  double currentTimeUTC;        /* current UTC time of start of plot      */
  double timeOffset;            /* Offset startTime - frame start         */
  double tmin;                  /* current starting time of time Window   */
  int previousGPS;              /* GPS time of previous frame read        */
  long realUnits;               /* flag to say that we plot in real Units */
  double slope;                 /* slope between units and ADC#           */
  double bias;                  /* bias  between units and ADC#           */

  double sumvalue;              /* accumulating input values              */
  double corrupt;               /* flag to indicate if last data used are corrupted */
  long   iFirst;                /* flag for first UHplotFTSim pass        */
  long   i;                     /* vector index                           */
  long   j;                     /* vector index                           */
  long   itime;                 /* index for X (time axis)                */
  long   itimeView;             /* index for plot refresh                 */
  long   nBinTime;              /* X (time axis) bin number               */
  long   dNbin;                 /* nbr of bins to shift if refresh plot   */
  double refresh_percent;       /* refresh period of plot (% of timeSize) */
  long   refresh_nBin;          /* refresh period of plot (nbr of x bins) */
  float  *buff;                 /* buffer for passing data to FFT         */
  float  *buffTot;              /* buffer for PACK/UNPACK of plot values  */
  int hidden;
};


/*-------------------------------------- UHpCoheTime ---------------------*/
struct UHpCoheTime{ 
  long   iClock;                /* clock number                           */
  long   id;                    /*  histo id                              */
  char   *name;                 /* name used in ROOT                      */
  char   *title;                /* histo title                            */
  long   nBinP2;                /* Y bin number (power of 2)              */
  double timeSize;              /* X time axis size (sec)                 */
  long   resampFactor;          /* sampling freq / resampling freq        */
  long   t0;                    /* starting time (in clock count)         */
  struct USignal *input1;       /* hold input parameters                  */
  struct USignal *input2;       /* hold input parameters                  */
  char   *options;              /* plot options
                                   (must begin with intervalle value)     */
                                /* ------ end_of_SIO parameters ----------*/
  long   nodc;                  /* flag>0 if spectrum computed without DC */
  long   nBin;                  /* histogram bin number                   */
  double dnu;                   /* frequency bin width                    */
  double dt;                    /* sampling period                        */
  double fmin;                  /* min frequency bin of the spectrum plot */
  double fmax;                  /* max frequency bin of the spectrum plot */
  double zmin;                  /* min value on z axis (for colors scale) */
  double zmax;                  /* max value on z axis (for colors scale) */
  long nAverage;                /* number of FFT performed                */
  long maxAverage;              /* number of FFT performed before reset   */
  double norm;                  /* normalization factor                   */
  struct UArrayCD *vect1;       /* vector                                 */
  struct UArrayCD *vect2;       /* vector                                 */
  struct UArrayD  *window;      /* hanning window                         */
  struct UArrayD *module;       /* fft results                            */
  struct UArrayD *phase;        /* fft results                            */
  struct UArrayD *rer;          /* fft results                            */
  struct UArrayD *imr;          /* fft results                            */
  struct UArrayD *mod1sq;       /* fft results                            */
  struct UArrayD *mod2sq;       /* fft results                            */
  struct UFrUnpack *FrUnpack1;  /* used if no input signal                */
  struct UFrUnpack *FrUnpack2;  /* used if no input signal                */
  long   numPicture;            /* number to place the picture            */
  char plotOpt[30];             /* plot options                           */
  long   intervalle;            /* intervalle = number of clock tick 
                                   between two hplot call (if =< 0 the
                                   histo are just saved at the end        */
  double currentTimeUTC;        /* current UTC time of start of plot      */
  double timeOffset;            /* Offset startTime - frame start         */
  double tmin;                  /* current starting time of time Window   */
  int previousGPS;              /* GPS time of previous frame read        */
  long realUnits;               /* flag to say that we plot in real Units */
  double slope1;                /* slope between units and ADC#           */
  double bias1;                 /* bias  between units and ADC#           */
  double slope2;                /* slope between units and ADC#           */
  double bias2;                 /* bias  between units and ADC#           */

  double sumvalue1;             /* accumulating input values              */
  double sumvalue2;             /* accumulating input values              */
  double corrupt;               /* flag to indicate if last data used are corrupted */
  long   iFirst;                /* flag for first UHplotFTSim pass        */
  long   i;                     /* vector index                           */
  long   j;                     /* vector index                           */
  long   itime;                 /* index for X (time axis)                */
  long   itimeView;             /* index for plot refresh                 */
  long   nBinTime;              /* X (time axis) bin number               */
  long   dNbin;                 /* nbr of bins to shift if refresh plot   */
  double refresh_percent;       /* refresh period of plot (% of timeSize) */
  long   refresh_nBin;          /* refresh period of plot (nbr of x bins) */
  float  *buff1;                /* buffer for passing data to FFT         */
  float  *buff2;                /* buffer for passing data to FFT         */
  float  *buffTot;              /* buffer for PACK/UNPACK of plot values  */
  int hidden;
};


/*-------------------------------------- UHpTfctTime ---------------------*/
struct UHpTfctTime{ 
  long   iClock;                /* clock number                           */
  long   id;                    /*  histo id                              */
  char   *name;                 /* name used in ROOT                      */
  char   *title;                /* histo title                            */
  long   nBinP2;                /* Y bin number (power of 2)              */
  double timeSize;              /* X time axis size (sec)                 */
  long   resampFactor;          /* sampling freq / resampling freq        */
  long   t0;                    /* starting time (in clock count)         */
  struct USignal *input1;       /* hold input parameters                  */
  struct USignal *input2;       /* hold input parameters                  */
  char   *options;              /* plot options
                                   (must begin with intervalle value)     */
                                /* ------ end_of_SIO parameters ----------*/
  long   nodc;                  /* flag>0 if spectrum computed without DC */
  long   nBin;                  /* histogram bin number                   */
  double dnu;                   /* frequency bin width                    */
  double dt;                    /* sampling period                        */
  double fmin;                  /* min frequency bin of the spectrum plot */
  double fmax;                  /* max frequency bin of the spectrum plot */
  double zmin;                  /* min value on z axis (for colors scale) */
  double zmax;                  /* max value on z axis (for colors scale) */
  long nAverage;                /* number of FFT performed                */
  long maxAverage;              /* number of FFT performed before reset   */
  double norm;                  /* normalization factor                   */
  struct UArrayD  *window;      /* hanning window                         */
  struct UArrayCD *vect1;       /* vector                                 */
  struct UArrayCD *vect2;       /* vector                                 */
  struct UArrayD *module;       /* fft results                            */
  struct UArrayD *phase;        /* fft results                            */
  struct UArrayD *rer;          /* fft results                            */
  struct UArrayD *imr;          /* fft results                            */
  struct UArrayD *mod1sq;       /* fft results                            */
  struct UFrUnpack *FrUnpack1;  /* used if no input signal                */
  struct UFrUnpack *FrUnpack2;  /* used if no input signal                */
  long   numPicture;            /* number to place the picture            */
  char plotOpt[30];             /* plot options                           */
  long   intervalle;            /* intervalle = number of clock tick 
                                   between two hplot call (if =< 0 the
                                   histo are just saved at the end        */
  double currentTimeUTC;        /* current UTC time of start of plot      */
  double timeOffset;            /* Offset startTime - frame start         */
  double tmin;                  /* current starting time of time Window   */
  int previousGPS;              /* GPS time of previous frame read        */
  long realUnits;               /* flag to say that we plot in real Units */
  double slope1;                /* slope between units and ADC#           */
  double bias1;                 /* bias  between units and ADC#           */
  double slope2;                /* slope between units and ADC#           */
  double bias2;                 /* bias  between units and ADC#           */

  double sumvalue1;             /* accumulating input values              */
  double sumvalue2;             /* accumulating input values              */
  double corrupt;               /* flag to indicate if last data used are corrupted */
  long   iFirst;                /* flag for first UHplotFTSim pass        */
  long   i;                     /* vector index                           */
  long   j;                     /* vector index                           */
  long   itime;                 /* index for X (time axis)                */
  long   itimeView;             /* index for plot refresh                 */
  long   nBinTime;              /* X (time axis) bin number               */
  long   dNbin;                 /* nbr of bins to shift if refresh plot   */
  double refresh_percent;       /* refresh period of plot (% of timeSize) */
  long   refresh_nBin;          /* refresh period of plot (nbr of x bins) */
  float  *buff1;                /* buffer for passing data to FFT         */
  float  *buff2;                /* buffer for passing data to FFT         */
  float  *buffTot;              /* buffer for PACK/UNPACK of plot values  */
  int hidden;
};


/*------------------------------------- UHplot1DTime ---------------------*/
struct UHplot1DTime{ 
  long   iClock;                /* clock number                           */
  long   id;                    /*  histo id                              */
  char   *name;                 /* name used in ROOT                      */
  char   *title;                /* histo title                            */
  long   nyBin;                 /* bin number of y axis                   */
  double yMin;                  /* min y of the 1D plot                   */
  double yMax;                  /* max y of the 1D plot                   */
  double timeSize;              /* X time axis size (sec)                 */
  long   resampFactor;          /* sampling freq / resampling freq        */
  long   t0;                    /* starting time (in clock count)         */
  struct USignal *input;        /* hold input parameters                  */
  char   *options;              /* plot options
                                   (must begin with intervalle value)     */
                                /* ------ end_of_SIO parameters ----------*/
  long   nodc;                  /* flag>0 if spectrum computed without DC */
  double dt;                    /* sampling period                        */
  long   nBin;                  /* nbr of values between each stat reset  */
  double zmin;                  /* min value on z axis (for colors scale) */
  double zmax;                  /* max value on z axis (for colors scale) */
  struct UFrUnpack *FrUnpack;   /* used if no input signal                */
  long   numPicture;            /* number to place the picture            */
  char   plotOpt[30];           /* plot options                           */
  long   intervalle;            /* intervalle = number of clock tick 
                                   between two hplot call (if =< 0 the
                                   histo are just saved at the end        */
  double currentTimeUTC;        /* current UTC time of start of plot      */
  double timeOffset;            /* Offset startTime - frame start         */
  double tmin;                  /* current starting time of time Window   */
  int    previousGPS;           /* GPS time of previous frame read        */
  char   units[256];            /* copy of the FrAdcData units            */
  long   realUnits;             /* flag to say that we plot in real Units */
  double slope;                 /* slope between units and ADC#           */
  double bias;                  /* bias  between units and ADC#           */
  BOOL   adjust;                /* tell if the AUTO option is used        */

  double sumvalue;              /* accumulating input values              */
  double corrupt;               /* flag to indicate if last data used are corrupted */
  long   iFirst;                /* flag for first UHplot1DTSim pass       */
  long   i;                     /* vector index                           */
  long   j;                     /* vector index                           */
  long   itime;                 /* index for X (time axis)                */
  long   itimeView;             /* index for plot refresh                 */
  long   nBinTime;              /* X (time axis) bin number               */
  long   dNbin;                 /* nbr of bins to shift if refresh plot   */
  double refresh_percent;       /* refresh period of plot (% of timeSize) */
  long   refresh_nBin;          /* refresh period of plot (nbr of x bins) */
  float  *buff;                 /* buffer for passing data to 1DTime      */
  float  *buffTot;              /* buffer for PACK/UNPACK of plot values  */
  int hidden;
};


/*-------------------------------------- UHplotRaw ----------------------*/
struct UHplotRaw{ 
  long   iClock;                /* clock number                           */
  long   id;                    /* plot histo id                          */
  char   *name;                 /* name used in ROOT                      */
  char   *title;                /* histo title                            */
  long   maxAverage;            /* number of plots for average            */
  long   t0;                    /* starting time (in clock count)         */
  struct USignal *input;        /* hold input parameters                  */
  char   *options;              /* plot options
                                   (must begin with intervalle value)     */
                                /* ------ end_of_SIO parameters ----------*/
  double dt;                    /* sampling period                        */
  struct UFrUnpack *FrUnpack;   /* used if no input signal                */
  long numPicture;	        /* number to place the picture            */
  char plotOpt[30];             /* plot options                           */
  long   intervalle;            /* intervalle = number of clock tick 
                                   between two hplot call (if =< 0 the
                                   histo are just save at the end         */

  long   resampFactor;          /* for resampling (currently not used)    */
  long   nAverage;              /* current number of plots averaged       */
  long   realUnits;             /* flag to say that we plot in real Units */
  double slope;                 /* slope between units and ADC#           */
  double bias;                  /* bias  between units and ADC#           */
  long   x2av;                  /* flag to say that we average squares    */

  int previousGPS;              /* GPS time of previous frame read        */
  long   autoScale;             /* flag for Y or Z (if 2D) axis auto scaling */
  double zMin;                  /* lower y or z (if 2D) edge of the plot     */
  double zMax;                  /* upper y or z (if 2D) edge of the plot     */
  double yMin;                  /* lower y edge of the plot               */
  double yMax;                  /* upper y edge of the plot               */
  double xMin;                  /* lower x edge of the plot               */
  double xMax;                  /* upper x edge of the plot               */
  long   nBin;                  /* number of bins of the plot             */
  long   nxBin;                 /* number of x bins of the 2d plot        */
  long   nyBin;                 /* number of y bins of the 2d plot        */
  long   raw2d;                 /* flag: >0 if data are 2d vector (image) */

  double sumvalue;              /* accumulating input values              */
  double corrupt;               /* flag to indicate if last data used are corrupted */
  long   iFirst;                /* used to call Ini within Sim            */
  long   i;                     /* current bin number                     */
  long   j;                     /* vector index                           */
  float *buff;                  /* buffer for PACK/UNPACK of plot values  */
  int    nsame;                 /* number of plots superposed to this one */
  int    color;                 /* color of the plot                      */
  struct UHplotRaw *refplot;    /* pointer to the plot on which is superposed... */
  struct UHplotRaw *next;
  int hidden;
};


/*------------------------------------- UHplotRawTime ---------------------*/
struct UHplotRawTime{ 
  long   iClock;                /* clock number                           */
  long   id;                    /*  histo id                              */
  char   *name;                 /* name used in ROOT                      */
  char   *title;                /* histo title                            */
  long   nyBin;                 /* bin number of y axis                   */
  double yMin;                  /* min y of the 1D plot                   */
  double yMax;                  /* max y of the 1D plot                   */
  double timeSize;              /* X time axis size (sec)                 */
  long   resampFactor;          /* sampling freq / resampling freq        */
  long   t0;                    /* starting time (in clock count)         */
  struct USignal *input;        /* hold input parameters                  */
  char   *options;              /* plot options
                                   (must begin with intervalle value)     */
                                /* ------ end_of_SIO parameters ----------*/
  long   nodc;                  /* flag>0 if spectrum computed without DC */
  double dt;                    /* sampling period                        */
  long   nBin;                  /* nbr of values between each stat reset  */
  double zmin;                  /* min value on z axis (for colors scale) */
  double zmax;                  /* max value on z axis (for colors scale) */
  long   autoScale;             /* flag>0 if z axis is auto-scaled        */
  struct UFrUnpack *FrUnpack;   /* used if no input signal                */
  long   numPicture;            /* number to place the picture            */
  char   plotOpt[30];           /* plot options                           */
  long   intervalle;            /* intervalle = number of clock tick 
                                   between two hplot call (if =< 0 the
                                   histo are just saved at the end        */
  double currentTimeUTC;        /* current UTC time of start of plot      */
  double timeOffset;            /* Offset startTime - frame start         */
  double tmin;                  /* current starting time of time Window   */
  int    previousGPS;           /* GPS time of previous frame read        */
  char   units[256];            /* copy of the FrAdcData units            */
  long   realUnits;             /* flag to say that we plot in real Units */
  double slope;                 /* slope between units and ADC#           */
  double bias;                  /* bias  between units and ADC#           */
  BOOL   adjust;                /* tell if the AUTO min/max option is used */
  int    iFirst;                /* used to call Ini within Sim            */
  long   raw2d;                 /* flag: >0 if data are 2d vector (image) */
  long   x2av;                  /* flag to say that we average squares    */
  double corrupt;               /* flag to indicate if last data used are corrupted */
  long   nAverage;              /* counter for raw data packets averaging */
  long   maxAverage;            /* raw data packets to average            */
  long   itime;                 /* index for X (time axis)                */
  long   itimeView;             /* index for plot refresh                 */
  long   nBinTime;              /* X (time axis) bin number               */
  long   dNbin;                 /* nbr of bins to shift if refresh plot   */
  double refresh_percent;       /* refresh period of plot (% of timeSize) */
  long   refresh_nBin;          /* refresh period of plot (nbr of x bins) */
  float  *buff;                 /* buffer for passing data to RawTime     */
  float  *buffTot;              /* buffer for PACK/UNPACK of plot values  */
  int hidden;
};


void UHLibIni(void);

void UHdummyRead(struct UJfile *iFile);
void UHdummySim(struct UHdummy *dummy, long time);

void UHplotUpdateEnv (int plotNum, char *plotName, char *plotOptions, char *refplotName, int refplotNum, int nsame,
                      char *title, int color, float *buff, int nBin, double *corrupt);
void UHplotSetTime (double origin, char *plotName, double deltat, int nAverage, struct FrameH *frame);

void UHoptDecode(char *options, long *intervalle, char *plotOpt);
void UHoptPlopt(char *options, long numPicture, int subpad);
void UHplot(char* name, long numPicture, char *options, char *plotOpt, int subpad, int nsame);
void UHplotZoom(char* name, long numPicture, char *options, char *plotOpt, int xmin, int xmax, int subpad);
void UHplotZoom2 (char *name, long numPicture, char *options, char *plotOpt, int ymin, int ymax, int subpad);

void UHplotFFree(struct UHplotFFT *plotFFT);
void UHplotFIni(struct UHplotFFT *plotFFT,double dt);
void UHplotFIniFrame(struct UHplotFFT *plotFFT);
void UHplotFRead(struct UJfile *iFile);
void UHplotFSim(struct  UHplotFFT *plotFFT,long time);
void UHplotFEnd(struct UHplotFFT *plotFFT);

void UHplotTFree(struct UHplotTime *plotTime);
void UHplotTIni(struct UHplotTime *plotTime,double dt);
void UHplotTIniFrame (struct UHplotTime *plotTime);
void UHplotTRead(struct UJfile *iFile);
void UHplotTSim(struct UHplotTime *plotTime,long time);
void UHplotTEnd(struct UHplotTime *plotTime);

void UHplot1DEnd(struct UHplot1D *plot1D);
void UHplot1DFree(struct UHplot1D *plot1D);
void UHplot1DIni(struct UHplot1D *plot1D,double dt);
void UHplot1DIniFrame(struct UHplot1D *plot1D);
void UHplot1DRead(struct UJfile *iFile);
void UHplot1DSim(struct UHplot1D *plot1D,long time);
void UHplot1DEnd(struct UHplot1D *plot1D);

void UHplot2DFree(struct UHplot2D *plot2D);
void UHplot2DIni(struct UHplot2D *plot2D,double dt);
void UHplot2DIniFrame(struct UHplot2D *plot2D);
void UHplot2DRead(struct UJfile *iFile);
void UHplot2DSim(struct UHplot2D *plot2D,long time);
void UHplot2DEnd(struct UHplot2D *plot2D);

void UHplot3DFree(struct UHplot3D *plot3D);
void UHplot3DIni(struct UHplot3D *plot3D,double dt);
void UHplot3DIniFrame(struct UHplot3D *plot3D);
void UHplot3DRead(struct UJfile *iFile);
void UHplot3DSim(struct UHplot3D *plot3D,long time);
void UHplot3DEnd(struct UHplot3D *plot3D);

long UHplotGrafIni(long);

void UHpTfctFree(struct UHpTfct *pTfct);
void UHpTfctIni(struct UHpTfct *pTfct,double dt);
void UHpTfctIniFrame(struct UHpTfct *pTfct);
void UHpTfctRead(struct UJfile *iFile);
void UHpTfctSim(struct UHpTfct *pTfct,long time);
void UHpTfctEnd(struct UHpTfct *pTfct);

void UHpCoheFree(struct UHpCohe *pCohe);
void UHpCoheIni(struct UHpCohe *pCohe,double dt);
void UHpCoheIniFrame(struct UHpCohe *pCohe);
void UHpCoheRead(struct UJfile *iFile);
void UHpCoheSim(struct UHpCohe *pCohe,long time);
void UHpCoheEnd(struct UHpCohe *pCohe);

void UHpCorrFree(struct UHpCorr *pCorr);
void UHpCorrIni(struct UHpCorr *pCorr,double dt);
void UHpCorrIniFrame(struct UHpCorr *pCorr);
void UHpCorrRead(struct UJfile *iFile);
void UHpCorrSim(struct UHpCorr *pCorr,long time);
void UHpCorrEnd(struct UHpCorr *pCorr);

void UHsoundIni(struct UHsound *sound,double dt);
void UHsoundIniFrame(struct UHsound *sound);
void UHsoundRead(struct UJfile *iFile);
void UHsoundSim(struct UHsound *sound,long time);
void UHsoundEnd(struct UHsound *sound);

void UHplotFTFree(struct UHplotFFTTime *plotFFTTime);
void UHplotFTIni(struct UHplotFFTTime *plotFFTTime,double dt);
void UHplotFTIniFrame(struct UHplotFFTTime *plotFFTTime);
void UHplotFTRead(struct UJfile *iFile);
void UHplotFTSim(struct UHplotFFTTime *plotFFTTime, long time);
void UHplotFTEnd(struct UHplotFFTTime *plotFFTTime);

void UHpTfctTFree(struct UHpTfctTime *pTfctTime);
void UHpTfctTIni(struct UHpTfctTime *pTfctTime,double dt);
void UHpTfctTIniFrame(struct UHpTfctTime *pTfctTime);
void UHpTfctTRead(struct UJfile *iFile);
void UHpTfctTSim(struct UHpTfctTime *pTfctTime, long time);
void UHpTfctTEnd(struct UHpTfctTime *pTfctTime);

void UHpCoheTFree(struct UHpCoheTime *pCoheTime);
void UHpCoheTIni(struct UHpCoheTime *pCoheTime,double dt);
void UHpCoheTIniFrame(struct UHpCoheTime *pCoheTime);
void UHpCoheTRead(struct UJfile *iFile);
void UHpCoheTSim(struct UHpCoheTime *pCoheTime, long time);
void UHpCoheTEnd(struct UHpCoheTime *pCoheTime);

void UHplot1DTFree(struct UHplot1DTime *plot1DTime);
void UHplot1DTIni(struct UHplot1DTime *plot1DTime,double dt);
void UHplot1DTIniFrame(struct UHplot1DTime *plot1DTime);
void UHplot1DTRead(struct UJfile *iFile);
void UHplot1DTSim(struct UHplot1DTime *plot1DTime, long time);
void UHplot1DTEnd(struct UHplot1DTime *plot1DTime);

void UHplotRawFree(struct UHplotRaw *plotRaw);
void UHplotRawIni(struct UHplotRaw *plotRaw,double dt);
void UHplotRawRead(struct UJfile *iFile);
void UHplotRawSim(struct UHplotRaw *plotRaw,long time);
void UHplotRawEnd(struct UHplotRaw *plotRaw);

void UHplotRawTFree(struct UHplotRawTime *plotRawTime);
void UHplotRawTIni(struct UHplotRawTime *plotRawTime,double dt);
void UHplotRawTRead(struct UJfile *iFile);
void UHplotRawTSim(struct UHplotRawTime *plotRawTime, long time);
void UHplotRawTEnd(struct UHplotRawTime *plotRawTime);

double DyTimeNeeded2 (char *fctName, char *flag);

extern struct UJstructH *UHplotFFTUJstructH;
extern struct UJstructH *UHplotTimeUJstructH;
extern struct UJstructH *UHplot1DUJstructH;
extern struct UJstructH *UHplot2DUJstructH;
extern struct UJstructH *UHplot3DUJstructH;
extern struct UJstructH *UHpTfctUJstructH;
extern struct UJstructH *UHpCoheUJstructH;
extern struct UJstructH *UHpCorrUJstructH;
extern struct UJstructH *UHsoundUJstructH;
extern struct UJstructH *UHplotFFTTimeUJstructH;
extern struct UJstructH *UHpTfcTimeUJstructH;
extern struct UJstructH *UHpCoheTimeUJstructH;
extern struct UJstructH *UHplot1DTimeUJstructH;
extern struct UJstructH *UHplotRawUJstructH;
extern struct UJstructH *UHplotRawTimeUJstructH;
extern struct UJstructH *UHdummyUJstructH;

#endif

