/**************************************************/
/*                                                */
/* File: VDyPlot2.cc                              */
/* Copyright (C) 2003, D. Buskulic, D. Verkindt   */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// VDyPlot2: class describing a plot (representation) of a series        //
// Used to adapt the behaviour of TH2F to datadisplay needs             //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

using namespace std;

#include <fstream>
#include <iostream>
#include <sys/time.h>
#include <time.h>
#include "TSystem.h"
#include "TROOT.h"
#include "TClass.h"
#include "VDyPlot2.h"
#include "TString.h"
#include "TGaxis.h"
#include "TCanvas.h"
#include "TStyle.h"
#include "TFile.h"
#include "TKey.h"
#include "TObject.h"
#include "dataDisplay.h"
#include "VDyPlot.h"
R__EXTERN TCanvas* gDDCanvas;

VDyPlot2*  gLastModifiedPlot2=0; 
VDyPlot2*  gLastModifiedPlotY2=0; 

ClassImp(VDyPlot2)

//______________________________________________________________________________
VDyPlot2::VDyPlot2() : TH2F()
{
   alreadyDrawn  = 0;
   numpad        = 0;
   numsubpad     = 0;
   superposed    = 0;
   mTimeFlag     = 0;
   mTimeString   = "";
   mTimePave     = 0;
   mTimeX1       = 0.03;
   mTimeY1       = 0.01;
   mTimeX2       = 0.97;
   mTimeY2       = 0.08;
   mTimeUTC      = 0.;
   mOldFirstXbin = GetXaxis()->GetFirst();
   mOldLastXbin  = GetXaxis()->GetLast();
   mOldFirstYbin = GetYaxis()->GetFirst();
   mOldLastYbin  = GetYaxis()->GetLast();
   mOldMinY      = 0.;
   mOldMaxY      = 0.;
   mPreviousMinY = 0.;
   mPreviousMaxY = 0.;
   mIndicator.Set(0);
}

//______________________________________________________________________________
VDyPlot2::VDyPlot2(Text_t* name, Text_t* title, Int_t nbinsx, Axis_t xlow, Axis_t xup, Int_t nbinsy, Axis_t ylow, Axis_t yup)
           :TH2F(name,title,nbinsx,xlow,xup,nbinsy,ylow,yup)
{
   char *str;

   alreadyDrawn  = 0;
   numpad        = 0;
   numsubpad     = 0;
   superposed    = 0;
   mTimeFlag     = 0;
   mTimeString   = "";
   mTimePave     = 0;
   mTimeX1       = 0.03;
   mTimeY1       = 0.01;
   mTimeX2       = 0.97;
   mTimeY2       = 0.08;
   mTimeUTC      = 0.;
   mOldFirstXbin = GetXaxis()->GetFirst();
   mOldLastXbin  = GetXaxis()->GetLast();
   mOldFirstYbin = GetYaxis()->GetFirst();
   mOldLastYbin  = GetYaxis()->GetLast();
   mOldMinY      = GetMinimum();
   mOldMaxY      = GetMaximum();
   mIndicator.Set(fNcells);

   str = strstr(title,SEPTITLE);
   if (str) strcpy(mPlotType,&str[0]+2);
   else     strcpy(mPlotType,"NULL");
   if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug > 0) printf("VDyPlot2> Created VDyPlot2 %s=%s \n", GetName(), GetTitle());
}

//______________________________________________________________________________
VDyPlot2::VDyPlot2(VDyPlot2 &plot)
{
   ((VDyPlot2&)plot).Copy(*this);
}

//______________________________________________________________________________
void VDyPlot2::Copy(TObject& newvsp)
{
   TH2F::Copy((VDyPlot2&)newvsp);
}

//______________________________________________________________________________
void VDyPlot2::Draw(Option_t* option)
{
// Draw the plot. Added the drawing of the time string, with respect to
// the TH2F base class

 char dbgmsg[LSTRING_MAX];

 sprintf(dbgmsg, "VDyPlot2:Draw> plot=%s=%s=pad%d", GetName(), GetTitle(), numpad);
 DyDebugPrint((const char *)dbgmsg,0);

 if (gPad && mTimeFlag  && strstr(option,"SAME") == NULL)
    {if (gPad->GetBottomMargin() < mTimeY2+0.10) gPad->SetBottomMargin(mTimeY2+0.10);}

 if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug > 0)
    {printf("VDyPlot2Draw> plot=%s=%s  pad=%d  option=%s \n", GetName(), GetTitle(), numpad, option);}

/**
 if (!alreadyDrawn) {TH2F::Draw(option); alreadyDrawn = 1; Dy->nplotdrawn++;}
**/

 if (strstr(option,"SAME") != NULL) superposed = 1;
 else                               superposed = 0;

 TH2F::Draw(option);

 if (gPad && mTimeFlag && strstr(option,"SAME") == NULL)
    {
     if (!mTimePave)
        {
         mTimePave = new TPaveText(mTimeX1, mTimeY1, mTimeX2, mTimeY2,"bNDC");
         mTimePave->ResetBit(kCanDelete);
         mTimePave->SetName("DyTimeString");
         mTimePave->AddText(mTimeString.Data());
//         if (strstr(mTimeString.Data(),"dt:") != NULL) {mTimePave->SetTextSize(0.04); mTimePave->SetTextAlign(12);}
//         if (strstr(mTimeString.Data(),"dt:") != NULL) {mTimePave->SetX1NDC(0.07); mTimePave->SetX2NDC(0.93);}
         mTimePave->SetFillColor(0);
         mTimePave->Draw("NDC");
        }
     else
        {
         mTimePave->Clear();
         mTimePave->AddText(mTimeString.Data());
//         if (strstr(mTimeString.Data(),"dt:") != NULL) {mTimePave->SetX1NDC(0.07); mTimePave->SetX2NDC(0.93);}
         mTimePave->SetFillColor(0);
         if (alreadyDrawn == 0) mTimePave->Draw("NDC");
        }
    } 
}


//______________________________________________________________________________
char *VDyPlot2::GetObjectInfo(Int_t px, Int_t py) const
{
//   Redefines TH1::GetObjectInfo.
//   Displays the histogram info (bin number, contents, integral up to bin
//   corresponding to cursor position px,py
//   Switches to the right pad before doing so.
//   Correction of a ROOT bug (feature ?).
//
   static char info[128];
   TVirtualPad* savepad;
   savepad = gPad;
   gPad = (gPad->GetCanvas())->GetSelectedPad();

   if (!gPad) return (char*)"";
   Double_t x  = gPad->PadtoX(gPad->AbsPixeltoX(px));
   Double_t y  = gPad->PadtoY(gPad->AbsPixeltoY(py));
   Double_t x1 = gPad->PadtoX(gPad->AbsPixeltoX(px+1));
   Double_t y1 = gPad->PadtoY(gPad->AbsPixeltoY(py+1));
   Int_t binx,binxmin,binx1;
   Int_t biny,binymin,biny1;
   
   binx    = fXaxis.FindFixBin(x);
   binxmin = fXaxis.GetFirst();
   binx1   = fXaxis.FindFixBin(x1);
// special case if more than 1 bin in x per pixel
   if (binx1-binx>1 && GetDimension() == 1) {
      Double_t binval=GetBinContent(binx);
      Int_t binnear=binx;
      for (Int_t ibin=binx+1; ibin<binx1; ibin++) {
         Double_t binvaltmp = GetBinContent(ibin);
         if (TMath::Abs(y-binvaltmp) < TMath::Abs(y-binval)) {
            binval=binvaltmp;
            binnear=ibin;
         }
      }
      binx = binnear;
   }

   biny    = fYaxis.FindFixBin(y);
   binymin = fYaxis.GetFirst();
   biny1   = fYaxis.FindFixBin(y1);
// special case if more than 1 bin in y per pixel
   if (biny1-biny>1 && GetDimension() == 1) {
      Double_t binval=GetBinContent(biny);
      Int_t binnear=biny;
      for (Int_t ibin=biny+1; ibin<biny1; ibin++) {
         Double_t binvaltmp = GetBinContent(ibin);
         if (TMath::Abs(y-binvaltmp) < TMath::Abs(y-binval)) {
            binval=binvaltmp;
            binnear=ibin;
         }
      }
      biny = binnear;
   }

   sprintf(info,"x=%g y=%g binx=%d biny=%d value=%g",x,y,binx,biny, GetBinContent(binx,biny));

   gPad = savepad;
   return info;
}


//______________________________________________________________________________
void VDyPlot2::SavePrimitive(ofstream &out, Option_t *option)
{
 // Save primitive as a C++ statement(s) on output stream out
 // Note the following restrictions in the code generated:
 // - variable bin size not implemented
 // - Objects in list of functions not saved (fits)
 // Adapted for Data Display: DO NOT save the bin contents, unnecessary

 char dbgmsg[LSTRING_MAX];

 sprintf(dbgmsg, "VDyPlot2:SavePrimitive> plot=%s=%s=pad%d", GetName(), GetTitle(), numpad);
 DyDebugPrint((const char *)dbgmsg,0);

 char quote = '"';
 out<<"   "<<endl;
 out<<"   "<<"TH1"<<" *";

 out<<GetName()<<" = new "<<ClassName()<<"("<<quote<<GetName()<<quote<<","<<quote<<GetTitle()<<quote
               <<","<<GetXaxis()->GetNbins()
               <<","<<GetXaxis()->GetXmin()
               <<","<<GetXaxis()->GetXmax();

 if (fDimension > 1) {
            out<<","<<GetYaxis()->GetNbins()
               <<","<<GetYaxis()->GetXmin()
               <<","<<GetYaxis()->GetXmax();
 }
 if (fDimension > 2) {
            out<<","<<GetZaxis()->GetNbins()
               <<","<<GetZaxis()->GetXmin()
               <<","<<GetZaxis()->GetXmax();
 }
            out<<");"<<endl;

 if (TMath::Abs(GetBarOffset()) > 1e-5)  {out<<"   "<<GetName()<<"->SetBarOffset("<<GetBarOffset()<<");"<<endl;}
 if (TMath::Abs(GetBarWidth()-1) > 1e-5) {out<<"   "<<GetName()<<"->SetBarWidth("<<GetBarWidth()<<");"<<endl;}
 if (fMinimum != -1111)                  {out<<"   "<<GetName()<<"->SetMinimum("<<fMinimum<<");"<<endl;}
 if (fMaximum != -1111)                  {out<<"   "<<GetName()<<"->SetMaximum("<<fMaximum<<");"<<endl;}
 if (fNormFactor != 0)                   {out<<"   "<<GetName()<<"->SetNormFactor("<<fNormFactor<<");"<<endl;}
 if (fEntries != 0)                      {out<<"   "<<GetName()<<"->SetEntries("<<fEntries<<");"<<endl;}
 if (fDirectory == 0)                    {out<<"   "<<GetName()<<"->SetDirectory(0);"<<endl;}
 if (TestBit(kNoStats))                  {out<<"   "<<GetName()<<"->SetStats(0);"<<endl;}
 if (fOption.Length() != 0)              {out<<"   "<<GetName()<<"->SetOption("<<quote<<fOption.Data()<<quote<<");"<<endl;}

 Int_t bin;
//    for (bin=0;bin<fNcells;bin++) {
//       Double_t bc = GetBinContent(bin);
//       if (bc) {
//          out<<"   "<<GetName()<<"->SetBinContent("<<bin<<","<<bc<<");"<<endl;
//       }
//    }
//    if (fSumw2.fN) {
//       for (bin=0;bin<fNcells;bin++) {
//          Double_t be = GetBinError(bin);
//          if (be) {
//             out<<"   "<<GetName()<<"->SetBinError("<<bin<<","<<be<<");"<<endl;
//          }
//       }
//    }
 Int_t ncontours = GetContour();
 if (ncontours > 0) {
    out<<"   "<<GetName()<<"->SetContour("<<ncontours<<");"<<endl;
    for (bin=0;bin<ncontours;bin++) {
       out<<"   "<<GetName()<<"->SetContourLevel("<<bin<<","<<GetContourLevel(bin)<<");"<<endl;
    }
 }

 SaveFillAttributes(out,GetName(),0,1001);
 SaveLineAttributes(out,GetName(),1,1,1);
 SaveMarkerAttributes(out,GetName(),1,1,1);
 fXaxis.SaveAttributes(out,GetName(),"->GetXaxis()");
 fYaxis.SaveAttributes(out,GetName(),"->GetYaxis()");
 fZaxis.SaveAttributes(out,GetName(),"->GetZaxis()");
 TString opt = option;
 opt.ToLower();
 if (!opt.Contains("nodraw")) {
    out<<"   "<<GetName()<<"->Draw("
    <<quote<<option<<quote<<");"<<endl;
 }
}

//______________________________________________________________________________
void VDyPlot2::SetIndicator(int bin, short colorValue)
{
// Sets the value for indicator on bin. This will be used to draw a colored vertical
// line indicating something went wrong.

    if (bin < 0) return;
    mIndicator.AddAt((short)colorValue,bin);
}

//______________________________________________________________________________
void VDyPlot2::Paint(Option_t* option)
{
// Paint the plot and add indicator lines

 double origin;
 char title[256];
 double indicymin, indicymax, indicxmin,indicxmax;
 Float_t savexloffset;
 char dbgmsg[LSTRING_MAX];

 sprintf(dbgmsg, "VDyPlot2:Paint> plot=%s=%s=pad%d", GetName(), GetTitle(), numpad);
 DyDebugPrint(dbgmsg,0);
 DyDebugTimeNeeded(dbgmsg,"start",4);

 savexloffset = GetXaxis()->GetLabelOffset();
 if (gPad->GetLogx() == 1) GetXaxis()->SetLabelOffset(savexloffset+0.005);

// Paint the plot
 
 TH2F::Paint(option);


// If a zoom on X axis was made, set new timeString

 if ((mOldFirstXbin != GetXaxis()->GetFirst() || mOldLastXbin != GetXaxis()->GetLast()) && strstr(mPlotType,"TIME") != NULL)
    {
     TPaveText* timepave = (TPaveText*)gPad->FindObject("DyTimeString");
     if (timepave)
        {
         double zoom_min2 = gPad->GetUxmin();
         origin = GetTimeUTC() + zoom_min2;
         if (FRGPSDELTA <= 13 && origin-FRGPSOFF+FRGPSDELTA>=820108813) origin = origin-FRGPSOFF+FRGPSDELTA+1;
         else                                                           origin = origin-FRGPSOFF+FRGPSDELTA;
         strcpy(title, timepave->GetLine(0)->GetTitle());
         DyGpsToTimeString (origin, NULL, 0, 0, &title[0]);
         timepave->Clear();
         TString timeString = title;
         timepave->AddText(timeString.Data());
         timepave->SetFillColor(0);
         timepave->Draw("NDC");
        }
    }

// Add colored indicator lines

 double zoom_min = gPad->GetUxmin();
 double zoom_max = gPad->GetUxmax();
 indicymin = gPad->GetUymin(); /** + (gPad->GetUymax()-gPad->GetUymin())/3; **/
 indicymax = gPad->GetUymax(); /** - (gPad->GetUymax()-gPad->GetUymin())/3; **/

 int flag = 0;
 for (int i=1;i<fNcells;i++)
    {
     if (i>1 && mIndicator[i-1] > 0 && mIndicator[i] != mIndicator[i-1])
        {
         indicxmax = GetXaxis()->GetBinUpEdge(i-1);
         if (indicxmax > zoom_max) indicxmax = zoom_max;
         if (indicxmin >= zoom_min  &&  indicxmax <= zoom_max  &&  indicxmin < indicxmax)
            {gVirtualX->SetFillStyle(1001);
             gVirtualX->SetFillColor(mIndicator[i-1]); //is redefined in TPad::PaintBox
             gPad->PaintBox(indicxmin, indicymin, indicxmax, indicymax);
            }
         flag = 0;
        }
     if (mIndicator[i] > 0  &&  flag == 0)
        {flag = 1;
         indicxmin = GetXaxis()->GetBinLowEdge(i);
         if (indicxmin < zoom_min) indicxmin = zoom_min;
        }
    }

 GetXaxis()->SetLabelOffset(savexloffset);


// If a zoom on X axis was made, set this plot as last modified plot

 if (mOldFirstXbin != GetXaxis()->GetFirst() || mOldLastXbin != GetXaxis()->GetLast())
    {
     mOldFirstXbin = GetXaxis()->GetFirst();
     mOldLastXbin = GetXaxis()->GetLast();
     gLastModifiedPlot2 = this;
     strcpy(Dy->gLastTitle,gLastModifiedPlot2->GetTitle()); 
//     if (Dy->Order == STOP) UpdateTimeFreq();
     if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>2) 
        {printf("VDyPlot2Paint> ZOOMX plot=%s=%s  pad=%d  type=%s  mOldFirstXbin=%d mOldLastXbin=%d  GetMin=%g GetMax=%g  gLastModifiedPlot2=%s\n",
                 GetName(), GetTitle(), numpad, mPlotType, mOldFirstXbin, mOldLastXbin, GetMinimum(), GetMaximum(), gLastModifiedPlot2->GetName());
        }
    } 


// If a zoom on Y axis was made, set this plot as last modified plot

 if (mOldFirstYbin != GetYaxis()->GetFirst() || mOldLastYbin != GetYaxis()->GetLast())
    {
     mOldFirstYbin = GetYaxis()->GetFirst();
     mOldLastYbin = GetYaxis()->GetLast();
     gLastModifiedPlotY2 = this;
     strcpy(Dy->gLastTitleY,gLastModifiedPlotY2->GetTitle()); 
//     if (Dy->Order == STOP) UpdateTimeFreq();
     if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>2) 
        {printf("VDyPlot2Paint> ZOOMY plot=%s=%s  pad=%d  type=%s  mOldFirstYbin=%d mOldLastYbin=%d  GetMin=%g GetMax=%g  gLastModifiedPlotY2=%s\n",
                 GetName(), GetTitle(), numpad, mPlotType, mOldFirstYbin, mOldLastYbin, GetMinimum(), GetMaximum(), gLastModifiedPlotY2->GetName());
        }
    }

 if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>2)
    {printf("VDyPlot2Paint> Paint done for plot=%s=%s  pad=%d  type=%s  gLastModifiedPlotY2=%s\n",
            GetName(), GetTitle(), numpad, mPlotType, gLastModifiedPlotY2->GetName());
    }

 DyDebugTimeNeeded(dbgmsg,"stop",4);
}


//______________________________________________________________________________
void VDyPlot2::ZoomAsLast()
{
// Zoom with the same range as the last plot

 if (gLastModifiedPlot2)
    {GetXaxis()->SetRange(gLastModifiedPlot2->GetXaxis()->GetFirst(), gLastModifiedPlot2->GetXaxis()->GetLast());}

 if (gLastModifiedPlotY2)
    {GetYaxis()->SetRange(gLastModifiedPlotY2->GetYaxis()->GetFirst(), gLastModifiedPlotY2->GetYaxis()->GetLast());}
}


//______________________________________________________________________________
void VDyPlot2::ExportAscii(const char* fileName)
{
// Create a file containing plot data in ASCII format: col1=x col2=y

 int i, j, nBinx, nBiny;
 double x, y, gtime, value;
 char realName[256];
 FILE *fp_ascii;

 if (strcmp(fileName,"Default_file_name_=_Plot_name") == 0) sprintf(realName,"%s.txt",GetTitle());
 else                                                       strcpy(realName, fileName);

 fp_ascii = fopen(realName,"w");
 if (fp_ascii == NULL) {printf("VDyPlot2ExportAscii> Could not open file %s \n", realName); return;}

 nBinx = GetNbinsX();
 nBiny = GetNbinsY();
 gtime = DyTimeFormatToGps(GetXaxis()->GetTimeFormat());

 for (i=0; i<nBinx; i++)
    {
     x = GetXaxis()->GetBinCenter(i+1);
     for (j=0;j<nBiny;j++)
        { 
         y = GetYaxis()->GetBinCenter(j+1);
         value = GetBinContent(i+1, j+1);
         if (strstr(GetTitle(),"TIME") != NULL) fprintf(fp_ascii,"%f  %g  %g \n", gtime+x, y, value);
         else                                   fprintf(fp_ascii,"%g  %g  %g \n", x, y, value);
        }
    }

 printf("VDyPlot2ExportAscii> Exported data of %s into ASCII file %s \n", GetTitle(), realName);
 fclose(fp_ascii);

 return;
}


//______________________________________________________________________________
void VDyPlot2::SuperposeExternalPlot(const char *fileName, const char *plotName)
{
// Superpose to the plot an other one read from file

 int nBinx, nBiny, nBinx2, nBiny2, i, j, ok, color;
 double value;
 char dummy[LSTRING_MAX], *dytimestring;
 TPaveText *timepave;
 const char *basename;

   DyDebugPrint(__FUNCTION__,0);

// Update the pad
   TVirtualPad* pad = gPad;

   TFile *fp = new TFile(fileName);

   if (fp == NULL)
      {printf("VDyPlot2SuperPoseExternalPlot> file %s not found\n", fileName);
       return;
      }

   TH1 *h2 = (TH1 *)(fp->Get(plotName));

   if (h2 == NULL)
      {
       printf("VDyPlot2SuperPoseExternalPlot> plot %s not found in file %s\n", plotName, fileName);
       printf("VDyPlot2SuperPoseExternalPlot> Available plot in this file are:\n");
       fp->GetListOfKeys()->Print();
       TKey *key;
       TIter nextkey(fp->GetListOfKeys());
       while ((key = (TKey*)nextkey()))
         {
          TObject *obj = (TObject *)(key->ReadObj());
          if (obj->IsA()->InheritsFrom("TH1"))
             {TH1 *h = (TH1*)obj; 
              printf("VDyPlot2SuperPoseExternalPlot> %s: %s=%s \n", obj->IsA()->GetName(), h->GetName(), h->GetTitle());
             }
          else printf("VDyPlot2SuperPoseExternalPlot> %s \n", obj->IsA()->GetName());
         }
       fp->Close();
       return;
      }

   h2->SetDirectory(gROOT);
   fp->Close();

   basename = gSystem->BaseName(fileName);
   sprintf(dummy, "%s%s%s%s", GetName(), SEPTITLE, basename, REF_SUFFIX);
   h2->SetName(dummy);
   if (h2->GetLineColor() < 2) h2->SetLineColor(Dy->refplot_color);
   color = h2->GetLineColor();
   DyXformAddToRefList((char *)fileName,color);

   nBinx = GetNbinsX();
   nBiny = GetNbinsY();
   nBinx2 = h2->GetNbinsX();
   nBiny2 = h2->GetNbinsY();

   if (nBinx != nBinx2 || nBiny != nBiny2)
      {printf("VDyPlot2SuperPoseExternalPlot> Cannot superpose %s=%s to %s=%s. nBinx or nBiny differ.\n", h2->GetName(), h2->GetTitle(), GetName(), GetTitle());}

   for (i=0; i<nBinx; i++)
      {for (j=0; j<nBiny; j++)
          {value = GetBinContent(i+1, j+1) - h2->GetBinContent(i+1, j+1);
           SetBinContent (i+1, j+1, value);
          }
      }

   if (h2->GetXaxis()->GetTimeFormat())
      {
       dytimestring = (char *)malloc(256);
       ok = GetTimeString(dytimestring, h2->GetXaxis()->GetTimeFormat());
       if (ok)
          {timepave = new TPaveText(0.02, 0.06, 0.98, 0.11,"bNDC");
           sprintf(dummy,"DyRefString%s%s", SEPTITLE, basename);
           timepave->SetName(dummy);
           timepave->AddText(dytimestring);
           timepave->SetTextFont(DY_FONT_HELVETICA);
           timepave->SetTextColor(color);
           timepave->SetFillColor(0);
           timepave->Draw("NDC");
          }
       free(dytimestring);
      }

   pad->Modified();
   return;
}


//______________________________________________________________________________
void VDyPlot2::RemoveRefPlot()
{
// Delete the reference plots superposed to the plot

 int padmodified;
 TObject *obj;
 char filename[LSTRING_MAX], dummy[LSTRING_MAX], *str, *str2;
 TPaveText *timepave;

   DyDebugPrint(__FUNCTION__,0);

   TIter  next((gPad)->GetListOfPrimitives());
   padmodified = 0;

   while ((obj = next()))
       {
        if (obj->InheritsFrom(TH1::Class()))
           {
            TH1* href = (TH1 *)obj;
            str2 = strstr(href->GetName(),REF_SUFFIX);
            if (str2 == NULL) continue;
            str = strstr(href->GetName(),SEPTITLE);
            strncpy(filename, str+strlen(SEPTITLE), str2-str-strlen(SEPTITLE));
            filename[str2-str-strlen(SEPTITLE)] = '\0';
            sprintf(dummy,"DyRefString%s%s", SEPTITLE, filename);
            timepave = (TPaveText*)(gPad->FindObject(dummy));
            if (timepave) delete timepave;
            delete href;
            padmodified = 1;
           }
       }

   gPad->Modified();
   return;
}


//______________________________________________________________________________
int VDyPlot2::GetTimeString (char *dytimestring, const char *timeformat)
{
// Get the GPS time string from the plot time format

 char dummy[256], *str;
 double gtime;

 gtime = DyTimeFormatToGps(timeformat);
 if (gtime == 0.) return(0);

 DyGpsToTimeString ((int)gtime, NULL, 0, 0, dytimestring);

 return(1);
}


/*-------------------------------- UpdateTimeFreq ----------*/
void VDyPlot2::UpdateTimeFreq()
{
// Zoom all freq plots with the same range as the last plot

   TAxis* axis;
   TAxis* axislast;
   int padmodified, padmodified2;
   Axis_t plotymin;
   Axis_t plotymax;

   if (strcmp(mPlotType,"FFTTIME")!=0 && strcmp(mPlotType,"TRFCTTIME")!=0 && strcmp(mPlotType,"COHETIME")!=0) return;

   axislast = GetYaxis();
   plotymin = axislast->GetBinLowEdge(axislast->GetFirst());
   plotymax = axislast->GetBinLowEdge(axislast->GetLast())
              + 9.99*axislast->GetBinWidth(axislast->GetLast())/10;

   if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
     {
      TObject *obj;
      TIter    next(gDDCanvas->GetListOfPrimitives());

      /*--- Loop on all the objects of the root Canvas ---*/
      while ((obj = next()))
        {
         if (obj->InheritsFrom(TPad::Class()))
            {
             TObject *obj2;
             TIter    next(((TPad *)obj)->GetListOfPrimitives());
             padmodified = 0;

             /*--- If Pad encountered, loop on all the objects of this pad ---*/

              while ((obj2 = next()))
                 {
                  if (obj2->InheritsFrom(VDyPlot::Class()))
                     {
                      VDyPlot* h = (VDyPlot *)obj2;
                      axis = h->GetXaxis();

                      /*--- If plot encountered is FFT, COHE or TRFCT type, modify it ---*/

                      if (strcmp(h->GetPlotType(),"FFT")==0 || strcmp(h->GetPlotType(),"COHE")==0 || strcmp(h->GetPlotType(),"TRFCT")==0)
                         {
                          if (axis->FindBin(plotymin) < axis->FindBin(plotymax))
                             {axis->SetRange(axis->FindBin(plotymin), axis->FindBin(plotymax));
                              padmodified++;
                             }
                         }
                     }

                  /*--- If SubPad encountered, loop on all the objects of this Subpad ---*/

                  else if (obj2->InheritsFrom(TPad::Class()))
                     {
                      TObject *obj3;
                      TIter    next(((TPad *)obj2)->GetListOfPrimitives());
                      padmodified2 = 0;
                      while ((obj3 = next()))
                         {
                          /*--- If plot encountered is COHE or TRFCT, modify it ---*/

                          if (obj3->InheritsFrom(VDyPlot::Class()))
                             {
                              VDyPlot* h = (VDyPlot *)obj3;

                              const char *drawOption = h->GetDrawOption();
                              if (drawOption && (strstr(drawOption,"same") != NULL || strstr(drawOption,"SAME") != NULL)) continue;

                              axis = h->GetXaxis();

                              if (strcmp(h->GetPlotType(),"FFT")==0 || strcmp(h->GetPlotType(),"COHE")==0 || strcmp(h->GetPlotType(),"TRFCT")==0)
                                 {
                                  if (axis->FindBin(plotymin) < axis->FindBin(plotymax))
                                     {axis->SetRange(axis->FindBin(plotymin), axis->FindBin(plotymax));
                                      padmodified2++;
                                     }
                                 }
                             }
                          if (padmodified2>0) break;
                         }
//                   Update the subpad
                     if (padmodified2 > 0)
                        {
                         ((TPad *)obj2)->Modified();
                         ((TPad *)obj2)->Update();
                         }
                     }

                  /*--- If plot 2D encountered, change it only if it is of the same type as last modified plot ---*/

                  else if (obj2->InheritsFrom(VDyPlot2::Class()))
                     {
                      VDyPlot2* h = (VDyPlot2 *)obj2;
                      axis = h->GetYaxis();

                       if (strcmp(h->mPlotType,"FFTTIME")==0 || strcmp(h->mPlotType,"TRFCTTIME")==0 || strcmp(h->mPlotType,"COHETIME")==0)
                         {
                          if (axis->FindBin(plotymin) < axis->FindBin(plotymax))
                            {axis->SetRange(axis->FindBin(plotymin), axis->FindBin(plotymax));
                             padmodified = 1;
                            }
                         }
                     }

                  if (padmodified>0) break;
                 }
//           Update the pad
             if (padmodified > 0)
                {
                 ((TPad *)obj)->Modified();
                 ((TPad *)obj)->Update();
                }
            }
        }
    }
}


