#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <time.h>


/*------------------------------------ compute_pgcd ---*/

int compute_pgcd (int nbr1, int nbr2)
{
 int reste, dummy;
 reste = 2;

 if (nbr2 > nbr1) {dummy=nbr1; nbr1=nbr2; nbr2=dummy;}

 while (reste > 0)
     {
      reste = nbr1 % nbr2;
      if (reste > 0)
         {nbr1 = nbr2;
          nbr2 = reste;
         }
     }

 return nbr2;
}


/*------------------------------------ DyPPCM ---*/

double DyPPCM (double *freqClock, int nclock)
{
 int i;
 double previous_ppcm, ppcm, pgcd;
 double freqmax, freqmin, scale;
 double newfreq;

 if (nclock < 2) return(0);

 freqmin = 1e+30;
 freqmax = 0.;
 for (i=0; i<nclock; i++)
    {if (freqClock[i] < freqmin && freqClock[i] > 0) freqmin = freqClock[i];
     if (freqClock[i] > freqmax) freqmax = freqClock[i];
    }

 scale = 1.;
 if (freqmin < 1.) scale = pow(10,6-floor(-log10(freqmax)));
 printf("scale=%f  freqmin=%f  freqmax=%f \n", scale, freqmin, freqmax);

 ppcm = freqClock[0]*scale;
 for (i=1; i<nclock; i++)
    {
     pgcd = (double)compute_pgcd((int)ppcm,(int)(freqClock[i]*scale));
     previous_ppcm = ppcm;
     ppcm = ppcm/pgcd * freqClock[i]*scale;
     printf("previous_ppcm=%f  freqClock[%d]=%d (%f) : pgcd=%f  ppcm=%f\n", previous_ppcm, i, (int)(freqClock[i]*scale), freqClock[i], pgcd, ppcm);
    }

 newfreq = ppcm/scale;
 return(newfreq);
}


/*---------------------------------------------------------------*/
int main(long argc, char **argv)
{
 double freq[10];
 double newfreq;

 freq[0]=20000;
 freq[1]=16384;
 freq[2]=0.1;
 if (argc > 1) freq[0] = atof(argv[1]);
 if (argc > 2) freq[1] = atof(argv[2]);
 if (argc > 3) freq[2] = atof(argv[3]);

 printf("%f/%f=%f  (int)(%f/%f)=%d  ceil(%f/%f)=%f  floor(%f/%f)=%f \n",
         freq[0], freq[1], freq[0]/freq[1], freq[0], freq[1], (int)(freq[0]/freq[1]), freq[0], freq[1], ceil(freq[0]/freq[1]), freq[0], freq[1], floor(freq[0]/freq[1]));
 printf("Computing PPCM for the following frequencies: %f (1/%f)  %f (1/%f)  %f (1/%f)\n", freq[0], 1./freq[0], freq[1], 1./freq[1], freq[2], 1./freq[2]);

 newfreq = DyPPCM(freq, 3);

 printf("Frequencies %f %f %f are all submultiples of %f\n", freq[0], freq[1], freq[2], newfreq);

 return(0);
}

