
/**************************************************/
/*                                                */
/* File: DyPlot.c                                 */
/* Copyright (C) 2003, D. Verkindt                */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include <fcntl.h> 
#include <stdarg.h>
#include <dirent.h>
#include <assert.h>
#include <unistd.h>
#include "dataDisplay.h"

#ifdef MEMWATCH
#    include "memwatch.h"
#endif


/*                        ***********                     */
/*                        *  PLOTS  *                     */
/*                        ***********                     */


/*------------------------------------------- DyXformSetPlot ---*/
                                            /*  called from Main Browser  */
void DyXformSetPlot (FL_OBJECT *obj, long arg)
{
 long i, i0, nvar, id;
 char dummy[LSTRING_MAX];
 FL_OBJECT *browser;

 DyDebugPrint(__FUNCTION__,0);

 if (Dy->Frame == NULL) {printf("DyXformSetPlot> no frame in memory \n"); return;}

 i0 = -1;

 /*--- Loop over browser and make the plot for the lines selected ---*/

 if (fl_get_browser(mainbrowser->sms) != 0)
    {browser = mainbrowser->sms;
     nvar = fl_get_browser_maxline(browser);
     for (i=0; i<nvar; i++)
        {if (fl_isselected_browser_line(browser,i+1))
            {DyXformSetVar(browser,i+1);
             DyXformSetOnePlot(obj,i);
             i0 = i;
            }
        }
    }

 if (fl_get_browser(mainbrowser->adc) != 0)
    {browser = mainbrowser->adc;
     nvar = fl_get_browser_maxline(browser);
     for (i=0; i<nvar; i++)
        {if (fl_isselected_browser_line(browser,i+1))
            {DyXformSetVar(browser,i+1);
             DyXformSetOnePlot(obj,i);
             i0 = i;
            }
        }
    }

 if (fl_get_browser(mainbrowser->proc) != 0)
    {browser = mainbrowser->proc;
     nvar = fl_get_browser_maxline(browser);
     for (i=0; i<nvar; i++)
        {if (fl_isselected_browser_line(browser,i+1))
            {DyXformSetVar(browser,i+1);
             DyXformSetOnePlot(obj,i);
             i0 = i;
            }
        }
    }

 if (fl_get_browser(mainbrowser->sim) != 0)
    {browser = mainbrowser->sim;
     nvar = fl_get_browser_maxline(browser);
     for (i=0; i<nvar; i++)
        {if (fl_isselected_browser_line(browser,i+1))
            {DyXformSetVar(browser,i+1);
             DyXformSetOnePlot(obj,i);
             i0 = i;
            }
        }
    }

 if (i0 == -1) DyInfoLine ("No channel selected");

 if (Dy->y2d == OFF && i0>=0)
    {id = fl_get_browser_maxline(mainbrowser->plots);
     sprintf(dummy, "Made a new %s plot", Dy->typeName[Plot[id]->type]);
     DyInfoLine (dummy);
     DyXformAutoDeselect(mainbrowser->sms, 0);
     DyXformAutoDeselect(mainbrowser->adc, 0);
     DyXformAutoDeselect(mainbrowser->proc, 0);
     DyXformAutoDeselect(mainbrowser->sim, 0);
    }

 return;
}



/*------------------------------------------- DyXformSetOnePlot ---*/
                                                        /*  called from DyXformSetPlot  */
void DyXformSetOnePlot (FL_OBJECT *obj, long arg)
{
 long id;
 double dt;
 struct FrAdcData *adc;
 struct FrProcData *proc;
 struct FrSimData *sim;
 struct FrSerData *sms;

 DyDebugPrint(__FUNCTION__,0);

 if (Dy->Frame == NULL) {printf("DyXformSetOnePlot> no frame in memory \n"); return;}
      
 id = fl_get_browser_maxline(mainbrowser->plots);
 id++;

 if (obj == mainbrowser->plotRaw)     Plot[id]->type = PLOTRAW;
 if (obj == mainbrowser->plotTime)    Plot[id]->type = PLOTTIME;
 if (obj == mainbrowser->plotCorr)    Plot[id]->type = PLOTCORR;
 if (obj == mainbrowser->plotFFT)     Plot[id]->type = PLOTFFT;
 if (obj == mainbrowser->plotTrFct)   Plot[id]->type = PLOTTR;
 if (obj == mainbrowser->plotCohe)    Plot[id]->type = PLOTCOHE;
 if (obj == mainbrowser->plot1D)      Plot[id]->type = PLOT1D;
 if (obj == mainbrowser->plot2D)      Plot[id]->type = PLOT2D;
 if (obj == mainbrowser->plot3D)      Plot[id]->type = PLOT3D;
 if (obj == mainbrowser->plotAudio)   Plot[id]->type = PLOTAUDIO;
 if (obj == mainbrowser->plotFFTTime) Plot[id]->type = PLOTFFTTIME;
 if (obj == mainbrowser->plot1DTime)  Plot[id]->type = PLOT1DTIME;
 if (obj == mainbrowser->plotRawTime) Plot[id]->type = PLOTRAWTIME;
 if (obj == mainbrowser->plotTrFctTime) Plot[id]->type = PLOTTRTIME;
 if (obj == mainbrowser->plotCoheTime)  Plot[id]->type = PLOTCOHETIME;

 /*--- If first pass for a 2D plot or if not a 2D plot ---*/

 if (Dy->y2d == OFF)
    {
/**
     if (Dy->smsvarName)    sprintf(Plot[id]->vVarx, "%s%s%s", Dy->smsName, SEPSMS, Dy->smsvarName);
**/
     if (Dy->smsvarName)    sprintf(Plot[id]->vVarx, "%s%s%s", Dy->smsName, SEP, Dy->smsvarName);
     else if (Dy->adcName)  sprintf(Plot[id]->vVarx, "%s", Dy->adcName);
     else if (Dy->procName) sprintf(Plot[id]->vVarx, "%s", Dy->procName);
     else if (Dy->simName)  sprintf(Plot[id]->vVarx, "%s", Dy->simName);
     else                  {printf("DyXformSetOnePlot> not been able to get variable x\n"); return;}
     sprintf(Plot[id]->name, "%s", Plot[id]->vVarx);
    }

 /*--- If second pass for a 2D plot ---*/

 if (Dy->y2d == ON)
    {
/**
     if (Dy->smsvarName)    sprintf(Plot[id]->vVary, "%s%s%s", Dy->smsName, SEPSMS, Dy->smsvarName);
**/
     if (Dy->smsvarName)    sprintf(Plot[id]->vVary, "%s%s%s", Dy->smsName, SEP, Dy->smsvarName);
     else if (Dy->adcName)  sprintf(Plot[id]->vVary, "%s", Dy->adcName);
     else if (Dy->procName) sprintf(Plot[id]->vVary, "%s", Dy->procName);
     else if (Dy->simName)  sprintf(Plot[id]->vVary, "%s", Dy->simName);
     else                  {printf("DyXformSetOnePlot> not been able to get variable y\n"); return;}
     if (Plot[id]->type == PLOTTR || Plot[id]->type == PLOTTRTIME)
        {sprintf(Plot[id]->name, "%s%s%s", Plot[id]->vVarx, SEP2VARTR, Plot[id]->vVary);}
     else
        {sprintf(Plot[id]->name, "%s%s%s", Plot[id]->vVarx, SEP2VAR, Plot[id]->vVary);}
    }

 /*--- If third pass for a 3D plot ---*/

 if (Dy->y2d == ON3)
    {
/**
     if (Dy->smsvarName)    sprintf(Plot[id]->vVarz, "%s%s%s", Dy->smsName, SEPSMS, Dy->smsvarName);
**/
     if (Dy->smsvarName)    sprintf(Plot[id]->vVarz, "%s%s%s", Dy->smsName, SEP, Dy->smsvarName);
     else if (Dy->adcName)  sprintf(Plot[id]->vVarz, "%s", Dy->adcName);
     else if (Dy->procName) sprintf(Plot[id]->vVarz, "%s", Dy->procName);
     else if (Dy->simName)  sprintf(Plot[id]->vVarz, "%s", Dy->simName);
     else                  {printf("DyXformSetOnePlot> not able to get variable z\n"); return;}
     if (Plot[id]->type != PLOT3D)
        {printf("DyXformSetOnePlot> plottype=%d=%s not a 3D plot \n", Plot[id]->type, Dy->typeName[Plot[id]->type]);
         return;
        }
     sprintf(Plot[id]->name, "%s%s%s%s%s", Plot[id]->vVarx, SEP2VAR, Plot[id]->vVary, SEP2VAR, Plot[id]->vVarz);
    }

 /*--- Look if we need a new clock (Frame->dt is clock for serial data) ---*/

 if (Dy->smsName && Dy->smsvarName)
    {
     if (Dy->y2d == ON)
        {
         sms = FrSerDataFind(Dy->Frame, Dy->smsName, NULL);
         if (sms && sms->sampleRate > 0.)
            {Plot[id]->freqClock2 = sms->sampleRate;}
         else
            {
             dt = DyGetFramedt(Dy->Frame);
             if (dt > 0)
                {printf("DyXformSetOnePlot> sampleRate of sms %s is 0. I put Frame->dt=%g instead.\n", sms->name, dt);
                 Plot[id]->freqClock2 = 1./dt;
                }
             else
                {printf("  DySetPlot: sms not found and Frame->dt=0. I force freqClock2 to 1.\n");
                 Plot[id]->freqClock2 = 1.;
                }
            }

         if (Plot[id]->freqClock2 > Plot[id]->freqClock)
            {Plot[id]->freqClock12 = Plot[id]->freqClock2;
             if (Plot[id]->type == PLOT2D || Plot[id]->type == PLOT3D) Plot[id]->freqClock12 = Plot[id]->freqClock;
            }
         else
            {Plot[id]->freqClock12 = Plot[id]->freqClock;
             if (Plot[id]->type == PLOT2D || Plot[id]->type == PLOT3D) Plot[id]->freqClock12 = Plot[id]->freqClock2;
            }
        }

     if (Dy->y2d == OFF)
        {
         sms = FrSerDataFind(Dy->Frame, Dy->smsName, NULL);
         if (sms && sms->sampleRate > 0.)
            {Plot[id]->freqClock = sms->sampleRate;}
         else
            {
             dt = DyGetFramedt(Dy->Frame);
             if (dt > 0)
                {printf("DyXformSetOnePlot> sampleRate of sms %s is 0. I put Frame->dt=%g instead.\n", sms->name, dt);
                 Plot[id]->freqClock = 1./dt;
                }
             else
                {printf("DyXformSetOnePlot> sms not found and Frame->dt=0. I force freqClock to 1.\n");
                 Plot[id]->freqClock = 1.;
                }
            }
        }

    }

 if (Dy->adcName)
    {if ((adc = FrAdcDataFind(Dy->Frame, Dy->adcName)) != NULL)
        {
         if (Dy->y2d == ON)
            {
             Plot[id]->freqClock2 = adc->sampleRate;
             if (Plot[id]->freqClock2 > Plot[id]->freqClock)
                {Plot[id]->freqClock12 = Plot[id]->freqClock2;
                 if (Plot[id]->type == PLOT2D || Plot[id]->type == PLOT3D) Plot[id]->freqClock12 = Plot[id]->freqClock;
                }
             else
                {Plot[id]->freqClock12 = Plot[id]->freqClock;
                 if (Plot[id]->type == PLOT2D || Plot[id]->type == PLOT3D) Plot[id]->freqClock12 = Plot[id]->freqClock2;
                }
            }
         if (Dy->y2d == OFF)  Plot[id]->freqClock = adc->sampleRate;
        }
    }

 if (Dy->procName)
    {if ((proc = FrProcDataFind(Dy->Frame, Dy->procName)) != NULL)
        {
         if (Dy->y2d == ON)
            {
             Plot[id]->freqClock2 = 1./proc->data->dx[0];
             if (Plot[id]->freqClock2 > Plot[id]->freqClock)
                {Plot[id]->freqClock12 = Plot[id]->freqClock2;
                 if (Plot[id]->type == PLOT2D || Plot[id]->type == PLOT3D) Plot[id]->freqClock12 = Plot[id]->freqClock;
                }
             else
                {Plot[id]->freqClock12 = Plot[id]->freqClock;
                 if (Plot[id]->type == PLOT2D || Plot[id]->type == PLOT3D) Plot[id]->freqClock12 = Plot[id]->freqClock2;
                }
            }
         if (Dy->y2d == OFF)  Plot[id]->freqClock = 1./proc->data->dx[0];
        }
    }

 if (Dy->simName)
    {if ((sim = FrSimDataFind(Dy->Frame, Dy->simName)) != NULL)
        {
         if (Dy->y2d == ON)
            {
             Plot[id]->freqClock2 = 1./sim->data->dx[0];
             if (Plot[id]->freqClock2 > Plot[id]->freqClock)
                {Plot[id]->freqClock12 = Plot[id]->freqClock2;
                 if (Plot[id]->type == PLOT2D || Plot[id]->type == PLOT3D) Plot[id]->freqClock12 = Plot[id]->freqClock;
                }
             else
                {Plot[id]->freqClock12 = Plot[id]->freqClock;
                 if (Plot[id]->type == PLOT2D || Plot[id]->type == PLOT3D) Plot[id]->freqClock12 = Plot[id]->freqClock2;
                }
            }
         if (Dy->y2d == OFF)  Plot[id]->freqClock = 1./sim->data->dx[0];
        }
    }

 if (Plot[id]->type == PLOTRAW || Plot[id]->type == PLOTRAWTIME) Plot[id]->freqClock = 1./DyGetFramedt(Dy->Frame);


 /*--- Second variable of 2D or TrFct plot has been found ---*/

 if (Dy->y2d == ON3 && Plot[id]->type == PLOT3D) Dy->y2d = ON2;

 if (Dy->y2d == ON && Plot[id]->type == PLOT3D)
    {if (Plot[id]->type == PLOT3D) DyInfoLine ("Made a new 3D plot");
     Dy->y2d = ON3;
    }

 if (Dy->y2d == ON && Plot[id]->type != PLOT3D)
    {if (Plot[id]->type == PLOT2D)       DyInfoLine ("Made a new 2D plot");
     if (Plot[id]->type == PLOTTR)       DyInfoLine ("Made a new Transfert Function plot");
     if (Plot[id]->type == PLOTCOHE)     DyInfoLine ("Made a new Coherence plot");
     if (Plot[id]->type == PLOTTRTIME)   DyInfoLine ("Made a new Transfert Function vs Time plot");
     if (Plot[id]->type == PLOTCOHETIME) DyInfoLine ("Made a new Coherence vs Time plot");
     if (Plot[id]->type == PLOTCORR)     DyInfoLine ("Made a new Correlation Function plot");
     Dy->y2d = ON2;
    }

 /*--- Need a second variable if it is a 2D or TrFct plot ---*/

 if (obj == mainbrowser->plot3D        && Dy->y2d == OFF)  Dy->y2d = ON;
 if (obj == mainbrowser->plot2D        && Dy->y2d == OFF)  Dy->y2d = ON;
 if (obj == mainbrowser->plotTrFct     && Dy->y2d == OFF)  Dy->y2d = ON;
 if (obj == mainbrowser->plotCohe      && Dy->y2d == OFF)  Dy->y2d = ON;
 if (obj == mainbrowser->plotCorr      && Dy->y2d == OFF)  Dy->y2d = ON;
 if (obj == mainbrowser->plotTrFctTime && Dy->y2d == OFF)  Dy->y2d = ON;
 if (obj == mainbrowser->plotCoheTime  && Dy->y2d == OFF)  Dy->y2d = ON;
    
 /*--- Put default parameter in plot structure and update plots list ---*/

 if (Dy->y2d == ON2) Dy->y2d = OFF;

 if (Dy->y2d == OFF)
    {DyPutDefaultInPlot(id);
     DyGetDefaultFromPlot(id);
     Plot[id]->superposed = OFF;
     DyXformUpdatePlotBrowser(id, ADD_PLOT);
    }

 return;
}


/*------------------------------------------- DyXformSetOnePlotVarY ---*/
                                                 /*  called from Main Browser  */
void DyXformSetOnePlotVarY (FL_OBJECT *obj, long arg)
{
 long i, id;
 double dt;
 struct FrAdcData *adc;
 struct FrProcData *proc;
 struct FrSimData *sim;
 struct FrSerData *sms;

 if (Dy->y2d == OFF) return;

 DyDebugPrint(__FUNCTION__,0);

 if (Dy->Frame == NULL) {printf("DyXformSetOnePlotVarY> no frame in memory \n"); return;}

 i = fl_get_browser(obj);
 if (i <= 0) return;

 DyXformSetVar(obj,i);

 id = fl_get_browser_maxline(mainbrowser->plots);
 id++;

/**
 if (Dy->smsName && Dy->smsvarName) sprintf(Plot[id]->vVary, "%s%s%s", Dy->smsName, SEPSMS, Dy->smsvarName);
**/
 if (Dy->smsName && Dy->smsvarName) sprintf(Plot[id]->vVary, "%s%s%s", Dy->smsName, SEP, Dy->smsvarName);
 else if (Dy->adcName)              sprintf(Plot[id]->vVary, "%s", Dy->adcName);
 else if (Dy->procName)             sprintf(Plot[id]->vVary, "%s", Dy->procName);
 else if (Dy->simName)              sprintf(Plot[id]->vVary, "%s", Dy->simName);
 else                              {printf("DyXformSetOnePlotVarY> not able to get variable x\n"); return;}

 if (Plot[id]->type == PLOTTR || Plot[id]->type == PLOTTRTIME)
    {sprintf(Plot[id]->name, "%s%s%s", Plot[id]->vVarx, SEP2VARTR, Plot[id]->vVary);}
 else if (Plot[id]->type == PLOT3D)
    {sprintf(Plot[id]->name, "%s%s%s%s%s", Plot[id]->vVarx, SEP2VAR, Plot[id]->vVary, SEP2VAR, Plot[id]->vVarz);}
 else
    {sprintf(Plot[id]->name, "%s%s%s", Plot[id]->vVarx, SEP2VAR, Plot[id]->vVary);}


 /*--- Look if we need a new clock (Frame->dt is clock for serial data) ---*/

 if (Dy->smsName && Dy->smsvarName)
    {sms = FrSerDataFind(Dy->Frame, Dy->smsName, NULL);
     if (sms && sms->sampleRate > 0.)
        {Plot[id]->freqClock2 = sms->sampleRate;}
     else
        {
         dt = DyGetFramedt(Dy->Frame);
         if (dt > 0)
            {printf("DyXformSetOnePlotVarY> sampleRate of sms %s is 0. I put Frame->dt=%g instead.\n", sms->name, dt);
             Plot[id]->freqClock2 = 1./dt;
            }
         else
            {printf("DyXformSetOnePlotVarY> sms not found and Frame->dt=0. I force freqClock2 to 1.\n");
             Plot[id]->freqClock2 = 1.;
            }
        }
    }

 if (Dy->adcName)
    {if ((adc = FrAdcDataFind(Dy->Frame, Dy->adcName)) != NULL) Plot[id]->freqClock2 = adc->sampleRate;}

 if (Dy->procName)
    {if ((proc = FrProcDataFind(Dy->Frame, Dy->procName)) != NULL) Plot[id]->freqClock2 = 1./proc->data->dx[0];}

 if (Dy->simName)
    {if ((sim = FrSimDataFind(Dy->Frame, Dy->simName)) != NULL) Plot[id]->freqClock2 = 1./sim->data->dx[0];}

 if (Dy->smsName || Dy->adcName || Dy->procName || Dy->simName)
    {if (Plot[id]->freqClock2 > Plot[id]->freqClock) Plot[id]->freqClock12 = Plot[id]->freqClock2;
     else                                            Plot[id]->freqClock12 = Plot[id]->freqClock;
    }

 /*--- Second variable of 2D or TrFct plot has been found ---*/

 if (Plot[id]->type == PLOT3D)       DyInfoLine ("Made a new 3D plot");
 if (Plot[id]->type == PLOT2D)       DyInfoLine ("Made a new 2D plot");
 if (Plot[id]->type == PLOTTR)       DyInfoLine ("Made a new Transfert Function plot");
 if (Plot[id]->type == PLOTCOHE)     DyInfoLine ("Made a new Coherence plot");
 if (Plot[id]->type == PLOTTRTIME)   DyInfoLine ("Made a new Transfert Function vs Time plot");
 if (Plot[id]->type == PLOTCOHETIME) DyInfoLine ("Made a new Coherence vs Time plot");
 if (Plot[id]->type == PLOTCORR)     DyInfoLine ("Made a new Correlation Function plot");

 /*--- Put default parameter in plot structure and update plots list ---*/

 DyPutDefaultInPlot(id);
 DyGetDefaultFromPlot(id);
 Plot[id]->superposed = OFF;
 DyXformUpdatePlotBrowser(id, ADD_PLOT);
 Dy->y2d = OFF;

 DyXformAutoDeselect(mainbrowser->sms, 0);
 DyXformAutoDeselect(mainbrowser->adc, 0);
 DyXformAutoDeselect(mainbrowser->proc, 0);
 DyXformAutoDeselect(mainbrowser->sim, 0);

 return;
}


/*-------------------------------------------- DyXformEditPlot ---*/
/*                                  called when OK button of plot panel is pressed  */

void DyXformEditPlot (FL_OBJECT *obj, long arg)
{
 long id, nplot, i, j;
 char plotname[LSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);

 id = fl_get_browser(mainbrowser->plots);

 if (id <= 0)
    {nplot = fl_get_browser_maxline(mainbrowser->plots);
     for (i=1; i<=nplot; i++) {if (fl_isselected_browser_line(mainbrowser->plots,i)) {id = i; break;}}
    }
 if (id <= 0) return;

 if (Dy->Clone == ON)
    {nplot = fl_get_browser_maxline(mainbrowser->plots);
     for (i=1; i<=nplot; i++)
        {if (Plot[i]->type == Plot[id]->type)
            {strcpy(plotname,Plot[i]->name);
             DyXformPanelToPlot(i);
             strcpy(Plot[i]->name,plotname);
             DyXformUpdatePlotBrowser(i, REPLACE_PLOT);
            }
        }
    }

 else
    {nplot = fl_get_browser_maxline(mainbrowser->plots);
     for (i=1; i<=nplot; i++)
        {if (fl_isselected_browser_line(mainbrowser->plots,i))
            {strcpy(plotname,Plot[i]->name);
             DyXformPanelToPlot(i);
             strcpy(Plot[i]->name,plotname);
             DyXformUpdatePlotBrowser(i, REPLACE_PLOT);
             if (Plot[i]->type < 10000)
                {if (i < nplot && Plot[i]->superposed == OFF && Plot[i+1]->superposed == ON)
                    {j=i+1;
                     while (j<=nplot && Plot[j]->superposed == ON)
                        {DyCopyPlotForSame(Plot[j], Plot[i]);
                         j++;
                        }
                    }
                 else if (Plot[i]->superposed == ON)
                    {j = i-1;
                     while (j>0 && Plot[j]->superposed == ON) j--;
                     DyCopyPlotForSame(Plot[i], Plot[j]);
                    }
                }
            }
        }
    }

 DyXformAutoDeselect(mainbrowser->plots,0);
 DyXformClosePanel(obj,arg);

 return;
}


/*-------------------------------------------- DyXformGetPlotPanel ---*/
/*                 called when Edit or Edit Clone button is pressed   */

void DyXformGetPlotPanel (FL_OBJECT *obj, long arg)
{
 long id, i, nplot;
 char dummy[LSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);

 id = fl_get_browser(mainbrowser->plots);

 if (id <= 0)
    {nplot = fl_get_browser_maxline(mainbrowser->plots);
     for (i=1; i<=nplot; i++)
        {if (fl_isselected_browser_line(mainbrowser->plots,i)) {id = i; break;}}
    }

 if (id <= 0) return;

 if (arg == 0)  Dy->Clone = OFF;
 if (arg == 1)  Dy->Clone = ON;

 DyPlotInit(0,0);           /* Init of the Plot 0 (intermediate plot)           */
 DyCopyPlot(0,id);          /* Fill the Plot 0 with the current plot parameters */
 DyXformPlotToPanel(id);    /* Put current plot parameters into the panel       */

/**
 if (Plot[id]->type == PLOTCORR) DyXformRaisePanel(NULL,PLOTTIME);
 else                            DyXformRaisePanel(NULL,Plot[id]->type);
**/

 DyXformCleanPanel(NULL,Plot[id]->type);
 DyXformRaisePanel(NULL,Plot[id]->type);

 sprintf(dummy, "Edited plot %ld: name=%s  type=%s", id, Plot[id]->name, Dy->typeName[Plot[id]->type]);
 DyInfoLine (dummy);

 return;
}


/*-------------------------------------------- DyXformRemovePlot ---*/

void DyXformRemovePlot (FL_OBJECT *obj, long arg)
{
 char strnew[LSTRING_MAX];
 long nplot, i, i0, j, lastSelected=0;
 char dummy[LSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);

 nplot = fl_get_browser_maxline(mainbrowser->plots);
 if (nplot <= 0) return;

 for (i=1; i<=nplot-1; i++)
    {
     if (fl_isselected_browser_line(mainbrowser->plots,i) && Plot[i]->superposed == OFF)
        {for (j=i+1; j<=i+1; j++)
            {if (Plot[j]->type == Plot[i]->type && Plot[j]->superposed == ON)
                {Plot[j]->superposed = OFF;
                 sprintf (strnew, "%ld %s %s", i, Dy->typeName[Plot[j]->type], Plot[j]->name);
                 fl_replace_browser_line(mainbrowser->plots, i, strnew);
                }
             else break;
            }
        }
    }

 i0 = 1;
 while (i0 > 0)
   {
    nplot = fl_get_browser_maxline(mainbrowser->plots);
    if (nplot <= 0) break;

    i0 = -1;
    for (i=1; i<=nplot; i++) {if (fl_isselected_browser_line(mainbrowser->plots,i)) {i0 = i; break;}}

    if (i0 <= 0) break;
    else         lastSelected = i0;

    for (i=i0; i<=nplot-1; i++) DyCopyPlot(i,i+1);

    for (i=i0; i<=nplot-1; i++)
      {sprintf (strnew, "%ld %s %s", i, Dy->typeName[Plot[i]->type], Plot[i]->name);
       fl_replace_browser_line(mainbrowser->plots,i, strnew);
       if (fl_isselected_browser_line(mainbrowser->plots,i+1)) fl_select_browser_line(mainbrowser->plots,i);
       else                                                    fl_deselect_browser_line(mainbrowser->plots,i);
      }

   fl_delete_browser_line(mainbrowser->plots,nplot);
  }

 nplot = fl_get_browser_maxline(mainbrowser->plots);
 if (nplot > 0 && lastSelected > 0) fl_select_browser_line(mainbrowser->plots,lastSelected);

 if (nplot > 0)
    {for (i=1; i<=nplot; i++)
        {if (Plot[i]->superposed == ON)
            {sprintf (strnew, "%ld %s %s (s)", i, Dy->typeName[Plot[i]->type], Plot[i]->name);
             fl_replace_browser_line(mainbrowser->plots, i, strnew);
            }
        }
    }

 return;
}



/*------------------------------------------- DyXformClonePlot ---*/
                                                 /*  called from Main Browser  */
void DyXformClonePlot (FL_OBJECT *obj, long arg)
{
 int id, nplot, i;

 DyDebugPrint(__FUNCTION__,0);

 nplot = fl_get_browser_maxline(mainbrowser->plots);
 if (nplot <= 0) return;

 for (i=1; i<=nplot; i++)
    {if (fl_isselected_browser_line(mainbrowser->plots,i))
        {id = fl_get_browser_maxline(mainbrowser->plots);
         id++;
         DyCopyPlot (id,i);
         DyXformUpdatePlotBrowser(id, ADD_PLOT);
        }
    }

 return;
}


/*------------------------------------------ DyCopyPlot ---*/

void DyCopyPlot (long idout, long idin)
{
 char dummy[LSTRING_MAX];

 sprintf(dummy,"DyCopyPlot> in=%d=%s  out=%d=%s", idin, Plot[idin]->name, idout, Plot[idout]->name);
 DyDebugPrint(dummy,0);

/***
 memcpy(Plot[idout], Plot[idin], sizeof(struct DyPlot));
***/

 strcpy (Plot[idout]->name,  Plot[idin]->name);
 strcpy (Plot[idout]->vVarx, Plot[idin]->vVarx);
 strcpy (Plot[idout]->vVary, Plot[idin]->vVary);

 Plot[idout]->type          = Plot[idin]->type;
 Plot[idout]->hidden        = Plot[idin]->hidden;

 Plot[idout]->yoffset       = Plot[idin]->yoffset;
 Plot[idout]->yscale        = Plot[idin]->yscale;

 Plot[idout]->timeSize      = Plot[idin]->timeSize;
 Plot[idout]->tsize         = Plot[idin]->tsize;
 Plot[idout]->sampFreq      = Plot[idin]->sampFreq;
 Plot[idout]->minmax_resample = Plot[idin]->minmax_resample;
 Plot[idout]->tstep         = Plot[idin]->tstep;
 Plot[idout]->dfreq         = Plot[idin]->dfreq;
 Plot[idout]->dtime         = Plot[idin]->dtime;

 Plot[idout]->nBin          = Plot[idin]->nBin;
 Plot[idout]->nBinP2        = Plot[idin]->nBinP2;
 Plot[idout]->xnBin         = Plot[idin]->xnBin;
 Plot[idout]->xMin          = Plot[idin]->xMin;
 Plot[idout]->xMax          = Plot[idin]->xMax;

 Plot[idout]->ynBin         = Plot[idin]->ynBin;
 Plot[idout]->yMin          = Plot[idin]->yMin;
 Plot[idout]->yMax          = Plot[idin]->yMax;
 Plot[idout]->nSampling     = Plot[idin]->nSampling;

 Plot[idout]->zMax          = Plot[idin]->zMax;

 Plot[idout]->crossCohe_fmin = Plot[idin]->crossCohe_fmin;
 Plot[idout]->crossCohe_fmax = Plot[idin]->crossCohe_fmax;
 Plot[idout]->crossCohe      = Plot[idin]->crossCohe;

 Plot[idout]->filterMin     = Plot[idin]->filterMin;
 Plot[idout]->filterMax     = Plot[idin]->filterMax;

 Plot[idout]->refresh_percent = Plot[idin]->refresh_percent;

 Plot[idout]->refreshPeriod = Plot[idin]->refreshPeriod;
 Plot[idout]->average_fft   = Plot[idin]->average_fft;
 Plot[idout]->refresh_fft   = Plot[idin]->refresh_fft;
 Plot[idout]->resampFactor  = Plot[idin]->resampFactor;
 Plot[idout]->t0            = Plot[idin]->t0;
 Plot[idout]->duration      = Plot[idin]->duration;
 Plot[idout]->sound16bits   = Plot[idin]->sound16bits;
 Plot[idout]->soundonline   = Plot[idin]->soundonline;

 Plot[idout]->nodc          = Plot[idin]->nodc;
 Plot[idout]->bits          = Plot[idin]->bits;

 Plot[idout]->gridx         = Plot[idin]->gridx;
 Plot[idout]->gridy         = Plot[idin]->gridy;
 Plot[idout]->logx          = Plot[idin]->logx;
 Plot[idout]->logy          = Plot[idin]->logy;

 Plot[idout]->gridx2        = Plot[idin]->gridx2;
 Plot[idout]->gridy2        = Plot[idin]->gridy2;
 Plot[idout]->logx2         = Plot[idin]->logx2;
 Plot[idout]->logy2         = Plot[idin]->logy2;

 Plot[idout]->logz          = Plot[idin]->logz;
 Plot[idout]->stat          = Plot[idin]->stat;
 Plot[idout]->type2d        = Plot[idin]->type2d;
 Plot[idout]->autoEdge      = Plot[idin]->autoEdge;
 Plot[idout]->autoScale     = Plot[idin]->autoScale;
 Plot[idout]->modulesqrt    = Plot[idin]->modulesqrt;
 Plot[idout]->realUnits     = Plot[idin]->realUnits;
 Plot[idout]->superposed    = Plot[idin]->superposed;
 Plot[idout]->hertz         = Plot[idin]->hertz;
 Plot[idout]->rmsfft        = Plot[idin]->rmsfft;
 Plot[idout]->decayfft      = Plot[idin]->decayfft;

 Plot[idout]->freqClock     = Plot[idin]->freqClock;
 Plot[idout]->freqClock2    = Plot[idin]->freqClock2;
 Plot[idout]->freqClock12   = Plot[idin]->freqClock12;
 Plot[idout]->clockN        = Plot[idin]->clockN;
 Plot[idout]->clockN2       = Plot[idin]->clockN2;

 return;
}


/*--------------------------------------------------- DyCopyPlotForSame ---*/
/*  called from DyXformEditPlot for each plot superposedto the plot edited */

void DyCopyPlotForSame (struct DyPlot *target, struct DyPlot *source)
{
 long nbin;
 char dummy[LSTRING_MAX];

 if (source == NULL || target == NULL) return;

 sprintf(dummy,"DyCopyPlotForSame> in=%s  out=%s", source->name, target->name);
 DyDebugPrint(dummy,0);

 target->superposed = ON;

 target->bits       = source->bits;
 target->logx       = source->logx;
 target->logy       = source->logy;
 target->gridx      = source->gridx;
 target->gridy      = source->gridy;

 target->yMin       = source->yMin;
 target->yMax       = source->yMax;
 target->ynBin      = source->ynBin;
 target->tstep      = source->tstep;
 target->xnBin      = source->xnBin;
 target->nBinP2     = source->nBinP2;

 if (target->type == PLOTTIME) target->tsize = source->tsize;

 target->refreshPeriod = (long)(target->tstep*target->sampFreq*target->tsize / 100.);

 if (target->type == PLOTFFT || target->type == PLOTTR || target->type == PLOTCOHE)
    {target->xnBin = (int)(target->tsize * target->sampFreq);
     nbin = 2; target->nBinP2 = 0;
     while (nbin <= target->xnBin)  {nbin *= 2; target->nBinP2++;}
     target->xnBin = 1 << target->nBinP2;
     target->tsize = (double)(target->xnBin) / target->sampFreq;
     target->average_fft = source->average_fft;
     target->refresh_fft = source->refresh_fft;
    }

 if (target->type == PLOTTIME)
    {target->xnBin = (int)(target->tsize * target->freqClock);}

 if (target->type == PLOT1D || target->type == PLOT2D || target->type == PLOT3D)
    {target->refreshPeriod = (long)(target->tstep * target->freqClock);}

 if (target->type == PLOTTR || target->type == PLOTCOHE)
    {target->refreshPeriod = (long)(target->tstep*target->freqClock12*target->tsize / 100.);}

/**
 target->refreshPeriod = source->refreshPeriod / (long)(source->freqClock/target->freqClock + 0.5);
 target->nBin          = source->nBin;
 target->sampFreq      = source->sampFreq;
 target->resampFactor  = (long)(target->freqClock / target->sampFreq + 0.5);
 if (target->resampFactor == 0) target->resampFactor = 1;
**/

 return;
}


/*--------------------------------------- DyXformPlotToPanel ---*/

void DyXformPlotToPanel (long id)
{
 long plottype;
 char dummy[LSTRING_MAX];

 sprintf(dummy,"DyXformPlotToPanel> plot%d  name=%s  type=%s", id, Plot[id]->name, Dy->typeName[Plot[id]->type]);
 DyDebugPrint(dummy,0);

 plottype = Plot[id]->type;

 if (plottype == PLOTRAW)
    {
     strcpy(dummy,"\0");
     fl_set_input (plotraw->name, Plot[id]->name);
     sprintf(dummy, "%g", Plot[id]->yMin);
     fl_set_input (plotraw->yMin, dummy);
     sprintf(dummy, "%g", Plot[id]->yMax);
     fl_set_input (plotraw->yMax, dummy);
     sprintf(dummy, "%ld", Plot[id]->average_fft);
     fl_set_input (plotraw->average_fft, dummy);
     fl_set_button (plotraw->autoScale, Plot[id]->autoScale);
     fl_set_button (plotraw->realUnits, Plot[id]->realUnits);
     fl_set_button (plotrawtime->x2av, Plot[id]->x2av); 
     fl_set_button (plotraw->gridx, Plot[id]->gridx);
     fl_set_button (plotraw->gridy, Plot[id]->gridy);
     fl_set_button (plotraw->logx,  Plot[id]->logx);
     fl_set_button (plotraw->logy,  Plot[id]->logy);
    }

 if (plottype == PLOTRAWTIME)
    {
     fl_set_input (plotrawtime->name, Plot[id]->name);
     sprintf (dummy, "%ld", Plot[id]->xnBin);
     fl_set_input (plotrawtime->xnBin, dummy);
     sprintf(dummy, "%g", Plot[id]->xMin);
     fl_set_input (plotrawtime->xMin, dummy);
     sprintf(dummy, "%g", Plot[id]->xMax);
     fl_set_input (plotrawtime->xMax, dummy);
     sprintf(dummy, "%g", Plot[id]->zMin);
     fl_set_input (plotrawtime->zmin, dummy);
     sprintf(dummy, "%g", Plot[id]->zMax);
     fl_set_input (plotrawtime->zmax, dummy);
     sprintf(dummy, "%g", Plot[id]->refresh_percent);
     fl_set_input (plotrawtime->refresh_percent, dummy);
     sprintf(dummy,"%g",Plot[id]->timeSize);
     fl_set_input (plotrawtime->timeSize, dummy);
     sprintf(dummy,"%d",Plot[id]->average_fft);
     fl_set_input (plotrawtime->average_fft, dummy);
     fl_set_button (plotrawtime->autoScale, Plot[id]->autoScale);
     fl_set_button (plotrawtime->realUnits, Plot[id]->realUnits);
     fl_set_button (plotrawtime->x2av, Plot[id]->x2av); 
     fl_set_button (plotrawtime->gridx, Plot[id]->gridx);
     fl_set_button (plotrawtime->gridy, Plot[id]->gridy);
     fl_set_button (plotrawtime->gridz, Plot[id]->gridz);
     fl_set_button (plotrawtime->logx,  Plot[id]->logx);
     fl_set_button (plotrawtime->logy,  Plot[id]->logy);
     fl_set_button (plotrawtime->logz,  Plot[id]->logz);
    }

 if (plottype == PLOTTIME  ||  plottype == PLOTCORR)
    {
     strcpy(dummy,"\0");
     fl_set_input (plottime->name, Plot[id]->name);
     sprintf(dummy,"%g",Plot[id]->tsize);
     fl_set_input (plottime->tsize, dummy);
     sprintf(dummy,"%g",Plot[id]->tstep);
     fl_set_input (plottime->tstep, dummy);
     sprintf(dummy,"%g",Plot[id]->sampFreq);
     fl_set_input (plottime->sampFreq, dummy);
     sprintf(dummy, "%g", Plot[id]->yMin);
     fl_set_input (plottime->yMin, dummy);
     sprintf(dummy, "%g", Plot[id]->yMax);
     fl_set_input (plottime->yMax, dummy);
     sprintf(dummy, "%g", Plot[id]->filterMin);
     fl_set_input (plottime->filterMin, dummy);
     sprintf(dummy, "%g", Plot[id]->filterMax);
     fl_set_input (plottime->filterMax, dummy);
     sprintf(dummy,"%g",Plot[id]->yoffset);
     fl_set_input (plottime->yoffset, dummy);
     sprintf(dummy,"%g",Plot[id]->yscale);
     fl_set_input (plottime->yscale, dummy);
     fl_set_button (plottime->bits, Plot[id]->bits);
     fl_set_button (plottime->nodc, Plot[id]->nodc);
     fl_set_button (plottime->autoScale, Plot[id]->autoScale);
     fl_set_button (plottime->realUnits, Plot[id]->realUnits);
     fl_set_button (plottime->gridx, Plot[id]->gridx);
     fl_set_button (plottime->gridy, Plot[id]->gridy);
     fl_set_button (plottime->logx,  Plot[id]->logx);
     fl_set_button (plottime->logy,  Plot[id]->logy);
     fl_set_button (plottime->minmax_resample, Plot[id]->minmax_resample);
    }

 if (plottype == PLOTFFT)
    {
     fl_set_input (plotfft->name, Plot[id]->name);
     sprintf(dummy,"%g",Plot[id]->tsize);
     fl_set_input (plotfft->tsize, dummy);
     sprintf(dummy,"%g",Plot[id]->tstep);
     fl_set_input (plotfft->tstep, dummy);
     sprintf(dummy,"%g",Plot[id]->sampFreq);
     fl_set_input (plotfft->sampFreq, dummy);
     sprintf(dummy, "%g", Plot[id]->xMin);
     fl_set_input (plotfft->xMin, dummy);
     sprintf(dummy, "%g", Plot[id]->xMax);
     fl_set_input (plotfft->xMax, dummy);
     sprintf(dummy, "%ld", Plot[id]->average_fft);
     fl_set_input (plotfft->average_fft, dummy);
     sprintf(dummy, "%ld", Plot[id]->refresh_fft);
     fl_set_input (plotfft->refresh_fft, dummy);
     sprintf(dummy, "%g", Plot[id]->yMin);
     fl_set_input (plotfft->yMin, dummy);
     sprintf(dummy, "%g", Plot[id]->yMax);
     fl_set_input (plotfft->yMax, dummy);
     fl_set_button (plotfft->hertz, Plot[id]->hertz);
     fl_set_button (plotfft->rmsfft, Plot[id]->rmsfft);
     fl_set_button (plotfft->decayfft, Plot[id]->decayfft);
     fl_set_button (plotfft->nodc, Plot[id]->nodc);
     fl_set_button (plotfft->realUnits, Plot[id]->realUnits);
     fl_set_button (plotfft->autoScale, Plot[id]->autoScale);
     fl_set_button (plotfft->gridx, Plot[id]->gridx);
     fl_set_button (plotfft->gridy, Plot[id]->gridy);
     fl_set_button (plotfft->logx,  Plot[id]->logx);
     fl_set_button (plotfft->logy,  Plot[id]->logy);
     fl_set_button (plotfft->minmax_resample, Plot[id]->minmax_resample);
    }

 if (plottype == PLOTTR  ||  plottype == PLOTCOHE)
    {
     fl_set_input (plottrfct->name, Plot[id]->name);
     sprintf(dummy,"%g",Plot[id]->tsize);
     fl_set_input (plottrfct->tsize, dummy);
     sprintf(dummy,"%g",Plot[id]->tstep);
     fl_set_input (plottrfct->tstep, dummy);
     sprintf(dummy,"%g",Plot[id]->sampFreq);
     fl_set_input (plottrfct->sampFreq, dummy);
     sprintf(dummy, "%g", Plot[id]->xMin);
     fl_set_input (plottrfct->xMin, dummy);
     sprintf(dummy, "%g", Plot[id]->xMax);
     fl_set_input (plottrfct->xMax, dummy);
     sprintf(dummy, "%ld", Plot[id]->average_fft);
     fl_set_input (plottrfct->average_fft, dummy);
     sprintf(dummy, "%ld", Plot[id]->refresh_fft);
     fl_set_input (plottrfct->refresh_fft, dummy);
     sprintf(dummy, "%g", Plot[id]->yMin);
     fl_set_input (plottrfct->yMin, dummy);
     sprintf(dummy, "%g", Plot[id]->yMax);
     fl_set_input (plottrfct->yMax, dummy);
     sprintf(dummy, "%g", Plot[id]->crossCohe_fmin);
     fl_set_input (plottrfct->crossCohe_fmin, dummy);
     sprintf(dummy, "%g", Plot[id]->crossCohe_fmax);
     fl_set_input (plottrfct->crossCohe_fmax, dummy);
     fl_set_button (plottrfct->crossCohe, Plot[id]->crossCohe);
     fl_set_button (plottrfct->nodc, Plot[id]->nodc);
     fl_set_button (plottrfct->realUnits, Plot[id]->realUnits);
     fl_set_button (plottrfct->autoScale, Plot[id]->autoScale);
     fl_set_button (plottrfct->modulesqrt, Plot[id]->modulesqrt);
     fl_set_button (plottrfct->decayfft, Plot[id]->decayfft);
     fl_set_button (plottrfct->gridx, Plot[id]->gridx);
     fl_set_button (plottrfct->gridy, Plot[id]->gridy);
     fl_set_button (plottrfct->logx,  Plot[id]->logx);
     fl_set_button (plottrfct->logy,  Plot[id]->logy);
     fl_set_button (plottrfct->gridx2, Plot[id]->gridx2);
     fl_set_button (plottrfct->gridy2, Plot[id]->gridy2);
     fl_set_button (plottrfct->logx2,  Plot[id]->logx2);
     fl_set_button (plottrfct->logy2,  Plot[id]->logy2);
     fl_set_button (plottrfct->minmax_resample, Plot[id]->minmax_resample);
    }

 if (plottype == PLOTFFTTIME || plottype == PLOTTRTIME || plottype == PLOTCOHETIME)
    {
     fl_set_input (plotffttime->name, Plot[id]->name);
     sprintf(dummy,"%g",Plot[id]->dfreq);
     fl_set_input (plotffttime->dfreq, dummy);
     sprintf(dummy,"%g",Plot[id]->dtime);
     fl_set_input (plotffttime->dtime, dummy);
     sprintf(dummy,"%g",Plot[id]->tstep);
     fl_set_input (plotffttime->tstep, dummy);
     sprintf(dummy,"%g",Plot[id]->sampFreq);
     fl_set_input (plotffttime->sampFreq, dummy);
     sprintf(dummy, "%g", Plot[id]->xMin);
     fl_set_input (plotffttime->xMin, dummy);
     sprintf(dummy, "%g", Plot[id]->xMax);
     fl_set_input (plotffttime->xMax, dummy);
     sprintf(dummy, "%g", Plot[id]->zMin);
     fl_set_input (plotffttime->zmin, dummy);
     sprintf(dummy, "%g", Plot[id]->zMax);
     fl_set_input (plotffttime->zmax, dummy);
     sprintf(dummy, "%g", Plot[id]->refresh_percent);
     fl_set_input (plotffttime->refresh_percent, dummy);
     sprintf(dummy,"%g",Plot[id]->timeSize);
     fl_set_input (plotffttime->timeSize, dummy);
     fl_set_button (plotffttime->autoScale, Plot[id]->autoScale);
     fl_set_button (plotffttime->realUnits, Plot[id]->realUnits);
     fl_set_button (plotffttime->nodc, Plot[id]->nodc);
     fl_set_button (plotffttime->gridx, Plot[id]->gridx);
     fl_set_button (plotffttime->gridy, Plot[id]->gridy);
     fl_set_button (plotffttime->gridz, Plot[id]->gridz);
     fl_set_button (plotffttime->logx,  Plot[id]->logx);
     fl_set_button (plotffttime->logy,  Plot[id]->logy);
     fl_set_button (plotffttime->logz,  Plot[id]->logz);
     fl_set_button (plotffttime->minmax_resample, Plot[id]->minmax_resample);
    }

 if (plottype == PLOT1DTIME)
    {
     fl_set_input (plot1dtime->name, Plot[id]->name);
     sprintf(dummy,"%g",Plot[id]->tsize);
     fl_set_input (plot1dtime->tsize, dummy);
     sprintf(dummy,"%g",Plot[id]->sampFreq);
     fl_set_input (plot1dtime->sampFreq, dummy);
     sprintf (dummy, "%ld", Plot[id]->xnBin);
     fl_set_input (plot1dtime->xnBin, dummy);
     sprintf(dummy, "%g", Plot[id]->xMin);
     fl_set_input (plot1dtime->xMin, dummy);
     sprintf(dummy, "%g", Plot[id]->xMax);
     fl_set_input (plot1dtime->xMax, dummy);
     sprintf(dummy, "%g", Plot[id]->zMin);
     fl_set_input (plot1dtime->zmin, dummy);
     sprintf(dummy, "%g", Plot[id]->zMax);
     fl_set_input (plot1dtime->zmax, dummy);
     sprintf(dummy, "%g", Plot[id]->refresh_percent);
     fl_set_input (plot1dtime->refresh_percent, dummy);
     sprintf(dummy,"%g",Plot[id]->timeSize);
     fl_set_input (plot1dtime->timeSize, dummy);
     fl_set_button (plot1dtime->autoEdge, Plot[id]->autoEdge);
     fl_set_button (plot1dtime->autoScale, Plot[id]->autoScale);
     fl_set_button (plot1dtime->realUnits, Plot[id]->realUnits);
     fl_set_button (plot1dtime->nodc, Plot[id]->nodc);
     fl_set_button (plot1dtime->gridx, Plot[id]->gridx);
     fl_set_button (plot1dtime->gridy, Plot[id]->gridy);
     fl_set_button (plot1dtime->gridz, Plot[id]->gridz);
     fl_set_button (plot1dtime->logx,  Plot[id]->logx);
     fl_set_button (plot1dtime->logy,  Plot[id]->logy);
     fl_set_button (plot1dtime->logz,  Plot[id]->logz);
     fl_set_button (plot1dtime->minmax_resample, Plot[id]->minmax_resample);
    }

 if (plottype == PLOT1D)
    {
     fl_set_input (plot1d->name, Plot[id]->name);
     sprintf (dummy,"%g",Plot[id]->tstep);
     fl_set_input (plot1d->tstep, dummy);
     sprintf(dummy,"%g",Plot[id]->sampFreq);
     fl_set_input (plot1d->sampFreq, dummy);
     sprintf (dummy, "%ld", Plot[id]->xnBin);
     fl_set_input (plot1d->xnBin, dummy);
     sprintf (dummy, "%g", Plot[id]->xMin);
     fl_set_input (plot1d->xMin, dummy);
     sprintf (dummy, "%g", Plot[id]->xMax);
     fl_set_input (plot1d->xMax, dummy);
     fl_set_button (plot1d->autoEdge, Plot[id]->autoEdge);
     fl_set_button (plot1d->realUnits, Plot[id]->realUnits);
     fl_set_button (plot1d->gridx, Plot[id]->gridx);
     fl_set_button (plot1d->gridy, Plot[id]->gridy);
     fl_set_button (plot1d->logx, Plot[id]->logx);
     fl_set_button (plot1d->logy, Plot[id]->logy);
     fl_set_button (plot1d->stat, Plot[id]->stat);
     fl_set_button (plot1d->minmax_resample, Plot[id]->minmax_resample);
    }

 if (plottype == PLOT2D || plottype == PLOT3D)
    {
     fl_set_input (plot2d->name, Plot[id]->name);
     sprintf(dummy,"%g",Plot[id]->tstep);
     fl_set_input (plot2d->tstep, dummy);
     sprintf(dummy, "%ld", Plot[id]->xnBin);
     fl_set_input (plot2d->xnBin, dummy);
     sprintf(dummy, "%g", Plot[id]->xMin);
     fl_set_input (plot2d->xMin, dummy);
     sprintf(dummy, "%g", Plot[id]->xMax);
     fl_set_input (plot2d->xMax, dummy);
     sprintf(dummy, "%ld", Plot[id]->ynBin);
     fl_set_input (plot2d->ynBin, dummy);
     sprintf(dummy, "%g", Plot[id]->yMin);
     fl_set_input (plot2d->yMin, dummy);
     sprintf(dummy, "%g", Plot[id]->yMax);
     fl_set_input (plot2d->yMax, dummy);
     sprintf(dummy, "%ld", Plot[id]->nSampling);
     fl_set_input (plot2d->nSampling, dummy);
     fl_set_button (plot2d->autoEdge, Plot[id]->autoEdge);
     fl_set_button (plot2d->realUnits, Plot[id]->realUnits);
     fl_set_button (plot2d->gridx, Plot[id]->gridx);
     fl_set_button (plot2d->gridy, Plot[id]->gridy);
     fl_set_button (plot2d->logx,  Plot[id]->logx);
     fl_set_button (plot2d->logy,  Plot[id]->logy);
     fl_set_button (plot2d->logz,  Plot[id]->logz);
     fl_set_button (plot2d->stat,  Plot[id]->stat);
     if (Plot[id]->type2d == COL)   fl_set_button (plot2d->col,   ON);
     if (Plot[id]->type2d == CONT)  fl_set_button (plot2d->cont,  ON);
     if (Plot[id]->type2d == LEGO)  fl_set_button (plot2d->lego,  ON);
     if (Plot[id]->type2d == SURF)  fl_set_button (plot2d->surf,  ON);
     if (Plot[id]->type2d == SCAT)  fl_set_button (plot2d->scat,  ON);
    }

 if (plottype == PLOTAUDIO)
    {
     fl_set_input (plotaudio->name, Plot[id]->name);
     sprintf(dummy, "%ld", Plot[id]->t0);
     fl_set_input (plotaudio->t0, dummy);
     sprintf(dummy, "%ld", Plot[id]->duration);
     fl_set_input (plotaudio->duration, dummy);
     fl_set_button (plotaudio->sound16bits, Plot[id]->sound16bits);
     fl_set_button (plotaudio->soundonline, Plot[id]->soundonline);
    }

 return;
}


/*----------------------------------------- DyXformGetGeneParPanel ---*/

void DyXformGetGeneParPanel (FL_OBJECT *obj, long arg)
{
 char dummy[LSTRING_MAX];
 int i0, id, nPlot;
 static int firstTime=1;

 DyDebugPrint(__FUNCTION__,0);

 if (firstTime == 1)
    {
     firstTime = 0;

     nPlot = fl_get_browser_maxline(mainbrowser->plots);

     i0 = nPlot;
     for (id=1; id<=nPlot; id++)
        {
         if (Plot[id]->type == PLOTTIME    ||  Plot[id]->type == PLOTFFT  ||
             Plot[id]->type == PLOTTR      ||  Plot[id]->type == PLOTCOHE ||
             Plot[id]->type == PLOTFFTTIME ||  Plot[id]->type == PLOTCORR || 
             Plot[id]->type == PLOTTRTIME  ||  Plot[id]->type == PLOTCOHETIME)
            {i0 = id; break;}
        }

     strcpy(dummy,"\0");

     sprintf(dummy,"%g",Plot[i0]->tsize);
     fl_set_input (plotgenepar->tsize, dummy);

     printf("DyXformGetGeneParPanel> i0=%d ploti0=%s tstep=%g \n", i0, Plot[i0]->name, Plot[i0]->tstep);

     sprintf(dummy,"%g",Plot[i0]->tstep);
     fl_set_input (plotgenepar->tstep, dummy);

     sprintf(dummy,"%g",Plot[i0]->sampFreq);
     fl_set_input (plotgenepar->sampFreq, dummy);

     fl_set_button (plotgenepar->autoEdge, Plot[i0]->autoEdge);
     fl_set_button (plotgenepar->realUnits, Plot[i0]->realUnits);
     fl_set_button (plotgenepar->nodc, Plot[i0]->nodc);
     fl_set_button (plotgenepar->gridx, Plot[i0]->gridx);
     fl_set_button (plotgenepar->gridy, Plot[i0]->gridy);
     fl_set_button (plotgenepar->logx, Plot[i0]->logx);
     fl_set_button (plotgenepar->logy, Plot[i0]->logy);
     fl_set_button (plotgenepar->minmax_resample, Plot[i0]->minmax_resample);
    }

 Plot[0]->sampFreq  = atof(fl_get_input(plotgenepar->sampFreq));
 Plot[0]->tsize     = atof(fl_get_input(plotgenepar->tsize));
 Plot[0]->tstep     = atof(fl_get_input(plotgenepar->tstep));

 Plot[0]->autoScale = fl_get_button (plotgenepar->autoScale);
 Plot[0]->autoEdge  = fl_get_button (plotgenepar->autoEdge);
 Plot[0]->realUnits = fl_get_button (plotgenepar->realUnits);
 Plot[0]->decayfft  = fl_get_button (plotgenepar->decayfft);
 Plot[0]->nodc      = fl_get_button (plotgenepar->nodc);
 Plot[0]->gridx     = fl_get_button (plotgenepar->gridx);
 Plot[0]->gridy     = fl_get_button (plotgenepar->gridy);
 Plot[0]->logx      = fl_get_button (plotgenepar->logx);
 Plot[0]->logy      = fl_get_button (plotgenepar->logy);
 Plot[0]->minmax_resample = fl_get_button (plotgenepar->minmax_resample);

 DyInfoLine ("Edited general parameters of all plots");
 DyXformRaisePanel(NULL,GENEPAR);

 return;
}


/*----------------------------------------- DyXformGeneParToPlot ---*/

void DyXformGeneParToPlot (FL_OBJECT *obj, long arg)
{
 long nbin, id, nPlot;
 double tsize=0., tstep=0.;
 double sampFreq=0.;
 long nodc;
 long gridx, gridy, logx, logy;
 long autoEdge, autoScale, realUnits, decayfft, minmax_resample, all_clone;
 double freqClock;

 DyDebugPrint(__FUNCTION__,0);

 sampFreq  = atof(fl_get_input(plotgenepar->sampFreq));
 tsize     = atof(fl_get_input(plotgenepar->tsize));
 tstep     = atof(fl_get_input(plotgenepar->tstep));

 autoScale = fl_get_button (plotgenepar->autoScale);
 autoEdge  = fl_get_button (plotgenepar->autoEdge);
 realUnits = fl_get_button (plotgenepar->realUnits);
 nodc      = fl_get_button (plotgenepar->nodc);
 decayfft  = fl_get_button (plotgenepar->decayfft);
 gridx     = fl_get_button (plotgenepar->gridx);
 gridy     = fl_get_button (plotgenepar->gridy);
 logx      = fl_get_button (plotgenepar->logx);
 logy      = fl_get_button (plotgenepar->logy);
 minmax_resample = fl_get_button (plotgenepar->minmax_resample);

 all_clone = fl_get_button (plotgenepar->all_clone);

 nPlot = fl_get_browser_maxline(mainbrowser->plots);

 for (id=1; id<=nPlot; id++)
    {
     if (Plot[id]->type == PLOTTIME    ||  Plot[id]->type == PLOTFFT  ||
         Plot[id]->type == PLOTTR      ||  Plot[id]->type == PLOTCOHE ||
         Plot[id]->type == PLOTFFTTIME ||  Plot[id]->type == PLOTCORR ||
         Plot[id]->type == PLOTTRTIME  ||  Plot[id]->type == PLOTCOHETIME)
        {
         if (Plot[0]->sampFreq != sampFreq  ||  all_clone > 0)
            {
             if (Plot[id]->type == PLOTTR  ||  Plot[id]->type == PLOTCOHE) freqClock = Plot[id]->freqClock12;
             else                                                          freqClock = Plot[id]->freqClock;
             Plot[id]->sampFreq = sampFreq;
             if (Plot[id]->sampFreq <= 0  ||  Plot[id]->sampFreq > freqClock)
                {Plot[id]->sampFreq = freqClock;}
             Plot[id]->resampFactor = (long)(freqClock / Plot[id]->sampFreq + 0.5);
             Plot[id]->sampFreq     = freqClock / (double)Plot[id]->resampFactor;
            }

         if (Plot[0]->tsize != tsize  ||  all_clone > 0)
            {
             Plot[id]->tsize = tsize;
             if (Plot[id]->tsize <= 0.)  Plot[id]->tsize = 1.;
             Plot[id]->xnBin  = (long)(Plot[id]->tsize * Plot[id]->freqClock) / Plot[id]->resampFactor;
             if (Plot[id]->xnBin < 32) Plot[id]->xnBin = 32;
             nbin = 2; Plot[id]->nBinP2 = 0;
             while (nbin <= Plot[id]->xnBin)  {nbin *= 2; Plot[id]->nBinP2++;}
             Plot[id]->xnBin = 1 << Plot[id]->nBinP2;
             Plot[id]->tsize = Plot[id]->xnBin * (double)Plot[id]->resampFactor / Plot[id]->freqClock;
            }

         if (Plot[0]->tstep != tstep  ||  all_clone > 0)
            {
             Plot[id]->tstep = tstep;
             if (Plot[id]->tstep <= 0.  ||  Plot[id]->tstep > 100.)  Plot[id]->tstep = 100.;
             Plot[id]->refreshPeriod = (long)(Plot[id]->tstep * Plot[id]->freqClock
                                              * Plot[id]->tsize / 100.) / Plot[id]->resampFactor;
            }
        }

     if (Plot[0]->minmax_resample != minmax_resample ||  all_clone > 0)  Plot[id]->minmax_resample = minmax_resample;
     if (Plot[0]->autoEdge  != autoEdge   ||  all_clone > 0)  Plot[id]->autoEdge  = autoEdge;
     if (Plot[0]->autoScale != autoScale  ||  all_clone > 0)  Plot[id]->autoScale = autoScale;
     if (Plot[0]->realUnits != realUnits  ||  all_clone > 0)  Plot[id]->realUnits = realUnits;
     if (Plot[0]->nodc      != nodc       ||  all_clone > 0)  Plot[id]->nodc      = nodc;
     if (Plot[0]->decayfft  != decayfft   ||  all_clone > 0)  Plot[id]->decayfft  = decayfft;
     if (Plot[0]->gridx     != gridx      ||  all_clone > 0) {Plot[id]->gridx = gridx; Plot[id]->gridx2 = gridx;}
     if (Plot[0]->gridy     != gridy      ||  all_clone > 0) {Plot[id]->gridy = gridy; Plot[id]->gridy2 = gridy;}
     if (Plot[0]->logx      != logx       ||  all_clone > 0) {Plot[id]->logx = logx; Plot[id]->logx2 = logx;}
     if (Plot[0]->logy      != logy       ||  all_clone > 0) {Plot[id]->logy = logy; Plot[id]->logy2 = logy;}
    }

 DyInfoLine ("Updated general parameters in all plots");
 DyXformClosePanel (plotgenepar->ok,0);

 return;
}


/*----------------------------------------- DyXformPutDefaultInAllPlots ---*/

void DyXformPutDefaultInAllPlots (FL_OBJECT *obj, long arg)
{
 long id, nPlot, answer=1;

 answer = fl_show_question ("Do you really want to put default parameters in all the plots ?",1);
 if (!answer) return;

 nPlot = fl_get_browser_maxline(mainbrowser->plots);

 for (id=1; id<=nPlot; id++) DyPutDefaultInPlot(id);

 DyInfoLine ("Put default values in all Plots");
 DyXformClosePanel (plotgenepar->close,0);

 return;
}


/*----------------------------------------- DyXformPanelToPlot ---*/

long DyXformPanelToPlot (long id)
{
 char name[LSTRING_MAX], *varxName, *varyName, dummy[LSTRING_MAX];
 long plottype, i, nplot;
 long xnBin=0, ynBin=0, average_fft=0, nbin;
 double xMin=0., xMax=0., yMin=0., yMax=0.;
 double tsize=0., tstep=0., timeSize=0., dtime=0., dfreq=0.;
 double sampFreq=0., crossCohe_fmin, crossCohe_fmax;
 long crossCohe;
 long sound16bits, soundonline, x2av, nodc, hertz, rmsfft, decayfft, t0=0, duration=0;
 long gridx, gridy, gridz, logx, logy, logz, bits;
 long gridx2, gridy2, logx2, logy2;
 long stat, type2d, autoEdge, autoScale, modulesqrt, realUnits, nSampling;
 double freqMin, filterMin, filterMax, zMin, zMax, refresh_percent;
 double yoffset, yscale, freqmin;
 long refresh_fft, minmax_resample;

 sprintf(dummy,"DyXformPanelToPlot> plot%d: name=%s  type=%s", id, Plot[id]->name, Dy->typeName[Plot[id]->type]);
 DyDebugPrint(dummy,0);

 plottype = Plot[id]->type;

 gridx = OFF;  gridy  = OFF;  nodc     = OFF;
 logx  = OFF;  logy   = OFF;  logz     = OFF; bits      = OFF;
 stat  = OFF;  type2d = 0;    autoEdge = OFF; autoScale = OFF; realUnits = OFF; minmax_resample = OFF;

 if (plottype == PLOTRAW)
    {
     if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>0) printf("DyXformPanelToPlot> Get parameters from panel plotraw \n");
     strcpy (name, fl_get_input(plotraw->name));
     if (strcmp(Plot[0]->name, name) == 0)  strcpy(Plot[id]->name,name);

     yMin = atof(fl_get_input(plotraw->yMin));
     if (Plot[0]->yMin != yMin)  Plot[id]->yMin = yMin;

     yMax = atof(fl_get_input(plotraw->yMax));
     if (Plot[0]->yMax != yMax)  Plot[id]->yMax = yMax;

     average_fft = atol(fl_get_input (plotraw->average_fft));
     if (Plot[0]->average_fft != average_fft)  Plot[id]->average_fft = average_fft;
     if (Plot[id]->average_fft < 0)  Plot[id]->average_fft = 0;  /* 0 is for infinite average */

     autoScale = fl_get_button (plotraw->autoScale);
     realUnits = fl_get_button (plotraw->realUnits);
     x2av      = fl_get_button (plotrawtime->x2av);

     gridx     = fl_get_button (plotraw->gridx);
     gridy     = fl_get_button (plotraw->gridy);
     logx      = fl_get_button (plotraw->logx);
     logy      = fl_get_button (plotraw->logy);
    }


 if (plottype == PLOTRAWTIME)
    {
     if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>0) printf("DyXformPanelToPlot> Get parameters from panel plotrawtime \n");
     strcpy (name, fl_get_input(plotrawtime->name));
     if (strcmp(Plot[0]->name, name) == 0)  strcpy(Plot[id]->name, name);

     xnBin = atol(fl_get_input(plotrawtime->xnBin));
     if (Plot[0]->xnBin != xnBin) Plot[id]->xnBin = xnBin;
     if (fl_get_button (plotrawtime->xnBin_clone) == ON)  Plot[id]->xnBin = xnBin;
     if (Plot[id]->xnBin < 2)  Plot[id]->xnBin = Plot[id]->nBin / 4;

     xMin = atof(fl_get_input(plotrawtime->xMin));
     if (Plot[0]->xMin != xMin)  Plot[id]->xMin = xMin;
     if (fl_get_button (plotrawtime->xMin_clone) == ON)  Plot[id]->xMin = xMin;

     xMax = atof(fl_get_input(plotrawtime->xMax));
     if (Plot[0]->xMax != xMax)  Plot[id]->xMax = xMax;
     if (fl_get_button (plotrawtime->xMax_clone) == ON)  Plot[id]->xMax = xMax;
     if (Plot[id]->xMax < Plot[id]->xMin) Plot[id]->xMax = Plot[id]->xMin;

     zMin = atof(fl_get_input(plotrawtime->zmin));
     if (Plot[0]->zMin != zMin)  Plot[id]->zMin = zMin;
     if (fl_get_button (plotrawtime->zmin_clone) == ON)  Plot[id]->zMin = zMin;

     zMax = atof(fl_get_input(plotrawtime->zmax));
     if (Plot[0]->zMax != zMax)  Plot[id]->zMax = zMax;
     if (fl_get_button (plotrawtime->zmax_clone) == ON)  Plot[id]->zMax = zMax;

     average_fft = atol(fl_get_input (plotrawtime->average_fft));
     if (Plot[0]->average_fft != average_fft)  Plot[id]->average_fft = average_fft;
     if (fl_get_button (plotrawtime->average_fft_clone) == ON)  Plot[id]->average_fft = average_fft;
     if (Plot[id]->average_fft < 0)  Plot[id]->average_fft = 0;  /* 0 is for infinite average */

     refresh_percent = atof(fl_get_input(plotrawtime->refresh_percent));
     if (Plot[0]->refresh_percent != refresh_percent)  Plot[id]->refresh_percent = refresh_percent;
     if (plottype == PLOTRAWTIME && fl_get_button (plotrawtime->refresh_percent_clone) == ON)  Plot[id]->refresh_percent = refresh_percent;

     timeSize = atof(fl_get_input(plotrawtime->timeSize));
     if (Plot[0]->timeSize != timeSize)  Plot[id]->timeSize = timeSize;
     if (fl_get_button (plotrawtime->timeSize_clone) == ON)  Plot[id]->timeSize = timeSize;
     if (Plot[id]->timeSize < 0.) Plot[id]->timeSize = 256. * Dy->Frame->dt;

     autoScale = fl_get_button (plotrawtime->autoScale);
     realUnits = fl_get_button (plotrawtime->realUnits);
     x2av      = fl_get_button (plotrawtime->x2av);

     gridx     = fl_get_button (plotrawtime->gridx);
     gridy     = fl_get_button (plotrawtime->gridy);
     gridz     = fl_get_button (plotrawtime->gridz);
     logx      = fl_get_button (plotrawtime->logx);
     logy      = fl_get_button (plotrawtime->logy);
     logz      = fl_get_button (plotrawtime->logz);
    }


 if (plottype == PLOTTIME  ||  plottype == PLOTCORR)
    {
     if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>0) printf("DyXformPanelToPlot> Get parameters from panel plottime \n");
     strcpy (name, fl_get_input(plottime->name));
     if (strcmp(Plot[0]->name, name) == 0)  strcpy(Plot[id]->name,name);

     yoffset = atof(fl_get_input(plottime->yoffset));
     if (Plot[0]->yoffset != yoffset)  Plot[id]->yoffset = yoffset;
     if (fl_get_button (plottime->yoffset_clone) == ON)  Plot[id]->yoffset = yoffset;

     yscale = atof(fl_get_input(plottime->yscale));
     if (Plot[0]->yscale != yscale)  Plot[id]->yscale = yscale;
     if (fl_get_button (plottime->yscale_clone) == ON)  Plot[id]->yscale = yscale;

     tsize = atof(fl_get_input(plottime->tsize));
     if (Plot[0]->tsize != tsize)  Plot[id]->tsize = tsize;
     if (fl_get_button (plottime->tsize_clone) == ON)  Plot[id]->tsize = tsize;
     if (Plot[id]->tsize <= 0.)  Plot[id]->tsize = 1.;

     sampFreq = atof(fl_get_input(plottime->sampFreq));
     if (Plot[0]->sampFreq != sampFreq)  Plot[id]->sampFreq = sampFreq;
     if (fl_get_button (plottime->sampFreq_clone) == ON)  Plot[id]->sampFreq = sampFreq;

     if (Plot[id]->sampFreq <= 0.  ||  Plot[id]->sampFreq > Plot[id]->freqClock)
        {Plot[id]->sampFreq = Plot[id]->freqClock;}

     Plot[id]->resampFactor = (long)(Plot[id]->freqClock / Plot[id]->sampFreq + 0.5);
     Plot[id]->sampFreq     = Plot[id]->freqClock / (double)Plot[id]->resampFactor;

     Plot[id]->xnBin = (long)(Plot[id]->tsize * Plot[id]->freqClock);

     if (plottype == PLOTCORR)
        {if (Plot[id]->xnBin < 256)
            {Plot[id]->xnBin = 256;
             Plot[id]->tsize = (double)Plot[id]->xnBin * (double)Plot[id]->resampFactor
                             / Plot[id]->freqClock;
            }
         nbin = 2; Plot[id]->nBinP2 = 0;
         while (nbin <= Plot[id]->xnBin)  {nbin *= 2; Plot[id]->nBinP2++;}
         Plot[id]->xnBin = 1 << Plot[id]->nBinP2;
        }

     tstep = atof(fl_get_input(plottime->tstep));
     if (Plot[0]->tstep != tstep)  Plot[id]->tstep = tstep;
     if (fl_get_button (plottime->tstep_clone) == ON)  Plot[id]->tstep = tstep;

     if (Plot[id]->tstep <= 0.  ||  Plot[id]->tstep > 100.)  Plot[id]->tstep = 100.;
     Plot[id]->refreshPeriod  = (long)(Plot[id]->tstep * Plot[id]->freqClock
                                       * Plot[id]->tsize / 100.) / Plot[id]->resampFactor;

     if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>1)
        {printf("DyXformPanelToPlot: id=%ld Plot[id]->refreshPeriod=%ld  Plot[id]->tstep=%g  Plot[id]->freqClock=%g"
                " Plot[id]->tsize=%g  Plot[id]->resampFactor=%ld  num=%g \n",
                id, Plot[id]->refreshPeriod, Plot[id]->tstep, Plot[id]->freqClock,
                Plot[id]->tsize, Plot[id]->resampFactor, Plot[id]->tstep * Plot[id]->freqClock * Plot[id]->tsize / 100.);
        }

     if (Plot[id]->refreshPeriod == 0)
        {Plot[id]->refreshPeriod = 1;
         Plot[id]->tstep = (double)Plot[id]->refreshPeriod * ((double)Plot[id]->resampFactor
                         / Plot[id]->freqClock) * (100. / Plot[id]->tsize);
        }

     yMin = atof(fl_get_input(plottime->yMin));
     if (Plot[0]->yMin != yMin)  Plot[id]->yMin = yMin;
     if (fl_get_button (plottime->yMin_clone) == ON)  Plot[id]->yMin = yMin;

     yMax = atof(fl_get_input(plottime->yMax));
     if (Plot[0]->yMax != yMax)  Plot[id]->yMax = yMax;
     if (fl_get_button (plottime->yMax_clone) == ON)  Plot[id]->yMax = yMax;

     filterMin = atof(fl_get_input(plottime->filterMin));
     if (Plot[0]->filterMin != filterMin)  Plot[id]->filterMin = filterMin;
     if (fl_get_button (plottime->filterMin_clone) == ON)  Plot[id]->filterMin = filterMin;

     filterMax = atof(fl_get_input(plottime->filterMax));
     if (Plot[0]->filterMax != filterMax)  Plot[id]->filterMax = filterMax;
     if (fl_get_button (plottime->filterMin_clone) == ON)  Plot[id]->filterMax = filterMax;

     bits      = fl_get_button (plottime->bits);
     nodc      = fl_get_button (plottime->nodc);
     autoScale = fl_get_button (plottime->autoScale);
     realUnits = fl_get_button (plottime->realUnits);

     gridx     = fl_get_button (plottime->gridx);
     gridy     = fl_get_button (plottime->gridy);
     logx      = fl_get_button (plottime->logx);
     logy      = fl_get_button (plottime->logy);
     minmax_resample = fl_get_button (plottime->minmax_resample);
    }


 if (plottype == PLOTFFT)
    {
     if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>0) printf("DyXformPanelToPlot> Get parameters from panel plotfft \n");
     strcpy (name, fl_get_input(plotfft->name));

     if (strcmp(Plot[0]->name, name) != 0)
        {if ((strstr(Plot[0]->name, SEP2VAR) != 0 && strstr(name, SEP2VAR) != 0) ||
             (strstr(Plot[0]->name, SEP2VARTR) != 0 && strstr(name, SEP2VARTR) != 0) ||
             (strstr(Plot[0]->name, SEP2VAR) == 0 && strstr(Plot[0]->name, SEP2VARTR) == 0 &&
              strstr(name, SEP2VAR) == 0 && strstr(name, SEP2VARTR) == 0))
            {
             DyDecodePlotName(name, &varxName, &varyName);
             strcpy(Plot[id]->vVarx, varxName);
             if (varyName) strcpy(Plot[id]->vVary, varyName);
             DyUpdatePlotName(id);
             DyUpdatePlotParam (Dy->Frame,id);
            }
        }
     else
        {strcpy(Plot[id]->name, name);}

     sampFreq = atof(fl_get_input(plotfft->sampFreq));
     if (Plot[0]->sampFreq != sampFreq)  Plot[id]->sampFreq = sampFreq;
     if (fl_get_button (plotfft->sampFreq_clone) == ON)  Plot[id]->sampFreq = sampFreq;

     if (Plot[id]->sampFreq <= 0  ||  Plot[id]->sampFreq > Plot[id]->freqClock)
        {Plot[id]->sampFreq = Plot[id]->freqClock;}

     Plot[id]->resampFactor = (long)(Plot[id]->freqClock / Plot[id]->sampFreq + 0.5);
     Plot[id]->sampFreq     = Plot[id]->freqClock / (double)Plot[id]->resampFactor;

     tsize = atof(fl_get_input(plotfft->tsize));
     if (Plot[0]->tsize != tsize)  Plot[id]->tsize = tsize;
     if (fl_get_button (plotfft->tsize_clone) == ON)  Plot[id]->tsize = tsize;

     if (Plot[id]->tsize <= 0.)  Plot[id]->tsize = 1.;

     Plot[id]->xnBin  = (long)(Plot[id]->tsize * Plot[id]->sampFreq);
     if (Plot[id]->xnBin < 32)     Plot[id]->xnBin = 32;

     nbin = 2; Plot[id]->nBinP2 = 0;
     while (nbin <= Plot[id]->xnBin)  {nbin *= 2; Plot[id]->nBinP2++;}
     Plot[id]->xnBin = 1 << Plot[id]->nBinP2;

     Plot[id]->tsize = Plot[id]->xnBin / Plot[id]->sampFreq;

     tstep = atof(fl_get_input(plotfft->tstep));
     if (Plot[0]->tstep != tstep)  Plot[id]->tstep = tstep;
     if (fl_get_button (plotfft->tstep_clone) == ON)  Plot[id]->tstep = tstep;
     if (Plot[id]->tstep <= 0.  ||  Plot[id]->tstep > 100.)  Plot[id]->tstep = 100.;
     Plot[id]->refreshPeriod = (long)(Plot[id]->tstep * Plot[id]->sampFreq * Plot[id]->tsize / 100.);
  
     xMin = atof(fl_get_input(plotfft->xMin));
     if (Plot[0]->xMin != xMin)  Plot[id]->xMin = xMin;
     if (fl_get_button (plotfft->xMin_clone) == ON)  Plot[id]->xMin = xMin;
     if (Plot[id]->xMin < 0.  ||  Plot[id]->xMin > Plot[id]->sampFreq / 2.)
              {Plot[id]->xMin = 0.;}

     xMax = atof(fl_get_input(plotfft->xMax));
     if (Plot[0]->xMax != xMax)  Plot[id]->xMax = xMax;
     if (fl_get_button (plotfft->xMax_clone) == ON)  Plot[id]->xMax = xMax;
     if (Plot[id]->xMax < Plot[id]->xMin  ||  Plot[id]->xMax > Plot[id]->sampFreq / 2.)
              {Plot[id]->xMax = Plot[id]->sampFreq / 2.;}

     average_fft = atol(fl_get_input (plotfft->average_fft));
     if (Plot[0]->average_fft != average_fft)  Plot[id]->average_fft = average_fft;
     if (fl_get_button (plotfft->average_fft_clone) == ON)  Plot[id]->average_fft = average_fft;
     if (Plot[id]->average_fft < 0)  Plot[id]->average_fft = 0;  /* 0 is for infinite average */

     refresh_fft = atol(fl_get_input(plotfft->refresh_fft));
     if (Plot[0]->refresh_fft != refresh_fft) Plot[id]->refresh_fft = refresh_fft;
     if (fl_get_button (plotfft->refresh_fft_clone) == ON)  Plot[id]->refresh_fft = refresh_fft;
     if (Plot[id]->refresh_fft <= 0) Plot[id]->refresh_fft = Plot[id]->average_fft;

     yMin = atof(fl_get_input(plotfft->yMin));
     if (Plot[0]->yMin != yMin)  Plot[id]->yMin = yMin;
     if (fl_get_button (plotfft->yMin_clone) == ON)  Plot[id]->yMin = yMin;

     yMax = atof(fl_get_input(plotfft->yMax));
     if (Plot[0]->yMax != yMax)  Plot[id]->yMax = yMax;
     if (fl_get_button (plotfft->yMax_clone) == ON)  Plot[id]->yMax = yMax;

     hertz     = fl_get_button (plotfft->hertz);
     rmsfft    = fl_get_button (plotfft->rmsfft);
     decayfft  = fl_get_button (plotfft->decayfft);
     nodc      = fl_get_button (plotfft->nodc);
     realUnits = fl_get_button (plotfft->realUnits);
     autoScale = fl_get_button (plotfft->autoScale);

     gridx     = fl_get_button (plotfft->gridx);
     gridy     = fl_get_button (plotfft->gridy);
     logx      = fl_get_button (plotfft->logx);
     logy      = fl_get_button (plotfft->logy);
     minmax_resample = fl_get_button (plotfft->minmax_resample);
    }


 if (plottype == PLOTTR   ||  plottype == PLOTCOHE)
    {
     if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>0) printf("DyXformPanelToPlot> Get parameters from panel plottrfct \n");
     strcpy (name, fl_get_input(plottrfct->name));
     if (strcmp(Plot[0]->name, name) == 0)  strcpy(Plot[id]->name, name);

     sampFreq = atof(fl_get_input(plottrfct->sampFreq));
     if (Plot[0]->sampFreq != sampFreq)  Plot[id]->sampFreq = sampFreq;
     if (fl_get_button (plottrfct->sampFreq_clone) == ON)  Plot[id]->sampFreq = sampFreq;

     if (Plot[id]->freqClock > Plot[id]->freqClock2)
        {freqMin = Plot[id]->freqClock2;
         Plot[id]->freqClock12 = Plot[id]->freqClock;
        }
     else
        {freqMin = Plot[id]->freqClock;
         Plot[id]->freqClock12 = Plot[id]->freqClock2;
        }

     if (Plot[id]->sampFreq <= 0  ||  Plot[id]->sampFreq > Plot[id]->freqClock12)
        {Plot[id]->sampFreq = Plot[id]->freqClock12;}

     Plot[id]->resampFactor = (long)(Plot[id]->freqClock12 / Plot[id]->sampFreq + 0.5);
     Plot[id]->sampFreq     = Plot[id]->freqClock12 / (double)Plot[id]->resampFactor;

     tsize = atof(fl_get_input(plottrfct->tsize));
     if (Plot[0]->tsize != tsize)  Plot[id]->tsize = tsize;
     if (fl_get_button (plottrfct->tsize_clone) == ON)  Plot[id]->tsize = tsize;
     if (Plot[id]->tsize <= 0.)  Plot[id]->tsize = 1.;

     Plot[id]->xnBin = (long)(Plot[id]->tsize * Plot[id]->sampFreq);
     if (Plot[id]->xnBin < 32) Plot[id]->xnBin = 32;

     nbin = 2; Plot[id]->nBinP2 = 0;
     while (nbin <= Plot[id]->xnBin)  {nbin *= 2; Plot[id]->nBinP2++;}
     Plot[id]->xnBin = 1 << Plot[id]->nBinP2;

     Plot[id]->tsize = (double)(Plot[id]->xnBin) / Plot[id]->sampFreq;

     tstep = atof(fl_get_input(plottrfct->tstep));
     if (Plot[0]->tstep != tstep)  Plot[id]->tstep = tstep;
     if (fl_get_button (plottrfct->tstep_clone) == ON)  Plot[id]->tstep = tstep;
     if (Plot[id]->tstep <= 0.  ||  Plot[id]->tstep > 100.)  Plot[id]->tstep = 100.;
     Plot[id]->refreshPeriod = (long)(Plot[id]->tstep * Plot[id]->sampFreq * Plot[id]->tsize / 100.);
  
     xMin = atof(fl_get_input(plottrfct->xMin));
     if (Plot[0]->xMin != xMin)  Plot[id]->xMin = xMin;
     if (fl_get_button (plottrfct->xMin_clone) == ON)  Plot[id]->xMin = xMin;
     if (Plot[id]->xMin < 0.  ||  Plot[id]->xMin > Plot[id]->sampFreq / 2.) Plot[id]->xMin = 0.;

     xMax = atof(fl_get_input(plottrfct->xMax));
     if (Plot[0]->xMax != xMax)  Plot[id]->xMax = xMax;
     if (fl_get_button (plottrfct->xMax_clone) == ON)  Plot[id]->xMax = xMax;
     freqmin = Plot[id]->sampFreq;
     if (Plot[id]->freqClock < freqmin)  freqmin = Plot[id]->freqClock;
     if (Plot[id]->freqClock2 < freqmin) freqmin = Plot[id]->freqClock2;
     if (Plot[id]->xMax < Plot[id]->xMin  ||  Plot[id]->xMax > freqmin / 2.) Plot[id]->xMax = freqmin / 2.;


     crossCohe_fmin = atof(fl_get_input(plottrfct->crossCohe_fmin));
     if (Plot[0]->crossCohe_fmin != crossCohe_fmin)  Plot[id]->crossCohe_fmin = crossCohe_fmin;
     if (fl_get_button (plottrfct->crossCohe_fmin_clone) == ON)  Plot[id]->crossCohe_fmin = crossCohe_fmin;
     if (Plot[id]->crossCohe_fmin < 0.  ||  Plot[id]->crossCohe_fmin > Plot[id]->sampFreq / 2.) Plot[id]->crossCohe_fmin = 0.;

     crossCohe_fmax = atof(fl_get_input(plottrfct->crossCohe_fmax));
     if (Plot[0]->crossCohe_fmax != crossCohe_fmax)  Plot[id]->crossCohe_fmax = crossCohe_fmax;
     if (fl_get_button (plottrfct->crossCohe_fmax_clone) == ON)  Plot[id]->crossCohe_fmax = crossCohe_fmax;
     freqmin = Plot[id]->sampFreq;
     if (Plot[id]->freqClock < freqmin)  freqmin = Plot[id]->freqClock;
     if (Plot[id]->freqClock2 < freqmin) freqmin = Plot[id]->freqClock2;
     if (Plot[id]->crossCohe_fmax < Plot[id]->crossCohe_fmin  ||  Plot[id]->crossCohe_fmax > freqmin / 2.) Plot[id]->crossCohe_fmax = freqmin / 2.;

     average_fft = atol(fl_get_input (plottrfct->average_fft));
     if (Plot[0]->average_fft != average_fft)  Plot[id]->average_fft = average_fft;
     if (fl_get_button (plottrfct->average_fft_clone) == ON)  Plot[id]->average_fft = average_fft;
     if (Plot[id]->average_fft < 0)  Plot[id]->average_fft = 0;  /* 0 is for infinite average */

     refresh_fft = atol(fl_get_input(plottrfct->refresh_fft));
     if (Plot[0]->refresh_fft != refresh_fft) Plot[id]->refresh_fft = refresh_fft;
     if (fl_get_button (plottrfct->refresh_fft_clone) == ON)  Plot[id]->refresh_fft = refresh_fft;
     if (Plot[id]->refresh_fft <= 0) Plot[id]->refresh_fft = Plot[id]->average_fft;

     yMin = atof(fl_get_input(plottrfct->yMin));
     if (Plot[0]->yMin != yMin)  Plot[id]->yMin = yMin;
     if (fl_get_button (plottrfct->yMin_clone) == ON)  Plot[id]->yMin = yMin;

     yMax = atof(fl_get_input(plottrfct->yMax));
     if (Plot[0]->yMax != yMax)  Plot[id]->yMax = yMax;
     if (fl_get_button (plottrfct->yMax_clone) == ON)  Plot[id]->yMax = yMax;

     crossCohe = fl_get_button (plottrfct->crossCohe);

     nodc      = fl_get_button (plottrfct->nodc);
     realUnits = fl_get_button (plottrfct->realUnits);
     autoScale = fl_get_button (plottrfct->autoScale);
     modulesqrt = fl_get_button (plottrfct->modulesqrt);
     decayfft  = fl_get_button (plottrfct->decayfft);

     gridx     = fl_get_button (plottrfct->gridx);
     gridy     = fl_get_button (plottrfct->gridy);
     logx      = fl_get_button (plottrfct->logx);
     logy      = fl_get_button (plottrfct->logy);

     gridx2    = fl_get_button (plottrfct->gridx2);
     gridy2    = fl_get_button (plottrfct->gridy2);
     logx2     = fl_get_button (plottrfct->logx2);
     logy2     = fl_get_button (plottrfct->logy2);
     minmax_resample = fl_get_button (plottrfct->minmax_resample);
    }

 /** HIHI: Mettre une fonction par plot et rendre plus visible que Plot[0] est l'etat avant edition du panel **/

 if (plottype == PLOTFFTTIME || plottype == PLOTTRTIME || plottype == PLOTCOHETIME)
    {
     if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>0) printf("DyXformPanelToPlot> Get parameters from panel plotffttime \n");
     strcpy (name, fl_get_input(plotffttime->name));
     if (strcmp(Plot[0]->name, name) == 0)  strcpy(Plot[id]->name, name);

     sampFreq = atof(fl_get_input(plotffttime->sampFreq));
     if (Plot[0]->sampFreq != sampFreq)  Plot[id]->sampFreq = sampFreq;
     if (fl_get_button (plotffttime->sampFreq_clone) == ON)  Plot[id]->sampFreq = sampFreq;

     if (plottype == PLOTFFTTIME)
        {
         if (Plot[id]->sampFreq <= 0  ||  Plot[id]->sampFreq > Plot[id]->freqClock)
            {Plot[id]->sampFreq = Plot[id]->freqClock;}

         Plot[id]->resampFactor = (long)(Plot[id]->freqClock / Plot[id]->sampFreq +0.5);
         Plot[id]->sampFreq     = Plot[id]->freqClock / (double)Plot[id]->resampFactor;
        }

     if (plottype == PLOTTRTIME || plottype == PLOTCOHETIME)
        {
         if (Plot[id]->freqClock > Plot[id]->freqClock2)
            {freqMin = Plot[id]->freqClock2;
             Plot[id]->freqClock12 = Plot[id]->freqClock;
            }
         else
            {freqMin = Plot[id]->freqClock;
             Plot[id]->freqClock12 = Plot[id]->freqClock2;
            }

         if (Plot[id]->sampFreq <= 0  ||  Plot[id]->sampFreq > Plot[id]->freqClock12)
            {Plot[id]->sampFreq = Plot[id]->freqClock12;}

         Plot[id]->resampFactor = (long)(Plot[id]->freqClock12 / Plot[id]->sampFreq + 0.5);
         Plot[id]->sampFreq     = Plot[id]->freqClock12 / (double)Plot[id]->resampFactor;
        }

/*
     tsize = atof(fl_get_input(plotffttime->tsize));
*/
     dfreq = atof(fl_get_input(plotffttime->dfreq));
     if (Plot[0]->dfreq  != dfreq )  Plot[id]->dfreq  = dfreq ;
     if (fl_get_button (plotffttime->dfreq_clone) == ON)  Plot[id]->dfreq = dfreq;
     if (Plot[id]->dfreq  <= 0.)     Plot[id]->dfreq  = 0.5;
     Plot[id]->tsize = 0.5 / Plot[id]->dfreq;

     Plot[id]->xnBin  = (long)(Plot[id]->tsize * Plot[id]->sampFreq);
     if (Plot[id]->xnBin < 32)     Plot[id]->xnBin = 32;

     nbin = 2; Plot[id]->nBinP2 = 0;
     while (nbin <= Plot[id]->xnBin)  {nbin *= 2; Plot[id]->nBinP2++;}
     Plot[id]->xnBin = 1 << Plot[id]->nBinP2;

     Plot[id]->tsize = Plot[id]->xnBin / Plot[id]->sampFreq;

     tstep = atof(fl_get_input(plotffttime->tstep));
     if (Plot[0]->tstep != tstep)  Plot[id]->tstep = tstep;
     if (fl_get_button (plotffttime->tstep_clone) == ON)  Plot[id]->tstep = tstep;
     if (Plot[id]->tstep <= 0.  ||  Plot[id]->tstep > 100.)  Plot[id]->tstep = 100.;
     Plot[id]->refreshPeriod = (long)(Plot[id]->tstep * Plot[id]->sampFreq * Plot[id]->tsize / 100.);

     xMin = atof(fl_get_input(plotffttime->xMin));
     if (Plot[0]->xMin != xMin)  Plot[id]->xMin = xMin;
     if (fl_get_button (plotffttime->xMin_clone) == ON)  Plot[id]->xMin = xMin;
     if (Plot[id]->xMin < 0.  ||  Plot[id]->xMin > Plot[id]->sampFreq / 2.)
              {Plot[id]->xMin = 0.;}

     xMax = atof(fl_get_input(plotffttime->xMax));
     if (Plot[0]->xMax != xMax)  Plot[id]->xMax = xMax;
     if (fl_get_button (plotffttime->xMax_clone) == ON)  Plot[id]->xMax = xMax;

     if (plottype == PLOTTRTIME || plottype == PLOTCOHETIME)
        {freqmin = Plot[id]->sampFreq;
         if (Plot[id]->freqClock < freqmin)  freqmin = Plot[id]->freqClock;
         if (Plot[id]->freqClock2 < freqmin) freqmin = Plot[id]->freqClock2;
         if (Plot[id]->xMax < Plot[id]->xMin  ||  Plot[id]->xMax > freqmin / 2.) {Plot[id]->xMax = freqmin / 2.;}
        }
     else
        {if (Plot[id]->xMax < Plot[id]->xMin  ||  Plot[id]->xMax > Plot[id]->sampFreq / 2.) {Plot[id]->xMax = Plot[id]->sampFreq / 2.;}}

     zMin = atof(fl_get_input(plotffttime->zmin));
     if (Plot[0]->zMin != zMin)  Plot[id]->zMin = zMin;
     if (fl_get_button (plotffttime->zmin_clone) == ON)  Plot[id]->zMin = zMin;

     zMax = atof(fl_get_input(plotffttime->zmax));
     if (Plot[0]->zMax != zMax)  Plot[id]->zMax = zMax;
     if (fl_get_button (plotffttime->zmax_clone) == ON)  Plot[id]->zMax = zMax;

/**
     average_fft = atol(fl_get_input (plotffttime->average_fft));
**/
     dtime = atof(fl_get_input(plotffttime->dtime));
     if (Plot[0]->dtime  != dtime )  Plot[id]->dtime  = dtime;
     if (fl_get_button (plotffttime->dtime_clone) == ON)  Plot[id]->dtime = dtime;
     Plot[id]->average_fft = (int)(2.*Plot[id]->dfreq*Plot[id]->dtime / (Plot[id]->tstep/100.) + 0.5);
     if (Plot[id]->average_fft <= 0)
        {Plot[id]->average_fft = 1;
         Plot[id]->dtime = Plot[id]->tsize * (Plot[id]->tstep/100.) * Plot[id]->average_fft;
        }

     timeSize = atof(fl_get_input(plotffttime->timeSize));
     if (Plot[0]->timeSize != timeSize)  Plot[id]->timeSize = timeSize;
     if (fl_get_button (plotffttime->timeSize_clone) == ON)  Plot[id]->timeSize = timeSize;
     if (Plot[id]->timeSize < 0.) Plot[id]->timeSize = 10. * Plot[id]->tsize;

     refresh_percent = atof(fl_get_input(plotffttime->refresh_percent));
     if (Plot[0]->refresh_percent != refresh_percent)  Plot[id]->refresh_percent = refresh_percent;
     if (fl_get_button (plotffttime->refresh_percent_clone) == ON)  Plot[id]->refresh_percent = refresh_percent;
     if (Plot[id]->refresh_percent <= 0) Plot[id]->refresh_percent = 50.;

     autoScale = fl_get_button (plotffttime->autoScale);
     realUnits = fl_get_button (plotffttime->realUnits);
     nodc      = fl_get_button (plotffttime->nodc);

     gridx     = fl_get_button (plotffttime->gridx);
     gridy     = fl_get_button (plotffttime->gridy);
     gridz     = fl_get_button (plotffttime->gridz);
     logx      = fl_get_button (plotffttime->logx);
     logy      = fl_get_button (plotffttime->logy);
     logz      = fl_get_button (plotffttime->logz);
     minmax_resample = fl_get_button (plotffttime->minmax_resample);
    }


 if (plottype == PLOT1DTIME)
    {
     if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>0) printf("DyXformPanelToPlot> Get parameters from panel plot1dtime \n");
     strcpy (name, fl_get_input(plot1dtime->name));
     if (strcmp(Plot[0]->name, name) == 0)  strcpy(Plot[id]->name, name);

     sampFreq = atof(fl_get_input(plot1dtime->sampFreq));
     if (Plot[0]->sampFreq != sampFreq)  Plot[id]->sampFreq = sampFreq;
     if (fl_get_button (plot1dtime->sampFreq_clone) == ON)  Plot[id]->sampFreq = sampFreq;

     if (Plot[id]->sampFreq <= 0  ||  Plot[id]->sampFreq > Plot[id]->freqClock)
        {Plot[id]->sampFreq = Plot[id]->freqClock;}

     Plot[id]->resampFactor = (long)(Plot[id]->freqClock / Plot[id]->sampFreq + 0.5);
     Plot[id]->sampFreq     = Plot[id]->freqClock / (double)Plot[id]->resampFactor;

     tsize = atof(fl_get_input(plot1dtime->tsize));
     if (Plot[0]->tsize != tsize)  Plot[id]->tsize = tsize;
     if (fl_get_button (plot1dtime->tsize_clone) == ON)  Plot[id]->tsize = tsize;
     if (Plot[id]->tsize <= 0.)  Plot[id]->tsize = 1.;

     Plot[id]->nBin  = (long)(Plot[id]->tsize * Plot[id]->sampFreq);
     if (Plot[id]->nBin < 32)     Plot[id]->nBin = 32;

     Plot[id]->tsize = Plot[id]->nBin / Plot[id]->sampFreq;

     xnBin = atol(fl_get_input(plot1dtime->xnBin));
     if (Plot[0]->xnBin != xnBin) Plot[id]->xnBin = xnBin;
     if (fl_get_button (plot1dtime->xnBin_clone) == ON)  Plot[id]->xnBin = xnBin;
     if (Plot[id]->xnBin < 2)  Plot[id]->xnBin = Plot[id]->nBin / 4;

     xMin = atof(fl_get_input(plot1dtime->xMin));
     if (Plot[0]->xMin != xMin)  Plot[id]->xMin = xMin;
     if (fl_get_button (plot1dtime->xMin_clone) == ON)  Plot[id]->xMin = xMin;

     xMax = atof(fl_get_input(plot1dtime->xMax));
     if (Plot[0]->xMax != xMax)  Plot[id]->xMax = xMax;
     if (fl_get_button (plot1dtime->xMax_clone) == ON)  Plot[id]->xMax = xMax;
     if (Plot[id]->xMax < Plot[id]->xMin) Plot[id]->xMax = Plot[id]->xMin;

     zMin = atof(fl_get_input(plot1dtime->zmin));
     if (Plot[0]->zMin != zMin)  Plot[id]->zMin = zMin;
     if (fl_get_button (plot1dtime->zmin_clone) == ON)  Plot[id]->zMin = zMin;

     zMax = atof(fl_get_input(plot1dtime->zmax));
     if (Plot[0]->zMax != zMax)  Plot[id]->zMax = zMax;
     if (fl_get_button (plot1dtime->zmax_clone) == ON)  Plot[id]->zMax = zMax;

     refresh_percent = atof(fl_get_input(plot1dtime->refresh_percent));
     if (Plot[0]->refresh_percent != refresh_percent)  Plot[id]->refresh_percent = refresh_percent;
     if (fl_get_button (plot1dtime->refresh_percent_clone) == ON)  Plot[id]->refresh_percent = refresh_percent;
     if (Plot[id]->refresh_percent <= 0) Plot[id]->refresh_percent = 50.;

     timeSize = atof(fl_get_input(plot1dtime->timeSize));
     if (Plot[0]->timeSize != timeSize)  Plot[id]->timeSize = timeSize;
     if (fl_get_button (plot1dtime->timeSize_clone) == ON)  Plot[id]->timeSize = timeSize;
     if (Plot[id]->timeSize < 0.) Plot[id]->timeSize = 10. * Plot[id]->tsize;

     autoEdge  = fl_get_button (plot1dtime->autoEdge);
     autoScale = fl_get_button (plot1dtime->autoScale);
     realUnits = fl_get_button (plot1dtime->realUnits);
     nodc      = fl_get_button (plot1dtime->nodc);

     gridx     = fl_get_button (plot1dtime->gridx);
     gridy     = fl_get_button (plot1dtime->gridy);
     gridz     = fl_get_button (plot1dtime->gridz);
     logx      = fl_get_button (plot1dtime->logx);
     logy      = fl_get_button (plot1dtime->logy);
     logz      = fl_get_button (plot1dtime->logz);
     minmax_resample = fl_get_button (plot1dtime->minmax_resample);
    }


 if (plottype == PLOT1D)
    {
     if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>0) printf("DyXformPanelToPlot> Get parameters from panel plot1d\n");
     strcpy (name, fl_get_input(plot1d->name));
     if (strcmp(Plot[0]->name, name) == 0)  strcpy(Plot[id]->name, name);

     tstep = atof(fl_get_input(plot1d->tstep));
     if (Plot[0]->tstep != tstep)  Plot[id]->tstep = tstep;
     if (fl_get_button (plot1d->tstep_clone) == ON)  Plot[id]->tstep = tstep;

     Plot[id]->refreshPeriod = (long)(Plot[id]->tstep * Plot[id]->freqClock);
     if (Plot[id]->refreshPeriod <= 0)
        {Plot[id]->refreshPeriod = 1;
         Plot[id]->tstep = (double)Plot[id]->refreshPeriod / Plot[id]->freqClock;
        }

     sampFreq = atof(fl_get_input(plot1d->sampFreq));
     if (Plot[0]->sampFreq != sampFreq)  Plot[id]->sampFreq = sampFreq;
     if (fl_get_button (plot1d->sampFreq_clone) == ON)  Plot[id]->sampFreq = sampFreq;

     if (Plot[id]->sampFreq <= 0  ||  Plot[id]->sampFreq > Plot[id]->freqClock)
        {Plot[id]->sampFreq = Plot[id]->freqClock;}

     Plot[id]->resampFactor = (long)(Plot[id]->freqClock / Plot[id]->sampFreq + 0.5);
     Plot[id]->sampFreq     = Plot[id]->freqClock / (double)Plot[id]->resampFactor;

     xnBin = atol(fl_get_input(plot1d->xnBin));
     if (Plot[0]->xnBin != xnBin) Plot[id]->xnBin = xnBin;
     if (fl_get_button (plot1d->xnBin_clone) == ON)  Plot[id]->xnBin = xnBin;
     if (Plot[id]->xnBin < 2)  Plot[id]->xnBin = 100;

     xMin = atof(fl_get_input(plot1d->xMin));
     if (Plot[0]->xMin != xMin)  Plot[id]->xMin = xMin;
     if (fl_get_button (plot1d->xMin_clone) == ON)  Plot[id]->xMin = xMin;

     xMax = atof(fl_get_input(plot1d->xMax));
     if (Plot[0]->xMax != xMax)  Plot[id]->xMax = xMax;
     if (fl_get_button (plot1d->xMax_clone) == ON)  Plot[id]->xMax = xMax;
     if (Plot[id]->xMax <= Plot[id]->xMin)                Plot[id]->xMax = Plot[id]->xMin * 2.;
     if (Plot[id]->xMax <= 0.  &&  Plot[id]->xMin == 0.)  Plot[id]->xMax = 1.;

     autoEdge = fl_get_button (plot1d->autoEdge);
     realUnits = fl_get_button (plot1d->realUnits);

     gridx    = fl_get_button (plot1d->gridx);
     gridy    = fl_get_button (plot1d->gridy);
     logx     = fl_get_button (plot1d->logx);
     logy     = fl_get_button (plot1d->logy);
     stat     = fl_get_button(plot1d->stat);
     minmax_resample = fl_get_button (plot1d->minmax_resample);
    }


 if (plottype == PLOT2D || plottype == PLOT3D)
    {
     if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>0) printf("DyXformPanelToPlot> Get parameters from panel plot2d \n");
     strcpy (name, fl_get_input(plot2d->name));
     if (strcmp(Plot[0]->name, name) == 0)  strcpy(Plot[id]->name, name);

     tstep = atof(fl_get_input(plot2d->tstep));
     if (Plot[0]->tstep != tstep)  Plot[id]->tstep = tstep;
     if (fl_get_button (plot2d->tstep_clone) == ON)  Plot[id]->tstep = tstep;
     Plot[id]->refreshPeriod  = (long)(Plot[id]->tstep * Plot[id]->freqClock12);
     if (Plot[id]->refreshPeriod <= 0)
        {Plot[id]->refreshPeriod = 1;
         Plot[id]->tstep = (double)Plot[id]->refreshPeriod / Plot[id]->freqClock12;
        }

     xnBin = atol(fl_get_input(plot2d->xnBin));
     if (Plot[0]->xnBin != xnBin)  Plot[id]->xnBin = xnBin;
     if (fl_get_button (plot2d->xnBin_clone) == ON)  Plot[id]->xnBin = xnBin;
     if (Plot[id]->xnBin <    4)   Plot[id]->xnBin =    4;
     if (Plot[id]->xnBin > 4096)   Plot[id]->xnBin = 4096;

     xMin = atof((const char *)fl_get_input(plot2d->xMin));
     if (Plot[0]->xMin != xMin)  Plot[id]->xMin = xMin;
     if (fl_get_button (plot2d->xMin_clone) == ON)  Plot[id]->xMin = xMin;

     xMax = atof(fl_get_input(plot2d->xMax));
     if (Plot[0]->xMax != xMax)  Plot[id]->xMax = xMax;
     if (fl_get_button (plot2d->xMax_clone) == ON)  Plot[id]->xMax = xMax;

     if (Plot[id]->xMax <= Plot[id]->xMin)                Plot[id]->xMax = Plot[id]->xMin * 2.;
     if (Plot[id]->xMax <= 0.  &&  Plot[id]->xMin == 0.)  Plot[id]->xMax = 1.;

     ynBin = atol(fl_get_input(plot2d->ynBin));
     if (Plot[0]->ynBin != ynBin)  Plot[id]->ynBin = ynBin;
     if (fl_get_button (plot2d->ynBin_clone) == ON)  Plot[id]->ynBin = ynBin;
     if (Plot[id]->ynBin <    4)   Plot[id]->ynBin =    4;
     if (Plot[id]->ynBin > 4096)   Plot[id]->ynBin = 4096;

     yMin = atof(fl_get_input(plot2d->yMin));
     if (Plot[0]->yMin != yMin)  Plot[id]->yMin = yMin;
     if (fl_get_button (plot2d->yMin_clone) == ON)  Plot[id]->yMin = yMin;

     yMax = atof(fl_get_input(plot2d->yMax));
     if (Plot[0]->yMax != yMax)  Plot[id]->yMax = yMax;
     if (fl_get_button (plot2d->yMax_clone) == ON)  Plot[id]->yMax = yMax;
     if (Plot[id]->yMax <= Plot[id]->yMin)                Plot[id]->yMax = Plot[id]->yMin * 2.;
     if (Plot[id]->yMax <= 0.  &&  Plot[id]->yMin == 0.)  Plot[id]->yMax = 1.;

     nSampling = atoi(fl_get_input(plot2d->nSampling));
     if (Plot[0]->nSampling != nSampling)  Plot[id]->nSampling = nSampling;

     autoEdge = fl_get_button (plot2d->autoEdge);
     realUnits = fl_get_button (plot2d->realUnits);

     gridx    = fl_get_button (plot2d->gridx);
     gridy    = fl_get_button (plot2d->gridy);
     logx     = fl_get_button (plot2d->logx);
     logy     = fl_get_button (plot2d->logy);
     logz     = fl_get_button (plot2d->logz);
     stat     = fl_get_button (plot2d->stat);
     if (fl_get_button (plot2d->col))  type2d = COL;
     if (fl_get_button (plot2d->cont)) type2d = CONT;
     if (fl_get_button (plot2d->lego)) type2d = LEGO;
     if (fl_get_button (plot2d->surf)) type2d = SURF;
     if (fl_get_button (plot2d->scat)) type2d = SCAT;
    }


 if (plottype == PLOTAUDIO)
    {
     if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>0) printf("DyXformPanelToPlot> Get parameters from panel audio \n");
     strcpy (name, fl_get_input(plotaudio->name));
     if (strcmp(Plot[0]->name, name) == 0)  strcpy(Plot[id]->name, name);

     t0 = atol(fl_get_input(plotaudio->t0));
     if (Plot[0]->t0 != t0)  Plot[id]->t0 = t0;
     if (fl_get_button (plotaudio->t0_clone) == ON)  Plot[id]->t0 = t0;
     if (Plot[id]->t0 < 0)  Plot[id]->t0 = 0;

     duration = atol(fl_get_input(plotaudio->duration));
     if (Plot[0]->duration != duration)  Plot[id]->duration = duration;
     if (fl_get_button (plotaudio->duration_clone) == ON)  Plot[id]->duration = duration;
     if (Plot[id]->duration <= 0)  Plot[id]->duration = 10;

     sound16bits = fl_get_button (plotaudio->sound16bits);
     soundonline = fl_get_button (plotaudio->soundonline);
     if (soundonline == ON)
        {nplot = fl_get_browser_maxline(mainbrowser->plots);
         for (i=1; i<=nplot; i++)
            {if (i != id && Plot[i]->type == plottype && Plot[i]->soundonline == ON) Plot[i]->soundonline = OFF;}
        }
    }

 if (Plot[0]->crossCohe != crossCohe)  Plot[id]->crossCohe = crossCohe;

 if (Plot[0]->minmax_resample != minmax_resample) Plot[id]->minmax_resample = minmax_resample;

 if (Plot[0]->sound16bits != sound16bits) Plot[id]->sound16bits = sound16bits;
 if (Plot[0]->soundonline != soundonline) Plot[id]->soundonline = soundonline;
 if (Plot[0]->x2av      != x2av)       Plot[id]->x2av      = x2av;
 if (Plot[0]->rmsfft    != rmsfft)     Plot[id]->rmsfft    = rmsfft;
 if (Plot[0]->decayfft  != decayfft)   Plot[id]->decayfft  = decayfft;
 if (Plot[0]->hertz     != hertz)      Plot[id]->hertz     = hertz;
 if (Plot[0]->nodc      != nodc)       Plot[id]->nodc      = nodc;
 if (Plot[0]->bits      != bits)       Plot[id]->bits      = bits;

 if (Plot[0]->gridx     != gridx)      Plot[id]->gridx     = gridx;
 if (Plot[0]->gridy     != gridy)      Plot[id]->gridy     = gridy;
 if (Plot[0]->logx      != logx)       Plot[id]->logx      = logx;
 if (Plot[0]->logy      != logy)       Plot[id]->logy      = logy;

 if (Plot[0]->gridx2    != gridx2)     Plot[id]->gridx2    = gridx2;
 if (Plot[0]->gridy2    != gridy2)     Plot[id]->gridy2    = gridy2;
 if (Plot[0]->logx2     != logx2)      Plot[id]->logx2     = logx2;
 if (Plot[0]->logy2     != logy2)      Plot[id]->logy2     = logy2;

 if (Plot[0]->logz      != logz)       Plot[id]->logz      = logz;
 if (Plot[0]->stat      != stat)       Plot[id]->stat      = stat;
 if (Plot[0]->autoEdge  != autoEdge)   Plot[id]->autoEdge  = autoEdge;
 if (Plot[0]->autoScale != autoScale)  Plot[id]->autoScale = autoScale;
 if (Plot[0]->modulesqrt != modulesqrt)  Plot[id]->modulesqrt = modulesqrt;
 if (Plot[0]->realUnits != realUnits)  Plot[id]->realUnits = realUnits;
 if (Plot[0]->type2d    != type2d)     Plot[id]->type2d    = type2d;

 if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>0)
     {printf("DyXformPanelToPlot> Updated parameters in plot%ld (name=%s  type=%s) \n", id, Plot[id]->name, Dy->typeName[plottype]);}

 return(0);
}


/*------------------------------------------ DyUpdatePlotName ---*/

void DyUpdatePlotName (int id)
{
 char browserLine[LSTRING_MAX];

 if (Plot[id]->type == PLOT3D  ||  Plot[id]->type == PLOT2D   || Plot[id]->type == PLOTTR     || Plot[id]->type == PLOTCOHE || 
     Plot[id]->type == PLOTCORR || Plot[id]->type == PLOTTRTIME || Plot[id]->type == PLOTCOHETIME)
    {
     if (Plot[id]->type == PLOTTR || Plot[id]->type == PLOTTRTIME)
        {sprintf(Plot[id]->name, "%s%s%s", Plot[id]->vVarx, SEP2VARTR, Plot[id]->vVary);}
     else if (Plot[id]->type == PLOT3D)
        {sprintf(Plot[id]->name, "%s%s%s%s%s", Plot[id]->vVarx, SEP2VAR, Plot[id]->vVary, SEP2VAR, Plot[id]->vVarz);}
     else
        {sprintf(Plot[id]->name, "%s%s%s", Plot[id]->vVarx, SEP2VAR, Plot[id]->vVary);}
    }
 else
    {sprintf(Plot[id]->name, "%s", Plot[id]->vVarx);}

 if (Plot[id]->superposed == ON)
    {sprintf (browserLine, "%ld %s %s (s)", id, Dy->typeName[Plot[id]->type], Plot[id]->name);}
 else
    {sprintf (browserLine, "%ld %s %s", id, Dy->typeName[Plot[id]->type], Plot[id]->name);}

 if (Plot[id]->hidden == ON) sprintf (browserLine, "@C7 %s", browserLine);

 fl_replace_browser_line(mainbrowser->plots, id, browserLine);

 return;
}


/*------------------------------------------- DyUpdatePlotParam ---*/
/*
  Called when: an Update Channels is made
  Called why: to update plot parameters if corresponding channels parameters (like sampling frequency) have changed
  Call what: DyDecodeVar
*/

void DyUpdatePlotParam (struct FrameH *frame, int id)
{
 struct FrVect *vect, *vect2;
 struct FrSerData *sms;
 char dummy[LSTRING_MAX], *str;
 char   *smsName=NULL, *varName=NULL, *adcName=NULL, *procName=NULL, *simName=NULL;
 long nplot, i, j, imin, imax;
 int length, datatype, datatype_new;
 char namex[LSTRING_MAX], namey[LSTRING_MAX];

 if (frame == NULL) return;

 nplot = fl_get_browser_maxline(mainbrowser->plots);

 if (id >= 0) sprintf(dummy,"DyUpdatePlotParam> plot%d=%s=%s  frame=%d", id, Plot[id]->name, Dy->typeName[Plot[id]->type], frame->GTimeS);
 else         sprintf(dummy,"DyUpdatePlotParam> all %d plots  frame=%d", nplot, frame->GTimeS);
 DyDebugPrint(dummy,0);

 if (id < 0) {imin=1; imax=nplot;}
 else        {imin=id; imax=id;}

 /*--- Change of channels sampling frequency ---*/

 for (i=imin; i<=imax; i++)
    {
     if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>1)
        {printf("DyUpdatePlotParam> Updating plot%d=%s:  vVarx=%s  vVary=%s \n", i, Plot[i]->name, Plot[i]->vVarx, Plot[i]->vVary);}

     vect = FrameFindVect(frame, Plot[i]->vVarx);
     if (vect)
        {if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>1)
            {printf("DyUpdatePlotParam> plot%d=%s Changing frequency of varX from %g to %g\n", i, Plot[i]->name, Plot[i]->freqClock, 1/vect->dx[0]);}
         Plot[i]->freqClock = 1./vect->dx[0];
         continue;
        }
     vect = FrameFindVect(frame, Plot[i]->vVary);
     if (vect)
        {if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>1)
            {printf("DyUpdatePlotParam> plot%d=%s Changing frequency of varY from %g to %g\n", i, Plot[i]->name, Plot[i]->freqClock2, 1/vect->dx[0]);}
         Plot[i]->freqClock2 = 1./vect->dx[0];
         continue;
        }

     DyDecodeVar(Plot[i]->vVarx, &smsName, &varName, &adcName, &procName, &simName);
     if (smsName && varName && (sms=FrSerDataFind(frame, smsName, NULL)))
        {if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>1)
            {printf("DyUpdatePlotParam> plot%d=%s Changing frequency of varX from %g to %g\n", i, Plot[i]->name, Plot[i]->freqClock, sms->sampleRate);}
         if (sms->sampleRate > 0.) Plot[i]->freqClock = sms->sampleRate;
         else                      Plot[i]->freqClock = 1/frame->dt;
        }
     if (smsName)  {free(smsName);  smsName=NULL;}
     if (varName)  {free(varName);  varName=NULL;}
     if (adcName)  {free(adcName);  adcName=NULL;}
     if (procName) {free(procName); procName=NULL;}
     if (simName)  {free(simName);  simName=NULL;}

     DyDecodeVar(Plot[i]->vVary, &smsName, &varName,  &adcName, &procName, &simName);
     if (smsName && varName && (sms=FrSerDataFind(frame, smsName, NULL)))
        {if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>1)
            {printf("DyUpdatePlotParam> plot%d=%s Changing frequency of varY from %g to %g\n", i, Plot[i]->name, Plot[i]->freqClock2, sms->sampleRate);}
         if (sms->sampleRate > 0.) Plot[i]->freqClock2 = sms->sampleRate;
         else                      Plot[i]->freqClock2 = 1/frame->dt;
        }
     if (smsName)  {free(smsName);  smsName=NULL;}
     if (varName)  {free(varName);  varName=NULL;}
     if (adcName)  {free(adcName);  adcName=NULL;}
     if (procName) {free(procName); procName=NULL;}
     if (simName)  {free(simName);  simName=NULL;}
    }

 /*--- Update the fastest clock in case of 2 variables plot ---*/

 for (i=imin; i<=imax; i++)
    {if (Plot[i]->freqClock2 > Plot[i]->freqClock) Plot[i]->freqClock12 = Plot[i]->freqClock2;
     else                      Plot[i]->freqClock12 = Plot[i]->freqClock;
    }

 return;
}



/*------------------------------------------- DyPutDefaultInPlot ---*/

void DyPutDefaultInPlot (long id)
{
 int nBin, nBinP2, nplot, i;
 double vxmax, vxmin, vymax, vymin, value, dt, freqmin;
 char   *smsName=NULL, *varName=NULL, *adcName=NULL, *procName=NULL, *simName=NULL;
 char dummy[LSTRING_MAX];
 struct FrSerData  *sms;
 struct FrAdcData  *adc;
 struct FrSimData  *sim;
 struct FrProcData *proc;
 struct FrVect     *vect;
 long   plottype;

 if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>0) printf("DyPutDefaultInPlot \n");
 sprintf(dummy,"DyPutDefaultInPlot> plot%d  name=%s  type=%s", id, Plot[id]->name, Dy->typeName[Plot[id]->type]);
 DyDebugPrint(dummy,0);

 if (Dy->Frame == NULL) {printf("DyPutDefaultInPlot: no frame in memory \n"); return;}

 DyPlotInit(id,1);      /* Partial init of the Plot */

 plottype = Plot[id]->type;

 Plot[id]->filterMin = 0.;
 Plot[id]->filterMax = 0.;
 Plot[id]->minmax_resample = OFF;

 if (plottype == PLOTRAW)
    {
     DyDecodeVar(Plot[id]->vVarx, &smsName, &varName, &adcName, &procName, &simName);

     Plot[id]->nodc      = OFF;
     Plot[id]->gridx     = ON;   Plot[id]->gridy     = ON;
     Plot[id]->logx      = OFF;  Plot[id]->logy      = OFF;
     Plot[id]->autoScale = ON;   Plot[id]->realUnits = OFF;
     Plot[id]->x2av = OFF;

     Plot[id]->refreshPeriod = 1;
     Plot[id]->average_fft   = 1;

     if (smsName && varName)
        {
         FrSerDataGet(Dy->Frame, smsName, varName, &value);
         sms = FrSerDataFind(Dy->Frame, smsName, NULL);
         if (value > 0.)      {vymin = value*0.5; vymax = value*2.0;}
         else if (value < 0.) {vymin = value*2.0; vymax = value*0.5;}
         else                 {vymin = -0.2; vymax = 0.2;}
         Plot[id]->yMin = vymin - 2.*(vymax - vymin);
         Plot[id]->yMax = vymax + 2.*(vymax - vymin);
        }

     if (adcName || procName || simName)
        {
         if (adcName)  {adc  = FrAdcDataFind(Dy->Frame, adcName);    vect = adc->data;
                        if (Dy->InputType != INCM  &&  adc->slope != 0.) Plot[id]->realUnits = ON;}
         if (procName) {proc = FrProcDataFind (Dy->Frame, procName); vect = proc->data;}
         if (simName)  {sim  = FrSimDataFind(Dy->Frame, simName);    vect = sim->data;}

         DyGetMinMax(vect, &vymin, &vymax);

         if (adcName != NULL)
            {DyGetMinMaxFromTrend(Dy->Frame, adcName, &vymin, &vymax);}

         if (vymin > 0 && vymax/vymin > 10000.) Plot[id]->logy = ON;
         Plot[id]->yMin = vymin - 2.*(vymax - vymin);
         Plot[id]->yMax = vymax + 2.*(vymax - vymin);
        }
     if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>1)
        {printf("PutDefaultInPlot> RAW: plot%d=%s  tsize=%g  xnBin=%d  resampFactor=%d  refreshPeriod=%d \n",
                 id, Plot[id]->name, Plot[id]->tsize, Plot[id]->xnBin, Plot[id]->resampFactor, Plot[id]->refreshPeriod);
        }
    }


 if (plottype == PLOTRAWTIME)
    {
     DyDecodeVar(Plot[id]->vVarx, &smsName, &varName, &adcName, &procName, &simName);

     Plot[id]->gridx     = ON;  Plot[id]->gridy     = ON;
     Plot[id]->gridx2    = ON;  Plot[id]->gridy2    = ON;
     Plot[id]->logx      = OFF; Plot[id]->logy      = OFF; Plot[id]->logz = OFF;
     Plot[id]->logx2     = OFF; Plot[id]->logy2     = OFF;
     Plot[id]->autoScale = ON;  Plot[id]->realUnits = OFF;
     Plot[id]->x2av      = OFF;

     Plot[id]->resampFactor  = 1;
     Plot[id]->refreshPeriod = 1;
     Plot[id]->average_fft   = 1;

     if (adcName || procName || simName)
        {
         if (adcName)  {adc  = FrAdcDataFind(Dy->Frame, adcName);    vect = adc->data;
                        if (Dy->InputType != INCM  &&  adc->slope != 0.) Plot[id]->realUnits = ON;}
         if (procName) {proc = FrProcDataFind (Dy->Frame, procName); vect = proc->data;}
         if (simName)  {sim  = FrSimDataFind(Dy->Frame, simName);    vect = sim->data;}

         if (strstr(Plot[id]->vVarx,"spectro") != NULL)                  Plot[id]->logz = ON;
         else if (vect->unitX[0] && strstr(vect->unitX[0],"Hz") != NULL) Plot[id]->logz = ON;

         Plot[id]->xnBin = vect->nData;
         if (strstr(Plot[id]->vVarx,"spectro") != NULL) Plot[id]->xnBin = 512;
/**
         Plot[id]->xMin  = vect->startX[0];
         Plot[id]->xMax  = vect->startX[0]+vect->nData*vect->dx[0];
**/
         Plot[id]->xMin  = 0.; 
         Plot[id]->xMax  = 0.;

         Plot[id]->tsize = DyGetFramedt(Dy->Frame);
         if (strstr(Plot[id]->vVarx,"spectro") != NULL) Plot[id]->tsize = 240;

         if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>0)
            {printf("DyPutDefaultInPlot> RAWTIME: plot%d=%s  proc=%x  xnBin=%d  xMin=%g  xMax=%g  nData=%d  dx[0]=%g  startX[0]=%g  ",
                    id, Plot[id]->name, proc, Plot[id]->xnBin, Plot[id]->xMin, Plot[id]->xMax,
                    vect->nData, vect->dx[0], vect->startX[0]);
             if (proc) printf("proc->tRange=%g  fRange=%g \n", proc->tRange, proc->fRange);
             else      printf("\n");
            }
        }

     while (Plot[id]->xnBin > 512) Plot[id]->xnBin = Plot[id]->xnBin / 2;
     Plot[id]->timeSize = 150 * Plot[id]->tsize;
     Plot[id]->refresh_percent = 25.;  /* percentage of timeSize */   
    }


 if (plottype == PLOTTIME)
    {
     DyDecodeVar(Plot[id]->vVarx, &smsName, &varName, &adcName, &procName, &simName);

     Plot[id]->nodc      = OFF;
     Plot[id]->gridx     = ON;   Plot[id]->gridy     = ON;
     Plot[id]->logx      = OFF;  Plot[id]->logy      = OFF;
     Plot[id]->autoScale = ON;   Plot[id]->realUnits = ON;  Plot[id]->bits = OFF;
     Plot[id]->yoffset   = 0.;   Plot[id]->yscale  = 0;

     Plot[id]->resampFactor = 1;

     if (smsName && varName)
        {
         FrSerDataGet(Dy->Frame, smsName, varName, &value);
         sms = FrSerDataFind(Dy->Frame, smsName, NULL);
         if (value > 0.)      {vymin = value*0.5; vymax = value*2.0;}
         else if (value < 0.) {vymin = value*2.0; vymax = value*0.5;}
         else                 {vymin = -0.2; vymax = 0.2;}
         Plot[id]->yMin          = vymin - 2.*(vymax - vymin);
         Plot[id]->yMax          = vymax + 2.*(vymax - vymin);
         Plot[id]->xnBin         = 60;
         Plot[id]->refreshPeriod = Plot[id]->xnBin/10;
         Plot[id]->sampFreq      = sms->sampleRate;
         Plot[id]->tsize         = (double)(Plot[id]->xnBin * Plot[id]->resampFactor)
                                           / Plot[id]->freqClock;
         Plot[id]->tstep         = (double)(Plot[id]->refreshPeriod * Plot[id]->resampFactor)
                                           / Plot[id]->freqClock * (100. / Plot[id]->tsize);
        }

     if (adcName || procName || simName)
        {
         if (adcName)  {adc  = FrAdcDataFind(Dy->Frame, adcName);    vect = adc->data;
                        if (Dy->InputType != INCM  &&  adc->slope != 0.) Plot[id]->realUnits = ON;}
         if (procName) {proc = FrProcDataFind (Dy->Frame, procName); vect = proc->data;}
         if (simName)  {sim  = FrSimDataFind(Dy->Frame, simName);    vect = sim->data;}

         DyGetMinMax(vect, &vymin, &vymax);

         if (adcName != NULL)
            {DyGetMinMaxFromTrend(Dy->Frame, adcName, &vymin, &vymax);}

         if (vymin > 0 && vymax/vymin > 10000.) Plot[id]->logy = ON;
         Plot[id]->yMin          = vymin - 2.*(vymax - vymin);
         Plot[id]->yMax          = vymax + 2.*(vymax - vymin);
         Plot[id]->xnBin         = vect->nData;
         Plot[id]->refreshPeriod = Plot[id]->xnBin / 2;
         Plot[id]->sampFreq      = 1. / vect->dx[0];
         Plot[id]->tsize         = (double)(Plot[id]->xnBin * Plot[id]->resampFactor)
                                           / Plot[id]->freqClock;
         Plot[id]->tstep         = (double)(Plot[id]->refreshPeriod * Plot[id]->resampFactor )
                                           / Plot[id]->freqClock * (100. / Plot[id]->tsize);
        }
     if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>1)
        {printf("PutDefaultInPlot> TIME: plot%d=%s  tsize=%g  tstep=%g  xnBin=%d  resampFactor=%d  refreshPeriod=%d \n",
                 id, Plot[id]->name, Plot[id]->tsize, Plot[id]->tstep, Plot[id]->xnBin, Plot[id]->resampFactor, Plot[id]->refreshPeriod);
        }
    }


 if (plottype == PLOTCORR)
    {
     DyDecodeVar(Plot[id]->vVarx, &smsName, &varName, &adcName, &procName, &simName);

     Plot[id]->nodc      = OFF;
     Plot[id]->gridx     = ON;   Plot[id]->gridy     = ON;
     Plot[id]->logx      = OFF;  Plot[id]->logy      = OFF;
     Plot[id]->autoScale = ON;   Plot[id]->realUnits = ON;
     Plot[id]->yoffset   = 0.;   Plot[id]->yscale  = 0;

     Plot[id]->resampFactor = 1;
     Plot[id]->sampFreq     = Plot[id]->freqClock12 / (double) Plot[id]->resampFactor;
     Plot[id]->xMin         = 0.;
     Plot[id]->xMax         = 0.;
/***
     if (Plot[id]->freqClock < Plot[id]->freqClock2)
        {Plot[id]->resampFactor = (long)(Plot[id]->freqClock2 / Plot[id]->freqClock + 0.5);}
     else
        {Plot[id]->resampFactor = (long)(Plot[id]->freqClock / Plot[id]->freqClock2 + 0.5);}
***/
     if (smsName && varName)
        {
         FrSerDataGet(Dy->Frame, smsName, varName, &value);
         sms = FrSerDataFind(Dy->Frame, smsName, NULL);
         if (value > 0.)      {vymin = value*0.5; vymax = value*2.0;}
         else if (value < 0.) {vymin = value*2.0; vymax = value*0.5;}
         else                 {vymin = -0.2; vymax = 0.2;}
         Plot[id]->yMin          = vymin - 2.*(vymax - vymin);
         Plot[id]->yMax          = vymax + 2.*(vymax - vymin);
         Plot[id]->xnBin         = 60;
         Plot[id]->refreshPeriod = Plot[id]->xnBin/10;
         Plot[id]->sampFreq      = Plot[id]->freqClock12 / (double) Plot[id]->resampFactor;
         Plot[id]->tsize         = (double)(Plot[id]->xnBin) / Plot[id]->freqClock12;
         Plot[id]->tstep         = (double)(Plot[id]->refreshPeriod * Plot[id]->resampFactor)
                                           / Plot[id]->freqClock12 * (100. / Plot[id]->tsize);
        }

     if (adcName || procName || simName)
        {
         if (adcName)  {adc  = FrAdcDataFind(Dy->Frame, adcName);    vect = adc->data;
                        if (Dy->InputType != INCM  &&  adc->slope != 0.) Plot[id]->realUnits = ON;}
         if (procName) {proc = FrProcDataFind (Dy->Frame, procName); vect = proc->data;}
         if (simName)  {sim  = FrSimDataFind(Dy->Frame, simName);    vect = sim->data;}

         DyGetMinMax(vect, &vymin, &vymax);

         if (adcName != NULL)
            {DyGetMinMaxFromTrend(Dy->Frame, adcName, &vymin, &vymax);}

         if (vymin > 0 && vymax/vymin > 10000.) Plot[id]->logy = ON;
         Plot[id]->yMin          = vymin - 2.*(vymax - vymin);
         Plot[id]->yMax          = vymax + 2.*(vymax - vymin);

         Plot[id]->xnBin  = 2;
         Plot[id]->nBinP2 = 1;
         while (Plot[id]->xnBin <= vect->nData) {Plot[id]->xnBin *= 2; Plot[id]->nBinP2++;}

         if (Plot[id]->xnBin<64) {Plot[id]->xnBin = 64; Plot[id]->nBinP2 = 6;}

         DyDecodeVar(Plot[id]->vVary, &smsName, &varName, &adcName, &procName, &simName);

         if (adcName && Dy->InputType != INCM  &&  adc->slope != 0.) Plot[id]->realUnits = ON;
         if (adcName)  {adc  = FrAdcDataFind(Dy->Frame, adcName);    vect = adc->data;}
         if (procName) {proc = FrProcDataFind (Dy->Frame, procName); vect = proc->data;}
         if (simName)  {sim  = FrSimDataFind(Dy->Frame, simName);    vect = sim->data;}

         nBin   = Plot[id]->xnBin;
         nBinP2 = Plot[id]->nBinP2;

         Plot[id]->xnBin  = 2;
         Plot[id]->nBinP2 = 1;
         while (Plot[id]->xnBin <= vect->nData/2) {Plot[id]->xnBin *= 2; Plot[id]->nBinP2++;}
         if (nBin > Plot[id]->xnBin) {Plot[id]->xnBin = nBin; Plot[id]->nBinP2 = nBinP2;}

         if (Plot[id]->xnBin<64) {Plot[id]->xnBin = 64; Plot[id]->nBinP2 = 6;}

         Plot[id]->refreshPeriod = Plot[id]->xnBin / 10;
         if (Dy->InputType == INCM || Dy->InputType == INDD || Dy->InputType == INSHM)
            {Plot[id]->refreshPeriod = Plot[id]->xnBin / 2;}

         Plot[id]->tsize = (double)(Plot[id]->xnBin) / Plot[id]->freqClock12;
         Plot[id]->tstep = (double)(Plot[id]->refreshPeriod * Plot[id]->resampFactor)
                         / Plot[id]->freqClock12 * (100. / Plot[id]->tsize);
        }
     if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>1)
        {printf("PutDefaultInPlot> CORR: plot%d=%s  tsize=%g  tstep=%g  \n", id, Plot[id]->name, Plot[id]->tsize, Plot[id]->tstep);}
    }


 if (plottype == PLOTFFT)
    {
     DyDecodeVar(Plot[id]->vVarx, &smsName, &varName, &adcName, &procName, &simName);

     Plot[id]->rmsfft    = OFF;
     Plot[id]->decayfft  = OFF;
     Plot[id]->hertz     = OFF;
     Plot[id]->nodc      = ON;
     Plot[id]->gridx     = ON;   Plot[id]->gridy     = ON;
     Plot[id]->gridx2    = ON;   Plot[id]->gridy2    = ON;
     Plot[id]->logx      = OFF;  Plot[id]->logy      = ON;
     Plot[id]->logx2     = OFF;  Plot[id]->logy2     = OFF;
     Plot[id]->autoScale = ON;   Plot[id]->realUnits = ON;
     Plot[id]->yMin      = 0.;   Plot[id]->yMax      = 0.;

     Plot[id]->resampFactor  = 1;
     Plot[id]->sampFreq      = Plot[id]->freqClock / (double) Plot[id]->resampFactor;
     Plot[id]->xMin          = 0.;
     Plot[id]->xMax          = 0.;

     if (smsName && varName)
        {
         Plot[id]->xnBin         = 128;
         Plot[id]->nBinP2        = 7;
         Plot[id]->tsize         = (double)(Plot[id]->xnBin) / Plot[id]->sampFreq;
         Plot[id]->tstep         = 50.;
         Plot[id]->refreshPeriod =(int)(Plot[id]->xnBin * Plot[id]->tstep/100.);
         Plot[id]->average_fft   = 1;
         Plot[id]->refresh_fft   = 1;
        }

     if (adcName || procName || simName)
        {
         if (adcName)  {adc  = FrAdcDataFind(Dy->Frame, adcName);    vect = adc->data;
                        if (Dy->InputType != INCM  &&  adc->slope != 0.) Plot[id]->realUnits = ON;}
         if (procName) {proc = FrProcDataFind (Dy->Frame, procName); vect = proc->data;}
         if (simName)  {sim  = FrSimDataFind(Dy->Frame, simName);    vect = sim->data;}

         if (Plot[id]->freqClock > 9999.) Plot[id]->logx = ON;

         Plot[id]->xnBin  = 2;
         Plot[id]->nBinP2 = 1;
         while (Plot[id]->xnBin <= vect->nData/2) {Plot[id]->xnBin *= 2; Plot[id]->nBinP2++;}

         if (Plot[id]->xnBin<64) {Plot[id]->xnBin = 64; Plot[id]->nBinP2 = 6;}

         Plot[id]->tsize         = (double)(Plot[id]->xnBin) / Plot[id]->sampFreq;
         Plot[id]->tstep         = 50.;
         Plot[id]->refreshPeriod =(int)(Plot[id]->xnBin * Plot[id]->tstep/100.);
         Plot[id]->average_fft   = 1;
         Plot[id]->refresh_fft   = 1;
        }

     Plot[id]->dfreq = 0.5 / Plot[id]->tsize;
     Plot[id]->dtime = Plot[id]->tsize * (Plot[id]->tstep/100.) * Plot[id]->average_fft;

     if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>1)
        {printf("DyPutDefaultInPlot> FFT: plot%d=%s  tsize=%g  tstep=%g  average_fft=%d  dfreq=%g  dtime=%g \n",
                 id, Plot[id]->name, Plot[id]->tsize, Plot[id]->tstep, Plot[id]->average_fft, Plot[id]->dfreq, Plot[id]->dtime);
        }
    }


 if (plottype == PLOTTR)
    {
     DyDecodeVar(Plot[id]->vVarx, &smsName, &varName, &adcName, &procName, &simName);

     Plot[id]->rmsfft    = OFF;
     Plot[id]->decayfft  = OFF;
     Plot[id]->hertz     = OFF;
     Plot[id]->nodc      = ON;
     Plot[id]->gridx     = ON;   Plot[id]->gridy     = ON;
     Plot[id]->gridx2    = ON;   Plot[id]->gridy2    = ON;
     Plot[id]->logx      = OFF;  Plot[id]->logy      = ON;
     Plot[id]->logx2     = OFF;  Plot[id]->logy2     = OFF;
     Plot[id]->autoScale = ON;   Plot[id]->realUnits = ON;
     Plot[id]->yMin      = 0.;   Plot[id]->yMax      = 0.;
     Plot[id]->modulesqrt = OFF;

     Plot[id]->resampFactor = 1;
     Plot[id]->sampFreq     = Plot[id]->freqClock12 / (double) Plot[id]->resampFactor;
     freqmin = Plot[id]->sampFreq;
     if (Plot[id]->freqClock < freqmin)  freqmin = Plot[id]->freqClock;
     if (Plot[id]->freqClock2 < freqmin) freqmin = Plot[id]->freqClock2;
     Plot[id]->xMin         = 0.;
     Plot[id]->xMax         = freqmin / 2.;

     if (smsName && varName)
        {
         Plot[id]->xnBin         = 128;
         Plot[id]->nBinP2        = 7;
         Plot[id]->tsize         = (double)(Plot[id]->xnBin) / Plot[id]->freqClock12;
         Plot[id]->tstep         = 50.;
         Plot[id]->refreshPeriod =(int)(Plot[id]->xnBin * Plot[id]->tstep/100.);
         Plot[id]->average_fft   = 1;
         Plot[id]->refresh_fft   = 1;
        }

     if (adcName || procName || simName)
        {
         if (adcName)  {adc  = FrAdcDataFind(Dy->Frame, adcName);    vect = adc->data;
                        if (Dy->InputType != INCM  &&  adc->slope != 0.) Plot[id]->realUnits = ON;}
         if (procName) {proc = FrProcDataFind (Dy->Frame, procName); vect = proc->data;}
         if (simName)  {sim  = FrSimDataFind(Dy->Frame, simName);    vect = sim->data;}

         if (Plot[id]->freqClock12 > 9999.)  {Plot[id]->logx = ON; Plot[id]->logx2 = ON;}

         Plot[id]->xnBin  = 2;
         Plot[id]->nBinP2 = 1;
         while (Plot[id]->xnBin <= vect->nData/2) {Plot[id]->xnBin *= 2; Plot[id]->nBinP2++;}

         if (Plot[id]->xnBin<64) {Plot[id]->xnBin = 64; Plot[id]->nBinP2 = 6;}

         DyDecodeVar(Plot[id]->vVary, &smsName, &varName, &adcName, &procName, &simName);

         if (adcName) {adc  = FrAdcDataFind(Dy->Frame, adcName); vect = adc->data;
                       if (Dy->InputType != INCM  &&  adc->slope != 0.) Plot[id]->realUnits = ON;}
         if (procName) {proc = FrProcDataFind (Dy->Frame, procName); vect = proc->data;}
         if (simName)  {sim  = FrSimDataFind(Dy->Frame, simName);    vect = sim->data;}

         nBin   = Plot[id]->xnBin;
         nBinP2 = Plot[id]->nBinP2;

         Plot[id]->xnBin  = 2;
         Plot[id]->nBinP2 = 1;
         while (Plot[id]->xnBin <= vect->nData/2) {Plot[id]->xnBin *= 2; Plot[id]->nBinP2++;}
         if (nBin > Plot[id]->xnBin) {Plot[id]->xnBin = nBin; Plot[id]->nBinP2 = nBinP2;}

         if (Plot[id]->xnBin<64) {Plot[id]->xnBin = 64; Plot[id]->nBinP2 = 6;}

         Plot[id]->tsize         = (double)(Plot[id]->xnBin) / Plot[id]->freqClock12;
         Plot[id]->tstep         = 50.;
         Plot[id]->refreshPeriod =(int)(Plot[id]->xnBin * Plot[id]->tstep/100.);
         Plot[id]->average_fft   = 1;
         Plot[id]->refresh_fft   = 1;
        }

     Plot[id]->dfreq = 0.5 / Plot[id]->tsize;
     Plot[id]->dtime = Plot[id]->tsize * (Plot[id]->tstep/100.) * Plot[id]->average_fft;

     if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>1)
        {printf("DyPutDefaultInPlot> TRFCT: plot%d=%s  tsize=%g  tstep=%g  average_fft=%d  dfreq=%g  dtime=%g \n",
                 id, Plot[id]->name, Plot[id]->tsize, Plot[id]->tstep, Plot[id]->average_fft, Plot[id]->dfreq, Plot[id]->dtime);
        }
    }


 if (plottype == PLOTCOHE)
    {
     DyDecodeVar(Plot[id]->vVarx, &smsName, &varName, &adcName, &procName, &simName);

     Plot[id]->rmsfft    = OFF;
     Plot[id]->decayfft  = OFF;
     Plot[id]->hertz     = OFF;
     Plot[id]->nodc      = ON;
     Plot[id]->gridx     = ON;   Plot[id]->gridy     = ON;
     Plot[id]->gridx2    = ON;   Plot[id]->gridy2    = ON;
     Plot[id]->logx      = OFF;  Plot[id]->logy      = OFF;
     Plot[id]->logx2     = OFF;  Plot[id]->logy2     = OFF;
     Plot[id]->autoScale = OFF;   Plot[id]->realUnits = ON;
     Plot[id]->yMin      = 0.;   Plot[id]->yMax      = 0.;
     Plot[id]->crossCohe = OFF;
     Plot[id]->modulesqrt = OFF;

     Plot[id]->resampFactor = 1;
     Plot[id]->sampFreq     = Plot[id]->freqClock12 / (double) Plot[id]->resampFactor;
     freqmin = Plot[id]->sampFreq;
     if (Plot[id]->freqClock < freqmin)  freqmin = Plot[id]->freqClock;
     if (Plot[id]->freqClock2 < freqmin) freqmin = Plot[id]->freqClock2;
     Plot[id]->xMin         = 0.;
     Plot[id]->xMax         = freqmin / 2.;
     Plot[id]->crossCohe_fmin = 0.;
     Plot[id]->crossCohe_fmax = freqmin / 2.;

     if (smsName && varName)
        {
         Plot[id]->xnBin         = 128;
         Plot[id]->nBinP2        = 7;
         Plot[id]->tsize         = (double)(Plot[id]->xnBin) / Plot[id]->freqClock12;
         Plot[id]->tstep         = 50.;
         Plot[id]->refreshPeriod =(int)(Plot[id]->xnBin * Plot[id]->tstep/100.);
         Plot[id]->refresh_fft   = 1;
        }

     if (adcName || procName || simName)
        {
         if (adcName)  {adc  = FrAdcDataFind(Dy->Frame, adcName);    vect = adc->data;
                        if (Dy->InputType != INCM  &&  adc->slope != 0.) Plot[id]->realUnits = ON;}
         if (procName) {proc = FrProcDataFind (Dy->Frame, procName); vect = proc->data;}
         if (simName)  {sim  = FrSimDataFind(Dy->Frame, procName);   vect = sim->data;}

         if (Plot[id]->freqClock12 > 9999.)  {Plot[id]->logx = ON; Plot[id]->logx2 = ON;}

         Plot[id]->xnBin  = 2;
         Plot[id]->nBinP2 = 1;
         while (Plot[id]->xnBin <= vect->nData/2) {Plot[id]->xnBin *= 2; Plot[id]->nBinP2++;}

         if (Plot[id]->xnBin<64) {Plot[id]->xnBin = 64; Plot[id]->nBinP2 = 6;}

         DyDecodeVar(Plot[id]->vVary, &smsName, &varName, &adcName, &procName, &simName);

         if (adcName)  {adc  = FrAdcDataFind(Dy->Frame, adcName);    vect = adc->data;
                        if (Dy->InputType != INCM  &&  adc->slope != 0.) Plot[id]->realUnits = ON;}
         if (procName) {proc = FrProcDataFind (Dy->Frame, procName); vect = proc->data;}
         if (simName)  {sim  = FrSimDataFind(Dy->Frame, simName);    vect = sim->data;}

         nBin   = Plot[id]->xnBin;
         nBinP2 = Plot[id]->nBinP2;

         Plot[id]->xnBin  = 2;
         Plot[id]->nBinP2 = 1;
         while (Plot[id]->xnBin <= vect->nData/2) {Plot[id]->xnBin *= 2; Plot[id]->nBinP2++;}
         if (nBin > Plot[id]->xnBin) {Plot[id]->xnBin = nBin; Plot[id]->nBinP2 = nBinP2;}

         if (Plot[id]->xnBin<64) {Plot[id]->xnBin = 64; Plot[id]->nBinP2 = 6;}

         Plot[id]->tsize         = (double)(Plot[id]->xnBin) / Plot[id]->freqClock12;
         Plot[id]->tstep         = 50.;
         Plot[id]->refreshPeriod =(int)(Plot[id]->xnBin * Plot[id]->tstep/100.);
         Plot[id]->refresh_fft   = 1;
        }

     Plot[id]->average_fft = 100000;

     Plot[id]->dfreq = 0.5 / Plot[id]->tsize;
     Plot[id]->dtime = Plot[id]->tsize * (Plot[id]->tstep/100.) * Plot[id]->average_fft;

     if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>1)
        {printf("DyPutDefaultInPlot> COHE: plot%d=%s  tsize=%g  tstep=%g  average_fft=%d  dfreq=%g  dtime=%g \n",
                 id, Plot[id]->name, Plot[id]->tsize, Plot[id]->tstep, Plot[id]->average_fft, Plot[id]->dfreq, Plot[id]->dtime);
        }
    }


 if (plottype == PLOTFFTTIME)
    {
     DyDecodeVar(Plot[id]->vVarx, &smsName, &varName, &adcName, &procName, &simName);

     Plot[id]->rmsfft    = OFF;
     Plot[id]->decayfft  = OFF;
     Plot[id]->hertz     = OFF;
     Plot[id]->nodc      = ON;
     Plot[id]->gridx     = ON;   Plot[id]->gridy     = ON;
     Plot[id]->gridx2    = ON;   Plot[id]->gridy2    = ON;
     Plot[id]->logx      = OFF;  Plot[id]->logy      = ON;
     Plot[id]->logx2     = OFF;  Plot[id]->logy2     = OFF;
     Plot[id]->autoScale = ON;   Plot[id]->realUnits = ON;
     Plot[id]->yMin      = 0.;   Plot[id]->yMax      = 0.;

     Plot[id]->resampFactor  = 1;
     Plot[id]->sampFreq      = Plot[id]->freqClock / (double) Plot[id]->resampFactor;
     Plot[id]->xMin          = 0.;
     Plot[id]->xMax          = 0.;

     if (smsName && varName)
        {
         Plot[id]->xnBin         = 128;
         Plot[id]->nBinP2        = 7;
         Plot[id]->tsize         = (double)(Plot[id]->xnBin) / Plot[id]->sampFreq;
         Plot[id]->tstep         = 50.;
         Plot[id]->refreshPeriod =(int)(Plot[id]->xnBin * Plot[id]->tstep/100.);
         Plot[id]->average_fft   = 1;
         Plot[id]->refresh_fft   = 1;
        }

     if (adcName || procName || simName)
        {
         if (adcName)  {adc  = FrAdcDataFind(Dy->Frame, adcName);    vect = adc->data;
                        if (Dy->InputType != INCM  &&  adc->slope != 0.) Plot[id]->realUnits = ON;}
         if (procName) {proc = FrProcDataFind (Dy->Frame, procName); vect = proc->data;}
         if (simName)  {sim  = FrSimDataFind(Dy->Frame, simName);    vect = sim->data;}

         Plot[id]->xnBin  = 2;
         Plot[id]->nBinP2 = 1;
         while (Plot[id]->xnBin <= vect->nData/2) {Plot[id]->xnBin *= 2; Plot[id]->nBinP2++;}
         if (Plot[id]->xnBin < 64)  {Plot[id]->xnBin = 64;  Plot[id]->nBinP2 = 6;}
         if (Plot[id]->xnBin > 512) {Plot[id]->xnBin = 512; Plot[id]->nBinP2 = 9;}
         Plot[id]->tsize         = (double)(Plot[id]->xnBin) / Plot[id]->sampFreq;
         Plot[id]->tstep         = 10.;
         Plot[id]->refreshPeriod =(int)(Plot[id]->xnBin * Plot[id]->tstep/100.);
         Plot[id]->average_fft   = 10;
         Plot[id]->refresh_fft   = 1;
        }

     Plot[id]->timeSize        = 256. * Plot[id]->tsize * Plot[id]->average_fft * Plot[id]->tstep / 100.;
     Plot[id]->refresh_percent = 25.;  /* percentage of timeSize */
     Plot[id]->logx = OFF;
     Plot[id]->logy = OFF;
     Plot[id]->logz = ON;

     if (Plot[id]->freqClock > 999.) Plot[id]->logy = ON;

     Plot[id]->dfreq = 0.5 / Plot[id]->tsize;
     Plot[id]->dtime = Plot[id]->tsize * (Plot[id]->tstep/100.) * Plot[id]->average_fft;

     if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>1)
        {printf("DyPutDefaultInPlot> FFTTIME: plot%d=%s  tsize=%g  tstep=%g  average_fft=%d  dfreq=%g  dtime=%g \n",
                 id, Plot[id]->name, Plot[id]->tsize, Plot[id]->tstep, Plot[id]->average_fft, Plot[id]->dfreq, Plot[id]->dtime);
        }
    }


 if (plottype == PLOTTRTIME)
    {
     DyDecodeVar(Plot[id]->vVarx, &smsName, &varName, &adcName, &procName, &simName);

     Plot[id]->rmsfft    = OFF;
     Plot[id]->decayfft  = OFF;
     Plot[id]->hertz     = OFF;
     Plot[id]->nodc      = ON;
     Plot[id]->gridx     = ON;   Plot[id]->gridy     = ON;
     Plot[id]->gridx2    = ON;   Plot[id]->gridy2    = ON;
     Plot[id]->logx      = OFF;  Plot[id]->logy      = ON;
     Plot[id]->logx2     = OFF;  Plot[id]->logy2     = OFF;
     Plot[id]->autoScale = ON;   Plot[id]->realUnits = ON;
     Plot[id]->yMin      = 0.;   Plot[id]->yMax      = 0.;

     Plot[id]->resampFactor = 1;
     Plot[id]->sampFreq     = Plot[id]->freqClock12 / (double) Plot[id]->resampFactor;
     freqmin = Plot[id]->sampFreq;
     if (Plot[id]->freqClock < freqmin)  freqmin = Plot[id]->freqClock;
     if (Plot[id]->freqClock2 < freqmin) freqmin = Plot[id]->freqClock2;
     Plot[id]->xMin         = 0.;
     Plot[id]->xMax         = freqmin / 2.;

     if (smsName && varName)
        {
         Plot[id]->xnBin         = 128;
         Plot[id]->nBinP2        = 7;
         Plot[id]->tsize         = (double)(Plot[id]->xnBin) / Plot[id]->freqClock12;
         Plot[id]->tstep         = 20.;
         Plot[id]->refreshPeriod =(int)(Plot[id]->xnBin * Plot[id]->tstep/100.);
         Plot[id]->average_fft   = 1;
         Plot[id]->refresh_fft   = 1;
        }

     if (adcName || procName || simName)
        {
         if (adcName)  {adc  = FrAdcDataFind(Dy->Frame, adcName);    vect = adc->data;
                        if (Dy->InputType != INCM  &&  adc->slope != 0.) Plot[id]->realUnits = ON;}
         if (procName) {proc = FrProcDataFind (Dy->Frame, procName); vect = proc->data;}
         if (simName)  {sim  = FrSimDataFind(Dy->Frame, simName);    vect = sim->data;}

         Plot[id]->xnBin  = 2;
         Plot[id]->nBinP2 = 1;
         while (Plot[id]->xnBin <= vect->nData/2) {Plot[id]->xnBin *= 2; Plot[id]->nBinP2++;}

         if (Plot[id]->xnBin < 64)   {Plot[id]->xnBin = 64;   Plot[id]->nBinP2 = 6;}
         if (Plot[id]->xnBin > 1024) {Plot[id]->xnBin = 1024; Plot[id]->nBinP2 = 10;}

         DyDecodeVar(Plot[id]->vVary, &smsName, &varName, &adcName, &procName, &simName);

         if (adcName)  {adc  = FrAdcDataFind(Dy->Frame, adcName);    vect = adc->data;
                            if (Dy->InputType != INCM  &&  adc->slope != 0.) Plot[id]->realUnits = ON;}
         if (procName) {proc = FrProcDataFind (Dy->Frame, procName); vect = proc->data;}
         if (simName)  {sim  = FrSimDataFind(Dy->Frame, simName);    vect = sim->data;}

         nBin   = Plot[id]->xnBin;
         nBinP2 = Plot[id]->nBinP2;

         Plot[id]->xnBin  = 2;
         Plot[id]->nBinP2 = 1;
         while (Plot[id]->xnBin <= vect->nData/2) {Plot[id]->xnBin *= 2; Plot[id]->nBinP2++;}
         if (nBin > Plot[id]->xnBin) {Plot[id]->xnBin = nBin; Plot[id]->nBinP2 = nBinP2;}

         if (Plot[id]->xnBin < 64)   {Plot[id]->xnBin = 64;   Plot[id]->nBinP2 = 6;}
         if (Plot[id]->xnBin > 1024) {Plot[id]->xnBin = 1024; Plot[id]->nBinP2 = 10;}

         Plot[id]->tsize         = (double)(Plot[id]->xnBin) / Plot[id]->freqClock12;
         Plot[id]->tstep         = 10.;
         Plot[id]->refreshPeriod =(int)(Plot[id]->xnBin * Plot[id]->tstep/100.);
         Plot[id]->average_fft   = 10;
         Plot[id]->refresh_fft   = 1;
        }

     Plot[id]->timeSize        = 256. * Plot[id]->tsize * Plot[id]->average_fft * Plot[id]->tstep / 100.;
     Plot[id]->refresh_percent = 25.;  /* percentage of timeSize */
     Plot[id]->logx = OFF;
     Plot[id]->logy = OFF;
     Plot[id]->logz = ON;

     if (Plot[id]->freqClock12 > 9999.) Plot[id]->logy = ON;

     Plot[id]->dfreq = 0.5 / Plot[id]->tsize;
     Plot[id]->dtime = Plot[id]->tsize * (Plot[id]->tstep/100.) * Plot[id]->average_fft;

     if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>1)
        {printf("DyPutDefaultInPlot> TRFCTTIME: plot%d=%s  tsize=%g  tstep=%g  average_fft=%d  dfreq=%g  dtime=%g \n",
                 id, Plot[id]->name, Plot[id]->tsize, Plot[id]->tstep, Plot[id]->average_fft, Plot[id]->dfreq, Plot[id]->dtime);
        }
    }


 if (plottype == PLOTCOHETIME)
    {
     DyDecodeVar(Plot[id]->vVarx, &smsName, &varName, &adcName, &procName, &simName);

     Plot[id]->rmsfft    = OFF;
     Plot[id]->decayfft  = OFF;
     Plot[id]->hertz     = OFF;
     Plot[id]->nodc      = ON;
     Plot[id]->gridx     = ON;   Plot[id]->gridy     = ON;
     Plot[id]->gridx2    = ON;   Plot[id]->gridy2    = ON;
     Plot[id]->logx      = OFF;  Plot[id]->logy      = ON;
     Plot[id]->logx2     = OFF;  Plot[id]->logy2     = OFF;
     Plot[id]->autoScale = ON;   Plot[id]->realUnits = ON;
     Plot[id]->yMin      = 0.;   Plot[id]->yMax      = 0.;

     Plot[id]->resampFactor = 1;
     Plot[id]->sampFreq     = Plot[id]->freqClock12 / (double) Plot[id]->resampFactor;
     freqmin = Plot[id]->sampFreq;
     if (Plot[id]->freqClock < freqmin)  freqmin = Plot[id]->freqClock;
     if (Plot[id]->freqClock2 < freqmin) freqmin = Plot[id]->freqClock2;
     Plot[id]->xMin         = 0.;
     Plot[id]->xMax         = freqmin / 2.;

     if (smsName && varName)
        {
         Plot[id]->xnBin         = 128;
         Plot[id]->nBinP2        = 7;
         Plot[id]->tsize         = (double)(Plot[id]->xnBin) / Plot[id]->freqClock12;
         Plot[id]->tstep         = 20.;
         Plot[id]->refreshPeriod =(int)(Plot[id]->xnBin * Plot[id]->tstep/100.);
         Plot[id]->average_fft   = 10;
         Plot[id]->refresh_fft   = 1;
        }

     if (adcName || procName || simName)
        {
         if (adcName)  {adc  = FrAdcDataFind(Dy->Frame, adcName);    vect = adc->data;
                        if (Dy->InputType != INCM  &&  adc->slope != 0.) Plot[id]->realUnits = ON;}
         if (procName) {proc = FrProcDataFind (Dy->Frame, procName); vect = proc->data;}
         if (simName)  {sim  = FrSimDataFind(Dy->Frame, simName);    vect = sim->data;}

         Plot[id]->xnBin  = 2;
         Plot[id]->nBinP2 = 1;
         while (Plot[id]->xnBin <= vect->nData/2) {Plot[id]->xnBin *= 2; Plot[id]->nBinP2++;}

         if (Plot[id]->xnBin < 64)   {Plot[id]->xnBin = 64;   Plot[id]->nBinP2 = 6;}
         if (Plot[id]->xnBin > 1024) {Plot[id]->xnBin = 1024; Plot[id]->nBinP2 = 10;}

         DyDecodeVar(Plot[id]->vVary, &smsName, &varName, &adcName, &procName, &simName);

         if (adcName)  {adc  = FrAdcDataFind(Dy->Frame, adcName);    vect = adc->data;
                        if (Dy->InputType != INCM  &&  adc->slope != 0.) Plot[id]->realUnits = ON;}
         if (procName) {proc = FrProcDataFind (Dy->Frame, procName); vect = proc->data;}
         if (simName)  {sim  = FrSimDataFind(Dy->Frame, simName);    vect = sim->data;}

         nBin   = Plot[id]->xnBin;
         nBinP2 = Plot[id]->nBinP2;

         Plot[id]->xnBin  = 2;
         Plot[id]->nBinP2 = 1;
         while (Plot[id]->xnBin <= vect->nData/2) {Plot[id]->xnBin *= 2; Plot[id]->nBinP2++;}
         if (nBin > Plot[id]->xnBin) {Plot[id]->xnBin = nBin; Plot[id]->nBinP2 = nBinP2;}

         if (Plot[id]->xnBin < 64)   {Plot[id]->xnBin = 64;   Plot[id]->nBinP2 = 6;}
         if (Plot[id]->xnBin > 1024) {Plot[id]->xnBin = 1024; Plot[id]->nBinP2 = 10;}

         Plot[id]->tsize         = (double)(Plot[id]->xnBin) / Plot[id]->freqClock12;
         Plot[id]->tstep         = 10.;
         Plot[id]->refreshPeriod = (int)(Plot[id]->xnBin * Plot[id]->tstep/100.);
         Plot[id]->average_fft   = 10;
         Plot[id]->refresh_fft   = 1;
        }

     Plot[id]->timeSize        = 256. * Plot[id]->tsize * Plot[id]->average_fft * Plot[id]->tstep / 100.;
     Plot[id]->refresh_percent = 25.;  /* percentage of timeSize */
     Plot[id]->logx = OFF;
     Plot[id]->logy = OFF;
     Plot[id]->logz = OFF;

     if (Plot[id]->freqClock12 > 999.) Plot[id]->logy = ON;
 
     Plot[id]->dfreq = 0.5 / Plot[id]->tsize;
     Plot[id]->dtime = Plot[id]->tsize * (Plot[id]->tstep/100.) * Plot[id]->average_fft;

     if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>1)
        {printf("DyPutDefaultInPlot> COHETIME: plot%d=%s  tsize=%g  tstep=%g  average_fft=%d  dfreq=%g  dtime=%g \n",
                 id, Plot[id]->name, Plot[id]->tsize, Plot[id]->tstep, Plot[id]->average_fft, Plot[id]->dfreq, Plot[id]->dtime);
        }
    }


 if (plottype == PLOT1DTIME)
    {
     DyDecodeVar(Plot[id]->vVarx, &smsName, &varName, &adcName, &procName, &simName);

     Plot[id]->nodc   = ON;
     Plot[id]->gridx  = ON;   Plot[id]->gridy  = ON;
     Plot[id]->gridx2 = ON;   Plot[id]->gridy2 = ON;
     Plot[id]->logx   = OFF;  Plot[id]->logy   = OFF;
     Plot[id]->logx2  = OFF;  Plot[id]->logy2  = OFF;
     Plot[id]->autoScale = ON; Plot[id]->autoEdge = ON; Plot[id]->realUnits = ON;

     Plot[id]->resampFactor = 1;
     Plot[id]->sampFreq     = Plot[id]->freqClock / (double) Plot[id]->resampFactor;
     Plot[id]->xMin         = -1.e-15;
     Plot[id]->xMax         =  1.e-15;

     if (smsName && varName)
        {
         Plot[id]->nBin          = 60;
         Plot[id]->refreshPeriod = Plot[id]->nBin;
         Plot[id]->tsize         = (double)(Plot[id]->nBin) / Plot[id]->sampFreq;
        }

     if (adcName || procName || simName)
        {
         if (adcName)  {adc  = FrAdcDataFind(Dy->Frame, adcName);    vect = adc->data;
                        if (Dy->InputType != INCM  &&  adc->slope != 0.) Plot[id]->realUnits = ON;}
         if (procName) {proc = FrProcDataFind (Dy->Frame, procName); vect = proc->data;}
         if (simName)  {sim  = FrSimDataFind(Dy->Frame, simName);    vect = sim->data;}

         Plot[id]->nBin          = vect->nData/2;
         Plot[id]->refreshPeriod = Plot[id]->nBin;
         Plot[id]->tsize         = (double)(Plot[id]->nBin) / Plot[id]->sampFreq;
        }

     Plot[id]->xnBin           = 128;
     Plot[id]->timeSize        = 128. * Plot[id]->tsize ;
     Plot[id]->refresh_percent = 25.;  /* percentage of timeSize */

     Plot[id]->dfreq = 0.;
     Plot[id]->dtime = Plot[id]->tsize;
     if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>1)
        {printf("DyPutDefaultInPlot> 1DTIME: plot%d=%s  tsize=%g  tstep=%g  average_fft=%d  dfreq=%g  dtime=%g \n",
                 id, Plot[id]->name, Plot[id]->tsize, Plot[id]->tstep, Plot[id]->average_fft, Plot[id]->dfreq, Plot[id]->dtime);
        }
    }


 if (plottype == PLOT1D)
    {
     DyDecodeVar(Plot[id]->vVarx, &smsName, &varName, &adcName, &procName, &simName);

     Plot[id]->gridx = ON;  Plot[id]->gridy    = ON;
     Plot[id]->logx  = OFF; Plot[id]->logy     = OFF;
     Plot[id]->stat  = ON;  Plot[id]->autoEdge = ON;  Plot[id]->realUnits = ON;

     Plot[id]->resampFactor = 1;
     Plot[id]->sampFreq     = Plot[id]->freqClock / (double)Plot[id]->resampFactor;

     if (smsName && varName)
        {
         FrSerDataGet(Dy->Frame, smsName, varName, &value);
         if (value > 0.)      {vxmin = value*0.8; vxmax = value*1.2;}
         else if (value < 0.) {vxmin = value*1.2; vxmax = value*0.8;}
         else                 {vxmin = 0.33; vxmax = 0.33;}
         Plot[id]->xnBin         = 128;
         Plot[id]->xMin          = vxmin - (vxmax - vxmin);
         Plot[id]->xMax          = vxmax + (vxmax - vxmin);
         Plot[id]->refreshPeriod = 5;
         Plot[id]->tstep         = (double)Plot[id]->refreshPeriod / Plot[id]->freqClock;
        }

     if (adcName || procName || simName)
        {
         if (adcName)  {adc  = FrAdcDataFind(Dy->Frame, adcName);    vect = adc->data;
                        if (Dy->InputType != INCM  &&  adc->slope != 0.) Plot[id]->realUnits = ON;}
         if (procName) {proc = FrProcDataFind (Dy->Frame, procName); vect = proc->data;}
         if (simName)  {sim  = FrSimDataFind(Dy->Frame, simName);    vect = sim->data;}

        DyGetMinMax(vect, &vxmin, &vxmax);

         if (adcName != NULL)
            {DyGetMinMaxFromTrend(Dy->Frame, adcName, &vxmin, &vxmax);}

        if (vxmin > 0 && vxmax/vxmin > 1000.) {Plot[id]->logx = ON; Plot[id]->logy = ON;}
        if (vxmax <= vxmin)
           {Plot[id]->xMin = -1.e-15;
            Plot[id]->xMax =  1.e-15;
           }
        else
           {Plot[id]->xMin = vxmin - (vxmax - vxmin);
            Plot[id]->xMax = vxmax + (vxmax - vxmin);
           }
        Plot[id]->xnBin         = 128;
        Plot[id]->refreshPeriod = vect->nData * 2;
        if (Plot[id]->refreshPeriod < 64) Plot[id]->refreshPeriod = 64;
        Plot[id]->tstep         = (double)Plot[id]->refreshPeriod / Plot[id]->freqClock;
       }
     if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>1)
        {printf("DyPutDefaultInPlot> 1D: plot%d=%s  tsize=%g  tstep=%g  xnBin=%d  refreshPeriod=%d \n",
                 id, Plot[id]->name, Plot[id]->tsize, Plot[id]->tstep, Plot[id]->xnBin, Plot[id]->refreshPeriod);
        }
    }


 if (plottype == PLOT2D || plottype == PLOT3D)
    {
     Plot[id]->gridx  = ON;  Plot[id]->gridy    = ON;
     Plot[id]->logx   = OFF; Plot[id]->logy     = OFF; Plot[id]->logz      = OFF;
     Plot[id]->stat   = OFF; Plot[id]->autoEdge = ON;  Plot[id]->realUnits = ON;

     if (Dy->InputType == INCM) Plot[id]->type2d = SCAT;
     else                       Plot[id]->type2d = COL;

     DyDecodeVar(Plot[id]->vVarx, &smsName, &varName, &adcName, &procName, &simName);

     if (smsName && varName)
        {
         FrSerDataGet(Dy->Frame, smsName, varName, &value);
         if (value > 0.)      {vxmin = value*0.8; vxmax = value*1.2;}
         else if (value < 0.) {vxmin = value*1.2; vxmax = value*0.8;}
         else                 {vxmin = 0.33; vxmax = 0.33;}
         Plot[id]->xnBin         = 128;
         Plot[id]->xMin          = vxmin - (vxmax - vxmin);
         Plot[id]->xMax          = vxmax + (vxmax - vxmin);
         Plot[id]->refreshPeriod = 5;
         Plot[id]->tstep         = (double)Plot[id]->refreshPeriod / Plot[id]->freqClock12;
        }

     if (adcName || procName || simName)
        {
         if (adcName)  {adc  = FrAdcDataFind(Dy->Frame, adcName);    vect = adc->data;
                        if (Dy->InputType != INCM  &&  adc->slope != 0.) Plot[id]->realUnits = ON;}
         if (procName) {proc = FrProcDataFind (Dy->Frame, procName); vect = proc->data;}
         if (simName)  {sim  = FrSimDataFind(Dy->Frame, simName);    vect = sim->data;}

         DyGetMinMax(vect, &vxmin, &vxmax);

         if (adcName != NULL)
            {DyGetMinMaxFromTrend(Dy->Frame, adcName, &vxmin, &vxmax);}

         if (vxmax <= vxmin)
            {Plot[id]->xMin = -1.e-15;
             Plot[id]->xMax =  1.e-15;
            }
         else
            {Plot[id]->xMin = vxmin - (vxmax - vxmin);
             Plot[id]->xMax = vxmax + (vxmax - vxmin);
            }
         Plot[id]->xnBin         = 256;
         Plot[id]->refreshPeriod = vect->nData * 2;
         Plot[id]->tstep         = (double)Plot[id]->refreshPeriod / Plot[id]->freqClock12;
        }

     DyDecodeVar(Plot[id]->vVary, &smsName, &varName, &adcName, &procName, &simName);

     if (smsName && varName)
        {
         FrSerDataGet(Dy->Frame, smsName, varName, &value);
         if (value > 0.)      {vymin = value*0.8; vymax = value*1.2;}
         else if (value < 0.) {vymin = value*1.2; vymax = value*0.8;}
         else                 {vymin = 0.33; vymax = 0.33;}
         Plot[id]->ynBin         = 128;
         Plot[id]->yMin          = vymin - (vymax - vymin);
         Plot[id]->yMax          = vymax + (vymax - vymin);
         Plot[id]->yMin          = 0.;
         Plot[id]->yMax          = -1.;
         Plot[id]->refreshPeriod = 5;
         Plot[id]->tstep         = (double)Plot[id]->refreshPeriod / Plot[id]->freqClock12;
        }

     if (adcName || procName || simName)
        {
         if (adcName)  {adc  = FrAdcDataFind(Dy->Frame, adcName);    vect = adc->data;
                        if (Dy->InputType != INCM  &&  adc->slope != 0.) Plot[id]->realUnits = ON;}
         if (procName) {proc = FrProcDataFind (Dy->Frame, procName); vect = proc->data;}
         if (simName)  {sim  = FrSimDataFind(Dy->Frame, simName);    vect = sim->data;}

         DyGetMinMax(vect, &vymin, &vymax);

         if (adcName != NULL)
            {DyGetMinMaxFromTrend(Dy->Frame, adcName, &vymin, &vymax);}

         if (vymax <= vymin)
            {Plot[id]->yMin = -1.e-15;
             Plot[id]->yMax =  1.e-15;
            }
         else
            {Plot[id]->yMin = vymin - (vymax - vymin);
             Plot[id]->yMax = vymax + (vymax - vymin);
            }
         Plot[id]->ynBin = 256;
         if (vect->nData > 2 * Plot[id]->refreshPeriod)
            {Plot[id]->refreshPeriod = vect->nData * 2;}
         Plot[id]->tstep = (double)Plot[id]->refreshPeriod / Plot[id]->freqClock12;
        }
     if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>1)
        {printf("DyPutDefaultInPlot> 2D: plot%d=%s  tsize=%g  tstep=%g  xnBin=%d  refreshPeriod=%d \n",
                 id, Plot[id]->name, Plot[id]->tsize, Plot[id]->tstep, Plot[id]->xnBin, Plot[id]->refreshPeriod);
        }
    }


 if (plottype == PLOTAUDIO)
    {
     DyDecodeVar(Plot[id]->vVarx, &smsName, &varName, &adcName, &procName, &simName);

     nplot = fl_get_browser_maxline(mainbrowser->plots);
     for (i=1; i<=nplot; i++)
        {if (Plot[i]->type == plottype && Plot[i]->soundonline == ON) Plot[i]->soundonline = OFF;}
     Plot[id]->soundonline = ON; 
     Plot[id]->sound16bits = OFF; 
     Plot[id]->t0          = 0;
     Plot[id]->duration    = Dy->duration;
     if (Plot[id]->duration <= 0 || Plot[id]->duration > 10) Plot[id]->duration = 10;
     if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>1)
        {printf("DyPutDefaultInPlot> AUDIO: plot%d=%s  duration=%d  soundonline=%d  sound16bits=%d \n",
                 id, Plot[id]->name, Plot[id]->duration, Plot[id]->soundonline, Plot[id]->sound16bits);
        }
    }

 if (!adcName && !smsName && !varName && !procName)
    {printf("DyPutDefaultInPlot> plot %d of type %d does not exist \n", id, plottype);}

 if (id > 0)
    {DyConstrain(&(Plot[id]->yMin));
     DyConstrain(&(Plot[id]->yMax));
    }

 if (adcName)  free(adcName);
 if (smsName)  free(smsName);
 if (varName)  free(varName);
 if (procName) free(procName);

 return;
}


/*------------------------------------------- DyConstrain ---*/

void DyConstrain (double *value)
{
 if (*value >  1.e+33) *value =  1.e+33;
 if (*value < -1.e+33) *value = -1.e+33;
 if (*value<0  &&  *value > -1.e-33) *value = -1.e-33;
 if (*value>0  &&  *value <  1.e-33) *value =  1.e-33;

 return;
}



/*------------------------------------------- DyGetDefaultFromPlot ---*/
/*
  Called when: a plot is created and a plot of same type already exists
  Called why: to put in the new plot the same parameters as the existing plot
  Call what: NONE
*/

void DyGetDefaultFromPlot (long id)
{
 int nplot, i;
 long   plottype;
 char dummy[LSTRING_MAX];

 sprintf(dummy,"DyGetDefaultFromPlot> plot%d  name=%s  type=%s", id, Plot[id]->name, Dy->typeName[Plot[id]->type]);
 DyDebugPrint(dummy, 0);

 plottype = Plot[id]->type;

 /*--- If it is a RAW plot ---*/

 if (plottype == PLOTRAW)
    {
    }

 /*--- If it is a Raw-Time plot ---*/

 if (plottype == PLOTRAWTIME)
    {
    }

 /*--- If it is a TIME plot ---*/

 if (plottype == PLOTTIME  ||  plottype == PLOTCORR)
    {
     nplot = fl_get_browser_maxline (mainbrowser->plots);
     for (i=1; i<=nplot; i++)
        {if (i != id && Plot[i]->type == plottype && Plot[i]->freqClock == Plot[id]->freqClock)
            {Plot[id]->xnBin         = Plot[i]->xnBin;
             Plot[id]->tsize         = Plot[i]->tsize;
             Plot[id]->tstep         = Plot[i]->tstep;
             Plot[id]->sampFreq      = Plot[i]->sampFreq;
             Plot[id]->resampFactor  = Plot[i]->resampFactor;
             Plot[id]->refreshPeriod = Plot[i]->refreshPeriod;
             if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>0) printf("DyGetDefaultFromPlot> Put default in %s using parameters of %s \n", Plot[id]->name, Plot[i]->name);
             break;
            }
         }
    }

 /*--- If it is a FFT plot or FTTime plot or TRfct plot or Coherence plot ---*/

 if (plottype == PLOTFFT  ||  plottype == PLOTFFTTIME  ||
     plottype == PLOTTR   ||  plottype == PLOTCOHE     || plottype == PLOTTRTIME  ||  plottype == PLOTCOHETIME)
    {
     nplot = fl_get_browser_maxline (mainbrowser->plots);
     for (i=1; i<=nplot; i++)
        {if (i != id && Plot[i]->type == plottype && Plot[i]->freqClock == Plot[id]->freqClock)
            {Plot[id]->xnBin           = Plot[i]->xnBin;
             Plot[id]->nBinP2          = Plot[i]->nBinP2;
             Plot[id]->tsize           = Plot[i]->tsize;
             Plot[id]->tstep           = Plot[i]->tstep;
             Plot[id]->refreshPeriod   = Plot[i]->refreshPeriod;
             Plot[id]->refresh_percent = Plot[i]->refresh_percent;
             Plot[id]->refresh_fft     = Plot[i]->refresh_fft;
             Plot[id]->average_fft     = Plot[i]->average_fft;
             Plot[id]->dtime           = Plot[i]->dtime;
             Plot[id]->dfreq           = Plot[i]->dfreq;
             Plot[id]->timeSize        = Plot[i]->timeSize;
             Plot[id]->xMin            = Plot[i]->xMin;
             Plot[id]->xMax            = Plot[i]->xMax;
             Plot[id]->sampFreq        = Plot[i]->sampFreq;
             Plot[id]->resampFactor    = Plot[i]->resampFactor;
             Plot[id]->crossCohe_fmin  = Plot[i]->crossCohe_fmin;
             Plot[id]->crossCohe_fmax  = Plot[i]->crossCohe_fmax;
             Plot[id]->crossCohe       = Plot[i]->crossCohe;

             if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>0) printf("DyGetDefaultFromPlot> Put default in %s using parameters of %s \n", Plot[id]->name, Plot[i]->name);
             break;
            }
         }
    }


 /*--- If it is a 1D-Time plot ---*/

 if (plottype == PLOT1DTIME)
    {
     nplot = fl_get_browser_maxline (mainbrowser->plots);
     for (i=1; i<=nplot; i++)
        {if (i != id && Plot[i]->type == plottype && Plot[i]->freqClock == Plot[id]->freqClock)
            {Plot[id]->nBin = Plot[i]->nBin;
             Plot[id]->tsize = Plot[i]->tsize;
             Plot[id]->refreshPeriod = Plot[i]->refreshPeriod;
             Plot[id]->refresh_percent= Plot[i]->refresh_percent;
             Plot[id]->timeSize = Plot[i]->timeSize;
             Plot[id]->xMin = Plot[i]->xMin;
             Plot[id]->xMax = Plot[i]->xMax;
             Plot[id]->sampFreq      = Plot[i]->sampFreq;
             Plot[id]->resampFactor  = Plot[i]->resampFactor;
             if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>0) printf("DyGetDefaultFromPlot> Put default in %s using parameters of %s \n", Plot[id]->name, Plot[i]->name);
             break;
            }
         }
    }

 /*--- If it is a 1D plot ---*/

 if (plottype == PLOT1D)
    {
     nplot = fl_get_browser_maxline (mainbrowser->plots);
     for (i=1; i<=nplot; i++)
        {if (i != id && Plot[i]->type == plottype && Plot[i]->freqClock == Plot[id]->freqClock)
            {Plot[id]->xnBin = Plot[i]->xnBin;
             Plot[id]->refreshPeriod = Plot[i]->refreshPeriod;
             Plot[id]->xMin = Plot[i]->xMin;
             Plot[id]->xMax = Plot[i]->xMax;
             Plot[id]->sampFreq      = Plot[i]->sampFreq;
             Plot[id]->resampFactor  = Plot[i]->resampFactor;
             if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>0) printf("DyGetDefaultFromPlot> Put default in %s using parameters of %s \n", Plot[id]->name, Plot[i]->name);
             break;
            }
         }
   }

 /*--- If it is a 2D plot ---*/

 if (plottype == PLOT2D || plottype == PLOT3D)
    {
     nplot = fl_get_browser_maxline (mainbrowser->plots);
     for (i=1; i<=nplot; i++)
        {if (i != id && Plot[i]->type == plottype && Plot[i]->freqClock == Plot[id]->freqClock)
            {Plot[id]->xnBin = Plot[i]->xnBin;
             Plot[id]->ynBin = Plot[i]->ynBin;
             Plot[id]->tstep = Plot[i]->tstep;
             Plot[id]->refreshPeriod = Plot[i]->refreshPeriod;
             Plot[id]->xMin = Plot[i]->xMin;
             Plot[id]->xMax = Plot[i]->xMax;
             Plot[id]->yMin = Plot[i]->yMin;
             Plot[id]->yMax = Plot[i]->yMax;
             Plot[id]->sampFreq      = Plot[i]->sampFreq;
             Plot[id]->resampFactor  = Plot[i]->resampFactor;
             if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>0) printf("DyGetDefaultFromPlot> Put default in %s using parameters of %s \n", Plot[id]->name, Plot[i]->name);
             break;
            }
         }
    }

 /*--- If it is a AUDIO plot ---*/

 if (plottype == PLOTAUDIO)
    {
     nplot = fl_get_browser_maxline (mainbrowser->plots);
     for (i=1; i<=nplot; i++)
        {if (i != id && Plot[i]->type == plottype && Plot[i]->freqClock == Plot[id]->freqClock)
            {Plot[id]->duration = Plot[i]->duration;
             Plot[id]->t0       = Plot[i]->t0;
             if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>0) printf("DyGetDefaultFromPlot> Put default in %s using parameters of %s \n", Plot[id]->name, Plot[i]->name);
             break;
           }
        }
    }

 return;
}


/*------------------------------------------- DyPlotInit ---*/

void DyPlotInit (long id, long arg)
{
 char dummy[LSTRING_MAX];

 sprintf(dummy,"DyPlotInit> plot%d  name=%s  type=%s", id, Plot[id]->name, Dy->typeName[Plot[id]->type]);
 DyDebugPrint(dummy, 0);

 if (arg == 0)                    /* Init also parameters not seen in plot panel */
    {strcpy(Plot[id]->vVarx,"");
     strcpy(Plot[id]->vVary,"");
     strcpy(Plot[id]->name,"");
     Plot[id]->type       = NONE;
     Plot[id]->freqClock  = 0.;
     Plot[id]->freqClock2 = 0.;
     Plot[id]->freqClock12= 0.;
     Plot[id]->clockN     = 0;
     Plot[id]->clockN2    = 0;
     Plot[id]->clockN12   = 0;
     Plot[id]->superposed = OFF;
    }

 Plot[id]->xnBin         = 0;
 Plot[id]->xMin          = 0.;
 Plot[id]->xMax          = 0.;
 Plot[id]->ynBin         = 0;
 Plot[id]->yMin          = 0.;
 Plot[id]->yMax          = 0.;
 Plot[id]->zMax          = 0.;
 Plot[id]->nSampling     = 0.;
 Plot[id]->refreshPeriod = 0;
 Plot[id]->resampFactor  = 0;
 Plot[id]->sampFreq      = 0.;
 Plot[id]->tsize         = 0.;
 Plot[id]->tstep         = 0.;
 Plot[id]->timeSize      = 0.;
 Plot[id]->average_fft   = 0;
 Plot[id]->decayfft      = 0;
 Plot[id]->rmsfft        = 0;
 Plot[id]->t0            = 0;
 Plot[id]->duration      = 0;
 Plot[id]->dtime         = 0.;
 Plot[id]->dfreq         = 0;

 Plot[id]->nodc          = OFF;
 Plot[id]->bits          = OFF;

 Plot[id]->gridx         = OFF;
 Plot[id]->gridy         = OFF;
 Plot[id]->logx          = OFF;
 Plot[id]->logy          = OFF;
 Plot[id]->logz          = OFF;
 Plot[id]->stat          = OFF;
 Plot[id]->type2d        = 0;
 Plot[id]->autoEdge      = OFF;
 Plot[id]->autoScale     = OFF;
 Plot[id]->modulesqrt    = OFF;
 Plot[id]->x2av          = OFF;
 Plot[id]->minmax_resample = OFF;
 Plot[id]->hidden        = OFF;

 return;
}


/*--------------------------------------------------- DyXformSamePlot ---*/
/*   Called when Superpose button is pushed. Superpose to the first plot */
/*   selected all the other plots selected.                              */

void DyXformSamePlot (FL_OBJECT *obj, long arg)
{
 char strnew[LSTRING_MAX];
 long nplot, i, i0, iref, iref0, imax, imin, nbin;
 double maxFreq, minFreq;

 DyDebugPrint(__FUNCTION__, 0);

 nplot = fl_get_browser_maxline(mainbrowser->plots);
 if (nplot <= 0) return;

 /*--- First plot selected is the reference one over which the others will be superposed ---*/

 iref = -1;
 for (i=1; i<=nplot; i++)
    {if (fl_isselected_browser_line(mainbrowser->plots,i)) {iref = i; break;}}
 if (iref <= 0) return;

if (Plot[iref]->type == PLOT3D || Plot[iref]->type == PLOT2D || Plot[iref]->type == PLOTFFTTIME || Plot[iref]->type == PLOT1DTIME ||
    Plot[iref]->type == PLOTTRTIME || Plot[iref]->type == PLOTCOHETIME)
   {DyInfoLine("Plot 3D, 2D, FFTTIME, 1DTIME, TRFCTTIME OR COHETIME can not be superposed"); return;} 

 iref0 = iref;

 if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>1) printf("DyXformSamePlot> iref0=%ld\n",iref0);

 /*--- Superpose only plots of same type ---*/

 for (i=iref0+1; i<=nplot; i++)
    {if (fl_isselected_browser_line(mainbrowser->plots,i))
        {if (Plot[i]->type != Plot[iref]->type) fl_deselect_browser_line(mainbrowser->plots,i);}
    }

 /*--- Which plot has the maximal sampling frequency ---*/

 maxFreq = 0.;
 imax    = 0;
 for (i=iref0+1; i<=nplot; i++)
    {if (fl_isselected_browser_line(mainbrowser->plots,i)  && Plot[i]->freqClock > maxFreq)
        {maxFreq = Plot[i]->freqClock; imax = i;}
    }

 if (imax > iref  &&  Plot[imax]->freqClock > Plot[iref0]->freqClock)
    {DyCopyPlot(0,iref0);
     DyCopyPlot(iref0,imax);
     DyCopyPlot(imax,0);
     sprintf (strnew, "%ld %s %s (s)", imax, Dy->typeName[Plot[imax]->type], Plot[imax]->name);
     fl_replace_browser_line(mainbrowser->plots, imax, strnew);
     sprintf (strnew, "%ld %s %s", iref0, Dy->typeName[Plot[iref0]->type], Plot[iref0]->name);
     fl_replace_browser_line(mainbrowser->plots, iref0, strnew);
    }

 Plot[iref0]->superposed = OFF;
 Plot[iref0]->bits = OFF;

/*--- Put all the plots with the same minimal sampling frequency ---*/
/* Not used anymore but kept for a possible future usage */
/***
 minFreq = 9999990.;
 for (i=iref0; i<=nplot; i++)
    {if (fl_isselected_browser_line(mainbrowser->plots,i)  && Plot[i]->freqClock < minFreq)
        {minFreq = Plot[i]->freqClock; imin = i;}
    }

 for (i=iref0; i<=nplot; i++)
    {if (fl_isselected_browser_line(mainbrowser->plots,i) && Plot[i]->type == PLOTTIME)
        {Plot[i]->sampFreq     = Plot[imin]->sampFreq;
         Plot[i]->resampFactor = (long)(Plot[i]->freqClock / Plot[i]->sampFreq + 0.5);
        }
    }
***/

/*--- iref0 is the main reference for the parameters alignment ---*/
/*--- iref is temporary reference to put together all the plots superposed ---*/

 i0 = 1;
 while (i0 > 0)
   {
    i0 = -1;
    for (i=iref+1; i<=nplot; i++)
       {if (fl_isselected_browser_line(mainbrowser->plots,i)) {i0 = i; break;}}
    if (i0 <= 0) break;

    /*--- If needed, copy the plot in a temporary area and remove it from browser ---*/

    if (i0 > iref+1)
       {
        if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>1) printf("DyXformSamePlot> replace iref+1=%ld by i0=%ld\n",iref+1,i0);
        DyCopyPlot(0,i0);
        for (i=i0-1; i>iref; i--) DyCopyPlot(i+1,i);
        for (i=i0-1; i>iref; i--)
           {if (fl_isselected_browser_line(mainbrowser->plots,i))
               {fl_select_browser_line(mainbrowser->plots,i+1);}
            else
               {fl_deselect_browser_line(mainbrowser->plots,i+1);}
           }
        DyCopyPlot(iref+1,0);
       }

    DyCopyPlotForSame (Plot[iref+1], Plot[iref0]);

/* hihi: Take as iref0 the plot with smallest or highest sampFreq and adapt refreshPeriod */

    iref++;
   }

 for (i=1; i<=nplot; i++)
    {
     if (Plot[i]->superposed == ON)
        {if (Plot[i]->hidden == ON) sprintf (strnew, "@C7 %ld %s %s (s)", i, Dy->typeName[Plot[i]->type], Plot[i]->name);
         else                       sprintf (strnew, "%ld %s %s (s)", i, Dy->typeName[Plot[i]->type], Plot[i]->name);
        }
     else
        {if (Plot[i]->hidden == ON) sprintf (strnew, "@C7 %ld %s %s", i, Dy->typeName[Plot[i]->type], Plot[i]->name);
         else                       sprintf (strnew, "%ld %s %s", i, Dy->typeName[Plot[i]->type], Plot[i]->name);
        }
     fl_replace_browser_line(mainbrowser->plots, i, strnew);
    }

 if (iref+1 <= nplot  &&  iref == iref0)
    {fl_select_browser_line(mainbrowser->plots,iref+1);
     fl_deselect_browser_line(mainbrowser->plots,iref);
    }

 DyXformAutoDeselect(mainbrowser->plots,0);

 return;
}


/*------------------------------------------------------------ DyXformUnsamePlot ---*/
/*   Called when button Unsuperposed is pushed. Unsuperposed all the selected plots */

void DyXformUnsamePlot (FL_OBJECT *obj, long arg)
{
 char strnew[LSTRING_MAX];
 long nplot, i;

 DyDebugPrint(__FUNCTION__,0);

 nplot = fl_get_browser_maxline(mainbrowser->plots);
 if (nplot <= 0) return;

 for (i=1; i<=nplot; i++)
    {if (fl_isselected_browser_line(mainbrowser->plots,i) && Plot[i]->superposed == ON)
        {Plot[i]->superposed = OFF;
         if (Plot[i]->hidden == ON) sprintf (strnew, "@C7 %ld %s %s", i, Dy->typeName[Plot[i]->type], Plot[i]->name);
         else                       sprintf (strnew, "%ld %s %s", i, Dy->typeName[Plot[i]->type], Plot[i]->name);
         fl_replace_browser_line(mainbrowser->plots, i, strnew);
        }
    }

 DyXformAutoDeselect(mainbrowser->plots,0);

 return;
}


/*-------------------------------------------- DyXformExchangeVar ---*/
/*                  Exchange the 2 variables of the selected plots   */

void DyXformExchangeVar (FL_OBJECT *obj, long arg)
{
 char strnew[LSTRING_MAX], dummy[LSTRING_MAX];
 long nplot, i, clockN_dummy;
 double freqclock_dummy;

 DyDebugPrint(__FUNCTION__,0);

 nplot = fl_get_browser_maxline(mainbrowser->plots);
 if (nplot <= 0) return;

 for (i=1; i<=nplot; i++)
    {
     if (fl_isselected_browser_line(mainbrowser->plots,i) &&
         (Plot[i]->type == PLOTTR || Plot[i]->type == PLOTCOHE || Plot[i]->type == PLOT2D || Plot[i]->type == PLOT3D ||
          Plot[i]->type == PLOTTRTIME || Plot[i]->type == PLOTCOHETIME) )
        {
         strcpy(dummy, Plot[i]->vVarx);
         strcpy(Plot[i]->vVarx, Plot[i]->vVary);
         strcpy(Plot[i]->vVary, dummy);

         clockN_dummy      = Plot[i]->clockN;
         Plot[i]->clockN  = Plot[i]->clockN2;
         Plot[i]->clockN2 = clockN_dummy;

         freqclock_dummy      = Plot[i]->freqClock;
         Plot[i]->freqClock  = Plot[i]->freqClock2;
         Plot[i]->freqClock2 = freqclock_dummy;

         if (Plot[i]->type == PLOTTR || Plot[i]->type == PLOTTRTIME)
            {sprintf(Plot[i]->name, "%s%s%s", Plot[i]->vVarx, SEP2VARTR, Plot[i]->vVary);}
         else if (Plot[i]->type == PLOT3D)
            {sprintf(Plot[i]->name, "%s%s%s%s%s", Plot[i]->vVarx, SEP2VAR, Plot[i]->vVary, SEP2VAR, Plot[i]->vVarz);}
         else
            {sprintf(Plot[i]->name, "%s%s%s", Plot[i]->vVarx, SEP2VAR, Plot[i]->vVary);}

         sprintf (strnew, "%ld %s %s", i, Dy->typeName[Plot[i]->type], Plot[i]->name);
         fl_replace_browser_line(mainbrowser->plots, i, strnew);
        }
    }

 DyXformAutoDeselect(mainbrowser->plots,0);

 return;
}


/*-------------------------------------------- DyXformPlotUp ---*/
/*                            Move upwards the selected plots   */

void DyXformPlotUp (FL_OBJECT *obj, long arg)
{
 char strnew[LSTRING_MAX];
 long nplot, i, j, flag, upMade;

 if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>0) printf("DyXformPlotUp \n");

 nplot = fl_get_browser_maxline(mainbrowser->plots);
 if (nplot <= 0) return;

 for (i=1; i<=nplot-1; i++)
    {
     if (fl_isselected_browser_line(mainbrowser->plots,i) && Plot[i]->superposed == OFF)
        {for (j=i+1; j<=nplot; j++)
            {if (Plot[j]->type == Plot[i]->type && Plot[j]->superposed == ON)
                {fl_select_browser_line(mainbrowser->plots,j);}
             else break;
            }
        }
    }

 flag = 1;
 upMade = 1;
 while(flag > 0 && upMade > 0)
    {
     upMade = 0;
     for (i=2; i<=nplot; i++)
        {
         if (fl_isselected_browser_line(mainbrowser->plots,i) != 0 && 
             fl_isselected_browser_line(mainbrowser->plots,i-1) == 0)
            {
             upMade = 1;
             if (Plot[i-1]->superposed == OFF) flag = 0;
             if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>1) printf("DyXformPlotUp> Exchange plot %d and plot %d flag=%d\n", i, i-1, flag);

             DyCopyPlot(0,i);
             DyCopyPlot(i,i-1);
             DyCopyPlot(i-1,0);
             if (Plot[i]->superposed == ON)
                {if (Plot[i]->hidden == ON) sprintf (strnew, "@C7 %ld %s %s (s)", i, Dy->typeName[Plot[i]->type], Plot[i]->name);
                 else                       sprintf (strnew, "%ld %s %s (s)", i, Dy->typeName[Plot[i]->type], Plot[i]->name);
                }
             else
                {if (Plot[i]->hidden == ON) sprintf (strnew, "@C7 %ld %s %s", i, Dy->typeName[Plot[i]->type], Plot[i]->name);
                 else                       sprintf (strnew, "%ld %s %s", i, Dy->typeName[Plot[i]->type], Plot[i]->name);
                }
             fl_replace_browser_line(mainbrowser->plots, i, strnew);

             if (Plot[i-1]->superposed == ON)
                {if (Plot[i-1]->hidden == ON) sprintf (strnew, "@C7 %ld %s %s (s)", i-1, Dy->typeName[Plot[i-1]->type], Plot[i-1]->name);
                 else                         sprintf (strnew, "%ld %s %s (s)", i-1, Dy->typeName[Plot[i-1]->type], Plot[i-1]->name);
                }
             else
                {if (Plot[i-1]->hidden == ON) sprintf (strnew, "@C7 %ld %s %s", i-1, Dy->typeName[Plot[i-1]->type], Plot[i-1]->name);
                 else                         sprintf (strnew, "%ld %s %s", i-1, Dy->typeName[Plot[i-1]->type], Plot[i-1]->name);
                }
             fl_replace_browser_line(mainbrowser->plots, i-1, strnew);

             fl_deselect_browser_line(mainbrowser->plots,i);
             fl_select_browser_line(mainbrowser->plots,i-1);
            }
        }
    }

 return;
}


/*-------------------------------------------- DyXformPlotDown ---*/
/*                            Move downwards the selected plots   */

void DyXformPlotDown (FL_OBJECT *obj, long arg)
{
 char strnew[LSTRING_MAX];
 long nplot, i, j, flag, downMade;

 DyDebugPrint(__FUNCTION__,0);

 nplot = fl_get_browser_maxline(mainbrowser->plots);
 if (nplot <= 0) return;

 for (i=1; i<=nplot-1; i++)
    {
     if (fl_isselected_browser_line(mainbrowser->plots,i) && Plot[i]->superposed == OFF)
        {for (j=i+1; j<=nplot; j++)
            {if (Plot[j]->type == Plot[i]->type && Plot[j]->superposed == ON)
                {fl_select_browser_line(mainbrowser->plots,j);}
             else break;
            }
        }
    }

 for (i=nplot-1; i>=1; i--)
    {
     if (fl_isselected_browser_line(mainbrowser->plots,i) != 0 && 
         fl_isselected_browser_line(mainbrowser->plots,i+1) == 0)
        {
         if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>1) printf("DyXformPlotDown> step1> Exchange plot %d and plot %d \n", i, i+1);

         DyCopyPlot(0,i);
         DyCopyPlot(i,i+1);
         DyCopyPlot(i+1,0);
         if (Plot[i]->superposed == ON)
            {if (Plot[i]->hidden == ON) sprintf (strnew, "@C7 %ld %s %s (s)", i, Dy->typeName[Plot[i]->type], Plot[i]->name);
             else                       sprintf (strnew, "%ld %s %s (s)", i, Dy->typeName[Plot[i]->type], Plot[i]->name);
            }
         else
            {if (Plot[i]->hidden == ON) sprintf (strnew, "@C7 %ld %s %s", i, Dy->typeName[Plot[i]->type], Plot[i]->name);
             else                       sprintf (strnew, "%ld %s %s", i, Dy->typeName[Plot[i]->type], Plot[i]->name);
            }
         fl_replace_browser_line(mainbrowser->plots, i, strnew);

         if (Plot[i+1]->superposed == ON)
            {if (Plot[i+1]->hidden == ON) sprintf (strnew, "@C7 %ld %s %s (s)", i+1, Dy->typeName[Plot[i+1]->type], Plot[i+1]->name);
             else                         sprintf (strnew, "%ld %s %s (s)", i+1, Dy->typeName[Plot[i+1]->type], Plot[i+1]->name);
            }
         else
            {if (Plot[i+1]->hidden == ON) sprintf (strnew, "@C7 %ld %s %s", i+1, Dy->typeName[Plot[i+1]->type], Plot[i+1]->name);
             else                         sprintf (strnew, "%ld %s %s", i+1, Dy->typeName[Plot[i+1]->type], Plot[i+1]->name);
            }
         fl_replace_browser_line(mainbrowser->plots, i+1, strnew);

         fl_deselect_browser_line(mainbrowser->plots,i);
         fl_select_browser_line(mainbrowser->plots,i+1);
        }
    }

 flag = 1;
 downMade = 1;
 while(flag > 0 && downMade > 0)
    {
     downMade = 0;
     for (i=nplot-1; i>=1; i--)
        {
         if (fl_isselected_browser_line(mainbrowser->plots,i) != 0 && 
             fl_isselected_browser_line(mainbrowser->plots,i+1) == 0)
            {
             downMade = 1;
             if (Plot[i+1]->superposed == OFF) {flag = 0; continue;}
             if (((Dy->debugType)&(1<<DBG_PLOT)) && Dy->debug>1) printf("DyXformPlotDown> step2> Exchange plot %d and plot %d flag=%d\n", i, i+1, flag);

             DyCopyPlot(0,i);
             DyCopyPlot(i,i+1);
             DyCopyPlot(i+1,0);
             if (Plot[i]->superposed == ON)
                {if (Plot[i]->hidden == ON) sprintf (strnew, "@C7 %ld %s %s (s)", i, Dy->typeName[Plot[i]->type], Plot[i]->name);
                 else                       sprintf (strnew, "%ld %s %s (s)", i, Dy->typeName[Plot[i]->type], Plot[i]->name);
                }
             else
                {if (Plot[i]->hidden == ON) sprintf (strnew, "@C7 %ld %s %s", i, Dy->typeName[Plot[i]->type], Plot[i]->name);
                 else                       sprintf (strnew, "%ld %s %s", i, Dy->typeName[Plot[i]->type], Plot[i]->name);
                }
             fl_replace_browser_line(mainbrowser->plots, i, strnew);

             if (Plot[i+1]->superposed == ON)
                {if (Plot[i+1]->hidden == ON) sprintf (strnew, "@C7 %ld %s %s (s)", i+1, Dy->typeName[Plot[i+1]->type], Plot[i+1]->name);
                 else                         sprintf (strnew, "%ld %s %s (s)", i+1, Dy->typeName[Plot[i+1]->type], Plot[i+1]->name);
                }
             else
                {if (Plot[i+1]->hidden == ON) sprintf (strnew, "@C7 %ld %s %s", i+1, Dy->typeName[Plot[i+1]->type], Plot[i+1]->name);
                 else                         sprintf (strnew, "%ld %s %s", i+1, Dy->typeName[Plot[i+1]->type], Plot[i+1]->name);
                }
             fl_replace_browser_line(mainbrowser->plots, i+1, strnew);

             fl_deselect_browser_line(mainbrowser->plots,i);
             fl_select_browser_line(mainbrowser->plots,i+1);
            }
        }
    }

 return;
}


